function [theta,newbeta,Z3] = alpha_beta_alt_estimate(Z0,Z1,Z2,Z3,newbeta,Theta,Rmat,rvec,dcoint,q1,modelsdir,modelfile,d_coint,d_corel,d_coidet)

%
% this function is almost a replica of alpha_beta_restrict except we now allow for additional
% variables either in the cointegration space or outside
%

n = size(Theta,1);
r = size(newbeta,2);
T = size(Z0,2);

curr_ini = find_inisettings;

if curr_ini.convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
end;
ccrit = mystr2num(ch_convcrit);
%
% starting up the estimation by getting a restricted estimate of alpha and Psi
%

[theta,Sigma_theta,infomat,abort] = linrest(Z0,Z3,Theta,Rmat,rvec,0,1);

g2 = size(theta,2);
alphar = theta(:,g2-r+1:g2);
if g2>r;
   Psir = theta(:,1:g2-r);
else;
   Psir = [];
end;
neweps = Z0-(theta*Z3);
newlnl = loglike(neweps);
newOmega = (1/T)*neweps*neweps';
sivec = zeros(r,1);
%
% bH is needed to check the gradient of the likelihood function
% wrt the beta parameters
%
%
% Get the normalizing variables
%
file = strcat(modelsdir,'normalize-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.norm');
if fileexist(file)==1;
   normvec = loaddata(file);
else;
   normvec = [];
end;
H = [];
for i=1:r;
   file = strcat(modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.rest');
   if fileexist(file)==1;
      h = loaddata(file);
      if d_coint>0;
         h = [h zeros(size(h,1),d_coint);zeros(d_coint,size(h,2)) eye(d_coint)];
      elseif d_corel==1;
         if isempty(normvec)==0;
            H_fix = h(normvec(i,1),:);
            j = 1;
            while j<=size(h,1);
              if size(H_fix,1)<size(H_fix,2);
                 if j~=normvec(i,1);
                    H_test = [H_fix;h(j,:)];
                    if rank(H_test)==size(H_test,1);
                       H_fix = H_test;
                    end;
                 end;
                 j = j+1;
              else;
                 j = size(h,1)+1;
              end;
            end;
            h = h/H_fix;
            %
            h1 = h(:,1);
            if size(h,2)>1;
               h2 = h(:,2:size(h,2));
               h = [h1 h2 zeros(size(h2));h1 zeros(size(h2)) h2];
            else;
               h = [h1;h1]
            end;
         else;
            h = [h zeros(size(h));zeros(size(h)) h];
         end;
         %
         if d_coidet==1;
            %
            % check if there's a constant to remove
            %
            keep_rows = (1:1:size(h,1)-1);
            h_ch = h(size(h,1),:);
            keep_cols = 1;
            for i1=2:size(h_ch,2);
               if isequal(h_ch(1,i1),0)==1;
                  keep_cols = [keep_cols;i1];
               end;
            end;
            h = h(keep_rows,keep_cols);
         end;
      end;
      if i==1;
         bH = h;
      else;
         bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
      end;
      H = [H h];
      sivec(i,1) = size(h,2);
   end;
end;


%
% Setting up the normalizing restrictions
%

if isempty(normvec)==0;
   nq1 = size(newbeta,1);
   addrestmat = zeros(nq1*r,r);
   for i=1:r;
      thisnorm = normvec(i,1);
      addrestmat(thisnorm+nq1*(i-1),i) = 1;
   end;
else;
   addrestmat = [];
end;

%
% constructing restriction matrices for beta
%

if d_corel==0;
   Knr = commutation(n+q1+dcoint+d_coint,r);
else;
   if d_coidet==0;
      Knr = commutation(2*(n+q1+dcoint),r);
   else;
      Knr = commutation((2*(n+q1+dcoint))-1,r);
   end;
end;
bH_ort = null(bH','r');
bH_ort = [bH_ort addrestmat];
bRmat = bH_ort'*Knr';
if isempty(addrestmat)==1;
   brvec = zeros(size(bRmat,1),1);
else;
   brvec = [zeros(size(bRmat,1)-r,1);ones(r,1)];
end;
bH = null(bH_ort','r');

%
% Computing the gradient using the new theta matrix
%

M_01 = (1/T)*Z0*Z1';
M_11 = (1/T)*Z1*Z1';
if isempty(Z2)==0;
   M_21 = (1/T)*Z2*Z1';
   M_12 = M_21';
   M_22 = (1/T)*Z2*Z2';
else;
   M_21 = [];
   M_12 = [];
   M_22 = [];
end;
if rank(newOmega,eps)<size(newOmega,1);
   inv_newOmega = eye(n);
else;
   inv_newOmega = newOmega\eye(n);
end;
if isempty(M_21)==0;
   grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(M_01-(alphar*newbeta'*M_11)-(Psir*M_21)))*bH;
else;
   grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(M_01-(alphar*newbeta'*M_11)))*bH;
end;
grad = max(abs(grad));

%
% setting up iterations if needed
%
if grad>ccrit;
   it = 1;
   while (it<=curr_ini.maxit)&(grad>ccrit);
      oldgrad = grad;
      oldlnl = newlnl;
      %
      % create new R0 and R1 series using Psir, alphar, and newbeta
      %
      if isempty(Z2)==0;
         newR0 = Z0-(Psir+(alphar*newbeta'*(M_12/M_22)))*Z2;
         newR1 = Z1-(M_12/M_22)*Z2;
      else;
         newR0 = Z0;
         newR1 = Z1;
      end;
      %
      % Constructing unrestricted beta estimate
      %
      S_00_new = (1/T)*newR0*newR0';
      S_10_new = (1/T)*newR1*newR0';
      S_11_new = (1/T)*newR1*newR1';
      S_01_new = (1/T)*newR0*newR1';
      alpharort = null(alphar');
      alpharbar = alphar/(alphar'*alphar);
      S_11_alpha = S_11_new-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_01_new)));
      S_10_alpha = S_10_new-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_00_new)));
      S_00_alpha = S_00_new-(S_00_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_00_new)));
      newbeta = (S_11_alpha)\(S_10_alpha*alpharbar);
      %
      % Computing restricted newbeta estimate using R0tilde and R1tilde
      %
      R0tilde = alpharbar'*(eye(n)-(S_00_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'))))*newR0;
      R1tilde = newR1-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\alpharort'))*newR0;
      [thisbeta,sig_beta,infmat,newabort] = linrest(R0tilde,R1tilde,newbeta',bRmat,brvec,0,1);
      newbeta = thisbeta';
      if isempty(Z2)==0;
         Z3 = [Z2;newbeta'*Z1];
      else;
         Z3 = newbeta'*Z1;
      end;
      [theta,Sigma_theta,infomat,newabort,inv_Omega] = linrest(Z0,Z3,theta,Rmat,rvec,0,1);
      g2 = size(theta,2);
      alphar = theta(:,g2-r+1:g2);
      if g2>r;
         Psir = theta(:,1:g2-r);
      else;
         Psir = [];
      end;
      neweps = Z0-(theta*Z3);
      newlnl = loglike(neweps);
      newOmega = (1/T)*neweps*neweps';
      if curr_ini.betagrad==0;
         grad = abs((newlnl-oldlnl)/oldlnl);
         oldgrad = 0;
      else;
         if isempty(M_21)==0;
            oldgrad = T*(vec(alphar))'*kron(eye(r),inv_Omega*(M_01-(alphar*newbeta'*M_11)-(Psir*M_21)))*bH;
         else;
            oldgrad = T*(vec(alphar))'*kron(eye(r),inv_Omega*(M_01-(alphar*newbeta'*M_11)))*bH;
         end;
         oldgrad = max(abs(oldgrad));
         if oldgrad>min([ccrit*10 1.0e-006]);
            oldgrad = 0;
         end;
         if rank(newOmega,eps)<size(newOmega,1);
            inv_newOmega = eye(n);
         else;
            inv_newOmega = newOmega\eye(n);
         end;
         if isempty(M_21)==0;
            grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(M_01-(alphar*newbeta'*M_11)-(Psir*M_21)))*bH;
         else;
            grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(M_01-(alphar*newbeta'*M_11)))*bH;
         end;
      end;
      grad = abs(max(abs(grad))-oldgrad);
      infomat = [it grad];
      it = it+1;
   end;
end;

%
% end of alpha_beta_alt_estimate.m
%
