function [pos,bplot] = bayes_parm_dist(pos,fig_tag,fig_name,p_title,tm,curr_ini,p_dist,param_name,col)

cdf_value = tm.cumdist;
my_lim = 1-tm.cbs/100;
l_lim = my_lim/2;
u_lim = (tm.cbs/100)+l_lim;

mean_val = (1/size(p_dist,1))*sum(p_dist);
med_val = median(p_dist);
draws = size(p_dist,1);
l_val = p_dist(ceil(l_lim*draws),1);
u_val = p_dist(ceil(u_lim*draws),1);

bplot = figure('Units','pixels', ...
   'Position',[pos 600 400], ...
   'NumberTitle','off', ...
   'Menubar','none', ...
   'Toolbar','none', ...
   'Tag',fig_tag, ...
   'Visible','off', ...
   'Name',fig_name);
%
pos = [pos(1)+22 pos(2)-22];
if pos(1,1)>(tm.realscrsz(3)-600);
   pos = [0 tm.realscrsz(4)-470];
end;
if pos(1,2)>(tm.realscrsz(4)-470);
   pos = [0 tm.realscrsz(4)-470];
elseif pos(1,2)<32;
   pos = [0 tm.realscrsz(4)-470];
end;
%
% obey centering and filling defaults
%
setfigureproperties(bplot);
%
[n1,n2] = hist(p_dist,tm.numcells);
n1 = n1/draws;
if (strcmp(tm.interpolate,'nearest')==1)|(strcmp(tm.interpolate,'linear')==1)|(strcmp(tm.interpolate,'spline')==1)|(strcmp(tm.interpolate,'cubic')==1);
   x_v = [];
   xsep = (n2(2)-n2(1))/100;
   for i1=1:size(n2,2);
      x_v = [x_v n2(i1)];
      if i1<size(n2,2);
         for j=1:99;
            x_v = [x_v (n2(i1)+(j*xsep))];
         end;
      end;
   end;
   x_v = sort(x_v);
end;
%
%
ylabelstr = 'Frequency';
leg_pos = 1;
if strcmp(tm.graph_type,'histogram')==1;
   if cdf_value==1;
      n1 = cumsum(n1);
      set(bplot,'Name',strrep(fig_name,'Density','Distribution'));
   end;
   %
   n3 = bar(n2,n1,0.75);
   this_axes = gca;
   if curr_ini.colors==1;
      set(n3,'FaceColor',[0 0 0.8],'EdgeColor','k','LineWidth',1,'Tag','autopatch');
   else;
      set(n3,'FaceColor',[0.5 0.5 0.5],'EdgeColor','k','LineWidth',1,'Tag','autopatch');
   end;
   myleg = [];
   if cdf_value==1;
      set(gca,'YLim',[0 1.1]);
   end;
else;
   ylabelstr = 'Density';
   %
   if (strcmp(tm.interpolate,'nearest')==1)|(strcmp(tm.interpolate,'linear')==1)|(strcmp(tm.interpolate,'spline')==1)|(strcmp(tm.interpolate,'cubic')==1);
      n3 = interp1(n2,n1,x_v,tm.interpolate);
      cdf_n3 = cdf(x_v,n3);
      n3 = (1/max(cdf_n3))*n3;
   else;
      xmin = min(p_dist);
      xmax = max(p_dist);
      xrange = xmax-xmin;
      ax = xmin-xrange/4;
      bx = xmax+xrange/4;
      if strcmp(tm.interpolate,'knorm')==1;
         h = 0;
      else;
         h = hns(p_dist,tm.interpolate);
      end;
      if tm.adaptivekde==1;
         [x_v,xa,n3,h,hl] = kdadapt(p_dist,tm.interpolate,512,h);
      else;
         [x_v,n3] = kdfft1(p_dist,tm.interpolate,512,h,0,[ax,bx]);
      end;
   end;
   [a,ind] = max(n3);
   mode_val = x_v(1,ind);
   if cdf_value==1;
      ylabelstr = 'Frequency';
      leg_pos = 4;
      n3 = cdf(x_v,n3);
      set(bplot,'Name',strrep(fig_name,'Density','Distribution'));
      p_title = strrep(p_title,'Density','Distribution');
   end;
   %
   y_lim = [0 1.1*max(n3)];
   %
   my_lines = [];
   %
   new_line = plot(x_v,n3,'LineWidth',1,'Color',col.vec_1,'Tag','parm-dist');
   my_lines = [my_lines new_line];
   hold('on');
   this_axes = gca;
   set(this_axes,'YLim',y_lim);
   set(this_axes,'XLim',[min(x_v) max(x_v)]);
   %

%   if isempty(est_parm)==0;
%      new_line = plot(parm*ones(1,2),y_lim,'Color',col.vec_5,'LineStyle','-','LineWidth',1);
%      my_lines = [my_lines new_line];
%      hold('on');
%   end;
   %
   % plotting mean, mode, median, and x percent confidence interval
   %
   new_line = plot(mean_val*ones(1,2),y_lim,'Color',col.vec_2,'LineStyle','--','LineWidth',1);
   my_lines = [my_lines new_line];
   hold('on');
   new_line = plot(mode_val*ones(1,2),y_lim,'Color',col.vec_5,'LineStyle','-','LineWidth',1);
   my_lines = [my_lines new_line];
   hold('on');
   new_line = plot(med_val*ones(1,2),y_lim,'Color',col.vec_3,'LineStyle','-.','LineWidth',1);
   my_lines = [my_lines new_line];
   hold('on');
   %
   % plotting edge line for confidence band
   %
   plot(l_val*ones(1,2),y_lim,'Color',col.vec_6,'LineStyle',':','LineWidth',1,'Tag','nolegend-lower');
   hold('on');
   plot(u_val*ones(1,2),y_lim,'Color',col.vec_6,'LineStyle',':','LineWidth',1,'Tag','nolegend-upper');
   x_lim = get(gca,'XLim');
   if (x_lim(1,1)<0)&(x_lim(1,2)>0);
      %
      % plot a vertical zero line
      %
      hold('on');
      plot(zeros(1,2),y_lim,'Color','k','LineStyle','-','LineWidth',1)
   end;
   myleg = legend_fcn(my_lines,param_name,'Mean','Mode','Median',leg_pos);
end;
setaxesfonts('Title',p_title,'XLabel','Value','YLabel',ylabelstr);
if isempty(myleg)==0;
   setaxesfonts(myleg);
   myleg = update_legend(myleg);
end;
set(this_axes,'UserData',myleg,'Layer','top');
toolbar_fcn(this_axes);
hold('off');

%
% end of bayes_parm_dist.m
%
