%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying graphs of forecasts, %
%          confidence bands, and outcomes (if any). Only out-of- %
%          sample forecasts are permitted                        %
%                                                                %
%          Function: bayes_var_forecastdlg                       %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       c_mod: structure with model info         %
%                       posterior_data: structure with parameter %
%                          draws from the posterior distribution %
%                                                                %
%          Output:                                               %
%                       h: handle                                %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by:          Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = bayes_var_forecastdlg(selector,c_mod,posterior_data)

if strcmp(selector,'init')==1;
   tm = c_mod;
   tm.posterior_data = posterior_data;
   tm.num_fdraws = 1000;
   bayesdlg = findobj('Type','figure','Tag','BayesGUI')';
   tm.controls = get(bayesdlg,'UserData');
   %
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
   %
   % check if all output files exist
   %
   res_exist = 1;
   for i=1:tm.save_draws
      if get(tm.controls.betarest,'Value')==1;
         dfbeta = dfbeta_fcn(tm);
         file = [curr_ini.simulatedir tm.modelfile '-betarest-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '.' num2str(dfbeta,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f')];
         if tm.prior_inf_alpha==1;
            file = [file '.' num2str(tm.lambda_alpha,'%0.3f')];
         end;
      else;
         file = [curr_ini.simulatedir tm.modelfile '-rank-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f') '.' num2str(tm.lambda_alpha,'%0.3f')];
      end;
      if tm.prior_inf_shortrun==1;
         file = [file '.' num2str(tm.lambda_b,'%0.3f') '.' num2str(tm.lambda_theta,'%0.3f') '.' num2str(tm.lambda_l,'%0.3f') '.' num2str(tm.lambda_e,'%0.3f')];
      end;
      if tm.prior_inf_deter==1;
         file = [file '.' num2str(tm.lambda_d,'%0.3f')];
      end;
      file = [file '-' num2str(tm.save_draws,'%0.0f') '.' num2str(i,'%0.0f') '.' num2str(tm.num_fdraws,'%0.0f') '.' num2str(tm.h,'%0.0f') '-' num2str(tm.d+tm.dcoint,'%0.0f') '.' num2str(tm.datad,'%0.0f') '.mat'];
      if fileexist(file)==0;
         res_exist = 0;
         break;
      end;
   end;
   %
   % create string with variables
   %
   tm.allvars = tm.myvars;
   for i=1:size(tm.myvars,1);
      tm.allvars = strvcat(tm.allvars,[strtrim(tm.myvars(i,:)) '(t)-' strtrim(tm.myvars(i,:)) '(t-1)']);
   end;
   %
   % now we add sums of levels and sums of first diffs if we have quarterly or monthly data
   %
   if (strcmp(tm.freq,'q')==1)|(strcmp(tm.freq,'m')==1);
      if strcmp(tm.freq,'q')==1;
         add_freq = '(t-4)';
         add_sum = '3';
      else;
         add_freq = '(t-12)';
         add_sum = '11';
      end;
      %
      for i=1:size(tm.myvars,1);
         tm.allvars = strvcat(tm.allvars,['sum(0,' add_sum ')'strtrim(tm.myvars(i,:)) '(t-i)']);
      end;
      %
      for i=1:size(tm.myvars,1);
         tm.allvars = strvcat(tm.allvars,[strtrim(tm.myvars(i,:)) '(t)-' strtrim(tm.myvars(i,:)) add_freq]);
      end;
      %
   end;
   %
   % create sample strings
   %
   p_str = 'year';
   if strcmp(tm.freq,'q')==1;
      p_str = 'quarter';
   elseif strcmp(tm.freq,'m')==1;
      p_str = 'month'
   end;
   tm.sample_string = ['1 ' p_str];
   for i=2:tm.h;
      tm.sample_string = strvcat(tm.sample_string,[num2str(i,'%0.0f') ' ' p_str 's']);
   end;
   %
   thisexit = 0;
else;
   thisexit = 1;
   forecastgui = findobj('Type','figure','Tag','BayesVARForecastGUI')';
   f_cast = get(forecastgui,'UserData');
   curr_ini = get(f_cast.firstvar,'UserData');
   tm = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      forecastgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','bayes_var_forecastdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-480)/2 (realscrsz(4)-220)/2 480 220], ...
         'Tag','BayesVARForecastGUI', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Graph Transformed Variable Forecasts', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','bayes_var_forecastdlg wayout', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(forecastgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 478 210],'Forecast Selections',96,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[426 160 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.forecastbig_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      %
      % Forecast period selection
      %

      f_cast.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 173 180 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Forecast number of periods:', ...
         'HorizontalAlignment','left', ...
         'Tag','PeriodText');

      f_cast.numperiods = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 155 330 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.sample_string, ...
         'HorizontalAlignment','center', ...
         'CallBack','bayes_var_forecastdlg checkfile', ...
         'Tag','GraphListbox',...
         'Value',tm.h);

      %
      % Select forecast variable
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 128 100 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Forecast variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','FormatText');


      f_cast.first_constant = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 110 50 20], ...
         'Style','edit', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','1', ...
         'Callback','bayes_var_forecastdlg check_constant', ...
         'HorizontalAlignment','right', ...
         'Tag','ConstantEdit');

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[66 108 10 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','x', ...
         'HorizontalAlignment','center', ...
         'Tag','DashText');

      f_cast.firstvar = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[76 110 100 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.allvars, ...
         'Callback','bayes_var_forecastdlg checkvar', ...
         'HorizontalAlignment','center', ...
         'Tag','TypePopup',...
         'Value',1);

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[176 108 10 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','+', ...
         'HorizontalAlignment','center', ...
         'Tag','DashText');

      f_cast.constant = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[186 110 50 20], ...
         'Style','edit', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','0', ...
         'Callback','bayes_var_forecastdlg checkconstant', ...
         'HorizontalAlignment','right', ...
         'Tag','ConstantEdit');

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 108 10 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','x', ...
         'HorizontalAlignment','center', ...
         'Tag','DashText');

      f_cast.secondvar = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[246 110 100 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.allvars, ...
         'Callback','bayes_var_forecastdlg checkvar', ...
         'HorizontalAlignment','center', ...
         'Tag','SecondTypePopup',...
         'Value',2);

      %
      % Select transformation function (53 pixels up)
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 83 210 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Transformation function:', ...
         'HorizontalAlignment','left', ...
         'Tag','TransformText');

      f_cast.transform = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 65 330 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','none|exp(variable)|exp(variable)-1|100*exp(variable)|100*(exp(variable)-1)|4*variable|12*variable|400*variable|1200*variable|(1+variable)^4|(1+variable)^4-1|100*(1+variable)^4|100*((1+variable)^4-1)|(1+variable)^12|(1+variable)^12-1|100*(1+variable)^12|100*((1+variable)^12-1)', ...
         'HorizontalAlignment','center', ...
         'Tag','TransformPopup',...
         'Value',1);

      %
      % Select number of draws per parameter draw
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 38 330 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Number of forecast draws per parameter draw:', ...
         'HorizontalAlignment','left', ...
         'Tag','NumdrawsText');

      f_cast.numdraws = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 20 330 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','100|200|300|400|500|600|700|800|900|1,000|1,250|1,500|1,750|2,000|2,500|3,000|3,500|4,000|4,500|5,000|5,500|6,000|6,500|7,000|7,500|8,000|8,500|9,000|9,500|10,000', ...
         'Callback','bayes_var_forecastdlg check_numdraws', ...
         'HorizontalAlignment','center', ...
         'Tag','NumDrawsPopup',...
         'Value',10);

      %
      % Create graph button
      %

      f_cast.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[362 45 100 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','bayes_var_forecastdlg showgraph', ...
         'Enable','off', ...
         'UserData','working', ...
         'Tag','ShowButton');

      if res_exist==1;
         set(f_cast.show,'Enable','on');
      end;

      f_cast.compute = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[362 70 100 20], ...
         'String','Compute', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','bayes_var_forecastdlg compute', ...
         'Enable','on', ...
         'Tag','ComputeButton');

      if tm.datad>0;
         f_cast.makedeter = uicontrol('Units','pixels', ...
            'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
            'Position',[362 95 100 20], ...
            'String','Create Dummy...', ...
            'FontSize',curr_ini.gui_fontsize, ...
            'FontName',curr_ini.gui_fontname, ...
            'FontWeight',curr_ini.gui_fontweight, ...
            'FontAngle',curr_ini.gui_fontangle, ...
            'CallBack','bayes_var_forecastdlg makedeter', ...
            'Enable','on', ...
            'Tag','MakeDeterButton');

         f_cast.displaydeter = uicontrol('Units','pixels', ...
            'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
            'Position',[362 120 100 20], ...
            'String','Display Dummy...', ...
            'FontSize',curr_ini.gui_fontsize, ...
            'FontName',curr_ini.gui_fontname, ...
            'FontWeight',curr_ini.gui_fontweight, ...
            'FontAngle',curr_ini.gui_fontangle, ...
            'CallBack','bayes_var_forecastdlg display_deter', ...
            'Enable','on', ...
            'Tag','DisplayDeterButton');
         file = [tm.modelsdir tm.modelfile '-deter-forecast-' num2str(tm.datad,'%0.0f') '-' num2str(tm.h,'%0.0f') '.mat'];
         if fileexist(file)==0;
            set(f_cast.compute,'Enable','off');
            set(f_cast.displaydeter,'Enable','off');
         end;
      end;

      %
      %  Done button
      %

      f_cast.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[362 20 100 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','bayes_var_forecastdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      set(forecastgui,'UserData',f_cast,...
         'HandleVisibility','callback');

   case 'display_deter'
      num_fp = get(f_cast.numperiods,'Value');
      file = [tm.modelsdir tm.modelfile '-deter-forecast-' num2str(tm.datad,'%0.0f') '-' num2str(num_fp,'%0.0f') '.mat'];
      a = load(file);
      deter_mat = a.deter_mat;
      %
      % construct date index for actual values and the forecasts
      %
      [dataindex,sample_beg,sample_end] = make_dataindex(tm.freq,tm.samplebeg,tm.sampleend);
      dataindex = dataindex';
      add_on = dataindex(1,2)-dataindex(1,1);
      forecast_index = (add_on:add_on:(add_on*num_fp));
      forecast_index = forecast_index+dataindex(1,tm.T);
      if curr_ini.colors==1;
         colorstr = 'b';
      else;
         colorstr = 'k';
      end;
      %
      deter_str = vecstr(tm.deter);
      deter_str = deter_str(tm.d-tm.datad+1:tm.d,:);
      my_plots = [];
      for i=1:size(deter_mat,1);
         thisplot = figure('Units','pixels', ...
            'Position',[(realscrsz(3)-512)/2 (realscrsz(4)-384)/2 512 384], ...
            'Menubar','none', ...
            'Toolbar','none', ...
            'NumberTitle','off', ...
            'Tag',['future-' strtrim(deter_str(i,:))], ...
            'Visible','off', ...
            'Name',['Time Series Plot of ' strtrim(deter_str(i,:))]);
         %
         % default figure properties
         %
         my_plots = [my_plots thisplot];
         setfigureproperties(thisplot);
         %
         [y1,y2] = stairs(forecast_index,deter_mat(i,:));
         lh = plot(y1,y2,'-','LineWidth',1,'Color',colorstr,'Tag',[strtrim(deter_str(i,:)) '-stairs']);
         hold('on');
         %
         % check if we should add a zero line
         %
         %
         y_lim = get(gca,'YLim');
         if (y_lim(1)<0)&(y_lim(2)>0);
            plot([forecast_index(1,1) forecast_index(1,num_fp)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
            hold('on');
         end;
         set(gca,'XLim',[min(forecast_index) max(forecast_index)]);
         setaxesfonts('Title',['Future values for ' strtrim(deter_str(i,:))]);
         toolbar_fcn(gca);
      end;
      if curr_ini.external==0;
         set(my_plots,'Visible','on');
      else;
         stats = [];
         file_names = '';
         for i=1:size(my_plots,2);
            file = [tm.outputdir tm.modelfile '-' get(my_plots(1,i),'Tag') num2str(tm.n) '.' num2str(tm.r) '-' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.num_fdraws,'%0.0f') '.' num2str(num_fp,'%0.0f')];
            set(0,'CurrentFigure',my_plots(1,i));
            stats = [stats autosavefigure(my_plots(1,i),file)];
            file_names = strvcat(file_names,file);
         end;
         if isempty(stats)==0;
            for i=1:size(stats,2);
               if stats(i)==0;
                  autoshowfigure(strtrim(file_names(i,:)));
               end;
            end;
         end;
      end;
   case 'makedeter'
      %
      % create values for extra deterministic variables and save to file
      %
      f_hor = get(f_cast.numperiods,'Value');
      file = [tm.modelsdir tm.modelfile '-deter-forecast-' num2str(tm.datad,'%0.0f') '-' num2str(f_hor,'%0.0f') '.mat'];
      if fileexist(file)==1;
         a = load(file);
         deter_mat = a.deter_mat;
      else;
         %
         % create baseline data
         %
         if tm.cointvalue==1;
            orig_data = tm.Z2(tm.d-tm.datad+1:tm.d,:);
         else;
            orig_data = tm.X1(tm.d-tm.datad+1:tm.d,:);
         end;
         deter_mat = zeros(tm.datad,f_hor);
         for i=1:tm.datad;
            %
            % try to determine the type of variable we have
            %
            val_1 = orig_data(i,tm.T);
            val_2 = orig_data(i,tm.T-1);
            if val_1==val_2;
               deter_mat(i,:) = val_2*ones(1,f_hor);
            else;
               deter_mat(i,:) = ((val_1-val_2)*(1:1:f_hor))+val_1;
            end;
         end;
      end;
      %
      % create the date string
      %
      if strcmp(tm.freq,'m')==1;
         add_on = 1/12;
      elseif strcmp(tm.freq,'q')==1;
         add_on = 1/4;
      else;
         add_on = 1;
      end;
      sample_end = tm.sampleend+(add_on*f_hor);
      sample_string = '';
      for i=tm.T+1:tm.T+f_hor;
         sample_string = strvcat(sample_string,getdate(tm.samplebeg,sample_end,tm.freq,i/(tm.T+f_hor)));
      end;
      %
      var_names = vecstr(tm.deter);
      var_names = var_names(tm.d-tm.datad+1:tm.d,:);
      %
      % Open the dialog for creating deterministic data and save it to file
      %
      set(forecastgui,'Visible','off');
      drawnow;
      %
      thisgraph = cond_forecast_data_fcn('init',tm,deter_mat,sample_string,'Edit Deterministic Data',var_names);
      killguipid(thisgraph);
      %
      if fileexist(file)==1;
         set(f_cast.compute,'Enable','on');
         if tm.datad>0;
            set(f_cast.displaydeter,'Enable','on');
         end;
      else;
         set(f_cast.compute,'Enable','off');
         if tm.datad>0;
            set(f_cast.displaydeter,'Enable','off');
         end;
      end;
      set(forecastgui,'Visible','on');
      drawnow;
   case 'check_numdraws'
      %
      % get number of forecast draws
      %
      f_str = get(f_cast.numdraws,'String');
      f_val = get(f_cast.numdraws,'Value');
      tm.num_fdraws = mystr2num(strrep(strtrim(f_str(f_val,:)),',',''));
      %
      % check if output files from the forecasts exists. if so enable the "Display" controls, else disable it
      %
      res_exist = 1;
      for i=1:tm.save_draws
         if get(tm.controls.betarest,'Value')==1;
            dfbeta = dfbeta_fcn(tm);
            file = [curr_ini.simulatedir tm.modelfile '-betarest-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '.' num2str(dfbeta,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f')];
            if tm.prior_inf_alpha==1;
               file = [file '.' num2str(tm.lambda_alpha,'%0.3f')];
            end;
         else;
            file = [curr_ini.simulatedir tm.modelfile '-rank-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f') '.' num2str(tm.lambda_alpha,'%0.3f')];
         end;
         if tm.prior_inf_shortrun==1;
            file = [file '.' num2str(tm.lambda_b,'%0.3f') '.' num2str(tm.lambda_theta,'%0.3f') '.' num2str(tm.lambda_l,'%0.3f') '.' num2str(tm.lambda_e,'%0.3f')];
         end;
         if tm.prior_inf_deter==1;
            file = [file '.' num2str(tm.lambda_d,'%0.3f')];
         end;
         file = [file '-' num2str(tm.save_draws,'%0.0f') '.' num2str(i,'%0.0f') '.' num2str(tm.num_fdraws,'%0.0f') '.' num2str(get(f_cast.numperiods,'Value'),'%0.0f') '-' num2str(tm.d+tm.dcoint,'%0.0f') '.' num2str(tm.datad,'%0.0f') '.mat'];
         if fileexist(file)==0;
            res_exist = 0;
            break;
         end;
      end;
      if res_exist==1;
         set(f_cast.show,'Enable','on');
      else;
         set(f_cast.show,'Enable','off');
      end;
   case 'checkfile'
      if tm.datad>0;
         file = [tm.modelsdir tm.modelfile '-deter-forecast-' num2str(tm.datad,'%0.0f') '-' num2str(get(f_cast.numperiods,'Value'),'%0.0f') '.mat'];
         if fileexist(file)==1;
            set(f_cast.compute,'Enable','on');
            if tm.datad>0;
               set(f_cast.displaydeter,'Enable','on');
            end;
         else;
            set(f_cast.compute,'Enable','off');
            if tm.datad>0;
               set(f_cast.displaydeter,'Enable','off');
            end;
         end;
      end;
      %
      % check if output files from the forecasts exists. if so enable the "Display" controls, else disable it
      %
      res_exist = 1;
      for i=1:tm.save_draws
         if get(tm.controls.betarest,'Value')==1;
            dfbeta = dfbeta_fcn(tm);
            file = [curr_ini.simulatedir tm.modelfile '-betarest-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '.' num2str(dfbeta,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f')];
            if tm.prior_inf_alpha==1;
               file = [file '.' num2str(tm.lambda_alpha,'%0.3f')];
            end;
         else;
            file = [curr_ini.simulatedir tm.modelfile '-rank-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f') '.' num2str(tm.lambda_alpha,'%0.3f')];
         end;
         if tm.prior_inf_shortrun==1;
            file = [file '.' num2str(tm.lambda_b,'%0.3f') '.' num2str(tm.lambda_theta,'%0.3f') '.' num2str(tm.lambda_l,'%0.3f') '.' num2str(tm.lambda_e,'%0.3f')];
         end;
         if tm.prior_inf_deter==1;
            file = [file '.' num2str(tm.lambda_d,'%0.3f')];
         end;
         file = [file '-' num2str(tm.save_draws,'%0.0f') '.' num2str(i,'%0.0f') '.' num2str(tm.num_fdraws,'%0.0f') '.' num2str(get(f_cast.numperiods,'Value'),'%0.0f') '-' num2str(tm.d+tm.dcoint,'%0.0f') '.' num2str(tm.datad,'%0.0f') '.mat'];
         if fileexist(file)==0;
            res_exist = 0;
            break;
         end;
      end;
      if res_exist==1;
         set(f_cast.show,'Enable','on');
      else;
         set(f_cast.show,'Enable','off');
      end;
   case 'showgraph'
      %
      % Determine the number of forecasts, which variable, and which type of variable
      %
      num_f = get(f_cast.numperiods,'Value');
      val_1 = get(f_cast.firstvar,'Value');
      val_2 = get(f_cast.secondvar,'Value');
      if val_1<=tm.n;
         var_1 = strtrim(tm.myvars(val_1,:));
         type_1 = 1;
      elseif (val_1>tm.n)&(val_1<=2*tm.n);
         val_1 = val_1-tm.n;
         var_1 = strtrim(tm.myvars(val_1,:));
         type_1 = 2;
      elseif (val_1>2*tm.n)&(val_1<3*tm.n);
         val_1 = val_1-(2*tm.n);
         var_1 = strtrim(tm.myvars(val_1,:));
         type_1 = 3;
      else;
         val_1 = val_1-(3*tm.n);
         var_1 = strtrim(tm.myvars(val_1,:));
         type_1 = 4;
      end;
      %
      % check the constants
      %
      %
      const_1 = mystr2num(strtrim(get(f_cast.first_constant,'String')));
      const_2 = mystr2num(strtrim(get(f_cast.constant,'String')));
      if isalmostequal(const_2,0)==0;
         if val_2<=tm.n;
            var_2 = strtrim(tm.myvars(val_2,:));
            type_2 = 1;
         elseif (val_2>tm.n)&(val_2<=2*tm.n);
            val_2 = val_2-tm.n;
            var_2 = strtrim(tm.myvars(val_2,:));
            type_2 = 2;
         elseif (val_2>2*tm.n)&(val_2<3*tm.n);
            val_2 = val_2-(2*tm.n);
            var_2 = strtrim(tm.myvars(val_2,:));
            type_2 = 3;
         else;
            val_2 = val_2-(3*tm.n);
            var_2 = strtrim(tm.myvars(val_2,:));
            type_2 = 4;
         end;
      else;
         val_2 = [];
         var_2 = [];
         type_2 = [];
      end;
      %
      if isempty(val_2)==0;
         if val_1==val_2;
            txt = 'You are not allowed to select the same base variable for both the first and the second variable. Please change your selections.';
            about(txt,'information','Graph Transformed Variable Forecasts',120,500);
            return;
         end;
      end;
      %
      %
      set(forecastgui,'Visible','off');
      drawnow;
      %
      % Now we can prepare the forecasts
      %
      trans_val = get(f_cast.transform,'Value');
      if (max(max(abs(tm.X0)))>709)&((trans_val>=2)&(trans_val<=5));
         ch_exp = 1;
         setlogtext(['Dividing variables by the factor ' num2str(tm.scale_endo,'%0.4f') ' in graphs...']);
      else;
         ch_exp = 0;
      end;
      num_fp = get(f_cast.numperiods,'Value');
      %
      % construct date index for actual values and the forecasts
      %
      [dataindex,sample_beg,sample_end] = make_dataindex(tm.freq,tm.samplebeg,tm.sampleend);
      dataindex = dataindex';
      add_on = dataindex(1,2)-dataindex(1,1);
      forecast_index = (add_on:add_on:(add_on*num_fp));
      forecast_index = forecast_index+dataindex(1,tm.T);
      %
      % initializing data matrices for the plot
      %
      mean_mat = zeros(1,num_fp);
      median_mat = zeros(1,num_fp);
      mode_mat = zeros(1,num_fp);
      upper_mat = zeros(1,num_fp);
      lower_mat = zeros(1,num_fp);
      if strcmp(tm.freq,'q')==1;
         sum_mat = zeros(num_fp,num_fp+3);
         for i=1:num_fp;
            sum_mat(i,i:i+3) = 1;
         end;
      elseif strcmp(tm.freq,'m')==1;
         sum_mat = zeros(num_fp,num_fp+11);
         for i=1:num_fp;
            sum_mat(i,i:i+11) = 1;
         end;
      end;
      %
      % create the actual values
      %
      if type_1==1;
         %
         % levels
         %
         x_mat = tm.X0;
         head_str = [num2str(const_1,'%0.2f') '*' strtrim(tm.myvars(val_1,:))];
         fig_name_str = [num2str(const_1,'%0.2f') '*' strtrim(tm.myvars(val_1,:))];
         tag_str = strtrim(tm.myvars(val_1,:));
      elseif type_1==2;
         %
         % first differences
         %
         x_mat = tm.X0(:,2:tm.T)-tm.X0(:,1:tm.T-1);
         head_str = [num2str(const_1,'%0.2f') '*\Delta' strtrim(tm.myvars(val_1,:))];
         fig_name_str = [num2str(const_1,'%0.2f') '*[' strtrim(tm.myvars(val_1,:)) '(t)-' strtrim(tm.myvars(val_1,:)) '(t-1)]'];
         tag_str = ['d.' strtrim(tm.myvars(val_1,:))];
      elseif type_1==3;
         if strcmp(tm.freq,'q')==1;
            temp_smat = zeros(tm.T-3,tm.T);
            for i=1:tm.T-3;
               temp_smat(i,i:i+3) = 1;
            end;
            x_mat = tm.X0*temp_smat';
            head_str = [num2str(const_1,'%0.2f') '*\Sigma_{0}^{3}' strtrim(tm.myvars(val_1,:))];
            fig_name_str = [num2str(const_1,'%0.2f') '*sum(0,3)' strtrim(tm.myvars(val_1,:))];
            tag_str = ['sum.' strtrim(tm.myvars(val_1,:))];
         elseif strcmp(tm.freq,'mm')==1;
            temp_smat = zeros(tm.T-11,tm.T);
            for i=1:tm.T-11;
               temp_smat(i,i:i+11) = 1;
            end;
            x_mat = tm.X0*temp_smat';
            head_str = [num2str(const_1,'%0.2f') '*\Sigma_{0}^{11}' strtrim(tm.myvars(val_1,:))];
            fig_name_str = [num2str(const_1,'%0.2f') '*sum(0,11)' strtrim(tm.myvars(val_1,:))];
            tag_str = ['sum.' strtrim(tm.myvars(val_1,:))];
         else;
            x_mat = tm.X0;
            head_str = [num2str(const_1,'%0.2f') '*' strtrim(tm.myvars(val_1,:))];
            fig_name_str = [num2str(const_1,'%0.2f') '*' strtrim(tm.myvars(val_1,:))];
            tag_str = strtrim(tm.myvars(val_1,:));
         end;
      elseif type_1==4;
         %
         % frequency diffs
         %
         if strcmp(tm.freq,'q')==1;
            x_mat = tm.X0(:,5:tm.T)-tm.X0(:,1:tm.T-4);
            head_str = [num2str(const_1,'%0.2f') '*\Delta_{4}' strtrim(tm.myvars(val_1,:))];
            fig_name_str = [num2str(const_1,'%0.2f') '*[' strtrim(tm.myvars(val_1,:)) '(t)-' strtrim(tm.myvars(val_1,:)) '(t-4)]'];
            tag_str = ['sum.d.' strtrim(tm.myvars(val_1,:))];
         elseif strcmp(tm.freq,'m')==1;
            x_mat = tm.X0(:,13:tm.T)-tm.X0(:,1:tm.T-12);
            head_str = [num2str(const_1,'%0.2f') '*\Delta_{12}' strtrim(tm.myvars(val_1,:))];
            fig_name_str = [num2str(const_1,'%0.2f') '*[' strtrim(tm.myvars(val_1,:)) '(t)-' strtrim(tm.myvars(val_1,:)) '(t-12)]'];
            tag_str = ['sum.d.' strtrim(tm.myvars(val_1,:))];
         else;
            x_mat = tm.X0(:,2:tm.T)-tm.X0(:,1:tm.T-1);
            head_str = [num2str(const_1,'%0.2f') '*\Delta' strtrim(tm.myvars(val_1,:))];
            fig_name_str = [num2str(const_1,'%0.2f') '*[' strtrim(tm.myvars(val_1,:)) '(t)-' strtrim(tm.myvars(val_1,:)) '(t-1)]'];
            tag_str = ['d.' strtrim(tm.myvars(val_1,:))];
         end;
      end;
      x_var1 = x_mat(val_1,:);
      if isempty(val_2)==0;
         if type_2==1;
            %
            % levels
            %
            x_mat = tm.X0;
            head_str = [head_str ' + ' num2str(const_2,'%0.2f') '*' strtrim(tm.myvars(val_2,:))];
            fig_name_str = [fig_name_str ' + ' num2str(const_2,'%0.2f') '*' strtrim(tm.myvars(val_2,:))];
            tag_str = [tag_str '-' strtrim(tm.myvars(val_2,:))];
         elseif type_2==2;
            %
            % first differences
            %
            x_mat = tm.X0(:,2:tm.T)-tm.X0(:,1:tm.T-1);
            head_str = [head_str ' + ' num2str(const_2,'%0.2f') '*\Delta' strtrim(tm.myvars(val_2,:))];
            fig_name_str = [fig_name_str ' + ' num2str(const_2,'%0.2f') '*[' strtrim(tm.myvars(val_2,:)) '(t)-' strtrim(tm.myvars(val_2,:)) '(t-1)]'];
            tag_str = [tag_str '-d.' strtrim(tm.myvars(val_2,:))];
         elseif type_2==3;
            if strcmp(tm.freq,'q')==1;
               temp_smat = zeros(tm.T-3,tm.T);
               for i=1:tm.T-3;
                  temp_smat(i,i:i+3) = 1;
               end;
               x_mat = tm.X0*temp_smat';
               head_str = [head_str ' + ' num2str(const_2,'%0.2f') '*\Sigma_{0}^{3}' strtrim(tm.myvars(val_2,:))];
               fig_name_str = [fig_name_str ' + ' num2str(const_2,'%0.2f') '*sum(0,3)' strtrim(tm.myvars(val_2,:))];
               tag_str = [tag_str '-sum.' strtrim(tm.myvars(val_2,:))];
            elseif strcmp(tm.freq,'mm')==1;
               temp_smat = zeros(tm.T-11,tm.T);
               for i=1:tm.T-11;
                  temp_smat(i,i:i+11) = 1;
               end;
               x_mat = tm.X0*temp_smat';
               head_str = [head_str ' + ' num2str(const_2,'%0.2f') '*\Sigma_{0}^{11}' strtrim(tm.myvars(val_2,:))];
               fig_name_str = [fig_name_str ' + ' num2str(const_2,'%0.2f') '*sum(0,11)' strtrim(tm.myvars(val_2,:))];
               tag_str = [tag_str '-sum.' strtrim(tm.myvars(val_2,:))];
            else;
               x_mat = tm.X0;
               head_str = [head_str ' + ' num2str(const_2,'%0.2f') '*' strtrim(tm.myvars(val_2,:))];
               fig_name_str = [fig_name_str ' + ' num2str(const_2,'%0.2f') '*' strtrim(tm.myvars(val_2,:))];
               tag_str = [tag_str '-' strtrim(tm.myvars(val_2,:))];
            end;
         elseif type_2==4;
            %
            % frequency diffs
            %
            if strcmp(tm.freq,'q')==1;
               x_mat = tm.X0(:,5:tm.T)-tm.X0(:,1:tm.T-4);
               head_str = [head_str ' + ' num2str(const_2,'%0.2f') '*\Delta_{4}' strtrim(tm.myvars(val_2,:))];
               fig_name_str = [fig_name_str ' + ' num2str(const_2,'%0.2f') '*[' strtrim(tm.myvars(val_2,:)) '(t)-' strtrim(tm.myvars(val_2,:)) '(t-4)]'];
               tag_str = [tag_str '-sum.d.' strtrim(tm.myvars(val_2,:))];
            elseif strcmp(tm.freq,'m')==1;
               x_mat = tm.X0(:,13:tm.T)-tm.X0(:,1:tm.T-12);
               head_str = [head_str ' + ' num2str(const_2,'%0.2f') '*\Delta_{12}' strtrim(tm.myvars(val_2,:))];
               fig_name_str = [fig_name_str ' + ' num2str(const_2,'%0.2f') '*[' strtrim(tm.myvars(val_2,:)) '(t)-' strtrim(tm.myvars(val_2,:)) '(t-12)]'];
               tag_str = [tag_str '-sum.d.' strtrim(tm.myvars(val_2,:))];
            else;
               x_mat = tm.X0(:,2:tm.T)-tm.X0(:,1:tm.T-1);
               head_str = [head_str ' + ' num2str(const_2,'%0.2f') '*\Delta' strtrim(tm.myvars(val_2,:))];
               fig_name_str = [fig_name_str ' + ' num2str(const_2,'%0.2f') '*[' strtrim(tm.myvars(val_2,:)) '(t)-' strtrim(tm.myvars(val_2,:)) '(t-1)]'];
               tag_str = [tag_str '-d.' strtrim(tm.myvars(val_2,:))];
            end;
         end;
         x_var2 = x_mat(val_2,:);
      else;
         x_var2 = 0;
      end;
      if (size(x_var1,2)~=size(x_var2,2))&(size(x_var2,2)>1);
         %
         % make sure they have the same dimentions
         %
         n1 = size(x_var1,2);
         n2 = size(x_var2,2);
         if n1>n2;
            %
            % correct x_var1
            %
            x_var1 = x_var1(1,n1-n2+1:n1);
         else;
            %
            % correct x_var2
            %
            x_var2 = x_var2(1,n2-n1+1:n2);
         end;
      end;
      x_var = (const_1*x_var1)+(const_2*x_var2);
      %
      % apply transformation function to x_var
      %
      if trans_val==2;
         if ch_exp==1;
            x_var = (1/tm.scale_endo)*x_var;
         end;
         x_var = exp(x_var);
         head_str = ['exp[' head_str ']'];
         fig_name_str = ['exp[' fig_name_str ']'];
         tag_str = ['exp-' tag_str];
      elseif trans_val==3;
         if ch_exp==1;
            x_var = (1/tm.scale_endo)*x_var;
         end;
         x_var = exp(x_var)-1;
         head_str = ['exp[' head_str ']-1'];
         fig_name_str = ['exp[' fig_name_str ']-1'];
         tag_str = ['exp1-' tag_str];
      elseif trans_val==4;
         if ch_exp==1;
            x_var = (1/tm.scale_endo)*x_var;
         end;
         x_var = 100*exp(x_var);
         head_str = ['100*exp[' head_str ']'];
         fig_name_str = ['100*exp[' fig_name_str ']'];
         tag_str = ['100exp-' tag_str];
      elseif trans_val==5;
         if ch_exp==1;
            x_var = (1/tm.scale_endo)*x_var;
         end;
         x_var = 100*(exp(x_var)-1);
         head_str = ['100*(exp[' head_str ']-1)'];
         fig_name_str = ['100*(exp[' fig_name_str ']-1)'];
         tag_str = ['100exp1-' tag_str];
      elseif trans_val==6;
         x_var = 4*x_var;
         head_str = ['4*[' head_str ']'];
         fig_name_str = ['4*[' fig_name_str ']'];
         tag_str = ['4-' tag_str];
      elseif trans_val==7;
         x_var = 12*x_var;
         head_str = ['12*[' head_str ']'];
         fig_name_str = ['12*[' fig_name_str ']'];
         tag_str = ['12-' tag_str];
      elseif trans_val==8;
         x_var = 400*x_var;
         head_str = ['400*[' head_str ']'];
         fig_name_str = ['400*[' fig_name_str ']'];
         tag_str = ['400-' tag_str];
      elseif trans_val==9;
         x_var = 1200*x_var;
         head_str = ['1200*[' head_str ']'];
         fig_name_str = ['1200*[' fig_name_str ']'];
         tag_str = ['1200-' tag_str];
      elseif trans_val==10;
         x_var = (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var);
         head_str = ['[1 + ' head_str ']^{4}'];
         fig_name_str = ['prod(1,4)[1 + ' fig_name_str ']'];
         tag_str = ['prod4-' tag_str];
      elseif trans_val==11;
         x_var = ((1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var))-1;
         head_str = ['[1 + ' head_str ']^{4} - 1'];
         fig_name_str = ['prod(1,4)[1 + ' fig_name_str ']-1'];
         tag_str = ['prod4-1-' tag_str];
      elseif trans_val==12;
         x_var = 100*((1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var));
         head_str = ['100*[1 + ' head_str ']^{4}'];
         fig_name_str = ['100*prod(1,4)[1 + ' fig_name_str ']'];
         tag_str = ['prod4-' tag_str];
      elseif trans_val==13;
         x_var = 100*(((1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var))-1);
         head_str = ['100*([1 + ' head_str ']^{4} - 1)'];
         fig_name_str = ['100*(prod(1,4)[1 + ' fig_name_str ']-1)'];
         tag_str = ['prod4-1-' tag_str];
      elseif trans_val==14;
         x_var = (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var);
         head_str = ['[1 + ' head_str ']^{12}'];
         fig_name_str = ['prod(1,12)[1 + ' fig_name_str ']'];
         tag_str = ['prod12-' tag_str];
      elseif trans_val==15;
         x_var = ((1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var))-1;
         head_str = ['[1 + ' head_str ']^{12} - 1'];
         fig_name_str = ['prod(1,12)[1 + ' fig_name_str ']-1'];
         tag_str = ['prod12-1-' tag_str];
      elseif trans_val==16;
         x_var = 100*((1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var));
         head_str = ['100*[1 + ' head_str ']^{12}'];
         fig_name_str = ['100*prod(1,12)[1 + ' fig_name_str ']'];
         tag_str = ['prod12-' tag_str];
      elseif trans_val==17;
         x_var = 100*(((1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var) .* (1+x_var))-1);
         head_str = ['100*([1 + ' head_str ']^{12} - 1)'];
         fig_name_str = ['100*(prod(1,12)[1 + ' fig_name_str ']-1)'];
         tag_str = ['prod12-1-' tag_str];
      end;
      %
      % adjust dataindex
      %
      if size(dataindex,2)>size(x_var,2);
         dataindex = dataindex(1,size(dataindex,2)-size(x_var,2)+1:tm.T);
      end;
      %
      % deter upper and lower points
      %
      up = (1+(tm.cbs/100))/2;
      up = floor(up*tm.save_draws*tm.num_fdraws);
      lp = (tm.save_draws*tm.num_fdraws)-up+1;
      %
      % setting up a wait dialog
      %
      txt = strvcat(' ','Loading data and transforming the forecasts to the desired expression.','Current Horizon:  1',['Current Variable: ' fig_name_str],' ','Please wait.');
      mywait = waitdlg(txt,'forecast','Forecasting',450,140,1);
      waitf = get(mywait,'UserData');
      drawnow;
      setlogtext(strvcat('Loading data and transforming...','Forecasting variable:',['   ' fig_name_str],' '));
      abort = '';
      for k=1:num_fp; % should be num_fp
         txt = strvcat(' ','Loading data and transforming the forecasts to the desired expression.',['Current Horizon:  ' num2str(k,'%0.0f')],['Current Variable: ' fig_name_str],' ','Please wait.');
         set(waitf.text,'String',txt);
         drawnow;
         abort = get(waitf.cancel,'UserData');
         if strcmp(abort,'cancel')==1;
            set(waitf.text,'UserData','done');
            delete(mywait);
            drawnow;
            set(forecastgui,'Visible','on');
            drawnow;
            return;
         end;
         f_vec = zeros(tm.save_draws*tm.num_fdraws,1);
         for i=1:tm.save_draws
            if get(tm.controls.betarest,'Value')==1;
               dfbeta = dfbeta_fcn(tm);
               file = [curr_ini.simulatedir tm.modelfile '-betarest-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '.' num2str(dfbeta,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f')];
               if tm.prior_inf_alpha==1;
                  file = [file '.' num2str(tm.lambda_alpha,'%0.3f')];
               end;
            else;
               file = [curr_ini.simulatedir tm.modelfile '-rank-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f') '.' num2str(tm.lambda_alpha,'%0.3f')];
            end;
            if tm.prior_inf_shortrun==1;
               file = [file '.' num2str(tm.lambda_b,'%0.3f') '.' num2str(tm.lambda_theta,'%0.3f') '.' num2str(tm.lambda_l,'%0.3f') '.' num2str(tm.lambda_e,'%0.3f')];
            end;
            if tm.prior_inf_deter==1;
               file = [file '.' num2str(tm.lambda_d,'%0.3f')];
            end;
            file = [file '-' num2str(tm.save_draws,'%0.0f') '.' num2str(i,'%0.0f') '.' num2str(tm.num_fdraws,'%0.0f') '.' num2str(num_fp,'%0.0f') '-' num2str(tm.d+tm.dcoint,'%0.0f') '.' num2str(tm.datad,'%0.0f') '.mat'];
            a = load(file);
            for j=1:tm.num_fdraws;
               drawnow;
               abort = get(waitf.cancel,'UserData');
               if strcmp(abort,'cancel')==1;
                  set(waitf.text,'UserData','done');
                  delete(mywait);
                  drawnow;
                  set(forecastgui,'Visible','on');
                  drawnow;
                  return;
               end;
               forecast_mat = a.res(j).forecast_mat;
               %
               % Handle the difference type of variables
               %
               if type_1==1;
                  %
                  % levels
                  %
                  f_mat = forecast_mat;
               elseif type_1==2;
                  %
                  % first differences
                  %
                  f_mat = [tm.X0(:,tm.T) forecast_mat];
                  f_mat = f_mat(:,2:num_fp+1)-f_mat(:,1:num_fp);
               elseif type_1==3;
                  %
                  % sum of levels to frequency
                  %
                  if strcmp(tm.freq,'q')==1;
                     f_mat = [tm.X0(:,tm.T-2:tm.T) forecast_mat];
                     f_mat = f_mat*sum_mat';
                  elseif strcmp(tm.freq,'m')==1;
                     f_mat = [tm.X0(:,tm.T-10:tm.T) forecast_mat];
                     f_mat = f_mat*sum_mat';
                  else;
                     f_mat = forecast_mat;
                  end;
               elseif type_1==4;
                  %
                  % sum of diffs to frequency = frequency diff
                  %
                  if strcmp(tm.freq,'q')==1;
                     f_mat = [tm.X0(:,tm.T-3:tm.T) forecast_mat];
                     f_mat = f_mat(:,5:num_fp+4)-f_mat(:,1:num_fp);
                  elseif strcmp(tm.freq,'m')==1;
                     f_mat = [tm.X0(:,tm.T-11:tm.T) forecast_mat];
                     f_mat = f_mat(:,13:num_fp+12)-f_mat(:,1:num_fp);
                  else;
                     f_mat = [tm.X0(:,tm.T) forecast_mat];
                     f_mat = f_mat(:,2:num_fp+1)-f_mat(:,1:num_fp);
                  end;
               end;
               f_var_1 = f_mat(val_1,k);
               if isempty(val_2)==0;
                  if type_2==1;
                     %
                     % levels
                     %
                     f_mat = forecast_mat;
                  elseif type_2==2;
                     %
                     % first differences
                     %
                     f_mat = [tm.X0(:,tm.T) forecast_mat];
                     f_mat = f_mat(:,2:num_fp+1)-f_mat(:,1:num_fp);
                  elseif type_2==3;
                     %
                     % sum of levels to frequency
                     %
                     if strcmp(tm.freq,'q')==1;
                        f_mat = [tm.X0(:,tm.T-2:tm.T) forecast_mat];
                        f_mat = f_mat*sum_mat';
                     elseif strcmp(tm.freq,'m')==1;
                        f_mat = [tm.X0(:,tm.T-10:tm.T) forecast_mat];
                        f_mat = f_mat*sum_mat';
                     else;
                        f_mat = forecast_mat;
                     end;
                  elseif type_2==4;
                     %
                     % sum of diffs to frequency = frequency diff
                     %
                     if strcmp(tm.freq,'q')==1;
                        f_mat = [tm.X0(:,tm.T-3:tm.T) forecast_mat];
                        f_mat = f_mat(:,5:num_fp+4)-f_mat(:,1:num_fp);
                     elseif strcmp(tm.freq,'m')==1;
                        f_mat = [tm.X0(:,tm.T-11:tm.T) forecast_mat];
                        f_mat = f_mat(:,13:num_fp+12)-f_mat(:,1:num_fp);
                     else;
                        f_mat = [tm.X0(:,tm.T) forecast_mat];
                        f_mat = f_mat(:,2:num_fp+1)-f_mat(:,1:num_fp);
                     end;
                  end;
                  f_var_2 = f_mat(val_2,k);
               else;
                  f_var_2 = 0;
               end;
               %
               % compute the value of the joint variable
               %
               f_vec(j+(tm.num_fdraws*(i-1)),1) = (const_1*f_var_1)+(const_2*f_var_2);
            end;
            clear('a');
         end;
         %
         % we have now the forecast of the desired variable for horizon k.
         % make any desired transformations to the variable
         %
         if trans_val==2;
            if ch_exp==1;
               f_vec = (1/tm.scale_endo)*f_vec;
            end;
            f_vec = exp(f_vec);
         elseif trans_val==3;
            if ch_exp==1;
               f_vec = (1/tm.scale_endo)*f_vec;
            end;
            f_vec = exp(f_vec)-1
         elseif trans_val==4;
            if ch_exp==1;
               f_vec = (1/tm.scale_endo)*f_vec;
            end;
            f_vec = 100*exp(f_vec);
         elseif trans_val==5;
            if ch_exp==1;
               f_vec = (1/tm.scale_endo)*f_vec;
            end;
            f_vec = 100*(exp(f_vec)-1);
         elseif trans_val==6;
            f_vec = 4*f_vec;
         elseif trans_val==7;
            f_vec = 12*f_vec;
         elseif trans_val==8;
            f_vec = 400*f_vec;
         elseif trans_val==9;
            f_vec = 1200*f_vec;
         elseif trans_val==10;
            f_vec = (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec);
         elseif trans_val==11;
            f_vec = ((1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec))-1;
         elseif trans_val==12;
            f_vec = 100*((1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec));
         elseif trans_val==13;
            f_vec = 100*(((1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec))-1);
         elseif trans_val==14;
            f_vec = (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec);
         elseif trans_val==15;
            f_vec = ((1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec))-1;
         elseif trans_val==16;
            f_vec = 100*((1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec));
         elseif trans_val==17;
            f_vec = 100*(((1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec) .* (1+f_vec))-1);
         end;
         %
         % now we sort
         %
         f_vec = sort(f_vec);
         lower_mat(1,k) = f_vec(lp,1);
         upper_mat(1,k) = f_vec(up,1);
         mean_mat(1,k) = (1/size(f_vec,1))*sum(f_vec);
         median_mat(1,k) = median(f_vec);
         %
         % compute the mode forecasts based on the Epanechnikov kernel density method
         %
         xmin = min(f_vec);
         xmax = max(f_vec);
         xrange = xmax-xmin;
         ax = xmin-xrange/4;
         bx = xmax+xrange/4;
         h_parm = hns(f_vec,'kepan');
         [x_v,y_v] = kdfft1(f_vec,'kepan',512,h_parm,0,[ax,bx]);
         [max_y,pos_max] = max(y_v);
         mode_mat(1,k) = x_v(1,pos_max);
      end;
      %
      % close the wait dialog
      %
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
      drawnow;
      %
      % we are now ready to plot the forecasts
      %
      if curr_ini.colors==1;
         colorstr1 = [0 0 0];
         colorstr2 = (1/255)*[255 255 177];
         colorstr3 = [1 0 0];
         colorstr4 = [0 1 0];
         colorstr5 = [0 0 1];
      else;
         colorstr1 = [0 0 0];
         colorstr2 = [0.6 0.6 0.6];
         colorstr3 = [0.45 0.45 0.45];
         colorstr4 = [0.3 0.3 0.3];
         colorstr5 = [0.15 0.15 0.15];
      end;
      %
      % show the forecast dialog
      %
      set(forecastgui,'Visible','on');
      drawnow;
      drawnow;
      %
      %
      realscrsz = get(0,'ScreenSize');
      pos = [(realscrsz(3)-612)/2 (realscrsz(4)-484)/2];
      %
      % Display results
      %
      fig_handle = figure('Units','pixels', ...
         'Position',[pos 612 484], ...
         'Menubar','none', ...
         'Toolbar','none', ...
         'NumberTitle','off', ...
         'Tag',['transformed-forecast-' tag_str '-'], ...
         'Visible','off', ...
         'Name',['Forecasts - ' fig_name_str]);
      %
      % default figure properties
      %
      setfigureproperties(fig_handle);
      %
      % Using a patch to display the confidence region
      %
      my_p = patch([forecast_index reverse(forecast_index)],[upper_mat reverse(lower_mat)],colorstr2);
      set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
      hold('on');
      %
      % actual values
      %
      line_act = plot(dataindex,x_var,'-','LineWidth',1,'Color',colorstr1,'Tag',tag_str);
      hold('on');
      %
      % add a vertical line at the end of the actual observations
      %
      plot(dataindex(1,size(dataindex,2))*ones(1,2),get(gca,'YLim'),':','LineWidth',1,'Color',[0.4 0.4 0.4],'Tag','nolegend-1');
      hold('on');
      %
      % mean forecasts
      %
      line_mean = plot(forecast_index,mean_mat,'-','LineWidth',1,'Color',colorstr3,'Tag',['mean-' tag_str]);
      hold('on');
      %
      % median forecasts
      %
      line_median = plot(forecast_index,median_mat,':','LineWidth',1,'Color',colorstr4,'Tag',['median-' tag_str]);
      hold('on');
      %
      % mode forecasts
      %
      line_mode = plot(forecast_index,mode_mat,'--','LineWidth',1,'Color',colorstr5,'Tag',['mode-' tag_str]);
      hold('on');
      %
      set(gca,'XLim',[dataindex(1,1) forecast_index(1,num_fp)],'Layer','top');
      %
      % plot a zero line if needed
      %
      y_lim = get(gca,'YLim');
      if (y_lim(1)<0)&(y_lim(2)>0);
         plot([dataindex(1,1) forecast_index(1,num_fp)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
         hold('on');
      end;
      %
      title_str = [num2str(num_fp,'%0.0f') ' forecasts for ' head_str ' with ' num2str(tm.c,'%0.0f') '% confidence region'];
      setaxesfonts('Title',title_str);
      %
      % create a legend
      %
      myleg = legend_fcn([line_act line_mean line_median line_mode my_p],'Actual','Mean','Median','Mode','Confidence',2);
      setaxesfonts(myleg);
      set(gca,'UserData',myleg);
      hold('off');
      toolbar_fcn(gca);
      %
      if curr_ini.external==0;
         set(fig_handle,'Visible','on');
         drawnow;
      else;
         file = [tm.outputdir tm.modelfile '-' get(fig_handle,'Tag') num2str(tm.n) '.' num2str(tm.r) '-' num2str(num_fp,'%0.0f') '-' num2str(tm.save_draws,'%0.0f') '-' num2str(tm.num_fdraws,'%0.0f')];
         set(0,'CurrentFigure',fig_handle);
         stat = autosavefigure(fig_handle,file);
         if isempty(stat)==0;
            if stat==0;
               autoshowfigure(file);
            end;
         end;
      end;
   case 'compute'
      %
      % we only create forecasts in levels. All transformations of the data can be applied to these forecasts
      %
      %
      set(forecastgui,'Visible','off');
      drawnow;
      %
      x_vec_orig = tm.X0(:,tm.T);
      if tm.p>1;
         for i=1:tm.p-1;
            x_vec_orig = [x_vec_orig;tm.X0(:,tm.T-i)];
         end;
      end;
      num_fp = get(f_cast.numperiods,'Value');
      if tm.cointvalue==1;
         %
         % The data in levels is located in tm.X0 and tm.X1
         %
         if tm.datad>0;
            file = [tm.modelsdir tm.modelfile '-deter-forecast-' num2str(tm.datad,'%0.0f') '-' num2str(num_fp,'%0.0f') '.mat'];
            a = load(file);
            D_mat = a.deter_mat;
         else;
            D_mat = [];
         end;
         %
         % Create the automatic deterministic variables (constant, trend, etc)
         %
         if tm.d>0;
            if tm.deterselect(1,1)==1;
               D_1 = ones(1,num_fp);
               D_2 = [];
            elseif tm.deterselect(1,2)==1;
               D_1 = [];
               D_2 = ones(1,num_fp);
            elseif tm.deterselect(1,3)==1;
               D_1 = [ones(1,num_fp);(tm.T+1:1:tm.T+num_fp)];
               D_2 = [];
            else;
               D_1 = ones(1,num_fp);
               D_2 = (tm.T+1:1:tm.T+num_fp);
            end;
            if tm.seasonal(1,1)==1;
               [seamat,seastring] = seasonal_fcn(tm.freq,size(tm.x0,1)+num_fp);
               add_D1 = seamat(size(tm.x0,1)+1:size(tm.x0,1)+num_fp,:)';
               D_1 = [D_1;add_D1];
            end;
         else;
            %
            % we have a restricted constant with no other automatic deterministics
            %
            D_1 = [];
            D_2 = ones(1,num_fp);
         end;
         D_mat = [D_1;D_mat;D_2];
         %
         % setting up the progress dialog
         %
         input_var.title = ['Progress for ' num2str(tm.save_draws*tm.num_fdraws,'%0.0f') ' forecast simulations'];
         input_var.facecolor = curr_ini.progress_facecolor;
         input_var.startfacecolor = curr_ini.progress_startfacecolor;
         input_var.edgecolor = curr_ini.progress_edgecolor;
         input_var.bgcolor = curr_ini.progress_bgcolor;
         input_var.stop = 0;
         input_var.clock = 1;
         input_var.label = 'Draw number:';
         input_var.name = 'Double Forecast Simulation';
         mywait = progressdlg(0,input_var);
         set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
         drawnow;
         txt = strvcat('Computing forecasts from a double simulation procedure',['     ' num2str(tm.num_fdraws,'%0.0f') ' draws per parameter draw'],['     ' num2str(tm.save_draws,'%0.0f') ' parameter draws from the Gibbs sampler'],['Simulation started at:  ' datestr(now,14)]);
         setlogtext(txt);
         %
         abort = '';
         for i=1:tm.save_draws;
            %
            % prepare forecasts from the error correction representation
            %
            delta = tm.posterior_data(i).delta;
            Gamma = tm.posterior_data(i).Gamma;
            alpha = tm.posterior_data(i).alpha;
            beta = tm.posterior_data(i).beta;
            Omega = tm.posterior_data(i).Omega;
            if tm.r>0;
               if size(beta,1)>tm.n;
                  delta = [delta alpha*beta(tm.n+1:size(beta,1),:)'];
                  beta = beta(1:tm.n,:);
               end;
            end;
            %
            % create the stacked VAR(1) matrix
            %
            if tm.r>0;
               Pi = alpha*beta';
            else;
               Pi = zeros(tm.n,tm.n);
            end;
            if tm.p>=2;
               Phi = Gamma(:,1:tm.n*(tm.p-1));
            end;
            ip = zeros(tm.p,tm.p);
            for i1=1:tm.p;
               for j1=1:tm.p;
                  if j1>=i1;
                     ip(j1,i1)=1;
                  end;
               end;
            end;
            %
            % Computing levels VAR coefficients from
            % VECM coefficients
            %
            ip = kron(ip,eye(tm.n));
            if tm.p>=2;
               M = [(Pi+eye(tm.n)) -Phi];
               M = M/ip;
            else;
               M = (Pi+eye(tm.n))/ip;
            end;
            %
            % we are now ready to start the iterations over the number of forecast draws per parameter draw
            %
            if curr_ini.symmetric==0;
               std_Omega = chol(Omega)';
            else;
               std_Omega = symm(Omega);
            end;
            for j=1:tm.num_fdraws; % should be tm.num_fdraws
               %
               % update the progress dialog
               %
               draw = j+(tm.num_fdraws*(i-1));
               abort = get(mywait,'UserData');
               if strcmp(abort,'cancel')==1;
                  break;
               else;
                  progressdlg([draw/(tm.save_draws*tm.num_fdraws) draw]);
               end;
               %
               % fix the random number generator state
               %
               if curr_ini.random_state==2;
                  randn('state',draw);
               elseif curr_ini.random_state==3;
                  randn('state',sum(100*clock));
               elseif curr_ini.random_state==4;
                  randn('state',sum(100*clock)+draw);
               end;
               residuals = randn(tm.n,50+num_fp);
               residuals = residuals(:,50+1:50+num_fp);
               forecast_mat = zeros(tm.n,num_fp);
               %
               % pick the latest observations for the levels variables
               %
               x_vec = x_vec_orig;
               for k=1:num_fp; % should be num_fp
                  f_vec = (delta*D_mat(:,k))+(M*x_vec)+(std_Omega*residuals(:,k));
                  forecast_mat(:,k) = f_vec;
                  if tm.p>1;
                     x_vec = [f_vec;x_vec(1:tm.n*(tm.p-1),1)];
                  else;
                     x_vec = f_vec;
                  end;
               end;
               res(j).forecast_mat = forecast_mat;
            end;
            if strcmp(abort,'cancel')==1;
               break;
            end;
            %
            % save the results to file
            %
            if get(tm.controls.betarest,'Value')==1;
               dfbeta = dfbeta_fcn(tm);
               file = [curr_ini.simulatedir tm.modelfile '-betarest-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '.' num2str(dfbeta,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f')];
               if tm.prior_inf_alpha==1;
                  file = [file '.' num2str(tm.lambda_alpha,'%0.3f')];
               end;
            else;
               file = [curr_ini.simulatedir tm.modelfile '-rank-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f') '.' num2str(tm.lambda_alpha,'%0.3f')];
            end;
            if tm.prior_inf_shortrun==1;
               file = [file '.' num2str(tm.lambda_b,'%0.3f') '.' num2str(tm.lambda_theta,'%0.3f') '.' num2str(tm.lambda_l,'%0.3f') '.' num2str(tm.lambda_e,'%0.3f')];
            end;
            if tm.prior_inf_deter==1;
               file = [file '.' num2str(tm.lambda_d,'%0.3f')];
            end;
            file = [file '-' num2str(tm.save_draws,'%0.0f') '.' num2str(i,'%0.0f') '.' num2str(tm.num_fdraws,'%0.0f') '.' num2str(num_fp,'%0.0f') '-' num2str(tm.d+tm.dcoint,'%0.0f') '.' num2str(tm.datad,'%0.0f') '.mat'];
            save(file,'res');
         end;
         %
         % Close the progress dialog
         %
         set(mywait,'UserData','done');
         close(mywait);
         drawnow;
         drawnow;
         if strcmp(abort,'cancel')==0;
            txt = ['Simulation finished at: ' datestr(now,14)];
            setlogtext(strvcat(txt,' '));
            set(f_cast.show,'Enable','on');
         else;
            %
            % delete all files if asked to
            %
            for i=1:tm.save_draws;
               if get(tm.controls.betarest,'Value')==1;
                  dfbeta = dfbeta_fcn(tm);
                  file = [curr_ini.simulatedir tm.modelfile '-betarest-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '.' num2str(dfbeta,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f')];
                  if tm.prior_inf_alpha==1;
                     file = [file '.' num2str(tm.lambda_alpha,'%0.3f')];
                  end;
               else;
                  file = [curr_ini.simulatedir tm.modelfile '-rank-forecast-' num2str(tm.scale_endo,'%0.4f') '-' num2str(tm.r,'%0.0f') '-' tm.determodel num2str(tm.burn_in,'%0.0f') '.' num2str(tm.save_draws,'%0.0f') '.' num2str(tm.save_freq,'%0.0f') '.' num2str(tm.discard_explosive,'%0.0f') '-' num2str(tm.prior_inf_shortrun,'%0.0f') '.' num2str(tm.prior_inf_deter,'%0.0f') '-' num2str(tm.prior_inf_alpha,'%0.0f') '-' num2str(tm.prior_inf_omega,'%0.0f') '.' num2str(tm.lambda_alpha,'%0.3f')];
               end;
               if tm.prior_inf_shortrun==1;
                  file = [file '.' num2str(tm.lambda_b,'%0.3f') '.' num2str(tm.lambda_theta,'%0.3f') '.' num2str(tm.lambda_l,'%0.3f') '.' num2str(tm.lambda_e,'%0.3f')];
               end;
               if tm.prior_inf_deter==1;
                  file = [file '.' num2str(tm.lambda_d,'%0.3f')];
               end;
               file = [file '-' num2str(tm.save_draws,'%0.0f') '.' num2str(i,'%0.0f') '.' num2str(tm.num_fdraws,'%0.0f') '.' num2str(num_fp,'%0.0f') '-' num2str(tm.d+tm.dcoint,'%0.0f') '.' num2str(tm.datad,'%0.0f') '.mat'];
               if fileexist(file)==1;
                  if i==1;
                     txt = 'Would you like to delete all files with forecasting results for the current setup?';
                     answer = query(txt,'question',120,'Delete Forecast Results...',400,'no');
                     if strcmp(answer,'Yes')==1;
                        setlogtext(strvcat('Simulation aborted...','Deleting output files...',' '));
                        set(f_cast.show,'Enable','off');
                     else;
                        setlogtext(strvcat('Simulation aborted...',' '));
                        break;
                     end;
                  end;
                  delete_file_fcn(file);
               else;
                  break;
               end;
            end;
         end;
      else;
         %
         % dealing with the VAR in levels
         %
         
      end;
      %
      % show the forecast dialog again
      %
      set(forecastgui,'Visible','on');
      drawnow;
      drawnow;
   case 'checkvar'
      f_var = get(f_cast.firstvar,'Value');
      s_var = get(f_cast.secondvar,'Value');
      if f_var==s_var;
         var_num = size(get(f_cast.firstvar,'String'),1);
         if s_var==var_num;
            num = 1;
         else;
            num = s_var+1;
         end;
         set(f_cast.secondvar,'Value',num);
      end;
   case 'check_constant'
      new_val = strtrim(get(f_cast.first_constant,'String'));
      ch_div = findstr(new_val,'/');
      if isempty(ch_div)==1;
         new_val = mystr2num(new_val);
      else;
         new_val = strrep(new_val,'/',' ');
         new_val = vecstr(new_val);
         if size(new_val,1)==2;
            new_val1 = mystr2num(strtrim(new_val(1,:)));
            new_val2 = mystr2num(strtrim(new_val(2,:)));
            if isalmostequal(new_val2,0)==0;
               new_val = new_val1/new_val2;
            else;
               new_val = [];
            end;
         else;
            new_val = [];
         end;
      end;
      if isempty(new_val)==1;
         set(f_cast.first_constant,'String','1');
      elseif isalmostequal(new_val,0);
         set(f_cast.first_constant,'String','1');
      end;
   case 'checkconstant'
      new_val = strtrim(get(f_cast.constant,'String'));
      ch_div = findstr(new_val,'/');
      if isempty(ch_div)==1;
         new_val = mystr2num(new_val);
      else;
         new_val = strrep(new_val,'/',' ');
         new_val = vecstr(new_val);
         if size(new_val,1)==2;
            new_val1 = mystr2num(strtrim(new_val(1,:)));
            new_val2 = mystr2num(strtrim(new_val(2,:)));
            if isalmostequal(new_val2,0)==0;
               new_val = new_val1/new_val2;
            else;
               new_val = [];
            end;
         else;
            new_val = [];
         end;
      end;
      if isempty(new_val)==1;
         set(f_cast.constant,'String','0');
      end;
   case 'done'
      set(f_cast.done,'UserData','done');
      set(forecastgui,'UserData',f_cast);
   case 'wayout'
      set(f_cast.done,'UserData','done');
%      if strcmp(get(f_cast.compute,'UserData'),'done')==1;
%         killguipid(forecastgui);
%      end;
end;

if nargout==1;
   h = forecastgui;
end;

if strcmp(selector,'wayout')==0;
   curr_ini.models = tm;
   set(f_cast.firstvar,'UserData',curr_ini);
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(forecastgui,'Name') ' ' fix_filepath([pwd '\icons\forecast.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(f_cast.icon,'UserData',pid);
      end;
   end;
   %
   set(forecastgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(forecastgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(f_cast.done);
   end;
   waitfor(f_cast.done,'UserData','done');
end;

%
% end of bayes_var_forecastdlg.m
%
