%
% Estimate beta under 2 additional restriction
%
function thisbeta = beta_3d_restrict(S_00,S_01,S_10,S_11,oldbeta,T,dcoint,modelsdir,modelfile,thisbijx,rowx,colx,thisbijy,rowy,coly);

thisbeta = [];

r = size(oldbeta,2);
n = size(S_00,1);
nq1 = size(oldbeta,1);
sivec = zeros(r,1);
q1 = nq1-n-dcoint;

file = strcat(modelsdir,'normalize-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.norm');
normvec = loaddata(file);

%
% setting up the restrictions
%
for i=1:r;
   file = strcat(modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.rest');
   if fileexist(file)==1;
      h = loaddata(file);
      if i==colx;
         %
         % Recreate h such that new restriction is properly taken into account
         %
         normelem = normvec(i,1);
         normmatrix = [h(normelem,:);h(rowx,:)];
         if size(h,2)>size(normmatrix,1);
            for k=1:(n+q1+dcoint);
               if (k~=normelem)&(k~=rowx);
                  checkHk = [normmatrix;h(k,:)];
                  if rank(checkHk*checkHk')==size(checkHk,1);
                     normmatrix = checkHk;
                     if size(normmatrix,1)==size(h,2);
                        break;
                     end;
                  end;
               end;
            end;
         end;
         %
         % ensuring that h has [1 0 ... 0]
         % for normalizing parameter and
         % [0 1 0 ... 0] for restricted parameter
         %
         h = h/normmatrix;
         %
         % imposing the restriction
         %
         hj = thisbijx*h(:,2);
         if size(h,2)>2;
            h = [h(:,1)+hj h(:,3:size(h,2))];
         else;
            h = h(:,1)+hj;
         end;
      end;
      if i==coly;
         %
         % Recreate h such that new restriction is properly taken into account
         %
         normelem = normvec(i,1);
         normmatrix = [h(normelem,:);h(rowy,:)];
         if size(h,2)>size(normmatrix,1);
            for k=1:(n+q1+dcoint);
               if (k~=normelem)&(k~=rowy);
                  checkHk = [normmatrix;h(k,:)];
                  if rank(checkHk*checkHk')==size(checkHk,1);
                     normmatrix = checkHk;
                     if size(normmatrix,1)==size(h,2);
                        break;
                     end;
                  end;
               end;
            end;
         end;
         %
         % ensuring that h has [1 0 ... 0]
         % for normalizing parameter and
         % [0 1 0 ... 0] for restricted parameter
         %
         h = h/normmatrix;
         %
         % imposing the restriction
         %
         hj = thisbijy*h(:,2);
         if size(h,2)>2;
            h = [h(:,1)+hj h(:,3:size(h,2))];
         else;
            h = h(:,1)+hj;
         end;
      end;
      if i==1;
         H = h;
         bH = h;
      else;
         H = [H h];
         bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
      end;
      sivec(i,1) = size(h,2);
   end;
end;

%
% Computing input values for oldbeta
%
S_bb = oldbeta'*oldbeta;
%
% Solving the eigenvalue problem
%
% Step 1: Diagonalize S_bb
%
[W,rho] = eig(S_bb);
rho = diag(rho,0);
[rho2,AscInd] = sort(abs(rho));
nr = size(AscInd,1);
DescInd = zeros(nr,1);
i1 = 1;
while i1<=nr;
   DescInd(i1)=AscInd(nr+1-i1);
   i1 =i1+1;
end;
%
% Resort eigenvalues and eigenvectors
% in descending order
%
rho = rho(DescInd);
sqrho = sqrt(rho);
rho = diag(rho,0);
sqrho = diag(sqrho,0);
W = W(:,DescInd);
inv_sqrho = eye(size(sqrho,1))/sqrho;
sqS_bb = W*inv_sqrho*W';
oldlambda = zeros(r,1);
i=1;
dfoldbeta = 0;
while i<=r;
   %
   % Selecting the restrictions from H
   %
   si = sivec(i,1);
   dfoldbeta = dfoldbeta+(nq1-r-si+1);
   sj = size(H,2);
   if i==1;
      Hsel = [eye(si);zeros(sj-si,si)];
   elseif i==r;
      Hsel = [zeros(sj-si,si);eye(si)];
   else;
      sk = sivec(1,1);
      j=2;
      while j<=i-1;
         sk = sk+sivec(j,1);
         j = j+1;
      end;
      Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
   end;
   Hi = H*Hsel;
   %
   % Step 2: Reformulating eigenvalue problem to
   %         calculating the eigenvalues for
   %
   %         adjS_bb = sqS_bb*oldbeta'*Hi*inverse(Hi'*Hi)*Hi'*oldbeta*sqS_bb
   %
   adjS_bb = sqS_bb*((oldbeta'*Hi)/(Hi'*Hi))*Hi'*oldbeta*sqS_bb;
   [U,lambdai] = eig(adjS_bb);
   lambdai = diag(lambdai,0);
   [lambdai2,AscInd] = sort(abs(lambdai));
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   i1 = 1;
   while i1<=nr;
      DescInd(i1)=AscInd(nr+1-i1);
      i1 =i1+1;
   end;
   %
   % Resort eigenvalues and eigenvectors
   % in descending order
   %
   lambdai = lambdai(DescInd);
   U = U(:,DescInd);
   %
   % Renormalize U such that V'S_bbV=I
   %
   V = sqS_bb*U;
   %
   % Computing the initial estimate of oldbetai
   % using the first eigenvector from the
   % solution to the above eigenvalue problem
   %
   oldbetai = oldbeta*V(:,1);
   oldlambda(i,1) = lambdai(1,1);
   if i==1;
      oldbetainit = oldbetai;
   else;
      oldbetainit = [oldbetainit oldbetai];
   end;
   i=i+1;
end;
for i=1:r;
   if oldlambda(i,1)==1;
      oldlambda(i,1) = 0;
   end;
end;
oldbetainit = real(oldbetainit);

curr_ini = find_inisettings;

if curr_ini.convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
end;
ccrit = mystr2num(ch_convcrit);
%
% Starting up the iterations
%
grad = 1;
it = 1;
while ((it<=curr_ini.maxit)&(grad>ccrit));
   i = 1;
   newlambda = zeros(r,1);
   while i<=r;
      %
      % Selecting the restrictions from H
      %
      si = sivec(i,1);
      sj = size(H,2);
      if i==1;
         Hsel = [eye(si);zeros(sj-si,si)];
         tau = oldbetainit(:,2:r);
      elseif i==r;
         Hsel = [zeros(sj-si,si);eye(si)];
         tau = oldbetainit(:,1:r-1);
      else;
         sk = sivec(1,1);
         j=2;
         while j<=i-1;
            sk = sk+sivec(j,1);
            j = j+1;
         end;
         Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
         tau = [oldbetainit(:,1:i-1) oldbetainit(:,i+1:r)];
      end;
      Hi = H*Hsel;
      %
      % Computing the S_ij_tau matrices
      %
      S_00_tau = S_00-(((S_01*tau)/(tau'*S_11*tau))*tau'*S_10);
      S_11_tau = S_11-(((S_11*tau)/(tau'*S_11*tau))*tau'*S_11);
      S_01_tau = S_01-(((S_01*tau)/(tau'*S_11*tau))*tau'*S_11);
      S_10_tau = S_10-(((S_11*tau)/(tau'*S_11*tau))*tau'*S_10);
      HS_11_tau = Hi'*S_11_tau*Hi;
      %
      % Solving the eigenvalue problem for fixed tau
      %
      % Step 1: Diagonalize HS_11_tau
      %
      [W,rho] = eig(HS_11_tau);
      rho = diag(rho,0);
      [rho2,AscInd] = sort(abs(rho));
      nr = size(AscInd,1);
      DescInd = zeros(nr,1);
      i1 = 1;
      while i1<=nr;
         DescInd(i1)=AscInd(nr+1-i1);
         i1 = i1+1;
      end;
      %
      % Resort eigenvalues and eigenvectors
      % in descending order
      %
      rho = rho(DescInd);
      sqrho = sqrt(rho);
      rho = diag(rho,0);
      sqrho = diag(sqrho,0);
      W = W(:,DescInd);
      %
      % Step 2: Reformulating eigenvalue problem to
      %         calculating the eigenvalues for
      %
      %         adjHS_11_tau =
      %        sqHS_11_tau*Hi'*S_10_tau*inverse(S_00_tau)*S_01_tau*Hi*sqHS_11_tau
      %
      inv_sqrho = eye(size(sqrho,1))/sqrho;
      sqHS_11_tau = W*inv_sqrho*W';
      adjHS_11_tau = sqHS_11_tau*Hi'*(S_10_tau/S_00_tau)*S_01_tau*Hi*sqHS_11_tau;
      [U,lambdai] = eig(adjHS_11_tau);
      lambdai = diag(lambdai,0);
      [lambdai2,AscInd] = sort(abs(lambdai));
      nr = size(AscInd,1);
      DescInd = zeros(nr,1);
      i1 = 1;
      while i1<=nr;
         DescInd(i1)=AscInd(nr+1-i1);
         i1 = i1+1;
      end;
      %
      % Resort eigenvalues and eigenvectors
      % in descending order
      %
      lambdai = lambdai(DescInd);
      U = U(:,DescInd);
      %
      % Renormalize U such that V'HS_11_tauV=I
      %
      V = sqHS_11_tau*U;
      newlambda(i,1) = lambdai(1,1);
      newbetai = Hi*V(:,1);
      %
      % Changing oldbetainit to include newbetai
      % for column i
      %
      if i==1;
         oldbetainit = [newbetai oldbetainit(:,2:r)];
      elseif i==r;
         oldbetainit = [oldbetainit(:,1:r-1) newbetai];
      else;
         oldbetainit = [oldbetainit(:,1:i-1) newbetai oldbetainit(:,i+1:r)];
      end;
      i = i+1;
   end;
   oldbetainit = real(oldbetainit);
   newlambda = real(newlambda);
   %
   % Calculate the convergence criterion using the
   % log-likelihood function
   %
   if curr_ini.betagrad==0;
      oldsum = sum(log(ones(r,1)-oldlambda));
      newsum = sum(log(ones(r,1)-newlambda));
      logdetS_00 = log(det(S_00));
      grad = abs(((newsum-oldsum)/(logdetS_00+oldsum)));
   else;
      %
      % computing the gradient vector
      %
      alphainit = (S_01*oldbetainit)/(oldbetainit'*S_11*oldbetainit);
      Omegainit = S_00-(((S_01*oldbetainit)/(oldbetainit'*S_11*oldbetainit))*oldbetainit'*S_10);
      inv_Omegainit = eye(size(Omegainit,1))/Omegainit;
      grad = T*vec(alphainit)'*kron(eye(r),inv_Omegainit*(S_01-(alphainit*oldbetainit'*S_11)))*bH;
      grad = max(abs(grad));
   end;
   oldlambda = newlambda;
   it=it+1;
end;
thisbeta = oldbetainit;
for i=1:r;
   normcoeffi = normvec(i,1);
   normbetai = thisbeta(normcoeffi,i);
   if abs(normbetai)<1.0e-006;
      normbetai = 1;
      mycheck = 0;
   end;
   for j=1:(n+q1+dcoint);
      thisbeta(j,i) = thisbeta(j,i)/normbetai;
   end;
end;

%
% end of beta_3d_restrict.m
%
