%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI for estimating beta under linear           %
%          restrictions                                          %
%                                                                %
%          Function: betagui                                     %
%                                                                %
%          Input:                                                %
%                    selector: case switch function              %
%                    curr_model: structure with model info       %
%                                                                %
%          Output:                                               %
%                    myhandle: handle                            %
%                                                                %
%          This version:        July 4, 2006                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function myhandle = betagui(selector,curr_model)

if strcmp(selector,'init')==1;
   cm = curr_model;
   %
   cm.running = 0;
   cm.cbs = cm.c;
   cm.dfbeta = 0;
   cm.dfalpha = 0;
   dfc = round(((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)/cm.n);
   cm.df = cm.n*(cm.p-1)+cm.r+cm.q1*cm.pe+cm.q0*(cm.pe+1)+cm.d+dfc;
   cm.bZ1t = normbeta(cm.beta)'*cm.Z1t;
   cm.bR1t = normbeta(cm.beta)'*cm.R1t;
   cm.oldbeta = cm.beta;
   cm.oldcval = 0;
   cm.restrict = [0 0];
   thisexit = 0;
   cm.estimated = 0;
   %   
   currbeta = showbeta(normbeta(cm.beta),cm.evars,strtrim([cm.xvars1 ' ' cm.restvec]));
   %
   % Test if H_i-modelfile.rest files exist in modelsdir
   %
   estimateenable = 'off';
   i = 1;
   while i<=cm.r;
      file = strcat(cm.modelsdir,'H_',num2str(i),'-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
      if fileexist(file)==1;
         i = i+1;
      else;
         break;
      end;
   end;
   %
   % if necessary test if normalize-modelfile.norm exists in modelsdir
   % and if so set cointbeta.estimate control to
   % being enabled
   %
   if i==cm.r+1;
      file = strcat(cm.modelsdir,'normalize-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.norm');
      if fileexist(file)==1;
         file = strcat(cm.modelsdir,'H_1-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
         checkH_1 = loaddata(file);
         if size(checkH_1,1)==size(cm.beta,1);
            estimateenable = 'on';
         end;
      end;
   end;
   %
   curr_ini = find_inisettings;
   if curr_ini.lagtest>0;
      lagenable = 'on';
   else;
      lagenable = 'off';
   end;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
   %
   % import images
   %
   button_images = read_images;
   %
   cm.alpha = (cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta);
   if isempty(cm.Z2)==0;
      cm.Psi = (cm.M_02/cm.M_22)-(cm.alpha*cm.beta'*(cm.M_12/cm.M_22));
      cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1)-(cm.Psi*cm.Z2);
      Z3 = [cm.Z2;cm.beta'*cm.Z1];
   else;
      cm.Psi = [];
      cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1);
      Z3 = cm.beta'*cm.Z1;
   end;
   Theta = [cm.Psi cm.alpha];
   cm.Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
   cm.oldTheta = Theta;
   inv_Z3 = eye(size(Z3,1))/(Z3*Z3');
   cm.oldSigma_theta = kron(inv_Z3,cm.Omega);
   lnl = loglike(cm.epsilon);
   cm.trendplace = 0;
   if cm.spectest(1,4)==1;
      if cm.deterselect(1,3)==1;
         cm.trendplace = 2;
      end;
   end;
   %
   % Checking eigenvalue of the companion matrix
   %
   eigenvalues = companion(cm.Z0,cm.Z1,cm.Z2,cm.r,cm.p,cm.n,cm.q1,cm.q0,cm.d,cm.T);
   eigenvalues = abs(sort(eigenvalues));
   i = size(eigenvalues,1);
   while i>=1;
      cm.evalue = eigenvalues(i,1);
      if strcmp(num2str(cm.evalue,'%1.10f'),'1.0000000000')==1;
         i = i-1;
      else;
         i = 0;
      end;
   end;
   %
   % get string matrices for model information
   %
   cm.model_info = getmodelinfo(cm,1);
   cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,lnl,cm.evalue);
   %
   if cm.evalue>=1;
      mypermcompenable = 'off';
   else;
      mypermcompenable = 'on';
   end;
   cmatenable = mypermcompenable;
   weakexoenable = 'on';
   if cm.r==cm.n;
      weakexoenable = 'off';
   end;
   if (cm.n==1)&(cm.q1+cm.q0==0);
      gcenable = 'off';
   else;
      gcenable = 'on';
   end;
   myalphaenable = 'off';
   file = strcat(cm.modelsdir,'G-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
   if fileexist(file)==1;
      myalphaenable = 'on';
   end;
   showpdfdoc = [];
   if (strcmp(curr_ini.pdfviewer,'')==0)|(strcmp(curr_ini.epsviewer,'')==0)|(strcmp(curr_ini.psviewer,'')==0);
      pdf_file = [pwd '\doc\generalized-impulse-response.pdf'];
      if isunix==1;
         pdf_file = strrep(pdf_file,'\','/');
      end;
      if fileexist(pdf_file)==1;
         showpdfdoc.gir = pdf_file;
      else;
         showpdfdoc.gir = '';
      end;
      pdf_file = [pwd '\doc\common-cycles.pdf'];
      if isunix==1;
         pdf_file = strrep(pdf_file,'\','/');
      end;
      if fileexist(pdf_file)==1;
         showpdfdoc.cc = pdf_file;
      else;
         showpdfdoc.cc = '';
      end;
   end;
else;
   thisexit = 1;
%   betaguidlg = get(0,'CurrentFigure');
   betaguidlg = findobj('Type','figure','Tag','BetaGUI')';
   cointbeta = get(betaguidlg,'UserData');
   curr_ini = get(cointbeta.viewdata,'UserData');
   button_images = curr_ini.images;
   cm = curr_ini.model;
   cm.cointbeta = cointbeta;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

scrsz = [1 1 1024 768];
realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';

      if cm.modeltype(1,1)==1;
         add_width = 20;
      else;
         add_width = 0;
      end;

      betaguidlg = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','betagui.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-(scrsz(3)/2)-add_width-100)/2 (realscrsz(4)-(scrsz(4)/2))/2 scrsz(3)/2+add_width+100 scrsz(4)/2], ...
         'Tag','BetaGUI', ...
         'Resize','off', ...
         'Name','Structural VAR - Restrictions on (alpha,beta)', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','betagui wayout', ...
         'Visible','off', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(betaguidlg,'DockControl','off');
      end;

      %
      % File menu
      %

      filemenu = uimenu('Parent',betaguidlg, ...
         'Label','&File', ...
         'Tag','FileMenu');

      %
      % Objects on File menu
      %

      cointbeta.viewdata = uimenu('Parent',filemenu, ...
         'Label','View &Data...', ...
         'Callback','betagui viewdata', ...
         'Enable','on', ...
         'Tag','ViewData');

      cointbeta.savecoint = uimenu('Parent',filemenu, ...
         'Label','Save &Cointegration Relations...', ...
         'Callback','betagui savecoint', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','SaveResid');

      cointbeta.savetc = uimenu('Parent',filemenu, ...
         'Label','Save &Transitory Components...', ...
         'Callback','betagui savetc', ...
         'Accelerator','T', ...
         'Enable',mypermcompenable, ...
         'Tag','SaveTC');

      cointbeta.saveresid = uimenu('Parent',filemenu, ...
         'Label','Save &Residuals...', ...
         'Callback','betagui saveresid', ...
         'Accelerator','D', ...
         'Enable','on', ...
         'Tag','SaveCoint');

      uimenu('Parent',filemenu, ...
         'Label','Center &Window', ...
         'Separator','on', ...
         'Accelerator','W', ...
         'Enable','on', ...
         'Callback','center_figure_fcn', ...
         'Tag','CenterMenu');

      uimenu('Parent',filemenu, ...
         'Label','R&eturn', ...
         'Callback','betagui quit', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuReturn');

      uimenu('Parent',filemenu, ...
         'Label','C&ontinue', ...
         'Callback','betagui continue', ...
         'Enable','on', ...
         'Accelerator','C', ...
         'Tag','MenuContinue');

      if cm.modeltype(1,1)==1;

         cointbeta.menustop = uimenu('Parent',filemenu, ...
            'Label','&Quit', ...
            'Separator','on', ...
            'Accelerator','Q', ...
            'Callback','betagui continue', ...
            'Enable','on', ...
            'Tag','MenuStop');

      end;

      %
      % Tools menu
      %

      testmenu = uimenu('Parent',betaguidlg, ...
         'Label','&Tools', ...
         'Tag','TestMenu');

      %
      % Objects on Tools menu
      %

      cointbeta.corrtest = uimenu('Parent',testmenu, ...
         'Label','&Serial Correlation...', ...
         'Callback','betagui corrtest', ...
         'Enable','on', ...
         'Accelerator','S', ...
         'Tag','CorrTest');

      cointbeta.archtest = uimenu('Parent',testmenu, ...
         'Label','&ARCH...', ...
         'Callback','betagui archtest', ...
         'Accelerator','A', ...
         'Enable','on', ...
         'Tag','ArchTest');

      cointbeta.normalitytest = uimenu('Parent',testmenu, ...
         'Label','&Normality...', ...
         'Callback','betagui normalitytest', ...
         'Accelerator','N', ...
         'Enable','on', ...
         'Tag','NormalityTest');

      cointbeta.constancytest = uimenu('Parent',testmenu, ...
         'Label','&Parameter Constancy...', ...
         'Callback','betagui constancytest', ...
         'Accelerator','P', ...
         'Enable','on', ...
         'Tag','ConstancyTest');

      cointbeta.lagorder = uimenu('Parent',testmenu, ...
         'Label','&Lag Order...', ...
         'Callback','betagui lagorder', ...
         'Accelerator','L', ...
         'Separator','on', ...
         'Enable',lagenable, ...
         'Tag','LagOrder');

      cointbeta.weakexo = uimenu('Parent',testmenu, ...
         'Label','&Weak Exogeneity...', ...
         'Callback','betagui weakexo', ...
         'Enable',weakexoenable, ...
         'Tag','WeakExogeneity');

      cointbeta.gctest = uimenu('Parent',testmenu, ...
         'Label','&Granger Causality...', ...
         'Callback','betagui gctest', ...
         'Accelerator','G', ...
         'Enable',gcenable, ...
         'Tag','GrangerCausality');

      if cm.n>2;
         cointbeta.multigctest = uimenu('Parent',testmenu, ...
            'Label','M&ulti-Step Granger Causality...', ...
            'Callback','betagui multigctest', ...
            'Enable','on', ...
            'Tag','MultiGrangerCausality');
      end;


      if ((cm.p>1)&(cm.n>2))|(cm.q0>0)|((cm.q1>0)&(cm.pe>0));
         cointbeta.exclude = uimenu('Parent',testmenu, ...
            'Label','&Exclusion', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','ExclusionMenu');
         
         cointbeta.firstdiffs = uimenu('Parent',cointbeta.exclude, ...
            'Label','&First Differences', ...
            'Callback','betagui firstdiffs', ...
            'Enable','on', ...
            'Tag','FirstDiffMenu');
      end;


      cointbeta.space = uimenu('Parent',testmenu, ...
         'Label','Co&integration Space', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','CointSpaceMenu');

      cointbeta.exclusion = uimenu('Parent',cointbeta.space, ...
         'Label','&Exclusion...', ...
         'Callback','betagui exclusion', ...
         'Enable','on', ...
         'Tag','CointExclusionMenu');

      cointbeta.stationarity = uimenu('Parent',cointbeta.space, ...
         'Label','&Stationarity...', ...
         'Callback','betagui stationarity', ...
         'Enable','on', ...
         'Tag','CointStationarityMenu');

      if (cm.p>1)&(cm.r<cm.n);
         cointbeta.common_cycles = uimenu('Parent',testmenu, ...
            'Label','Co&mmon Cycles...', ...
            'Callback','betagui common_cycles', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','ShowCommonCycles');
      end;

      if cm.datad>0;
         cointbeta.dummy = uimenu('Parent',testmenu, ...
            'Label','&Deterministic Variables', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','DummyVarMenu');

        cointbeta.dummyexclusion = uimenu('Parent',cointbeta.dummy, ...
         'Label','&Exclusion...', ...
         'Callback','betagui dummyexclusion', ...
         'Enable','on', ...
         'Tag','DummyExclusionMenu');

         cointbeta.dummylongrun = uimenu('Parent',cointbeta.dummy, ...
            'Label','&Long Run...', ...
            'Callback','betagui dummylongrun', ...
            'Enable','off', ...
            'Tag','DummyLongRunMenu');

         if cm.r<cm.n;
            set(cointbeta.dummylongrun,'Enable','on');
         end;

         if cm.datad>1;
            cointbeta.propdummy = uimenu('Parent',cointbeta.dummy, ...
            'Label','&Proportionality...', ...
            'Callback','betagui dummyprop', ...
            'Enable','on', ...
            'Tag','DummyPropMenu');
         end;

      end;

      cointbeta.cmat = uimenu('Parent',testmenu, ...
         'Label','&C-Matrix...', ...
         'Callback','betagui cmat', ...
         'Separator','on', ...
         'Accelerator','M', ...
         'Enable',cmatenable, ...
         'Tag','CMatrix');

      cointbeta.longrun_gir = uimenu('Parent',testmenu, ...
         'Label','L&ong Run GIRs...', ...
         'Callback','betagui longrun_gir', ...
         'Separator','off', ...
         'Enable',cmatenable, ...
         'Tag','LongRunGIRsMenu');

      cointbeta.residanalysis = uimenu('Parent',testmenu, ...
         'Label','&Residual Analysis...', ...
         'Callback','betagui residanalysis', ...
         'Accelerator','R', ...
         'Enable','on', ...
         'Tag','ResidAnalysis');

      cointbeta.shortdyn = uimenu('Parent',testmenu, ...
         'Label','S&hort Run Dynamics...', ...
         'Callback','betagui shortdyn', ...
         'Accelerator','Y', ...
         'Enable','on', ...
         'Tag','ShortDyn');

      cointbeta.distribution = uimenu('Parent',testmenu, ...
         'Label','Distri&bution...', ...
         'Callback','betagui distribution', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','DistributionMenu');

      %
      % Graphics menu
      %

      graphmenu = uimenu('Parent',betaguidlg, ...
         'Label','&Graphics', ...
         'Tag','GraphicsMenu');

      cointbeta.menugraph = uimenu('Parent',graphmenu, ...
         'Label','&Cointegration Relations...', ...
         'Callback','betagui graphs', ...
         'Enable','on', ...
         'Tag','MenuGraph');

      cointbeta.menupermcomp = uimenu('Parent',graphmenu, ...
         'Label','&Permanent and Transitory Components...', ...
         'Callback','betagui permcomp', ...
         'Accelerator','E', ...
         'Enable',mypermcompenable, ...
         'Tag','MenuPermComp');

      cointbeta.menuresiduals = uimenu('Parent',graphmenu, ...
         'Label','&Residuals...', ...
         'Callback','betagui residuals', ...
         'Accelerator','I', ...
         'Enable','on', ...
         'Tag','MenuResiduals');

      cointbeta.menufitted = uimenu('Parent',graphmenu, ...
         'Label','Fitted &Terms...', ...
         'Callback','betagui fitted', ...
         'Enable','on', ...
         'Tag','MenuFitted');


      cointbeta.forecast = uimenu('Parent',graphmenu, ...
         'Label','Forec&asting...', ...
         'Callback','betagui forecast', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuFitted');

      cointbeta.varforecast = uimenu('Parent',graphmenu, ...
         'Label','Forecast &Variable Transformations...', ...
         'Callback','betagui varforecast', ...
         'Enable','on', ...
         'Tag','MenuVARForecast');

      if cm.q1+cm.q0>0;
         set(cointbeta.forecast,'Enable','off');
         set(cointbeta.varforecast,'Enable','off');
      end;

      cointbeta.menuchow = uimenu('Parent',graphmenu, ...
         'Label','Recursive Ch&ow Tests', ...
         'Callback','betagui chow', ...
         'Separator','on', ...
         'Accelerator','O', ...
         'Enable','on', ...
         'Tag','MenuChow');

      cointbeta.menuploberger = uimenu('Parent',graphmenu, ...
         'Label','Recursive &Fluctuation Test', ...
         'Callback','betagui ploberger', ...
         'Accelerator','F', ...
         'Enable','on', ...
         'Tag','MenuPloberger');
         
      cointbeta.genimpulse = uimenu('Parent',graphmenu, ...
         'Label','&Generalized Impulse Responses...', ...
         'Callback','betagui genimpulse', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuGeneralizedImpulse');

%      if (isempty(cm.isdummy)==0)&(max(cm.isdummy)==1);
      if cm.datad>0;
         cointbeta.dummyimpulse = uimenu('Parent',graphmenu, ...
            'Label','&Dummy Impulse Responses...', ...
            'Callback','betagui dummyimpulse', ...
            'Enable','on', ...
            'Tag','MenuDummyImpulse');
      end;
         
      cointbeta.betagraph = uimenu('Parent',graphmenu, ...
         'Label','&Log-likelihood and Beta...', ...
         'Callback','betagui betagraph', ...
         'Separator','on', ...
         'Accelerator','K', ...
         'Enable','off', ...
         'Tag','MenuBetaGraph');

      cointbeta.companion = uimenu('Parent',graphmenu, ...
         'Label','Eigenvalues of Companion &Matrix', ...
         'Callback','betagui companion', ...
         'Enable','on', ...
         'Tag','MenuCompanion');

      %
      % Alpha menu
      %

      alphamenu = uimenu('Parent',betaguidlg, ...
         'Label','&Alpha', ...
         'Tag','AlphaMenu');

      cointbeta.menugenalpha = uimenu('Parent',alphamenu, ...
         'Label','&Restrict Alpha...', ...
         'Callback','betagui genalpha', ...
         'Enable','on', ...
         'Tag','MenuGenAlpha');

      cointbeta.menuestalpha = uimenu('Parent',alphamenu, ...
         'Label','&Estimate Alpha...', ...
         'Callback','betagui estalpha', ...
         'Enable',myalphaenable, ...
         'Tag','MenuEstAlpha');

      cointbeta.menualphaundo = uimenu('Parent',alphamenu, ...
         'Label','&Unrestrict Alpha', ...
         'Callback','betagui alphaundo', ...
         'Accelerator','U', ...
         'Enable','off', ...
         'Tag','MenuAlphaUndo');

      cointbeta.menuopenalpharest = uimenu('Parent',alphamenu, ...
         'Label','Open &Alpha Restrictions...', ...
         'Callback','betagui openalpharest', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuOpenAlpha');

      cointbeta.menusavealpharest = uimenu('Parent',alphamenu, ...
         'Label','&Save Alpha Restrictions...', ...
         'Callback','betagui savealpharest', ...
         'Enable','on', ...
         'Tag','MenuSaveAlpha');

      %
      % Beta menu
      %

      betamenu = uimenu('Parent',betaguidlg, ...
         'Label','&Beta', ...
         'Tag','BetaMenu');

      cointbeta.menurestrict = uimenu('Parent',betamenu, ...
         'Label','&Restrict Beta...', ...
         'Callback','betagui restrict', ...
         'Enable','on', ...
         'Tag','MenuRestrict');

      cointbeta.menuestimate = uimenu('Parent',betamenu, ...
         'Label','&Estimate Beta...', ...
         'Callback','betagui estimate', ...
         'Enable',estimateenable, ...
         'Tag','MenuEstimate');

      cointbeta.menuclear = uimenu('Parent',betamenu, ...
         'Label','&Unrestrict Beta', ...
         'Callback','betagui clear', ...
         'Accelerator','B', ...
         'Enable','off', ...
         'Tag','MenuClear');

      cointbeta.menuopenrest = uimenu('Parent',betamenu, ...
         'Label','Open &Beta Restrictions...', ...
         'Callback','betagui openrest', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuOpen');

      cointbeta.menusaverest = uimenu('Parent',betamenu, ...
         'Label','&Save Beta Restrictions...', ...
         'Callback','betagui saverest', ...
         'Enable','on', ...
         'Tag','MenuSave');


      tpstr = 'Estimate Beta Re&cursively';
      if curr_ini.recalpha==1;
         tpstr = 'Estimate (Alpha,Beta) Re&cursively';
      end;

      cointbeta.menurecursive = uimenu('Parent',betamenu, ...
         'Label',tpstr, ...
         'Callback','betagui recursive', ...
         'Separator','on', ...
         'Enable',estimateenable, ...
         'Tag','MenuRecursive');

      simulatemenu = uimenu('Parent',betaguidlg, ...
         'Label','B&ootstrap', ...
         'Tag','SimulateMenu');

      cointbeta.bootalpha = uimenu('Parent',simulatemenu, ...
         'Label','&Alpha, Delta, Gamma...', ...
         'Callback','betagui bootalpha', ...
         'Enable','on', ...
         'Tag','MenuBootAlpha');

      cointbeta.bootbeta = uimenu('Parent',simulatemenu, ...
         'Label','&Beta...', ...
         'Callback','betagui bootbeta', ...
         'Enable',estimateenable, ...
         'Tag','MenuBootBeta');

      cointbeta.bootnyblommenu = uimenu('Parent',simulatemenu, ...
         'Label','&Nyblom Distribution...', ...
         'Callback','betagui bootnyblomtest', ...
         'Enable',estimateenable, ...
         'Tag','MenuBootNyblom');
      
      if (curr_ini.nyblom==0)|(curr_ini.nyblombeta==0);
         set(cointbeta.bootnyblommenu,'Enable','off');
      end;

      cointbeta.bootploberger = uimenu('Parent',simulatemenu, ...
         'Label','&Recursive Fluctuation Test...', ...
         'Callback','betagui bootploberger', ...
         'Enable','on', ...
         'Tag','MenuBootPloberger');

      cointbeta.boot_specification = uimenu('Parent',simulatemenu, ...
         'Label','&Specification Tests...', ...
         'Callback','betagui boot_specification', ...
         'Enable','on', ...
         'Tag','MenuBootSpecification');

      cointbeta.markov_switching = uimenu('Parent',simulatemenu, ...
         'Label','&Markov Switching Test...', ...
         'Callback','betagui boot_markov', ...
         'Enable','on', ...
         'Tag','MenuBootMarkov');

      %
      % Help menu
      %

      helpmenu = uimenu('Parent',betaguidlg, ...
         'Label','&Help', ...
         'Tag','HelpMenu');

      cointbeta.beta_help = uimenu('Parent',helpmenu, ...
            'Label','&Restrictions on alpha,beta', ...
            'Callback','help_fcn(''Restrictionsonalphabeta'');', ...
            'Accelerator','H', ...
            'Separator','off', ...
            'Enable','on', ...
            'Tag','MenuBetaHelp');

      if isempty(showpdfdoc)==0;
         if strcmp(showpdfdoc.cc,'')==0;
            cointbeta.pdf_cc = uimenu('Parent',helpmenu, ...
               'Label','&Common Cycles - PDF file', ...
               'Callback','show_pdf', ...
               'Separator','on', ...
               'UserData',showpdfdoc.cc, ...
               'Enable','on', ...
               'Tag','MenuDocCCEstimate');
            sep_str = 'off';
         else;
            sep_str = 'on';
         end;
         %
         if strcmp(showpdfdoc.gir,'')==0;
            cointbeta.pdf_gir = uimenu('Parent',helpmenu, ...
               'Label','&Generalized Impulse Responses - PDF file', ...
               'Callback','show_pdf', ...
               'Separator',sep_str, ...
               'UserData',showpdfdoc.gir, ...
               'Enable','on', ...
               'Tag','MenuDocGIREstimate');
         end;
         %
      end;

%      xline([1 scrsz(4)/2-3 (scrsz(3)/2)],[0.5 1]);

      xline([1 scrsz(4)/2-1 (scrsz(3)/2)+100+add_width],[0.5 1]);

      %
      % Outer axis
      %

      axesbox([2 2 (scrsz(3)/2)+add_width+98 scrsz(4)/2-15],'Maximum Likelihood Results',135,[0.5 1]);

      %
      % Outer axis for showing beta
      %

      axesbox([14 201 scrsz(3)/2+add_width+72 154],'Current Beta Matrix',90,[0.5 1]);

      %
      % Outer axis for text model info control
      %

      axesbox([14 14 scrsz(3)/2+add_width+72 173],'Model Information',85,[0.5 1]);

      %
      % Listbox for showing estimated beta
      %

      cointbeta.betamat = uicontrol('Units','pixels', ...
         'BackgroundColor','white', ...
         'Position',[30 217 scrsz(3)/2+add_width+40 122], ...
         'Style','listbox', ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'Max',2, ...
         'Value',[], ...
         'String',currbeta, ...
         'Enable','inactive', ...
         'UserData',0, ...
         'HorizontalAlignment','center', ...
         'Tag','BetaListbox');

      %
      % Text control listing info about model
      %

      cointbeta.model_header = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 30 scrsz(3)/2+add_width+40 145], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',cm.model_info.header, ...
         'ForegroundColor',curr_ini.gui_foreground_color, ...
         'HorizontalAlignment','left', ...
         'Tag','ModelInfoHeaderText');

      head_ext = get(cointbeta.model_header,'Extent');
      if head_ext(1,4)<=145;
         new_pos = [30 30+(145-head_ext(1,4))/2 head_ext(1,3:4)];
      else;
         new_pos = [30 30 head_ext(1,3) 145];
      end;
      set(cointbeta.model_header,'Position',new_pos);

      cointbeta.model_info = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[35+new_pos(1,3) new_pos(1,2) scrsz(3)/2+add_width+35-new_pos(1,3) new_pos(1,4)], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',cm.model_info.info, ...
         'UserData',0, ...
         'ForegroundColor',curr_ini.gui_foreground_color, ...
         'HorizontalAlignment','left', ...
         'Tag','ModelInfoText');

      cointbeta.toolbar = uitoolbar('Parent',betaguidlg, ...
                        'HandleVisibility','off', ...
                        'Interruptible','on', ...
                        'Clipping','on');

      %
      % Quit button
      %

      cointbeta.quit = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Return', ...
                         'CData',button_images.return, ...
                         'ClickedCallback','betagui quit', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','QuitButton');

      %
      %  continue button
      %

      cointbeta.continue = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Continue', ...
                         'CData',button_images.continue, ...
                         'ClickedCallback','betagui continue', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData','waiting', ...
                         'Enable','on', ...
                         'Tag','ContinueButton');

      %
      % Graphics button
      %

      cointbeta.graphs = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Cointegration Relations', ...
                         'CData',button_images.graph, ...
                         'ClickedCallback','betagui graphs', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData','working', ...
                         'Enable','on', ...
                         'Tag','Graphics');

      cointbeta.permcomp = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Permanent and Transitory Components', ...
                         'CData',button_images.permcomp, ...
                         'ClickedCallback','betagui permcomp', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable',mypermcompenable, ...
                         'Tag','PermCompGraphics');

      cointbeta.residgraph = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Residuals', ...
                         'CData',button_images.histogram, ...
                         'ClickedCallback','betagui residuals', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','ResidGraphics');

      %
      % Forecasting button
      %

      cointbeta.forecast_button = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Forecasting', ...
                         'CData',button_images.forecast, ...
                         'ClickedCallback','betagui forecast', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','ForecastGraph');

      if cm.q1+cm.q0>0;
         set(cointbeta.forecast_button,'Enable','off');
      end;

      %
      % Chow test button
      %

      cointbeta.chow = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Recursive Chow Tests', ...
                         'CData',button_images.chow, ...
                         'ClickedCallback','betagui chow', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','RecChowGraph');

      cointbeta.ploberger = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Recursive Fluctuation Test', ...
                         'CData',button_images.fluctest, ...
                         'ClickedCallback','betagui ploberger', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','RecPlobGraph');

      %
      % restrict alpha button
      %

      cointbeta.genalpha = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Restrict Alpha', ...
                         'CData',button_images.alpharest, ...
                         'ClickedCallback','betagui genalpha', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'UserData',cm.oldSigma_theta, ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','RestrictAlphaButton');

      %
      % Estimate alpha button
      %

      cointbeta.estalpha = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Estimate Alpha', ...
                         'CData',button_images.alphaest, ...
                         'ClickedCallback','betagui estalpha', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'UserData',Theta, ...
                         'Clipping','on', ...
                         'Enable',myalphaenable, ...
                         'Tag','EstimateAlphaButton');

      %
      % Undo alpha button
      %

      cointbeta.alphaundo = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Unrestrict Alpha', ...
                         'CData',button_images.alphaundo, ...
                         'ClickedCallback','betagui alphaundo', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','off', ...
                         'Tag','AlphaUndoButton');

      cointbeta.openalpharest = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Open Alpha Restrictions', ...
                         'CData',button_images.openalpha, ...
                         'ClickedCallback','betagui openalpharest', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','AlphaOpenRestButton');

      cointbeta.savealpharest = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Save Alpha Restrictions', ...
                         'CData',button_images.savealpha, ...
                         'ClickedCallback','betagui savealpharest', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','AlphaSaveRestButton');

      %
      % restrict beta button
      %

      cointbeta.restrict = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Restrict Beta', ...
                         'CData',button_images.betarest, ...
                         'ClickedCallback','betagui restrict', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData',cm.restrict, ...
                         'Enable','on', ...
                         'Tag','RestrictButton');

      %
      % estimate beta button
      %

      cointbeta.estimate = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Estimate Beta', ...
                         'CData',button_images.betaest, ...
                         'ClickedCallback','betagui estimate', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData',cm.restrict, ...
                         'Enable',estimateenable, ...
                         'Tag','EstimateButton');

      %
      % unrestricted beta button
      %

      cointbeta.clear = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Unrestrict Beta', ...
                         'CData',button_images.betaundo, ...
                         'ClickedCallback','betagui clear', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData',thisexit, ...
                         'Enable','off', ...
                         'Tag','ClearButton');

      cointbeta.openrest = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Open Beta Restrictions', ...
                         'CData',button_images.openbetarest, ...
                         'ClickedCallback','betagui openrest', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData',thisexit, ...
                         'Enable','on', ...
                         'Tag','OpenButton');

      cointbeta.saverest = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Save Beta Restrictions', ...
                         'CData',button_images.savebeta, ...
                         'ClickedCallback','betagui saverest', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData',thisexit, ...
                         'Enable','on', ...
                         'Tag','SaveButton');

      %
      % Recursive estimation button
      %

      tpstr = 'Estimate Beta Recursively';
      if curr_ini.recalpha==1;
         tpstr = 'Estimate (Alpha,Beta) Recursively';
      end;

      cointbeta.recursive = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString',tpstr, ...
                         'CData',button_images.recbetaest, ...
                         'ClickedCallback','betagui recursive', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable',estimateenable, ...
                         'Tag','RecursiveButton');

      if cm.modeltype(1,1)==1;

         cointbeta.stop = uipushtool('Parent',cointbeta.toolbar, ...
                         'TooltipString','Quit', ...
                         'CData',button_images.close, ...
                         'ClickedCallback','betagui continue', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'UserData','continue', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','QuitButton');

      end;

      set(betaguidlg,'UserData',cointbeta,...
         'HandleVisibility','callback');

   case 'distribution'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         %
         thisgraph = distdlg('init',cm.cbs);
         killguipid(thisgraph);
         %
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;

   case 'bootploberger'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         pkk_bootstrap('init',cm,'nocoint');
         drawnow;
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'boot_markov'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         markov_bootstrap('init',cm);
         drawnow;
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'boot_specification'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         specification_bootstrap('init',cm,'nocoint');
         drawnow;
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'bootbeta'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         beta_bootstrap('init',cm);
         drawnow;
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'bootalpha'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         alpha_bootstrap('init',cm);
         drawnow;
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'bootnyblomtest'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         nyblom_bootstrap('init',cm,get(betaguidlg,'UserData'));
         drawnow;
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'betagraph'
      if cm.running==0;
         cm.running = 1;
         %
         % Find the free parameters of beta
         %
         bH = [];
         for i=1:cm.r;
            file = strcat(cm.modelsdir,'H_',num2str(i),'-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
               h = loaddata(file);
               if i==1;
                  bH = h;
               else;
                  bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
               end;
            end;
         end;
         thisR = null(bH','r');
         file = strcat(cm.modelsdir,'normalize-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.norm');
         h = loaddata(file);
         for i=1:cm.r;
            thisvec = zeros(size(bH,1),1);
            thisvec(h(i,1)+(i-1)*size(cm.beta,1),1) = 1;
            thisR = [thisR thisvec];
         end;
         R_orth = null(thisR');
         for i=1:size(R_orth,2);
            if max(R_orth(:,i))>0;
               R_orth(:,i) = (1/max(R_orth(:,i)))*R_orth(:,i);
            else;
               R_orth(:,i) = (1/min(R_orth(:,i)))*R_orth(:,i);
            end;
         end;
         cm.R_orth = round(R_orth);
         %
         % The matrix R_orth shows the free parameters of vec(beta)
         %
         set(betaguidlg,'Visible','off');
         drawnow;
         mybetagraphgui = betagraphdlg('init',cm);
         graphbeta = get(mybetagraphgui,'UserData');
         set(graphbeta.graph,'UserData','done');
         killguipid(mybetagraphgui);
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'residanalysis'
      if cm.running==0;
         cm.running = 1;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_residual_analysis(betaguidlg,cm,ind);
         cm.running = 0;
      end;
   case 'fitted'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         mytheta = get(cointbeta.estalpha,'UserData');
         thisgraph = fittedterms('init',cm,mytheta,cm.beta);
         fitted = get(thisgraph,'UserData');
         set(fitted.show,'UserData','done');
         killguipid(thisgraph);
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'companion'
      if cm.running==0;
         cm.running = 1;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_companion(cm.beta,cm.alpha,cm.Psi,cm,curr_ini,ind,0);
         cm.running = 0;
      end;
   case 'common_cycles'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         show_common_cycles(cm,curr_ini,cm.alpha,cm.Psi,cm.beta);
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'dummyprop'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         proportionality_dummies_dlg('init',cm,curr_ini);
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'dummylongrun'
      if cm.running==0;
         cm.running = 1;
         show_longrun_dummies(betaguidlg,cm,curr_ini);
         cm.running = 0;
      end;
   case 'dummyexclusion'
      if cm.running==0;
         cm.running = 1;
         show_exclude_dummies(betaguidlg,cm,curr_ini);
         cm.running = 0;
      end;
   case 'dummyimpulse'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         %
         alpha = cm.alpha;
         beta = cm.beta;
         Psi = cm.Psi;
         Omega = cm.Omega;
         %
         txt = 'Would you like to impose restrictions on the dummy variables?';
         if (cm.datad>1)|((cm.r<cm.n)&(cm.r>0));
            answer = query(txt,'question',120,'Structural VAR - Dummy Restrictions',300,'no');
         else;
            answer = 'No';
         end;
         if strcmp(answer,'Yes')==1;
            %
            % open a select restrictions dialog
            %
            tm = restrict_dummies_dlg('init',cm,curr_ini);
            if strcmp(tm.action,'cancel')==0;
               %
               % check if we should use new estimated
               %
               if (isfield(tm,'applied')==1)&(isempty(tm.applied)==0)&(ischar(tm.applied)==1)&(strcmp(tm.applied,'applied')==1);
                  if isfield(tm,'dm')==1;
                     txt = 'Are you sure you want to use the restricted estimates for the dummy impulse responses?';
                     answer = query(txt,'question',120,'Structural VAR - Dummy Restrictions',300,'yes');
                     if strcmp(answer,'Yes')==1;
                        alpha = tm.dm.alpha;
                        beta = tm.dm.beta;
                        Psi = tm.dm.Psi;
                        Omega = tm.dm.Omega;
                        Sigma_theta = tm.dm.Sigma_theta;
                     end;
                  end;
               end;
            else;
               %
               % no dummy impulses
               %
               set(betaguidlg,'Visible','on');
               drawnow;
               return;
            end;
         end;
         %
         setlogtext(strvcat(['Computing impulse responses for dummies up to period ' num2str(cm.h)],['with asymptotic ' num2str(cm.c) ' % confidence bands...']));
         abort = '';
         [cm.duir,cm.duir_std,cm.buir,cm.buir_std,abort] = dummy_impulses_fcn(cm,alpha,beta,Psi,Omega,Sigma_theta,1);
         %
         if strcmp(abort,'cancel')==0;
            %
            % get the value from the normal distribution corresponding to the
            % chosen levels of significance c
            %
            c_val = norm_inv((100+cm.c)/200);
            for i=1:size(cm.duir,2);
               for j=1:cm.n*cm.datad;
                  if abs(cm.duir_std(j,i))<1.0e-008;
                     cm.duir_std(j,i) = 0;
                  end;
               end;
            end;
            cm.duir_upper = cm.duir+(c_val*cm.duir_std);
            cm.duir_lower = cm.duir-(c_val*cm.duir_std);
            %
            if isempty(cm.buir)==0;
               for i=1:size(cm.buir,2);
                  for j=1:cm.r*cm.datad;
                     if abs(cm.buir_std(j,i))<1.0e-008;
                        cm.buir_std(j,i) = 0;
                     end;
                  end;
               end;
               cm.buir_upper = cm.buir+(c_val*cm.buir_std);
               cm.buir_lower = cm.buir-(c_val*cm.buir_std);
            else;
               cm.buir_upper = [];
               cm.buir_lower = [];
            end;
            %
            thisgraph = duir_graphdlg('init',cm);
            ir_graph = get(thisgraph,'UserData');
            set(ir_graph.show,'UserData','done');
            killguipid(thisgraph);
         end;
         %
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'genimpulse'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         setlogtext(strvcat(['Computing generalized impulse responses up to period ' num2str(cm.h)],['with asymptotic ' num2str(cm.c) ' % confidence bands...']));
         abort = '';
         %
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         %
         [cm.gir,cm.gir_std,cm.bir,cm.bir_std,abort] = generalized_impulses_fcn(cm,cm.alpha,cm.beta,cm.Psi,cm.Omega,Sigma_theta,1);
         if strcmp(abort,'cancel')==0;
            %
            % get the value from the normal distribution corresponding to the
            % chosen levels of significance c
            %
            c_val = norm_inv((100+cm.c)/200);
            for i=1:size(cm.gir,2);
               for j=1:cm.n*cm.n;
                  if abs(cm.gir_std(j,i))<1.0e-008;
                     cm.gir_std(j,i) = 0;
                  end;
               end;
            end;
            cm.gir_upper = cm.gir+(c_val*cm.gir_std);
            cm.gir_lower = cm.gir-(c_val*cm.gir_std);
            if isempty(cm.bir)==0;
               for i=1:size(cm.bir,2);
                  for j=1:cm.r*cm.n;
                     if abs(cm.bir_std(j,i))<1.0e-008;
                        cm.bir_std(j,i) = 0;
                     end;
                  end;
               end;
               cm.bir_upper = cm.bir+(c_val*cm.bir_std);
               cm.bir_lower = cm.bir-(c_val*cm.bir_std);
            else;
               cm.bir_upper = [];
               cm.bir_lower = [];
            end;
            thisgraph = gir_graphdlg('init',cm);
            ir_graph = get(thisgraph,'UserData');
            set(ir_graph.show,'UserData','done');
            killguipid(thisgraph);
         end;
         %
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'genalpha'
      if cm.running==0;
         cm.running = 1;
         %
         % Setting up a default G matrix
         %
         cm.G = zeros(1,6);
         %
         % Checking if these already exists a file, and if so
         % check it and then load it
         %
         file = strcat(cm.modelsdir,'G-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            cm.G = loaddata(file);
         end;
         %
         %  Hide current window
         %
         set(betaguidlg,'Visible','off');
         drawnow;
         rd = 'redraw';
         while strcmp(rd,'redraw')==1;
            thisgmat = gmatdlg('init',cm);
            gmat = get(thisgmat,'UserData');
            cancel = get(gmat.cancel,'UserData');
            rd = get(gmat.numres,'UserData');
            cm.G = get(gmat.text,'UserData');
            set(gmat.numelem,'UserData','done');
            killrestpid(thisgmat);
            drawnow;
         end;
         set(betaguidlg,'Visible','on');
         if cancel==0;
            file = strcat(cm.modelsdir,'G-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
               cm.G = loaddata(file);
            end;
            [Rmat,rvec] = rmatrix(cm.G,cm.n,cm.r);
            if rank(Rmat)==size(cm.G,1);
               set(cointbeta.menuestalpha,'Enable','on');
               set(cointbeta.estalpha,'Enable','on');
            end;
         end;
         cm.running = 0;
      end;
   case 'estalpha'
      if cm.running==0;
         cm.running = 1;
         if cm.beta==cm.oldbeta;
            txt = 'You have to identify the cointegration vectors before you are allowed to estimate alpha under restrictions.';
            about(txt,'information',['Estimate Alpha - Information'],120,500);
         else;
            set(betaguidlg,'Visible','off');
            drawnow;
            %
            % check if we need to reset some parameters
            %
            if (strcmp(get(cointbeta.alphaundo,'Enable'),'on')==1);
               cm.latestbeta = cm.beta;
               if curr_ini.reestbeta==1;
                  cm.beta = cm.newoldbeta;
               end;
            end;
            %
            unOmega1 = cm.S_00-(((cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta))*cm.beta'*cm.S_10);
            if cm.dfbeta>0;
               unOmega2 = cm.S_00-(((cm.S_01*cm.oldbeta)/(cm.oldbeta'*cm.S_11*cm.oldbeta))*cm.oldbeta'*cm.S_10);
            end;
            alpha = (cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta);
            if isempty(cm.Z2)==0;
               Psi = (cm.M_02/cm.M_22)-(alpha*cm.beta'*(cm.M_12/cm.M_22));
               Z3 = [cm.Z2;cm.beta'*cm.Z1];
            else;
               Psi = [];
               Z3 = cm.beta'*cm.Z1;
            end;
            Theta = [Psi alpha];
            file = strcat(cm.modelsdir,'G-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
            G = loaddata(file);
            [Rmat,rvec] = rmatrix(G,cm.n,cm.r);
            cm.dfalpha = size(G,1);
            if isempty(cm.Z2)==0;
               Rmat = [zeros(cm.dfalpha,size(cm.Z2,1)*cm.n) Rmat];
            end;
            cm.newbeta = cm.beta;
            if curr_ini.reestbeta==0;
               setlogtext(strvcat(' ','Computing restricted alpha conditional on beta...'));
               [theta,Sigma_theta,infomat,abort] = linrest(cm.Z0,Z3,Theta,Rmat,rvec,1);
            else;
               setlogtext(strvcat(' ','Computing restricted alpha and beta...'));
               [theta,Sigma_theta,cm.newbeta,newstdbeta,Z3,infomat,abort] = alpha_beta_restrict(cm.Z0,cm.Z1,cm.Z2,Z3,cm.newbeta,Theta,Rmat,rvec,cm.dcoint,cm.q1,cm.modelsdir,cm.modelfile,1);
            end;
            if strcmp(abort,'cancel')==0;
               if curr_ini.noprint==0;
                  txt = strvcat(' ','Computing tests...',' ','Please hold!');
                  mywait = waitdlg(txt,'information');
                  waitf = get(mywait,'UserData');
               end;
               test_str = ' of alpha ';
               if cm.dfbeta>0;
                  test_str = 's of alpha and beta ';
               end;
               setlogtext(strvcat(['Number of iterations:               ' num2str(infomat(1,1),'%0.0f')],['Value of the convergence criterion: ' num2str(infomat(1,2))],['Computing test' test_str ' restrictions...']));
               dfc1 = round((((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)-cm.dfbeta)/cm.n);
               dfa = round(cm.dfalpha/cm.n);
               res1 = cm.dfalpha;
               cm.df1 = cm.n*(cm.p-1)+cm.r+cm.q1*cm.p+cm.q0*(cm.p+1)+cm.d+dfc1-dfa;
               epsilon = cm.Z0-(theta*Z3);
               resOmega = (1/cm.T)*epsilon*epsilon';
               LR1 = cm.T*log(det(resOmega)/det(unOmega1));
               LRtests = LR1;
               pvals = [];
               g2 = size(theta,2);
               alphar = theta(:,g2-cm.r+1:g2);
               if g2>cm.r;
                  Psir = theta(:,1:g2-cm.r);
               else;
                  Psir = [];
               end;
               %
               % Compute eigenvalues from companion matrix
               %
               eigenvalues = companionrest(alphar,Psir,cm.newbeta,cm.p,cm.n,cm.d);
               eigenvalues2 = eigenvalues;
               eigenvalues = abs(sort(eigenvalues));
               i = size(eigenvalues,1);
               while i>=1;
                  cm.evalue = eigenvalues(i,1);
                  if strcmp(num2str(cm.evalue,'%1.10f'),'1.0000000000')==1;
                     i = i-1;
                  else;
                     i = 0;
                  end;
               end;
               alpharstd = sqrt(abs(diag( Sigma_theta(cm.n*(g2-cm.r)+1:cm.n*g2,cm.n*(g2-cm.r)+1:cm.n*g2) )));
               alpharstd = vecinv(alpharstd,cm.n);
               if curr_ini.sscorrect==0;
                  pval1 = 1-chis_cdf(LR1,res1);
                  pvals = pval1;
                  status = 1;
               else;
                  LR1 = ((cm.T-cm.df1)/(cm.T*res1))*LR1;
                  LRtests = LR1;
                  pval1 = 1-fdis_cdf(LR1,res1,cm.T-cm.df1);
                  pvals = pval1;
                  status = 1;
               end;
               cm.LRalpha = LR1;
               cm.df2 = [];
               if cm.dfbeta>0;
                  dfc2 = round(((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)/cm.n);
                  cm.df2 = cm.n*(cm.p-1)+cm.r+cm.q1*cm.p+cm.q0*(cm.p+1)+cm.d+dfc2-dfa;
                  res2 = cm.dfalpha+cm.dfbeta;
                  LR2 = cm.T*log(det(resOmega)/det(unOmega2));
                  if curr_ini.sscorrect==0;
                     LRtests = [LRtests LR2];
                     pval2 = 1-chis_cdf(LR2,res2);
                     pvals = [pvals pval2];
                  else;
                     LR2 = ((cm.T-cm.df2)/(cm.T*res2))*LR2;
                     LRtests = [LRtests LR2];
                     pval2 = 1-fdis_cdf(LR2,res2,cm.T-cm.df2);
                     pvals = [pvals pval2];
                  end;
               end;
               [lnl,ldOmega] = loglike(epsilon);
               if curr_ini.noprint==0;
                  sp_str = spacestr(curr_ini.decimals-2);
                  %
                  % write to output file
                  %
                  fout = fopen(cm.outputfile,'at');
                  if curr_ini.texoutputformat>=2;
                     if cm.dfbeta==0;
                        fprintf(fout,'\\subsection*{Restrictions on alpha}\n\n\\header{For exactly identified $\\beta$ with $r$=%0.0f}\n\n',cm.r);
                     else;
                        fprintf(fout,'\\subsection*{Restrictions on alpha}\n\n\\header{For over-identified $\\beta$ with $r$=%0.0f}\n\n',cm.r);
                     end;
                     fprintf(fout,'\\header{Estimated alpha coefficients:}\n\n');
                     myprintstr = '%s';
                     r_str = 'r';
                     for i=1:cm.r;
                        myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                        r_str = [r_str 'r'];
                     end;
                     fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                     for i=1:cm.n;
                        fprintf(fout,[myprintstr ' \\\\\n'],cm.myvars(i,:),alphar(i,:));
                     end;
                     fprintf(fout,'\\end{tabular}\n\n');
                     fprintf(fout,'\\header{with asymptotic standard errors:}\n\n');
                     fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                     for i=1:cm.n;
                        fprintf(fout,[myprintstr ' \\\\\n'],cm.myvars(i,:),alpharstd(i,:));
                     end;
                     fprintf(fout,'\\end{tabular}\n\n');
                     if curr_ini.reestbeta==1;
                        fprintf(fout,'\\header{Re-estimated restricted cointegration relations:}\n\n');
                        myprintstr = '%0.0f';
                        r_str = 'r';
                        j = 2;
                        while j<=size(cm.newbeta,1)+1;
                           myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                           r_str = [r_str 'r'];
                           j = j+1;
                        end;
                        myprintstr = [myprintstr ' \\\\\n'];
                        fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                        fprintf(fout,['relation & ' strrep(strtrim([cm.evars ' ' cm.xvars1 ' ' cm.restvec]),' ',' & ') ' \\\\\n']);
                        fprintf(fout,myprintstr,[(1:size(cm.newbeta,2))' cm.newbeta']');
                        fprintf(fout,['\\multicolumn{' num2str(size(r_str,2),'%0.0f') '}{c}{ } \\\\\n']);
                        fprintf(fout,['\\multicolumn{' num2str(size(r_str,2),'%0.0f') '}{c}{with asymptotic conditional standard errors:} \\\\\n']);
                        fprintf(fout,myprintstr,[(1:size(cm.newbeta,2))' newstdbeta']');
                        fprintf(fout,'\\end{tabular}\n\n');
                     end;
                     fprintf(fout,'\\vspace*{2mm}\n\n\\begin{tabular}{ll}\n');
                     fprintf(fout,'Number of iterations for algorithm: & %0.0f \\\\\n',infomat(1,1));
                     if curr_ini.reestbeta==0;
                        fprintf(fout,['Value of convergence criterion ($\\max |\\partial \\ln L/\\partial \\alpha|$): & ' num2str(infomat(1,2)) ' \\\\\n']);
                     else;
                        fprintf(fout,['Value of convergence criterion ($\\max |\\partial \\ln L/\\partial \\beta|$): & ' num2str(infomat(1,2)) ' \\\\\n']);
                     end;
                     fprintf(fout,'\\end{tabular}\n\n');
                     if cm.dfbeta==0;
                        fprintf(fout,'\\header{LR-type statistic}\n\n');
                        fprintf(fout,'\\begin{tabular}{ll}\n');
                        if curr_ini.sscorrect==0;
                           fprintf(fout,[' & LR(%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],res1,LR1);
                           fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],pval1);
                        else;
                           fprintf(fout,[' & F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],res1,cm.T-cm.df1,LR1);
                           fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],pval1);
                        end;
                        fprintf(fout,'\\end{tabular}\n\n');
                     else;
                        fprintf(fout,'\\header{LR-type statistics}\n\n');
                        fprintf(fout,'\\begin{tabular}{ll}\n');
                        fprintf(fout,'$H_0$: & $\\alpha$ \\& $\\beta$ restrictions are true \\\\\n');
                        fprintf(fout,'$H_A$: & $\\alpha$ restrictions are false \\\\[2mm]\n');
                        if curr_ini.sscorrect==0;
                           fprintf(fout,[' & LR(%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],res1,LR1);
                           fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],pval1);
                        else;
                           fprintf(fout,[' & F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],res1,cm.T-cm.df1,LR1);
                           fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\[3mm]\n'],pval1);
                        end;
                        fprintf(fout,'$H_0$: & $\\alpha$ \\& $\\beta$ restrictions are true \\\\\n');
                        fprintf(fout,'$H_A$: & $\\alpha$ or $\\beta$ restrictions are false \\\\[2mm]\n');
                        if curr_ini.sscorrect==0;
                           fprintf(fout,[' & LR(%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],res2,LR2);
                           fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],pval2);
                        else;
                           fprintf(fout,[' & F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],res2,cm.T-cm.df2,LR2);
                           fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],pval2);
                        end;
                        fprintf(fout,'\\end{tabular}\n\n');
                     end;
                     fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                  else;
                     if cm.dfbeta==0;
                        fprintf(fout,'RESTRICTIONS ON ALPHA (for exactly identified beta with rank=%0.0f)\n\n',cm.r);
                     else;
                        fprintf(fout,'RESTRICTIONS ON ALPHA (for over-identified beta with rank=%0.0f)\n\n',cm.r);
                     end;
                     fprintf(fout,'Estimated alpha coefficients:\n\n');
                     myprintstr = ' %s';
                     for i=1:cm.r;
                        myprintstr = [myprintstr '    %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                     end;
                     for i=1:cm.n;
                        fprintf(fout,[myprintstr '\n'],cm.myvars(i,:),alphar(i,:));
                     end;
                     fprintf(fout,'\nwith asymptotic standard errors:\n\n');
                     for i=1:cm.n;
                        fprintf(fout,[myprintstr '\n'],cm.myvars(i,:),alpharstd(i,:));
                     end;
                     fprintf(fout,'\n');
                     if curr_ini.reestbeta==1;
                        fprintf(fout,'Re-estimated restricted cointegration relations:\n');
                        fprintf(fout,'\n');
                        myprintstr = '%2.0f';
                        j = 2;
                        while j<=size(cm.newbeta,1)+1;
                           myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                           j = j+1;
                        end;
                        myprintstr = [myprintstr '\n'];
                        fprintf(fout,myprintstr,[(1:size(cm.newbeta,2))' cm.newbeta']');
                        fprintf(fout,'\n');
                        fprintf(fout,'with asymptotic conditional standard errors:\n');
                        fprintf(fout,'\n');
                        fprintf(fout,myprintstr,[(1:size(cm.newbeta,2))' newstdbeta']');
                        fprintf(fout,'\n');
                     end;
                     fprintf(fout,'Number of iterations for algorithm:                %0.0f\n',infomat(1,1));
                     if curr_ini.reestbeta==0;
                        fprintf(fout,['Value of convergence criterion (max|dlnL/dalpha|): ' num2str(infomat(1,2)) '\n\n']);
                     else;
                        fprintf(fout,['Value of convergence criterion (max|dlnL/dbeta|):  ' num2str(infomat(1,2)) '\n\n']);
                     end;
                     if cm.dfbeta==0;
                        fprintf(fout,'  LR-type statistic\n  ~~~~~~~~~~~~~~~~~\n\n');
                        if curr_ini.sscorrect==0;
                           fprintf(fout,['LR(%1.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],res1,LR1);
                           fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],pval1);
                        else;
                           fprintf(fout,['F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],res1,cm.T-cm.df1,LR1);
                           fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],pval1);
                        end;
                     else;
                        fprintf(fout,'  LR-type statistics\n  ~~~~~~~~~~~~~~~~~~\n\n');
                        fprintf(fout,'Null hypothesis:        alpha+beta restrictions are true\n');
                        fprintf(fout,'Alternative hypothesis: alpha restrictions are false\n\n');
                        if curr_ini.sscorrect==0;
                           fprintf(fout,['LR(%1.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],res1,LR1);
                           fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],pval1);
                        else;
                           fprintf(fout,['F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],res1,cm.T-cm.df1,LR1);
                           fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],pval1);
                        end;
                        fprintf(fout,'\nNull hypothesis:        alpha+beta restrictions are true\n');
                        fprintf(fout,'Alternative hypothesis: alpha or beta restrictions are false\n\n');
                        if curr_ini.sscorrect==0;
                           fprintf(fout,['LR(%1.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],res2,LR2);
                           fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],pval2);
                        else;
                           fprintf(fout,['F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],res2,cm.T-cm.df2,LR2);
                           fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],pval2);
                        end;
                     end;
                     fprintf(fout,'\n===============================================================================\n\n');
                  end;
                  fclose(fout);
                  %
                  if (curr_ini.shortdyn==1)&(curr_ini.noprint==0);
                     txt = strvcat(' ','Computing short run dynamics.','Please hold!');
                     set(waitf.text,'String',txt);
                     shortstr = shortrun(alphar,Psir,resOmega,cm.newbeta,cm.Z1,cm.Z2,cm.p,cm.pe,cm.d,cm.deterselect,cm.q0,cm.q1,cm.evars,cm.xvars0,cm.xvars1,cm.deter,cm.restvec,1,cm.outputfile,0,Sigma_theta);
                     fout = fopen(cm.outputfile,'at');
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                     else;
                        fprintf(fout,'\n===============================================================================\n\n');
                     end;
                     if cm.evalue<1;
                        if cm.r<cm.n;
                           C = cmat(Psir,alphar,cm.newbeta,epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,Sigma_theta);
                        else;
                           C = cmat_stat(Psir,alphar,cm.newbeta,epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,Sigma_theta);
                        end;
                        myC = showc(C(:,1:cm.n),cm.myvars);
                        myCstd = showc(C(:,cm.n+1:2*cm.n),cm.myvars);
                        myCt = showc(inf2nan(C(:,1:cm.n)./C(:,cm.n+1:2*cm.n)),cm.myvars);
                        if curr_ini.texoutputformat>=2;
                           myC = text_latextable(myC);
                           myCstd = text_latextable(myCstd);
                           myCstd = myCstd(2:size(myCstd,1),:);
                           myCt = text_latextable(myCt);
                           myCt = myCt(2:size(myCt,1),:);
                           myC = merge_latextable(myC,myCstd,myCt);
                           r_str = 'r';
                           for i=1:cm.n;
                              r_str = [r_str 'r'];
                           end;
                           fprintf(fout,'\\subsection*{The $C$ matrix}\n\n\\header{For $r$=%0.0f}\n\n',cm.r);
                           fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                           for i=1:(size(myC,1));
                              fprintf(fout,'%s\n',myC(i,:));
                           end;
                           fprintf(fout,'\\end{tabular}\n\n');
                           fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                        else;
                           fprintf(fout,['THE C-MATRIX\n\n(for rank=' num2str(cm.r) ',lags=' num2str(cm.p) ',#free beta params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.dfbeta,'%0.0f') ',#free alpha params=' num2str(cm.n*cm.r-cm.dfalpha,'%0.0f') ')\n\n']);
                           for i=1:(size(myC,1));
                              fprintf(fout,'%s\n',myC(i,:));
                           end;
                           fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
                           for i=1:(size(myC,1));
                              fprintf(fout,'%s\n',myCstd(i,:));
                           end;
                           fprintf(fout,'\nt-ratios:\n\n');
                           for i=1:(size(myC,1));
                              fprintf(fout,'%s\n',myCt(i,:));
                           end;
                           fprintf(fout,'\n===============================================================================\n\n');
                        end;
                     end;
                     fclose(fout);
                  end;
                  %
                  fout = fopen(cm.outputfile,'at');
                  if curr_ini.texoutputformat>=2;
                     fprintf(fout,'\\subsection*{Value of the maximized log-likelihood function:}\n\n');
                     fprintf(fout,'\\begin{tabular}{ll}\n');
                     fprintf(fout,[' & $\\ln L(\\mbox{\\rm lags}=%0.0f,r=%0.0f,\\mbox{\\rm \\#free } \\alpha \\mbox{\\rm \\hspace*{1mm} params}=%1.0f,\\mbox{\\rm \\#free } \\beta \\mbox{\\rm \\hspace*{1mm} params}=%1.0f)$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],cm.p,cm.r,cm.n*cm.r-cm.dfalpha,(cm.n+cm.q1+cm.dcoint-cm.r)*cm.r-cm.dfbeta,lnl);
                     fprintf(fout,[' & $\\ln(\\det(\\Omega))$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],ldOmega);
                     fprintf(fout,'\\end{tabular}\n\n');
                     fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                  else;
                     fprintf(fout,'Value of the maximized log-likelihood function:\n\n');
                     fprintf(fout,['   lnL(lags=%1.0f,rank=%1.0f,#free alpha params=%1.0f,#free beta params=%1.0f)=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],cm.p,cm.r,cm.n*cm.r-cm.dfalpha,(cm.n+cm.q1+cm.dcoint-cm.r)*cm.r-cm.dfbeta,lnl);
                     fprintf(fout,['   ln(det(Omega))=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],ldOmega);
                     fprintf(fout,'===============================================================================\n\n');
                  end;
                  fclose(fout);
                  %
                  % compute tests for additional deterministic variables
                  %
                  if (curr_ini.dummytests==1)&(cm.datad>0);
                     oldrestrict = cm.restrict;
                     oldepsilon = cm.epsilon;
                     cm.restrict(1,1) = 1;
                     cm.epsilon = epsilon;
                     txt = strvcat(' ','Computing exclusion tests for additional deterministics.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_exclude_dummies(cm,curr_ini);
                     if cm.r<cm.n;
                        txt = strvcat(' ','Computing long-run tests for additional deterministics.','Please hold!');
                        set(waitf.text,'String',txt);
                        print_longrun_dummies(cm,curr_ini);
                     end;
                     cm.restrict = oldrestrict;
                     cm.epsilon = oldepsilon;
                  end;
                  %
                  % Calculate lag-order test
                  %
                  if curr_ini.lagtest>0;
                     setlogtext('Computing lag order tests...');
                     txt = strvcat(' ','Computing lag order tests.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_lagordertest(cm,curr_ini,cm.newbeta,alphar,Psir,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df1,1);
                  end;
                  if curr_ini.gctest==1;
                     setlogtext('Computing Granger causality tests...');
                     txt = strvcat(' ','Computing Granger causality tests.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_gctest(cm,curr_ini,cm.newbeta,alphar,Psir,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df1,1);
                     if cm.n>2;
                        setlogtext('Computing multi-step Granger causality tests...');
                        txt = strvcat(' ','Computing multi-step Granger causality tests.','Please hold!');
                        set(waitf.text,'String',txt);
                        print_mgctest(cm,curr_ini,cm.newbeta,alphar,Psir,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,1,0);
                     end;
                  end;
                  if curr_ini.excludediffs==1;
                     if ((cm.p>1)&(cm.n>2))|(cm.q0>0)|((cm.q1>0)&(cm.pe>0));
                        setlogtext('Computing exclusion tests for first differences...');
                        txt = strvcat(' ','Computing exclusion tests for first differences.','Please hold!');
                        set(waitf.text,'String',txt);
                        print_firstdiffstests(cm,curr_ini,cm.newbeta,alphar,Psir,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.df1,0);
                     end;
                  end;
                  if cm.r<cm.n;
                     if curr_ini.weakexo==1;
                        setlogtext('Computing weak exogeneity tests...');
                        txt = strvcat(' ','Computing weak exogeneity tests.','Please hold!');
                        set(waitf.text,'String',txt);
                        print_weakexo(cm,curr_ini,cm.newbeta,alphar,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df1,1);
                     end;
                  end;
                  %
                  if (cm.r<cm.n)&(cm.p>1);
                     if curr_ini.commoncycles==1;
                        oldrestrict = cm.restrict;
                        oldepsilon = cm.epsilon;
                        cm.restrict(1,1) = 1;
                        cm.epsilon = epsilon;
                        %
                        setlogtext('Computing common cycle tests...');
                        txt = strvcat(' ','Computing common cycle tests.','Please hold!');
                        set(waitf.text,'String',txt);
                        print_commoncycles(cm,curr_ini,alphar,Psir,cm.newbeta);
                        %
                        cm.restrict = oldrestrict;
                        cm.epsilon = oldepsilon;
                     end;
                  end;
                  if curr_ini.restat==1;
                     txt = strvcat(' ','Computing residual statistics.','Please hold!');
                     set(waitf.text,'String',txt);
                  end;
                  %
                  % residual statistics
                  %
                  print_residual_analysis(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df1);
                  %
                  % specification tests
                  %
                  if sum(cm.spectest)>0;
                     txt = strvcat(' ','Computing specification tests.','Please hold!');
                     set(waitf.text,'String',txt);
                  end;
                  %
                  if cm.spectest(1,1)==1;
                     setlogtext('Computing serial correlation tests...');
                     print_corrtest(cm,curr_ini,epsilon,cm.newbeta,cm.dfbeta,cm.dfalpha,cm.df1);
                  end;
                  %
                  if cm.spectest(1,2)==1;
                     setlogtext('Computing ARCH tests...');
                     print_archtest(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df1);
                  end;
                  %
                  if cm.spectest(1,3)==1;
                     setlogtext('Computing normality tests...');
                     print_normalitytest(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df1);
                  end;
                  %
                  if cm.spectest(1,4)==1;
                     if rank(cm.newbeta'*cm.newbeta,1.0e-004)==cm.r;
                        if cm.r>0;
                           Zb = cm.newbeta'*cm.Z1;
                        else;
                           Zb = 0;
                        end;
                        fout = fopen(cm.outputfile,'at');
                        if curr_ini.texoutputformat>=2;
                           fprintf(fout,['\\subsection*{Parameter constancy tests}\n\n\\header{For $r$=' num2str(cm.r) ', lags=' num2str(cm.p) ', \\#free $\\beta$ params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.dfbeta,'%0.0f') ', \\#free $\\alpha$ params=' num2str(cm.n*cm.r-cm.dfalpha,'%0.0f') '}\n\n']);
                        else;
                           fprintf(fout,['PARAMETER CONSTANCY TESTS\n\n(for rank=' num2str(cm.r) ',lags=' num2str(cm.p) ',#free beta params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.dfbeta,'%0.0f') ',#free alpha params=' num2str(cm.n*cm.r-cm.dfalpha,'%0.0f') ')\n\n']);
                        end;
                        fclose(fout);
                        if curr_ini.lstrtest==1;
                           setlogtext('Computing LSTR1 constancy tests...');
                           print_lstr1test(cm,curr_ini,epsilon,Zb,cm.df1);
                        end;
                        %
%                        if (cm.q1+cm.q0==0)&(curr_ini.nyblom==1)&(curr_ini.nyblombeta==1);
                        if (curr_ini.nyblom==1)&(curr_ini.nyblombeta==1);
                           setlogtext('Computing Nyblom constancy tests for beta...');
                           old_restrict = cm.restrict;
                           cm.restrict = [1 1];
                           print_alpha_nyblom(cm,curr_ini);
                           cm.restrict = old_restrict;
                        end;
                        %
                        if cm.p+cm.r+cm.d>1;
                           if curr_ini.chow==1;
                              setlogtext('Computing Chow tests...');
                              print_chow(cm,curr_ini,cm.newbeta,1);
                           end;
                           %
                           setlogtext('Computing fluctuation tests...');
                           print_ploberger(cm,curr_ini,cm.newbeta,1);
                        end;
                     else;
                        fout = fopen(cm.outputfile,'at');
                        myrank = rank(cm.newbeta'*cm.newbeta,1.0e-004);
                        setlogtext(strvcat(' ','WARNING: The estimated beta matrix is badly identified...',['         Numerical rank is ' num2str(myrank(1,1)) '...']));
                        txt = ['The estimated restricted beta matrix is badly identified. The rank of beta''beta is ' num2str(myrank(1,1)) ', i.e. less than ' num2str(cm.r) ', when a tolerance level of 1.0e-004 is used. Please consider some other restrictions.'];
                        about(txt,'warning','Badly identified beta matrix',120,500);
                        if curr_ini.texoutputformat>=2;
                           fprintf(fout,'\\vspace*{3mm}\n\\newline\n\\noindent\n\\textbf{WARNING:} Parameter constancy tests have not been calculated. The\n');
                           fprintf(fout,'         estimated restricted $\\beta$ matrix is badly identified.\n');
                           fprintf(fout,'         The rank of $\\beta''\\beta$ is %0.0f, i.e. less than %0.0f, when\n',myrank,cm.r);
                           fprintf(fout,'         a tolerance level of 1.0e-004 is used.\n');
                           fprintf(fout,'\n\\ruler\n\n%%==============================================================================\n\n');
                        else;
                           fprintf(fout,'WARNING: Parameter constancy tests have not been calculated. The\n');
                           fprintf(fout,'         estimated restricted beta matrix is badly identified.\n');
                           fprintf(fout,'         The rank of beta''beta is %1.0f, i.e. less than %1.0f, when\n',myrank,cm.r);
                           fprintf(fout,'         a tolerance level of 1.0e-004 is used.\n');
                           fprintf(fout,'\n===============================================================================\n\n');
                        end;
                        fclose(fout);
                     end;
                  end;
                  %
               end;
               %
               % close wait dialogue
               %
               if curr_ini.noprint==0;
                  set(waitf.text,'UserData','done');
                  delete(mywait);
                  drawnow;
               end;
               %
               % display results
               %
               thisalphadlg = alphadlg('init',cm,infomat,pvals,LRtests,epsilon,Psir,alphar,Sigma_theta);
               alpharest = get(thisalphadlg,'UserData');
               userestalpha = get(alpharest.betamat,'UserData');
               set(alpharest.alphamat,'UserData','done');
               delete(thisalphadlg);
               drawnow;
               if userestalpha==1;
                  setlogtext(strvcat(' ','Using the restricted alpha and beta estimates...'));
                  %
                  % If deciding to use new alpha restrictions
                  %
                  cm.restrict(1,1) = 1;
                  set(cointbeta.menualphaundo,'Enable','on');
                  set(cointbeta.alphaundo,'Enable','on');
                  set(cointbeta.estalpha,'UserData',theta);
                  set(cointbeta.genalpha,'UserData',Sigma_theta);
                  set(cointbeta.markov_switching,'Enable','off');
                  cm.epsilon = epsilon;
                  cm.alpha = alphar;
                  cm.Psi = Psir;
                  cm.Omega = resOmega;
                  cm.df = cm.df1;
                  %
                  cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,lnl,cm.evalue);
                  set(cointbeta.model_info,'String',cm.model_info.info);
                  %
                  if curr_ini.reestbeta==1;
                     cm.newoldbeta = cm.beta;
                     cm.newoldstdbeta = get(cointbeta.model_info,'UserData');
                     cm.beta = cm.newbeta;
                     cm.bZ1t = cm.beta'*cm.Z1t;
                     cm.bR1t = cm.beta'*cm.R1t;
                     cm.stdbeta = newstdbeta;
                     set(cointbeta.betamat,'UserData',cm.newbeta);
                     set(cointbeta.model_info,'UserData',newstdbeta);
                     currbeta = showbeta(cm.beta,cm.evars,strtrim([cm.xvars1 ' ' cm.restvec]));
                     set(cointbeta.betamat,'String',currbeta);
                  end;
                  %
                  % Need to compute the eigenvalues of the companion
                  % matrix to ensure that permanent and transitory
                  % components should be computed, i.e. check the largest
                  % eigenvalue
                  %
                  if cm.evalue>=1;
                     set(cointbeta.cmat,'Enable','off');
                     set(cointbeta.longrun_gir,'Enable','off');
                     set(cointbeta.menupermcomp,'Enable','off');
                     set(cointbeta.permcomp,'Enable','off');
                     set(cointbeta.savetc,'Enable','off');
                  else;
                     set(cointbeta.cmat,'Enable','on');
                     set(cointbeta.longrun_gir,'Enable','on');
                     set(cointbeta.menupermcomp,'Enable','on');
                     set(cointbeta.permcomp,'Enable','on');
                     set(cointbeta.savetc,'Enable','on');
                  end;
                  if curr_ini.noprint==0;
                     fout = fopen(cm.outputfile,'at');
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,'\\header{You selected to use these $\\alpha$ parameters.}\n\n');
                     else;
                        fprintf(fout,'You selected to use these alpha parameters.\n\n');
                     end;
                     [eigenvalues1,AscInd] = sort(abs(eigenvalues2));
                     nr = size(AscInd,1);
                     DescInd = zeros(nr,1);
                     i = 1;
                     while i<=nr;
                        DescInd(i)=AscInd(nr+1-i);
                        i =i+1;
                     end;
                     eigenvalues2 = eigenvalues2(DescInd);
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,'\\subsection*{Properties of the stacked VAR(1) companion matrix:}\n\n');
                        fprintf(fout,'\\begin{tabular}{rrrr}\n');
                        fprintf(fout,'\\multicolumn{4}{c}{Eigenvalues} \\\\\n');
                        fprintf(fout,'number & real & imaginary & modulus \\\\\n');
                        fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(1:size(eigenvalues2,1))' real(eigenvalues2) imag(eigenvalues2) abs(eigenvalues2)]');
                        fprintf(fout,'\\end{tabular}\n\n\\ruler\n\n%%==============================================================================\n\n');
                     else;
                        fprintf(fout,'Properties of the stacked VAR(1) companion matrix:\n');
                        fprintf(fout,'Eigenvalues\n');
                        fprintf(fout,['number    real   ' sp_str '   imaginary   ' sp_str '  modulus\n\n']);
                        fprintf(fout,['%3.0f      %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f     %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f         %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(1:size(eigenvalues2,1))' real(eigenvalues2) imag(eigenvalues2) abs(eigenvalues2)]');
                        fprintf(fout,'\n===============================================================================\n\n');
                     end;
                     fclose(fout);
                  end;
               else;
                  if strcmp(get(cointbeta.alphaundo,'Enable'),'on')==1;
                     cm.beta = cm.latestbeta;
                  end;
                  setlogtext(strvcat(' ','Skipping the restricted alpha and beta estimates...'));
                  eigenvalues = companionrest(cm.alpha,cm.Psi,cm.beta,cm.p,cm.n,cm.d);
                  eigenvalues2 = eigenvalues;
                  eigenvalues = abs(sort(eigenvalues));
                  i = size(eigenvalues,1);
                  while i>=1;
                     cm.evalue = eigenvalues(i,1);
                     if strcmp(num2str(cm.evalue,'%1.10f'),'1.0000000000')==1;
                        i = i-1;
                     else;
                        i = 0;
                     end;
                  end;
               end;
            else;
               if strcmp(get(cointbeta.alphaundo,'Enable'),'on')==1;
                  cm.beta = cm.latestbeta;
               end;
               setlogtext('Cancelled estimation...');
            end;
            set(betaguidlg,'Visible','on');
         end;
         cm.running = 0;
      end;
   case 'alphaundo'
      if cm.running==0;
         cm.running = 1;
         setlogtext(strvcat(' ','Skipping the restricted alpha and beta estimates...'));
         if curr_ini.reestbeta==1;
            cm.beta = cm.newoldbeta;
            cm.stdbeta = cm.newoldstdbeta;
            cm.bZ1t = cm.beta'*cm.Z1t;
            cm.bR1t = cm.beta'*cm.R1t;
            set(cointbeta.betamat,'UserData',cm.beta);
            set(cointbeta.model_info,'UserData',cm.stdbeta);
            currbeta = showbeta(cm.beta,cm.evars,strtrim([cm.xvars1 ' ' cm.restvec]));
            set(cointbeta.betamat,'String',currbeta);
         end;
         if curr_ini.noprint==0;
            fout = fopen(cm.outputfile,'at');
            if curr_ini.texoutputformat>=2;
               fprintf(fout,'\\header{But then you changed your mind about $\\alpha$!}\n\n');
               fprintf(fout,'\\ruler\n\n%%******************************************************************************\n\n');
            else;
               fprintf(fout,'But then you changed your mind about alpha!\n\n');
               fprintf(fout,'*******************************************************************************\n\n');
            end;
            fclose(fout);
         end;
         alpha = (cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta);
         if isempty(cm.Z2)==0;
            Psi = (cm.M_02/cm.M_22)-(alpha*cm.beta'*(cm.M_12/cm.M_22));
            Z3 = [cm.Z2;cm.beta'*cm.Z1];
         else;
            Psi = [];
            Z3 = cm.beta'*cm.Z1;
         end;
         %
         % Compute eigenvalues from companion matrix
         %
         eigenvalues = companionrest(alpha,Psi,cm.beta,cm.p,cm.n,cm.d);
         eigenvalues2 = eigenvalues;
         eigenvalues = abs(sort(eigenvalues));
         i = size(eigenvalues,1);
         while i>=1;
            cm.evalue = eigenvalues(i,1);
            if strcmp(num2str(cm.evalue,'%1.10f'),'1.0000000000')==1;
               i = i-1;
            else;
               i = 0;
            end;
         end;
         Theta = [Psi alpha];
         cm.epsilon = cm.Z0-Theta*Z3;
         Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
         inv_Z3 = eye(size(Z3,1))/(Z3*Z3');
         Sigma_theta = kron(inv_Z3,Omega);
         dfc = round(((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r-cm.dfbeta)/cm.n);
         cm.df = cm.n*(cm.p-1)+cm.r+cm.q1*cm.pe+cm.q0*(cm.pe+1)+cm.d+dfc;
         cm.alpha = alpha;
         cm.Psi = Psi;
         cm.Omega = Omega;
         lnl = loglike(cm.epsilon);
         %
         cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,lnl,cm.evalue);
         set(cointbeta.model_info,'String',cm.model_info.info);
         %
         cm.restrict(1,1) = 0;
         set(cointbeta.menualphaundo,'Enable','off');
         set(cointbeta.alphaundo,'Enable','off');
         set(cointbeta.estalpha,'UserData',Theta);
         set(cointbeta.genalpha,'UserData',Sigma_theta);
         set(cointbeta.markov_switching,'Enable','on');
         cm.dfalpha = 0;
         cm.LRalpha = 0;
         cm.running = 0;
      end;
   case 'savealpharest'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         thissavealpharest = savealpharestgui('init',cm);
         savealpharest = get(thissavealpharest,'UserData');
         set(savealpharest.gmat,'UserData','done');
         killguipid(thissavealpharest);
         %
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'openalpharest'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         thisopenalpharest = openalpharestgui('init',cm);
         openalpharest = get(thisopenalpharest,'UserData');
         set(openalpharest.gmat,'UserData','done');
         killguipid(thisopenalpharest);
         %
         % check if restricts file exists
         %
         myalphaenable = 'off';
         file = strcat(cm.modelsdir,'G-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            myalphaenable = 'on';
         end;
         set(cointbeta.menuestalpha,'Enable',myalphaenable);
         set(cointbeta.estalpha,'Enable',myalphaenable);
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'restrict'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         cm.index = 1;
         normalize = ones(cm.r,1);
         file = strcat(cm.modelsdir,'normalize-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.norm');
         if fileexist(file)==1;
            normalize = loaddata(file);
         end;
         if size(normalize,2)>size(normalize,1);
            normalize = normalize';
         end;
         while cm.index<=cm.r;
            cm.Hmat = zeros(cm.n+cm.q1+cm.dcoint-cm.r+1,cm.n+cm.q1+cm.dcoint);
            file = strcat(cm.modelsdir,'H_',int2str(cm.index),'-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
              cm.Hmat = loaddata(file);
              cm.Hmat = cm.Hmat';
            end;
            cm.normvar = normalize(cm.index,1);
            rd = 'redraw';
            while strcmp(rd,'redraw')==1;
               thishmatfig = hmatgui('init',cm);
               hmats = get(thishmatfig,'UserData');
               cm.Hmat = get(hmats.text,'UserData');
               rd = get(hmats.rest,'UserData');
               newindex = get(hmats.cv,'Value');
               cancel = get(hmats.cancel,'UserData');
               cm.normvar = get(hmats.popup,'Value');
               set(hmats.popup,'UserData','done');
               killrestpid(thishmatfig);
               drawnow;
            end;
            if cancel<=1;
               normalize(cm.index,1) = cm.normvar;
               if newindex==cm.index;
                  cm.index = cm.index+1;
               else;
                  cm.index = newindex;
               end;
            elseif cancel==2;
               cm.index = cm.r+1;
            end;
         end;
         if size(normalize,1)==cm.r;
            savefile = strcat(cm.modelsdir,'normalize-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.norm');
            fid = fopen(savefile,'wt');
            fprintf(fid,'%1.0f\n',normalize);
            fclose(fid);
         end;
         %
         % Check if all the necessary files exist to estimate
         % beta under restrictions
         %
         i = 1;
         while i<=cm.r;
            file = strcat(cm.modelsdir,'H_',num2str(i),'-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
               i = i+1;
            else;
               break;
            end;
         end;
         %
         % if necessary test if normalize-modelfile.norm exists in modelsdir
         % and if so set cointbeta.estimate control to
         % being enabled else disabled
         %
         if i==cm.r+1;
            file = strcat(cm.modelsdir,'normalize-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.norm');
            if fileexist(file)==1;
               checkH_1 = loaddata(file);
               if size(checkH_1,1)==size(cm.beta,2);
                  set(cointbeta.estimate,'Enable','on');
                  set(cointbeta.menuestimate,'Enable','on');
                  set(cointbeta.bootbeta,'Enable','on');
                  if (curr_ini.nyblom==0)|(curr_ini.nyblombeta==0);
                     set(cointbeta.bootnyblommenu,'Enable','off');
                  else;
                     set(cointbeta.bootnyblommenu,'Enable','on');
                  end;
                  set(cointbeta.recursive,'Enable','on');
                  set(cointbeta.menurecursive,'Enable','on');
               else;
                  set(cointbeta.estimate,'Enable','off');
                  set(cointbeta.menuestimate,'Enable','off');
                  set(cointbeta.bootbeta,'Enable','off');
                  set(cointbeta.bootnyblommenu,'Enable','off');
                  set(cointbeta.recursive,'Enable','off');
                  set(cointbeta.menurecursive,'Enable','off');
               end;
            else;
               set(cointbeta.estimate,'Enable','off');
               set(cointbeta.menuestimate,'Enable','off');
               set(cointbeta.bootbeta,'Enable','off');
               set(cointbeta.bootnyblommenu,'Enable','off');
               set(cointbeta.recursive,'Enable','off');
               set(cointbeta.menurecursive,'Enable','off');
            end;
         end;
         set(cointbeta.clear,'UserData',thisexit);
         set(cointbeta.restrict,'UserData',cm.restrict);
         set(betaguidlg,'Visible','on');
         set(betaguidlg,'UserData',cointbeta);
         cm.running = 0;
      end;
   case 'estimate'
      if cm.running==0;
         cm.running = 1;
         status = testident(cm.modelsdir,cm.modelfile,cm.r,cm.n,cm.dcoint,cm.q1);
         if sum(status)==cm.r;
            unOmega = cm.S_00-(((cm.S_01*cm.oldbeta)/(cm.oldbeta'*cm.S_11*cm.oldbeta))*cm.oldbeta'*cm.S_10);
            %
            %  Hide current window
            %
            set(betaguidlg,'Visible','off');
            drawnow;
            setlogtext(strvcat(' ','Computing restricted beta estimates...'));
            [cm.betarest,newlambda,cm.newdfbeta,cm.stdbeta,cm.outputtxt,abort,infomat,conv_txt] = betarestrict(cm.S_00,cm.S_11,cm.S_01,cm.S_10,cm.oldbeta,cm.T,cm.modelsdir,cm.modelfile,1,cm.dcoint);
            if strcmp(abort,'cancel')==0;
               if (curr_ini.noprint==0)&((cm.newdfbeta>0)|((cm.r<cm.n)&(cm.p>1)&(curr_ini.commoncycles==1)));
                  txt = strvcat(' ','Computing tests...',' ','Please hold!');
                  mywait = waitdlg(txt,'information');
                  waitf = get(mywait,'UserData');
               end;
               %
               set(betaguidlg,'UserData',cointbeta);
               dfc = round((((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)-cm.newdfbeta)/cm.n);
               cm.df = cm.n*(cm.p-1)+cm.r+cm.q1*cm.pe+cm.q0*(cm.pe+1)+cm.d+dfc;
               %
               % Compute the LR test of the restrictions if there are any
               % overidentifying such
               %
               if cm.newdfbeta>0;
                  setlogtext(strvcat(['Number of iterations:               ' num2str(infomat(1,1),'%0.0f')],['Value of the convergence criterion: ' num2str(infomat(1,2))],'Computing test of beta restrictions...'));
                  rOmega = cm.S_00-(((cm.S_01*cm.betarest)/(cm.betarest'*cm.S_11*cm.betarest))*cm.betarest'*cm.S_10);
                  cm.LRbeta = cm.T*log(det(rOmega)/det(unOmega));
                  if curr_ini.sscorrect==0;
                     cm.pval = chis_cdf(cm.LRbeta,cm.newdfbeta);
                  else;
                     cm.LRbeta = ((cm.T-cm.df)/(cm.T*cm.newdfbeta))*cm.LRbeta;
                     cm.pval = fdis_cdf(cm.LRbeta,cm.newdfbeta,cm.T-cm.df);
                  end;
               else;
                  cm.LRbeta = [];
                  cm.pval = [];
               end;
               %
               % estimate residuals
               %
               setlogtext('Computing short-run dynamics...');
               alpha = (cm.S_01*cm.betarest)/(cm.betarest'*cm.S_11*cm.betarest);
               if isempty(cm.Z2)==0;
                  Psi = (cm.M_02/cm.M_22)-(alpha*cm.betarest'*(cm.M_12/cm.M_22));
                  cm.epsilon = cm.Z0-(alpha*cm.betarest'*cm.Z1)-(Psi*cm.Z2);
               else;
                  Psi = [];
                  cm.epsilon = cm.Z0-(alpha*cm.betarest'*cm.Z1);
               end;
               %
               % Compute eigenvalues from companion matrix
               %
               eigenvalues = companionrest(alpha,Psi,cm.betarest,cm.p,cm.n,cm.d);
               eigenvalues2 = eigenvalues;
               eigenvalues = abs(sort(eigenvalues));
               i = size(eigenvalues,1);
               while i>=1;
                  cm.evalue = eigenvalues(i,1);
                  if strcmp(num2str(cm.evalue,'%1.10f'),'1.0000000000')==1;
                     i = i-1;
                  else;
                     i = 0;
                  end;
               end;
               [lnl,ldOmega] = loglike(cm.epsilon);
               numbp = ((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)-cm.newdfbeta;
               if curr_ini.noprint==0;
                  sp_str = spacestr(curr_ini.decimals-2);
                  %
                  % write to output file
                  %
                  fout = fopen(cm.outputfile,'at');
                  if curr_ini.texoutputformat>=2;
                     fprintf(fout,'\\subsection*{Estimated restricted cointegration relations:}\n\n');
                     myprintstr = '%0.0f';
                     r_str = 'r';
                     j = 2;
                     while j<=size(cm.betarest,1)+1;
                        myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                        r_str = [r_str 'r'];
                        j = j+1;
                     end;
                     myprintstr = [myprintstr ' \\\\\n'];
                     fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                     fprintf(fout,['relation & ' strrep(strtrim([cm.evars ' ' cm.xvars1 ' ' cm.restvec]),' ',' & ') ' \\\\\n']);
                     fprintf(fout,myprintstr,[(1:size(cm.betarest,2))' cm.betarest']');
                     fprintf(fout,['\\multicolumn{' num2str(size(r_str,2),'%0.0f') '}{c}{ } \\\\\n']);
                     fprintf(fout,['\\multicolumn{' num2str(size(r_str,2),'%0.0f') '}{c}{with asymptotic conditional standard errors:} \\\\\n']);
                     fprintf(fout,myprintstr,[(1:size(cm.betarest,2))' cm.stdbeta']');
                     fprintf(fout,'\\end{tabular}\n\n');
                     %
                     % add iteration output
                     %
                     if isempty(conv_txt)==0;
                        fprintf(fout,'\\vspace*{2mm}\n\n');
                        for j=1:size(conv_txt,1);
                           fprintf(fout,'%s\n',strtrim(conv_txt(j,:)));
                        end;
                        fprintf(fout,'\n');
                     end;
                     if cm.newdfbeta>0;
                        fprintf(fout,'\\header{LR-type test statistic}\n\n');
                        fprintf(fout,'\\begin{tabular}{ll}\n');
                        if curr_ini.sscorrect==0;
                           fprintf(fout,[' & LR(%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],cm.newdfbeta,cm.LRbeta);
                        else;
                           fprintf(fout,[' & F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],cm.newdfbeta,cm.T-cm.df,cm.LRbeta);
                        end;
                        if isempty(cm.pval)==0;
                           fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],1-cm.pval);
                        end;
                        fprintf(fout,'\\end{tabular}\n\n');
                     end;
                     fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                  else;
                     fprintf(fout,'Estimated restricted cointegration relations:\n\n');
                     myprintstr = '%2.0f';
                     j = 2;
                     while j<=size(cm.betarest,1)+1;
                        myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                        j = j+1;
                     end;
                     myprintstr = [myprintstr '\n'];
                     fprintf(fout,myprintstr,[(1:size(cm.betarest,2))' cm.betarest']');
                     fprintf(fout,'\n');
                     fprintf(fout,'with asymptotic conditional standard errors:\n');
                     fprintf(fout,'\n');
                     fprintf(fout,myprintstr,[(1:size(cm.betarest,2))' cm.stdbeta']');
                     fprintf(fout,'\n');
                     %
                     % add iteration output
                     %
                     if isempty(conv_txt)==0;
                        for j=1:size(conv_txt,1);
                           fprintf(fout,'%s\n',strtrim(conv_txt(j,:)));
                        end;
                        fprintf(fout,'\n');
                     end;
                     if cm.newdfbeta>0;
                        fprintf(fout,'  LR-type test statistic\n  ~~~~~~~~~~~~~~~~~~~~~~\n\n');
                        if curr_ini.sscorrect==0;
                           fprintf(fout,['LR(%1.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],cm.newdfbeta,cm.LRbeta);
                        else;
                           fprintf(fout,['F(%1.0f,%1.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],cm.newdfbeta,cm.T-cm.df,cm.LRbeta);
                        end;
                        if isempty(cm.pval)==0;
                           fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],1-cm.pval);
                        end;
                        fprintf(fout,'\n');
                     end;
                     fprintf(fout,'===============================================================================\n\n');
                  end;
                  fclose(fout);
                  %
                  if (curr_ini.shortdyn==1)&(curr_ini.noprint==0);
                     if cm.newdfbeta>0;
                        txt = strvcat(' ','Computing short run dynamics.','Please hold!');
                        set(waitf.text,'String',txt);
                     end;
                     Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
                     shortstr = shortrun(alpha,Psi,Omega,cm.betarest,cm.Z1,cm.Z2,cm.p,cm.pe,cm.d,cm.deterselect,cm.q0,cm.q1,cm.evars,cm.xvars0,cm.xvars1,cm.deter,cm.restvec,1,cm.outputfile,0);
                     fout = fopen(cm.outputfile,'at');
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                     else;
                        fprintf(fout,'\n===============================================================================\n\n');
                     end;
                     if (cm.evalue<1)&(cm.newdfbeta>0);
                        res_str = 'for restricted';
                        if cm.r<cm.n;
                           C = cmat(Psi,alpha,cm.betarest,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1);
                        else;
                           C = cmat_stat(Psi,alpha,cm.betarest,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1);
                        end;
                        myC = showc(C(:,1:cm.n),cm.myvars);
                        myCstd = showc(C(:,cm.n+1:2*cm.n),cm.myvars);
                        myCt = showc(inf2nan(C(:,1:cm.n)./C(:,cm.n+1:2*cm.n)),cm.myvars);
                        if curr_ini.texoutputformat>=2;
                           myC = text_latextable(myC);
                           myCstd = text_latextable(myCstd);
                           myCstd = myCstd(2:size(myCstd,1),:);
                           myCt = text_latextable(myCt);
                           myCt = myCt(2:size(myCt,1),:);
                           myC = merge_latextable(myC,myCstd,myCt);
                           r_str = 'r';
                           for i=1:cm.n;
                              r_str = [r_str 'r'];
                           end;
                           fprintf(fout,'\\subsection*{The $C$ matrix}\n\n\\header{For $r$=%0.0f}\n\n',cm.r);
                           fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                           for i=1:(size(myC,1));
                              fprintf(fout,'%s\n',myC(i,:));
                           end;
                           fprintf(fout,'\\end{tabular}\n\n');
                           fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                        else;
                           fprintf(fout,'THE C-MATRIX (%s rank=%1.0f)\n\n',res_str,cm.r);
                           for i=1:(size(myC,1));
                              fprintf(fout,'%s\n',myC(i,:));
                           end;
                           fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
                           for i=1:(size(myC,1));
                              fprintf(fout,'%s\n',myCstd(i,:));
                           end;
                           fprintf(fout,'\nt-ratios:\n\n');
                           for i=1:(size(myC,1));
                              fprintf(fout,'%s\n',myCt(i,:));
                           end;
                           fprintf(fout,'\n===============================================================================\n\n');
                        end;
                     end;
                     fclose(fout);
                  end;
                  %
                  if cm.newdfbeta>0;
                     fout = fopen(cm.outputfile,'at');
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,'\\subsection*{Value of the maximized log-likelihood function:}\n\n');
                        fprintf(fout,'\\begin{tabular}{ll}\n');
                        fprintf(fout,[' & $\\ln L(\\mbox{\\rm lags}=%0.0f,r=%0.0f,\\mbox{\\rm \\#free } \\beta \\mbox{\\rm \\hspace*{1mm} parameters}=%0.0f)$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],cm.p,cm.r,numbp,lnl);
                        fprintf(fout,[' & $\\ln (\\det(\\Omega))$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],ldOmega);
                        fprintf(fout,'\\end{tabular}\n\n');
                        fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                     else;
                        fprintf(fout,'Value of the maximized log-likelihood function:\n\n');
                        fprintf(fout,['   lnL(lags=%1.0f,rank=%1.0f,#free beta parameters=%1.0f)=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],cm.p,cm.r,numbp,lnl);
                        fprintf(fout,['   ln(det(Omega))=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],ldOmega);
                        fprintf(fout,'===============================================================================\n\n');
                     end;
                     fclose(fout);
                  end;
                  %
                  % compute tests for additional deterministic variables
                  %
                  if (curr_ini.dummytests==1)&(cm.datad>0)&(cm.newdfbeta>0);
                     oldrestrict = cm.restrict;
                     cm.restrict(1,2) = 1;
                     txt = strvcat(' ','Computing exclusion tests for additional deterministics.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_exclude_dummies(cm,curr_ini);
                     if cm.r<cm.n;
                        txt = strvcat(' ','Computing long-run tests for additional deterministics.','Please hold!');
                        set(waitf.text,'String',txt);
                        print_longrun_dummies(cm,curr_ini);
                     end;
                     cm.restrict = oldrestrict;
                  end;
                  %
                  % Calculate lag-order test
                  %
                  if (curr_ini.lagtest>0)&(cm.newdfbeta>0);
                     txt = strvcat(' ','Computing lag order tests.','Please hold!');
                     set(waitf.text,'String',txt);
                     setlogtext('Computing lag order tests...');
                     print_lagordertest(cm,curr_ini,cm.betarest,alpha,Psi,cm.epsilon,[],cm.newdfbeta,0,cm.T-cm.df,0);
                  end;
                  %
                  if (curr_ini.gctest==1)&(cm.newdfbeta>0);
                     txt = strvcat(' ','Computing Granger causality tests.','Please hold!');
                     set(waitf.text,'String',txt);
                     setlogtext('Computing Granger causality tests...');
                     print_gctest(cm,curr_ini,cm.betarest,alpha,Psi,cm.epsilon,[],cm.newdfbeta,0,cm.T-cm.df,0);
                     if cm.n>2;
                        txt = strvcat(' ','Computing multi-step Granger causality tests.','Please hold!');
                        set(waitf.text,'String',txt);
                        setlogtext('Computing multi-step Granger causality tests...');
                        print_mgctest(cm,curr_ini,cm.betarest,alpha,Psi,cm.epsilon,[],cm.newdfbeta,0,0,0);
                     end;
                  end;
                  %
                  if (curr_ini.excludediffs==1)&(cm.newdfbeta>0);
                     if ((cm.p>1)&(cm.n>2))|(cm.q0>0)|((cm.q1>0)&(cm.pe>0));
                        setlogtext('Computing exclusion tests for first differences...');
                        print_firstdiffstests(cm,curr_ini,cm.betarest,alpha,Psi,cm.epsilon,[],cm.newdfbeta,0,cm.df,0);
                     end;
                  end;
                  %
                  if cm.r<cm.n;
                     if (curr_ini.weakexo==1)&(cm.newdfbeta>0);
                        txt = strvcat(' ','Computing weak exogeneity tests.','Please hold!');
                        set(waitf.text,'String',txt);
                        setlogtext('Computing weak exogeneity tests...');
                        print_weakexo(cm,curr_ini,cm.betarest,alpha,cm.epsilon,[],cm.newdfbeta,0,cm.T-cm.df,0);
                     end;
                  end;
                  %
                  if (cm.r<cm.n)&(cm.p>1);
                     if curr_ini.commoncycles==1;
                        oldrestrict = cm.restrict;
                        cm.restrict(1,2) = 1;
                        txt = strvcat(' ','Computing common cycle tests.','Please hold!');
                        set(waitf.text,'String',txt);
                        setlogtext('Computing common cycle tests...');
                        print_commoncycles(cm,curr_ini,alpha,Psi,cm.betarest);
                        cm.restrict = oldrestrict;
                     end;
                  end;
                  %
                  % residual statistics
                  %
                  if cm.newdfbeta>0;
                     txt = strvcat(' ','Computing residual statistics.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_residual_analysis(cm,curr_ini,cm.epsilon,cm.newdfbeta,0,cm.df);
                  end;
                  %
                  if (sum(cm.spectest)>0)&(cm.newdfbeta>0);
                     txt = strvcat(' ','Computing specification tests.','Please hold!');
                     set(waitf.text,'String',txt);
                  end;
                  %
                  % specification tests
                  %
                  if (cm.spectest(1,1)==1)&(cm.newdfbeta>0);
                     setlogtext('Computing serial correlation tests...');
                     print_corrtest(cm,curr_ini,cm.epsilon,cm.betarest,cm.newdfbeta,0,cm.df);
                  end;
                  %
                  if (cm.spectest(1,2)==1)&(cm.newdfbeta>0);
                     setlogtext('Computing ARCH tests...');
                     print_archtest(cm,curr_ini,cm.epsilon,cm.newdfbeta,0,cm.df);
                  end;
                  %
                  if (cm.spectest(1,3)==1)&(cm.newdfbeta>0);
                     setlogtext('Computing normality tests...');
                     print_normalitytest(cm,curr_ini,cm.epsilon,cm.newdfbeta,0,cm.df);
                  end;
                  %
                  if (cm.spectest(1,4)==1)&((cm.newdfbeta>0)|((curr_ini.nyblom==1)&(curr_ini.nyblombeta==1)));
                     if rank(cm.betarest'*cm.betarest,1.0e-004)==cm.r;
                        if cm.r>0;
                           Zb = cm.betarest'*cm.Z1;
                        else;
                           Zb = 0;
                        end;
                        fout = fopen(cm.outputfile,'at');
                        if curr_ini.texoutputformat>=2;
                           fprintf(fout,['\\subsection*{Parameter constancy tests}\n\n\\header{For $r$=' num2str(cm.r) ', lags=' num2str(cm.p) ', \\#free $\\beta$ params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.newdfbeta,'%0.0f') ', \\#free $\\alpha$ params=' num2str(cm.n*cm.r,'%0.0f') '}\n\n']);
                        else;
                           fprintf(fout,['PARAMETER CONSTANCY TESTS\n\n(for rank=' num2str(cm.r) ',lags=' num2str(cm.p) ',#free beta params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.newdfbeta,'%0.0f') ',#free alpha params=' num2str(cm.n*cm.r,'%0.0f') ')\n\n']);
                        end;
                        fclose(fout);
                        if (curr_ini.lstrtest==1)&(cm.newdfbeta>0);
                           setlogtext('Computing LSTR1 constancy tests...');
                           print_lstr1test(cm,curr_ini,cm.epsilon,Zb,cm.df);
                        end;
%                        if (cm.q1+cm.q0==0)&(curr_ini.nyblom==1)&(curr_ini.nyblombeta==1);
                        if (curr_ini.nyblom==1)&(curr_ini.nyblombeta==1);
                           setlogtext('Computing Nyblom constancy tests for beta...');
                           old_restrict = cm.restrict;
                           cm.restrict = [0 1];
                           print_nyblom(cm,curr_ini);
                           cm.restrict = old_restrict;
                        end;
                        %
                        if (cm.p+cm.r+cm.d>1)&(cm.newdfbeta>0);
                           if curr_ini.chow==1;
                              setlogtext('Computing Chow tests...');
                              print_chow(cm,curr_ini,cm.betarest,0);
                           end;
                           %
                           setlogtext('Computing fluctuation tests...');
                           print_ploberger(cm,curr_ini,cm.betarest,0);
                        end;
                        %
                     else;
                        setlogtext(strvcat(' ','WARNING: The estimated beta matrix is badly identified...',['         Numerical rank is ' num2str(myrank(1,1)) '...']));
                        fout = fopen(cm.outputfile,'at');
                        myrank = rank(cm.betarest'*cm.betarest,1.0e-004);
                        txt = ['The estimated restricted beta matrix is badly identified. The rank of beta''beta is ' num2str(myrank(1,1)) ', i.e. less than ' num2str(cm.r) ', when a tolerance level of 1.0e-004 is used. Please consider some other restrictions.'];
                        about(txt,'warning','Badly identified beta matrix',120,500);
                        if curr_ini.texoutputformat>=2;
                           fprintf(fout,'\\vspace*{2mm}\n\\newline\n\\noindent\n\\textbf{WARNING:} Parameter constancy tests have not been calculated. The\n');
                           fprintf(fout,'         estimated restricted beta matrix is badly identified.\n');
                           fprintf(fout,'         The rank of $\\beta''\\beta$ is %0.0f, i.e. less than %0.0f, when\n',myrank,cm.r);
                           fprintf(fout,'         a tolerance level of 1.0e-004 is used.\n\n');
                           fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                        else;
                           fprintf(fout,'WARNING: Parameter constancy tests have not been calculated. The\n');
                           fprintf(fout,'         estimated restricted beta matrix is badly identified.\n');
                           fprintf(fout,'         The rank of beta''beta is %1.0f, i.e. less than %1.0f, when\n',myrank,cm.r);
                           fprintf(fout,'         a tolerance level of 1.0e-004 is used.\n');
                           fprintf(fout,'\n===============================================================================\n\n');
                        end;
                        fclose(fout);
                     end;
                  end;
                  %
               end;
               %
               % close the wait dialog
               %
               if (curr_ini.noprint==0)&((cm.newdfbeta>0)|((cm.r<cm.n)&(cm.p>1)&(curr_ini.commoncycles==1)));
                  set(waitf.text,'UserData','done');
                  delete(mywait);
                  drawnow;
               end;
               %
               % display results
               %
               thisbetadlg = betadlg('init',cm,infomat,Psi,alpha);
               betar = get(thisbetadlg,'UserData');
               userestrict = get(betar.betamat,'UserData');
               set(betar.stdmat,'UserData','done');
               delete(thisbetadlg);
               drawnow;
               cm.estimated = 1;
               %
               % Check decision and update window if necessary
               %
               if curr_ini.noprint==0;
                  fout = fopen(cm.outputfile,'at');
               end;
               if userestrict==1;
                  setlogtext(strvcat(' ','Using the restricted beta estimates...'));
                  cm.restrict(1,2) = 1;
                  cm.dfbeta = cm.newdfbeta;
                  cm.beta = cm.betarest;
                  cm.bZ1t = cm.beta'*cm.Z1t;
                  cm.bR1t = cm.beta'*cm.R1t;
                  cm.alpha = (cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta);
                  if isempty(cm.Z2)==0;
                     cm.Psi = (cm.M_02/cm.M_22)-(cm.alpha*cm.beta'*(cm.M_12/cm.M_22));
                     cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1)-(cm.Psi*cm.Z2);
                     Z3 = [cm.Z2;cm.beta'*cm.Z1];
                  else;
                     cm.Psi = [];
                     cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1);
                     Z3 = cm.beta'*cm.Z1;
                  end;
                  Theta = [cm.Psi cm.alpha];
                  set(cointbeta.estalpha,'UserData',Theta);
                  cm.Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
                  inv_Z3 = eye(size(Z3,1))/(Z3*Z3');
                  Sigma_theta = kron(inv_Z3,cm.Omega);
                  set(cointbeta.genalpha,'UserData',Sigma_theta);
                  cm.restrict(1,1) = 0;
                  set(cointbeta.menualphaundo,'Enable','off');
                  set(cointbeta.alphaundo,'Enable','off');
                  cm.dfalpha = 0;
                  if cm.evalue>=1;
                     set(cointbeta.cmat,'Enable','off');
                     set(cointbeta.longrun_gir,'Enable','off');
                     set(cointbeta.menupermcomp,'Enable','off');
                     set(cointbeta.permcomp,'Enable','off');
                     set(cointbeta.savetc,'Enable','off');
                  else;
                     set(cointbeta.cmat,'Enable','on');
                     set(cointbeta.longrun_gir,'Enable','on');
                     set(cointbeta.menupermcomp,'Enable','on');
                     set(cointbeta.permcomp,'Enable','on');
                     set(cointbeta.savetc,'Enable','on');
                  end;
                  set(cointbeta.clear,'Enable','on');
                  set(cointbeta.space,'Enable','off');
                  set(cointbeta.menuclear,'Enable','on');
                  if cm.dfbeta<((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r);
                     set(cointbeta.betagraph,'Enable','on');
                  else;
                     set(cointbeta.betagraph,'Enable','off');
                  end;
                  %
                  cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,lnl,cm.evalue);
                  set(cointbeta.model_info,'String',cm.model_info.info);
                  %
                  set(cointbeta.betamat,'UserData',cm.betarest);
                  set(cointbeta.model_info,'UserData',cm.stdbeta);
                  currbeta = showbeta(cm.beta,cm.evars,strtrim([cm.xvars1 ' ' cm.restvec]));
                  set(cointbeta.betamat,'String',currbeta);
                  if curr_ini.noprint==0;
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,'\\header{You selected to use these cointegration vectors.}\n\n');
                     else;
                        fprintf(fout,'You selected to use these cointegration vectors.\n\n');
                     end;
                     if cm.newdfbeta>0;
                        [eigenvalues1,AscInd] = sort(abs(eigenvalues2));
                        nr = size(AscInd,1);
                        DescInd = zeros(nr,1);
                        i = 1;
                        while i<=nr;
                           DescInd(i)=AscInd(nr+1-i);
                           i =i+1;
                        end;
                        eigenvalues2 = eigenvalues2(DescInd);
                        if curr_ini.texoutputformat>=2;
                           fprintf(fout,'\\subsection*{Properties of the stacked VAR(1) companion matrix:}\n\n');
                           fprintf(fout,'\\begin{tabular}{rrrr}\n');
                           fprintf(fout,'\\multicolumn{4}{c}{Eigenvalues} \\\\\n');
                           fprintf(fout,'number & real & imaginary & modulus \\\\\n');
                           fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(1:size(eigenvalues2,1))' real(eigenvalues2) imag(eigenvalues2) abs(eigenvalues2)]');
                           fprintf(fout,'\\end{tabular}\n\n');
                        else;
                           fprintf(fout,'Properties of the stacked VAR(1) companion matrix:\n');
                           fprintf(fout,'Eigenvalues\n');
                           fprintf(fout,['number    real  ' sp_str '    imaginary   ' sp_str '  modulus\n\n']);
                           fprintf(fout,['%3.0f      %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f     %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f         %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(1:size(eigenvalues2,1))' real(eigenvalues2) imag(eigenvalues2) abs(eigenvalues2)]');
                           fprintf(fout,'\n');
                        end;
                     end;
                  end;
               else;
                  setlogtext(strvcat(' ','Skipping the restricted beta estimates...'));
                  if curr_ini.noprint==0;
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,'\\header{You selected to skip these cointegration vectors.}\n\n');
                     else;
                        fprintf(fout,'You selected to skip these cointegration vectors.\n\n');
                     end;
                  end;
                  %
                  Theta = get(cointbeta.estalpha,'UserData');
                  g2 = size(Theta,2);
                  if g2>cm.r;
                     cm.Psi = Theta(:,1:g2-cm.r);
                     cm.alpha = Theta(:,g2-cm.r+1:g2);
                     Z3 = [cm.Z2;cm.beta'*cm.Z1];
                  else;
                     cm.alpha = Theta;
                     cm.Psi = [];
                     Z3 = [cm.beta'*cm.Z1];
                  end;
                  cm.epsilon = cm.Z0-(Theta*Z3);
                  %
                  % need to extract current degrees of freedom
                  %
                  sl = size(cm.model_info.info,1);
                  cm.df = cm.T-mystr2num(strtrim(cm.model_info.info(sl-2,:)));
                  cm.Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
               end;
               if curr_ini.noprint==0;
                  if curr_ini.texoutputformat>=2;
                     fprintf(fout,'\\ruler\n\n%%******************************************************************************\n\n');
                  else;
                     fprintf(fout,'*******************************************************************************\n\n');
                  end;
                  fclose(fout);
               end;
               set(cointbeta.clear,'UserData',thisexit);
               set(cointbeta.restrict,'UserData',cm.restrict);
            else;
               setlogtext('Cancelled estimation...');
            end;
            set(betaguidlg,'Visible','on');
            set(betaguidlg,'UserData',cointbeta);
         else;
            unident = '';
            for i=1:cm.r;
               if status(i,1)==0;
                  if strcmp(unident,'')==1;
                     unident = ['beta(' num2str(i) ')'];
                     thisres = 'restriction';
                     thisfail = 'fails';
                  else;
                     unident = [unident ', beta(' num2str(i) ')'];
                     thisres = 'restrictions';
                     thisfail = 'fail';
                  end;
               end;
            end;
            setlogtext(strvcat(' ','ERROR: The restrictions are not identifying...'));
            txt = ['The restrictions you have selected for beta are not identifying. In particular, the ' thisres ' on ' unident ' ' thisfail ' the identification test.'];
            about(txt,'warning','Identification Test Failure',120,500,'identification');
         end;
         cm.running = 0;
      end;
   case 'clear'
      if cm.running==0;
         cm.dfbeta = 0;
         cm.LRbeta = [];
         cm.LRalpha = 0;
         setlogtext(strvcat(' ','Skipping the restricted beta estimates...'));
         set(cointbeta.clear,'UserData',thisexit);
         set(cointbeta.restrict,'UserData',cm.restrict);
         set(cointbeta.betamat,'UserData',0);
         set(cointbeta.model_info,'UserData',0);
         cm.beta = cm.oldbeta;
         cm.bZ1t = normbeta(cm.beta)'*cm.Z1t;
         cm.bR1t = normbeta(cm.beta)'*cm.R1t;
         alpha = (cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta);
         if isempty(cm.Z2)==0;
            Psi = (cm.M_02/cm.M_22)-(alpha*cm.beta'*(cm.M_12/cm.M_22));
            cm.epsilon = cm.Z0-(alpha*cm.beta'*cm.Z1)-(Psi*cm.Z2);
         else;
            Psi = [];
            cm.epsilon = cm.Z0-(alpha*cm.beta'*cm.Z1);
         end;
         cm.alpha = alpha;
         cm.Psi = Psi;
         cm.Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
         cm.restrict(1,1) = 0;
         cm.restrict(1,2) = 0;
         set(cointbeta.menualphaundo,'Enable','off');
         set(cointbeta.alphaundo,'Enable','off');
         set(cointbeta.estalpha,'UserData',cm.oldTheta);
         set(cointbeta.genalpha,'UserData',cm.oldSigma_theta);
         set(cointbeta.markov_switching,'Enable','on');
         cm.dfalpha = 0;
         eigenvalues = companion(cm.Z0,cm.Z1,cm.Z2,cm.r,cm.p,cm.n,cm.q1,cm.q0,cm.d,cm.T);
         eigenvalues = abs(sort(eigenvalues));
         i = size(eigenvalues,1);
         while i>=1;
            cm.evalue = eigenvalues(i,1);
            if strcmp(num2str(cm.evalue,'%1.10f'),'1.0000000000')==1;
               i = i-1;
            else;
               i = 0;
            end;
         end;
         if cm.evalue>=1;
            set(cointbeta.cmat,'Enable','off');
            set(cointbeta.longrun_gir,'Enable','off');
            set(cointbeta.menupermcomp,'Enable','off');
            set(cointbeta.permcomp,'Enable','off');
            set(cointbeta.savetc,'Enable','off');
         else;
            set(cointbeta.cmat,'Enable','on');
            set(cointbeta.longrun_gir,'Enable','on');
            set(cointbeta.menupermcomp,'Enable','on');
            set(cointbeta.permcomp,'Enable','on');
            set(cointbeta.savetc,'Enable','on');
         end;
         lnl = loglike(cm.epsilon);
         if curr_ini.noprint==0;
            fout = fopen(cm.outputfile,'at');
            if curr_ini.texoutputformat>=2;
               fprintf(fout,'\\header{But then you changed your mind about $\\beta$ and $\\alpha$!}\n\n');
               fprintf(fout,'\\ruler\n\n%%******************************************************************************\n\n');
            else;
               fprintf(fout,'But then you changed your mind about beta and alpha!\n\n');
               fprintf(fout,'*******************************************************************************\n\n');
            end;
            fclose(fout);
         end;
         cm.estimated = 1;
         dfc = round(((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)/cm.n);
         cm.df = cm.n*(cm.p-1)+cm.r+cm.q1*cm.pe+cm.q0*(cm.pe+1)+cm.d+dfc;
         %
         cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,lnl,cm.evalue);
         set(cointbeta.model_info,'String',cm.model_info.info);
         %
         set(cointbeta.clear,'Enable','off');
         set(cointbeta.space,'Enable','on');
         set(cointbeta.menuclear,'Enable','off');
         set(cointbeta.betagraph,'Enable','off');
         currbeta = showbeta(normbeta(cm.beta),cm.evars,strtrim([cm.xvars1 ' ' cm.restvec]));
         set(cointbeta.betamat,'String',currbeta);
         set(betaguidlg,'UserData',cointbeta);
      end;
   case 'quit'
      if cm.running==0;
         if curr_ini.sillyq==0;
            txt = 'Are you sure you want to return to the Cointegration Rank Tests dialog?';
            answer = query(txt,'question',120,'Restrictions on (alpha,beta)',350);
         else;
            answer = 'Yes';
         end;
         if strcmp(answer,'Yes');
            thisexit = 2;
            cm.restrict = [0 0];
            set(cointbeta.continue,'UserData','done');
         end;
         set(cointbeta.clear,'UserData',thisexit);
         set(cointbeta.restrict,'UserData',cm.restrict);
         set(betaguidlg,'UserData',cointbeta);
      end;
   case 'graphs'
      if cm.running==0;
         set(cointbeta.clear,'UserData',thisexit);
         set(cointbeta.restrict,'UserData',cm.restrict);
         set(betaguidlg,'Visible','off');
         drawnow;
         cm.running = 1;
         thisgraph = cointgraphdlg('init',cm,cm.bZ1t,cm.bR1t,cm.restrict(1,2));
         graph = get(thisgraph,'UserData');
         set(graph.show,'UserData','done');
         killguipid(thisgraph);
         set(betaguidlg,'Visible','on');
         set(betaguidlg,'UserData',cointbeta);
         cm.running = 0;
      end;
   case 'varforecast'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         thisgraph = var_forecastdlg('init',cm,cm.restrict);
         f_cast = get(thisgraph,'UserData');
         set(f_cast.show,'UserData','done');
         killguipid(thisgraph);
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'forecast'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         thisgraph = forecastdlg('init',cm,cm.restrict);
         f_cast = get(thisgraph,'UserData');
         set(f_cast.show,'UserData','done');
         killguipid(thisgraph);
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'residuals'
      if cm.running==0;
         cm.running = 1;
         ind = 0;
         if cm.dfbeta>0;
            ind = 1;
         end;
         set(betaguidlg,'Visible','off');
         drawnow;
         thisgraph = epsilondlg('init',cm,ind);
         epsi = get(thisgraph,'UserData');
         set(epsi.show,'UserData','done');
         killguipid(thisgraph);
         set(betaguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'ploberger'
      if cm.running==0;
         cm.running = 1;
            ind = 1;
            if cm.dfbeta==0;
               ind = 0;
            end;
            aind = 0;
            if cm.restrict(1,1)==1;
               aind = 1;
            end;
            show_ploberger(betaguidlg,cm.beta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'chow'
      if cm.running==0;
         cm.running = 1;
            ind = 1;
            if cm.dfbeta==0;
               ind = 0;
            end;
            aind = 0;
            if cm.restrict(1,1)==1;
               aind = 1;
            end;
            show_chow(betaguidlg,cm.beta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'permcomp'
      if cm.running==0;
         cm.running = 1;
         [y_t,perm_y,trans_y] = permcomp(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.x0,cm.x1,cm.x1_r,cm.x2,cm.d,cm.dcoint,cm.p,cm.pe,cm.q1,cm.q0);
         ind = 1;
         if cm.beta==cm.oldbeta;
            ind = 0;
         end;
         set(betaguidlg,'Visible','off');
         drawnow;
         thisgraph = pctcdlg('init',cm,y_t,perm_y,trans_y,ind);
         pctc = get(thisgraph,'UserData');
         set(pctc.show,'UserData','done');
         killguipid(thisgraph);
         set(betaguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'corrtest'
      if cm.running==0;
         cm.running = 1;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_corrtest(betaguidlg,cm,curr_ini,ind);
         cm.running = 0;
      end;
   case 'archtest'
      if cm.running==0;
         cm.running = 1;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_archtest(betaguidlg,cm,curr_ini,ind);
         cm.running = 0;
      end;
   case 'normalitytest'
      if cm.running==0;
         cm.running = 1;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_normalitytest(betaguidlg,cm,curr_ini,ind);
         cm.running = 0;
      end;
   case 'constancytest'
      if cm.running==0;
         cm.running = 1;
         if rank(cm.beta'*cm.beta,1.0e-004)==cm.r;
            ind = 1;
            if cm.dfbeta==0;
               ind = 0;
            end;
            aind = 0;
            if cm.restrict(1,1)==1;
               aind = 1;
            end;
            show_constancytest(betaguidlg,cm.beta,cm,curr_ini,ind,aind,0);
         else;
            myrank = rank(cm.beta'*cm.beta,1.0e-004);
            txt = ['The estimated beta matrix is badly scaled. The rank of beta''*beta is ' num2str(myrank(1,1)) ', i.e. less than ' num2str(cm.r) ', when a tolerance level of 1.0e-004 is used. Parameter constancy tests will not be calculated. Please consider some other restrictions.'];
            about(txt,'warning','Badly identified beta matrix',120,500);
         end;
         cm.running = 0;
      end;
   case 'lagorder'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         aind = 0;
         if cm.restrict(1,1)==1;
            aind = 1;
         end;
         show_lagordertest(betaguidlg,cm.beta,cm.alpha,cm.Psi,Sigma_theta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'firstdiffs'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_firstdiffstests(betaguidlg,cm.beta,cm.alpha,cm.Psi,Sigma_theta,cm,curr_ini,ind,0);
         cm.running = 0;
      end;
   case 'gctest'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         aind = 0;
         if cm.restrict(1,1)==1;
            aind = 1;
         end;
         show_gctest(betaguidlg,cm.beta,cm.alpha,cm.Psi,Sigma_theta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'multigctest'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         %
         this_graph = select_mgc('init',cm,cm.beta,Sigma_theta,cm.restrict(1,1),ind,0);
         mgc = get(this_graph,'UserData');
         set(mgc.show,'UserData','done');
         killguipid(this_graph);
         set(betaguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'savetc'
      if cm.running==0;
         cm.running = 1;
         addrest = 'rest-';
         if cm.dfbeta==0;
            addrest = '';
         end;
         [y_t,perm_y,trans_y] = permcomp(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.x0,cm.x1,cm.x1_r,cm.x2,cm.d,cm.dcoint,cm.p,cm.pe,cm.q1,cm.q0);
         myfile = [cm.outputdir 'trans.comp-' addrest num2str(cm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myN myT] = size(trans_y);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(cm.samplebeg,cm.sampleend,cm.freq,i/myT));
            end;
            setlogtext(['Saving transitory components to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myN;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '              tc_' cm.myvars(i,:)];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),trans_y(:,i)');
            end;
            fclose(fid);
         end;
         cm.running = 0;
      end;
   case 'savecoint'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,2)==1;
            CR = cm.beta'*cm.Z1t;
         else;
            CR = normbeta(cm.beta)'*cm.Z1t;
         end;
         addrest = 'rest-';
         if cm.dfbeta==0;
            addrest = '';
         end;
         myfile = [cm.outputdir 'coint-' addrest num2str(cm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myR myT] = size(CR);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(cm.samplebeg,cm.sampleend,cm.freq,i/myT));
            end;
            setlogtext(['Saving cointegration relations to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myR;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '               ci_' num2str(i,'%1.0f')];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),CR(:,i)');
            end;
            fclose(fid);
         end;
         cm.running = 0;
      end;
   case 'saveresid'
      if cm.running==0;
         cm.running = 1;
         addrest = 'rest-';
         if cm.dfbeta==0;
            addrest = '';
         end;
         myfile = [cm.outputdir 'epsilon-' addrest num2str(cm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myN myT] = size(cm.epsilon);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(cm.samplebeg,cm.sampleend,cm.freq,i/myT));
            end;
            setlogtext(['Saving residuals to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myN;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '             eps_' cm.myvars(i,:)];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),cm.epsilon(:,i)');
            end;
            fclose(fid);
         end;
         cm.running = 0;
      end;
   case 'viewdata'
      if cm.running==0;
         cm.running = 1;
         if matlabnumber>=6.0;
            myfile = {'*.txt;*.data;*.wk1;*.xls;*.mat;*.csv', 'Data Files (*.txt, *.data, *.wk1, *.xls, *.mat, *.csv)'; ...
                    '*.txt;*.data', 'Text Files (*.txt, *.data)'; ...
                    '*.mat', 'MatLab MAT Files (*.mat)'; ...
                    '*.wk1', 'Lotus 1-2-3 Spreadsheets (*.wk1)'; ...
                    '*.xls', 'Excel Spreadsheets (*.xls)'; ...
                    '*.csv', 'Comma Separated Value Files (*.csv)'; ...
                    '*.*', 'All Files (*.*)'};
         else;
            if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1);
               fix_fig = findobj('Tag','StructuralVAR');
               model = get(fix_fig,'UserData');
               afile = get(model.endogenous,'String');
               basedir = getpath(strtrim(afile));
               file_fltr = 'Data Files (*.txt, *.data, *.wk1, *.mat, *.csv)|*.txt;*.data;*.wk1;*.mat;*.csv|Text Files (*.txt, *.data)|*.txt;*.data|MatLab MAT Files (*.mat)|*.mat|Lotus 1-2-3 Spreadsheets (*.wk1)|*.wk1|Comma Separated Value Files (*.csv)|*.csv|All Files (*.*)|*.*|';
               best_pos = 1;
               title_txt = 'Open';
            else;
               fix_fig = findobj('Tag','StructuralVAR');
               model = get(fix_fig,'UserData');
               afile = get(model.endogenous,'String');
               afile = strtrim(afile);
               if fileexist(afile)==1;
                  adir = getpath(afile);
                  if strcmp(computer,'PCWIN')==1;
                     if strcmp(adir(1,size(adir,2)),'\')==0;
                        adir = [adir '\'];
                     end;
                  else;
                     if strcmp(adir(1,size(adir,2)),'/')==0;
                        adir = [adir '/'];
                     end;
                  end;
               else;
                  adir = cm.outputdir;
               end;
               myfile = [adir '*.txt;*.data;*.wk1;*.mat;*.csv'];
            end;
         end;
         if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1)&(matlabnumber<6.0);
            [openfile,openpath] = fileopen(basedir,file_fltr,1,title_txt);
         else;
            if matlabnumber>=6;
               [openfile,openpath] = uigetfile(myfile,'Open',getpath(cm.endofile));
            else;
               [openfile,openpath] = uigetfile(myfile,'Open');
            end;
         end;
         drawnow;
         if isnumeric(openfile)==0;
            openfile = strcat(openpath,openfile);
            set(betaguidlg,'Visible','off');
            drawnow;
            textgui(openfile,['View Data - ' openfile],[(realscrsz(3)-(realscrsz(4)-80))/2 1 realscrsz(4)-80 realscrsz(4)-54],'Data',25);
            set(betaguidlg,'Visible','on');
            drawnow;
         end;
         cm.running = 0;
      end;
   case 'longrun_gir'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         %
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         addrest = ' restricted';
         if cm.dfbeta==0;
            addrest = '';
         end;
         [lr_gir,lr_gir_std] = longrun_gir_fcn(cm.alpha,cm.beta,cm.Psi,cm.Omega,Sigma_theta,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d);
         ctext = strvcat(' ',['LONG RUN GENERALIZED IMPULSE RESPONSES (for' addrest ' rank=' num2str(cm.r,'%1.0f') ')']);
         numchar = size(ctext,2);
         addcorr = '';
         for i=1:numchar;
            addcorr = strcat(addcorr,'=');
         end;
         ctext = strvcat(ctext,addcorr,' ');
         ctext = strvcat(ctext,showc(lr_gir,cm.myvars),' ');
         ctext = strvcat(ctext,'Estimated asymptotic standard errors:',' ',showc(lr_gir_std,cm.myvars),' ','t-ratios:',' ',showc(inf2nan(lr_gir./lr_gir_std),cm.myvars),' ');
         drawnow;
         textgui(ctext,'Long Run GIRs',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         %
         set(betaguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'cmat'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         addrest = ' restricted';
         if cm.dfbeta==0;
            addrest = '';
         end;
         if cm.r<cm.n;
            C = cmat(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,Sigma_theta);
         else;
            C = cmat_stat(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,Sigma_theta);
         end;
         ctext = strvcat(' ',['THE C-MATRIX (for' addrest ' rank=' num2str(cm.r,'%1.0f') ')']);
         numchar = size(ctext,2);
         addcorr = '';
         for i=1:numchar;
            addcorr = strcat(addcorr,'=');
         end;
         ctext = strvcat(ctext,addcorr,' ');
         ctext = strvcat(ctext,showc(C(:,1:cm.n),cm.myvars),' ');
         ctext = strvcat(ctext,'Estimated asymptotic standard errors:',' ',showc(C(:,cm.n+1:2*cm.n),cm.myvars),' ','t-ratios:',' ',showc(inf2nan(C(:,1:cm.n)./C(:,cm.n+1:2*cm.n)),cm.myvars),' ');
         set(betaguidlg,'Visible','off');
         drawnow;
         textgui(ctext,'C-Matrix',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         set(betaguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'exclusion'
      if cm.running==0;
         cm.running = 1;
         show_exclusion(betaguidlg,cm,curr_ini);
         cm.running = 0;
      end;
   case 'stationarity'
      if cm.running==0;
         cm.running = 1;
         show_stationarity(betaguidlg,cm,curr_ini);
         cm.running = 0;
      end;
   case 'weakexo'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
            aind = 0;
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
            aind = 1;
         end;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_weakexo(betaguidlg,cm.beta,cm.alpha,Sigma_theta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'shortdyn'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,1)==0;
            Sigma_theta = [];
         else;
            Sigma_theta = get(cointbeta.genalpha,'UserData');
         end;
         shortstr = shortrun(cm.alpha,cm.Psi,cm.Omega,cm.beta,cm.Z1,cm.Z2,cm.p,cm.pe,cm.d,cm.deterselect,cm.q0,cm.q1,cm.evars,cm.xvars0,cm.xvars1,cm.deter,cm.restvec,1,cm.outputfile,1,Sigma_theta);
         if cm.dfbeta>0;
            myshortstr = shortstr(2,:);
            oldsize = size(myshortstr,2);
            myshortstr = strrep(myshortstr,'for rank=','for restricted rank=');
            newsize = size(myshortstr,2);
            myshortstr = myshortstr(1,1:newsize-11);
            kind = findstr(myshortstr,')');
            if isempty(kind)==0;
               shortstr(2,:) = myshortstr;
               shortstr(3,:) = strrep(shortstr(3,:),'=           ','============');
            end;
         end;
         set(betaguidlg,'Visible','off');
         drawnow;
         textgui(shortstr,'Short Run Dynamics',[(realscrsz(3)-800)/2 (realscrsz(4)-500)/2 800 500],'Results',38);
         set(betaguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'saverest'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         thissaverest = saverestgui('init',cm);
         saverest = get(thissaverest,'UserData');
         set(saverest.hmat,'UserData','done');
         killguipid(thissaverest);
         %
         set(betaguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'openrest'
      if cm.running==0;
         cm.running = 1;
         set(betaguidlg,'Visible','off');
         drawnow;
         thisopenrest = openrestgui('init',cm);
         openrest = get(thisopenrest,'UserData');
         set(openrest.hmat,'UserData','done');
         killguipid(thisopenrest);
         set(betaguidlg,'Visible','on');
         %
         % Test if H_i-modelfile.rest files exist in modelsdir
         %
         estimateenable = 'off';
         i = 1;
         while i<=cm.r;
            file = strcat(cm.modelsdir,'H_',num2str(i),'-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
               i = i+1;
            else;
               break;
            end;
         end;
         %
         % if necessary test if normalize-modelfile.norm exists in modelsdir
         % and if so set cointbeta.estimate control to
         % being enabled
         %
         if i==cm.r+1;
            file = strcat(cm.modelsdir,'normalize-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.norm');
            if fileexist(file)==1;
               file = strcat(cm.modelsdir,'H_1-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
               checkH_1 = loaddata(file);
               if size(checkH_1,1)==size(cm.beta,1);
                  estimateenable = 'on';
               end;
            end;
         end;
         if strcmp(estimateenable,'on')==1;
            set(cointbeta.menuestimate,'Enable','on');
            set(cointbeta.bootbeta,'Enable','on');
            if (curr_ini.nyblom==0)|(curr_ini.nyblombeta==0);
               set(cointbeta.bootnyblommenu,'Enable','off');
            else;
               set(cointbeta.bootnyblommenu,'Enable','on');
            end;
            set(cointbeta.menurecursive,'Enable','on');
            set(cointbeta.estimate,'Enable','on');
            set(cointbeta.recursive,'Enable','on');
         else;
            set(cointbeta.menuestimate,'Enable','off');
            set(cointbeta.bootbeta,'Enable','off');
            set(cointbeta.bootnyblommenu,'Enable','off');
            set(cointbeta.menurecursive,'Enable','off');
            set(cointbeta.estimate,'Enable','off');
            set(cointbeta.recursive,'Enable','off');
         end;
         cm.running = 0;
      end;
   case 'recursive'
      if cm.running==0;
         cm.running = 1;
         show_recursivebeta(betaguidlg,cm,curr_ini);
         cm.running = 0;
      end;
   case 'continue';
      if cm.running==0;
         cm.running = 1;
         answer = '';
         set(cointbeta.clear,'UserData',thisexit);
         set(cointbeta.restrict,'UserData',cm.restrict);
         if cm.estimated==0;
            if curr_ini.sillyq==0;
               if (cm.modeltype(1,1)==1)&((gcbo==cointbeta.stop)|(gcbo==cointbeta.menustop));
                  answer = query('Are you sure you want to quit?','question',120,'Restrictions on (alpha,beta)',300);
                  if strcmp(answer,'Yes');
                     set(cointbeta.stop,'UserData','quit');
                  end;
               else;
                  txt = 'Are you sure you don''t want to estimate beta under linear restrictions?';
                  answer = query(txt,'question',120,'Restrictions on (alpha,beta)',350);
               end;
            else;
               answer = 'Yes';
            end;
            if strcmp(answer,'Yes');
               cm.estimated = 1;
            end;
         else;
            if curr_ini.sillyq==0;
               if (cm.modeltype(1,1)==1)&((gcbo==cointbeta.stop)|(gcbo==cointbeta.menustop));
                  answer = query('Are you sure you want to quit?','question',120,'Restrictions on (alpha,beta)',300);
                  if strcmp(answer,'Yes');
                     set(cointbeta.stop,'UserData','quit');
                  end;
               else;
                  answer = 'Yes';
               end;
            else;
               answer = 'Yes';
            end;
         end;
         if (cm.estimated==1)&(strcmp(answer,'Yes')==1);
            set(cointbeta.continue,'UserData','done');
            set(betaguidlg,'UserData',cointbeta);
         else;
            set(betaguidlg,'UserData',cointbeta);
         end;
         cm.running = 0;
      end;
   case 'wayout'
      if cm.running==0;
         cm.running = 1;
         set(cointbeta.clear,'UserData',thisexit);
         set(cointbeta.restrict,'UserData',cm.restrict);
         if cm.estimated==0;
            if curr_ini.sillyq==0;
               txt = 'Are you sure you don''t want to estimate beta under linear restrictions?';
               answer = query(txt,'question',120,'Restrictions on (alpha,beta)',350);
            else;
               answer = 'Yes';
            end;
            if strcmp(answer,'Yes');
               cm.estimated = 1;
            end;
         end;
         if cm.estimated==1;
            set(cointbeta.continue,'UserData','done');
            if strcmp(get(cointbeta.graphs,'UserData'),'done')==1;
               delete(betaguidlg);
            end;
         else;
            set(betaguidlg,'UserData',cointbeta);
         end;
         cm.running = 0;
      end;
end;

if strcmp(selector,'init')==1;
   set(betaguidlg,'Visible','on');
   if strcmp(cm.deterdel,'')==0;
      deterdelinfo = ['Due to linear dependence, the following deterministic variables were deleted: ' cm.deterdel '.'];
      set(cointbeta.model_info,'TooltipString',deterdelinfo);
      set(cointbeta.model_header,'TooltipString',deterdelinfo);
   end;
end;

if nargout==1;
   myhandle = betaguidlg;
end;

curr_ini.images = button_images;
curr_ini.model = cm;
set(cointbeta.viewdata,'UserData',curr_ini);

if thisexit==0;
   waitfor(cointbeta.continue,'UserData','done');
end;

%
% end of betagui.m
%
