function boot = bootstrap_sample(mm,beta,epsilon)

%
% create a bootstrap sample of desired length
%

T = size(epsilon,2);
Z0 = zeros(size(mm.Z0,1),T);
Z0_new = [];

Z1 = zeros(size(mm.Z1,1),T);

if isempty(mm.Z2)==0;
   Z2 = zeros(size(mm.Z2,1),T);
else;
   Z2 = [];
end;

if (size(beta,1)==0)|(size(beta,2)==0);
   beta = [];
end;
d_const = 0;
d_coint = 0;
d_corel = 0;
d_shortrun = 0;
d_coidet = 0;
if isfield(mm,'usealtmodel')==0;
   mm.usealtmodel = 0;
end;
if mm.usealtmodel==1;
   tempdummy = mm.altmodel.tempdummy;
   permdummy = mm.altmodel.permdummy;
   if tempdummy==1;
      d_begin = mm.altmodel.tempbegin;
      d_end = mm.altmodel.tempend;
      d_acc = mm.altmodel.tempacc;
      d_coint = mm.altmodel.tempcoint;
      d_corel = mm.altmodel.tempcorel;
      d_coidet = mm.altmodel.tempcoidet;
      d_shortrun = mm.altmodel.tempshortrun;
   else;
      d_begin = mm.altmodel.permbegin;
      d_end = mm.T;
      d_acc = mm.altmodel.permacc;
      d_coint = mm.altmodel.permcoint;
      d_corel = mm.altmodel.permcorel;
      d_coidet = mm.altmodel.permcoidet;
      d_shortrun = mm.altmodel.permshortrun;
   end;
   if mm.altmodel.relsample==1;
      d_begin = round(d_begin*(T/mm.T));
      d_end = round(d_end*(T/mm.T));
   else;
      if (d_end==mm.T)&(permdummy==1);
         d_end = T;
      end;
   end;
   if (d_corel==0)&(d_shortrun==0);
      d_const = 1-d_coint;
   end;
   %
   % create the dummy
   %
   if (d_begin>1)&(d_end<T);
      data_dummy = [zeros(d_begin-1,1);ones(d_end-d_begin+1,1);zeros(T-d_end,1)];
   elseif d_begin==1;
      data_dummy = [ones(d_end,1);zeros(T-d_end,1)];
   else;
      data_dummy = [zeros(d_begin-1,1);ones(T-d_begin+1,1)];
   end;
   if d_acc==1;
      data_dummy = cumsum(data_dummy);
   end;
end;

if T>mm.T;
   last_trend = 0;
   if mm.deterselect(1,4)==1;
      last_trend = mm.Z1(mm.n+1,mm.T);
      if size(mm.Z1,1)==2*(mm.n+1);
         last_trend = last_trend+mm.Z1(2*(mm.n+1),mm.T);
      end;
   elseif mm.deterselect(1,3)==1;
      last_trend = mm.Z2(2+d_const,mm.T);
   end;
end;

for i=1:T;
   %
   % create new Z1 and Z2
   %
   if i==1;
      Z0_old = mm.Z1(1:mm.n,1);
      Z1(:,1) = mm.Z1(:,1);
      if isempty(mm.Z2)==0;
         Z2(:,1) = mm.Z2(:,1);
      end;
   else;
      Z0_old = Z0_new;
      %
      % create new Z1
      %
      Z1_new = Z0_old;
      if size(mm.Z1,1)>mm.n;
         if i<=mm.T;
            if (d_corel==0)|(mm.usealtmodel==0);
               Z1_new = [Z1_new;mm.Z1(mm.n+1:size(mm.Z1,1),i)];
            else;
               if d_coidet==0;
                  Z1_add = mm.Z1(mm.n+1:((size(mm.Z1,1))/2),i)+mm.Z1(mm.n+1+((size(mm.Z1,1))/2):size(mm.Z1,1),i);
               else;
                  Z1_add = mm.Z1(mm.n+1:((size(mm.Z1,1)+1)/2),i)+[mm.Z1(mm.n+1+((size(mm.Z1,1)+1)/2):size(mm.Z1,1),i);0];
               end;
               Z1_new = [Z1_new;Z1_add];
            end;
         else;
            %
            % Add a constant or a trend
            %
            if isalmostequal(Z1(mm.n+1,i-1),1)==1;
               Z1_new = [Z1_new;1];
            else;
               Z1_new = [Z1_new;i-mm.T+last_trend];
            end;
            %
            % check if we're looking at an alternative model
            %
            if (mm.usealtmodel==1)&(d_coint==1);
               Z1_new = [Z1_new;data_dummy(i,1)];
            end;
         end;
      end;
      if (d_corel==0)|(mm.usealtmodel==0);
         Z1(:,i) =Z1_new;
      else;
         if d_coidet==0;
            Z1(:,i) = [(1-data_dummy(i,1))*Z1_new;data_dummy(i,1)*Z1_new];
         else;
            Z1(:,i) = [(1-data_dummy(i,1))*Z1_new(1:size(Z1_new,1)-1,1);Z1_new(size(Z1_new,1),1);data_dummy(i,1)*Z1_new(1:size(Z1_new,1)-1,1)];
         end;
      end;
      if isempty(Z2)==0;
         %
         % Create new Z2
         %
         % Step 1: deterministic variables
         %
         Z2_new = [];
         if i<=mm.T
            if mm.d>0;
               Z2_new = mm.Z2(1:mm.d,i);
            end;
         else;
            Z2_new = [];
            pos = 0;
            %
            % check if we should add a dummy first for an alternative model
            %
            if d_const==1;
               Z2_new = data_dummy(i,1);
               pos = 1;
            end;
            %
            % add a constant and/or a trend
            %
            if isalmostequal(Z2(1+d_const,i-1),1)==1;
               Z2_new = [Z2_new;1];
               pos = pos+1;
            end;
            if size(mm.Z2,1)>1;
               if isalmostequal(Z2(2+d_const,i-1),i-1-mm.T+last_trend)==1;
                  Z2_new = [Z2_new;i-mm.T+last_trend];
                  pos = pos+1;
               end;
            end;
            %
            % deal with seasonals
            %
            if mm.d>2+d_const;
               if strcmp(mm.freq,'q')==1;
                  Z2_new = [Z2_new;Z2(pos+1:pos+3,i-4)];
               elseif strcmp(mm.freq,'m')==1;
                  Z2_new = [Z2_new;Z2(pos+1:pos+11,i-12)];
               end;
            end;
         end;
         %
         % Step 2: lagged endogenous variables
         %
         if d_shortrun==0;
            if mm.p>1;
               %
               % lag 1
               %
               Z2_new = [Z2_new;Z0(:,i-1)];
               %
               % the remaining
               %
               if mm.p>2;
                  Z2_new = [Z2_new;Z2(mm.d+1:mm.d+(mm.n*(mm.p-2)),i-1)];
               end;
            end;
            %
            % Step 3: exogenous stochastic variables
            %
            if mm.q1+mm.q0>0;
               Z2_new = [Z2_new;mm.Z2(mm.d+(mm.n*(mm.p-1))+1:size(Z2,1),i)];
            end;
         else;
            Z2_extra = [];
            %  lagged endogenous variables
            if mm.p>1;
               Z2_extra = Z0(:,i-1);
               if mm.p>2;
                  Z2_extra = [Z2_extra;Z2(mm.d+1:mm.d+(mm.n*(mm.p-2)),i-1)];
               end;
            end;
            % exogenous stochastic variables
            if mm.q1+mm.q0>0;
               if isfield(mm,'exo')==0;
                  Z2_extra = [Z2_extra;mm.Z2(mm.d+(mm.n*(mm.p-1))+1:size(Z2,1),i)];
               else;
                  Z2_extra = [Z2_extra;mm.exo(:,i)];
               end;
            end;
            Z2_new = [Z2_new;Z2_extra*(1-data_dummy(i,1));Z2_extra*data_dummy(i,1)];
         end;
         if size(Z2,1)==size(Z2_new,1);
            Z2(:,i) = Z2_new;
         else;
            Z2(:,i) = Z2(:,i-1);
         end;
      end;
   end;
   %
   % Create the levels of the endogenous variables
   %
   if isempty(beta)==0;
      Z0_new = mm.alpha*beta'*Z1(:,i)+Z0_old+epsilon(:,i);
   else;
      Z0_new = Z0_old+epsilon(:,i);
   end;
   if isempty(Z2)==0;
      Z0_new = Z0_new+mm.Psi*Z2(:,i);
   end;
   %
   % Collecting endogenous variables in first differences
   %
   Z0(:,i) = (Z0_new-Z0_old);
end;

boot.Z0 = Z0;
if (mm.usealtmodel==0)|(d_const==1)|(d_shortrun==1);
   boot.Z1 = Z1;
elseif d_coint==1;
   %
   % we have a regime-shift dummy in the last row of Z1
   %
   boot.Z1 = Z1(1:size(Z1,1)-1,:);
elseif d_corel==1;
   %
   % we have a regime-shift in the DGP
   %
   if d_coidet==0;
      num_z1 = size(Z1,1)/2;
      boot.Z1 = Z1(1:num_z1,:)+Z1(num_z1+1:2*num_z1,:);
   else;
      num_z1 = (size(Z1,1)+1)/2;
      boot.Z1 = Z1(1:num_z1,:)+[Z1(num_z1+1:size(Z1,1),:);zeros(1,size(Z1,2))];
   end;
end;
if d_const==0;
   if d_shortrun==0;
      boot.Z2 = Z2;
   else;
      sum_Z2 = [];
      s_d = (1/2)*(size(Z2,1)-mm.d);
      if mm.d>0;
         sum_Z2 = [eye(mm.d) zeros(mm.d,2*s_d);zeros(s_d,mm.d) eye(s_d) eye(s_d)];
      else;
         sum_Z2 = [eye(s_d) eye(s_d)];
      end;
      boot.Z2 = sum_Z2*Z2;
   end;
else;
   %
   % we have a regime-shift dummy in the first row of Z2
   %
   if size(Z2)>1;
      boot.Z2 = Z2(2:size(Z2,1),:);
   else;
      boot.Z2 = [];
   end;
end;

%
% end of bootstrap_sample.m
%
