%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for showing restricted C            %
%          standard errors, test results and require the         %
%          user to select the new C or not                       %
%                                                                %
%          Function: cdlg                                        %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       cm: structure with model info            %
%                       infomat: summary information about       %
%                           about the estimation                 %
%                       pval: pvalue of test (if any)            %
%                       LR: LR test of restrictions              %
%                       epsilon: nxT matrix with residuals       %
%                       Psi: nxnq matrix                         %
%                       alpha: nxr matrix                        %
%                       beta: nq1xr matrix                       %
%                       Sigma_theta: covariance matrix for Psi   %
%                           and alpha                            %
%                       C: nxn long-run impact matrix            %
%                       Sigma_C: covariance matrix of C          %
%                       num_c_rest: number of effective C        %
%                           restrictions                         %
%                       evalue: largest non-unit eigenvalue of   %
%                           the companion matrix                 %
%                                                                %
%          Output:                                               %
%                       h = handle                               %
%                                                                %
%          This version:        July 4, 2006                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = cdlg(selector,cm,infomat,pval,LR,epsilon,Psi,alpha,beta,Sigma_theta,C,Sigma_C,num_c_rest,evalue)

if strcmp(selector,'init')==1;
   tm = cm;
   %
   curr_ini = find_inisettings;
   if curr_ini.lagtest>0;
      lagenable = 'on';
   else;
      lagenable = 'off';
   end;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
   tm.C = C;
   tm.Sigma_C = Sigma_C;
   tm.evalue = evalue;
   tm.num_c_rest = num_c_rest;
   tm.beta = beta;
   if isempty(tm.beta)==0;
      if tm.restrict(1,2)==1;
         tm.newbZ1t = tm.beta'*tm.Z1t;
         tm.newbR1t = tm.beta'*tm.R1t;
      else;
         tm.newbZ1t = normbeta(tm.beta)'*tm.Z1t;
         tm.newbR1t = normbeta(tm.beta)'*tm.R1t;
      end;
   else;
      tm.newbZ1t = [];
      tm.newbR1t = [];
   end;
   tm.epsilon = epsilon;
   tm.Psi = Psi;
   tm.alpha = alpha;
   tm.Sigma_theta = Sigma_theta;
   tm.restrict(1,3) = 1;
   %
   std_C = vecinv(sqrt(abs(diag(Sigma_C))),tm.n);
   curralpha = vecstr(tm.evars,':      ');
   curralpha = strvcat('C Matrix:',' ',curralpha);
   for j=1:tm.n;
      addtoalpha = '';
      for i=1:tm.n;
         if C(i,j)<0;
            if abs(C(i,j))>1.0e-005;
               addtoalpha = strvcat(addtoalpha,[num2str(C(i,j),'%1.4f') '  (' num2str(std_C(i,j),'%1.4f') ')     ']);
            else;
               addtoalpha = strvcat(addtoalpha,' 0       (-)      ');
            end;
         else;
            if abs(C(i,j))>1.0e-005;
               addtoalpha = strvcat(addtoalpha,[' ' num2str(C(i,j),'%1.4f') '  (' num2str(std_C(i,j),'%1.4f') ')     ']);
            else;
               addtoalpha = strvcat(addtoalpha,' 0       (-)      ');
            end;
         end;
      end;
      sizeadd = size(addtoalpha,2)-16;
      if j>9;
         sizeadd = sizeadd-1;
      end;
      paddalpha = '';
      for i=1:sizeadd-1;
         paddalpha = [paddalpha ' '];
      end;
      curralpha = [curralpha strvcat(' ',[paddalpha 'eps_' num2str(j)],addtoalpha)];
   end;
   curralpha = strvcat(curralpha,' ');
   %
   % import images
   %
   button_images = read_images;
   %
   tm.myrun = 0;
   if tm.restrict(1,2)==1;
      currbeta = showbeta(tm.beta,tm.evars,strtrim([tm.xvars1 ' ' tm.restvec]));
   else;
      currbeta = showbeta(normbeta(tm.beta),tm.evars,strtrim([tm.xvars1 ' ' tm.restvec]));
   end;
   myuserestrict = 0;
   thisexit = 0;
   add_s = '';
   if tm.num_c_rest>1;
      add_s = 's';
   end;
   %
   tm.model_info.header = strvcat('Number of iterations for algorithm:','Value of convergence criterion (max|R*vec(C)-r|):','Largest non-unit eigenvalue of the companion matrix:');
   tm.model_info.info = strvcat(num2str(infomat(1,1),'%0.0f'),num2str(infomat(1,2)),num2str(tm.evalue,'%0.4f'));
   if curr_ini.sscorrect==0;
      tm.model_info.header = strvcat(tm.model_info.header,['LR-test of ' num2str(tm.num_c_rest) ' C restriction' add_s ':']);
   else;
      tm.model_info.header = strvcat(tm.model_info.header,['F-test of (' num2str(tm.num_c_rest) ',' num2str(cm.T-cm.df1) ') C restriction' add_s ':']);
   end;
   tm.model_info.info = strvcat(tm.model_info.info,num2str(LR(1,1),'%0.4f'));
   if isempty(pval)==0;
      tm.model_info.header = strvcat(tm.model_info.header,'p-value:');
      tm.model_info.info = strvcat(tm.model_info.info,num2str(pval(1,1),'%0.4f'));
   end;
   %
   if tm.r<tm.n+tm.q1;
      if tm.evalue>=1;
         mypermcompenable = 'off';
      else;
         mypermcompenable = 'on';
      end;
   else;
      mypermcompenable = 'off';
   end;
   %
   cmatenable = 'on';
   weakexoenable = 'on';
   if tm.r==tm.n;
      weakexoenable = 'off';
   end;
   if (tm.n==1)&(tm.q1+tm.q0==0);
      gcenable = 'off';
   else;
      gcenable = 'on';
   end;
   dfc = round((((tm.n+tm.q1+tm.dcoint-tm.r)*tm.r)-tm.dfbeta)/tm.n);
   tm.df = tm.n*(tm.p-1)+tm.r+tm.q1*tm.pe+tm.q0*(tm.pe+1)+tm.d+dfc-round((tm.dfalpha+tm.num_c_rest)/tm.n);
else;
   thisexit = 1;
%   cdlggui = get(0,'CurrentFigure');
   cdlggui = findobj('Type','figure','Tag','C_DLG')';
   crest = get(cdlggui,'UserData');
   curr_ini = get(crest.viewdata,'UserData');
   button_images = curr_ini.images;
   tm = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

scrsz = [1 1 1024 768];
realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      cdlggui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','cdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-scrsz(3)/2)/2-50 (realscrsz(4)-435)/2 scrsz(3)/2+100 435], ...
         'Tag','C_DLG', ...
         'Resize','off', ...
         'Name','Restrictions on C - Results', ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'CloseRequestFcn','cdlg wayout', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(cdlggui,'DockControl','off');
      end;

      %
      % File menu
      %

      filemenu = uimenu('Parent',cdlggui, ...
         'Label','&File', ...
         'Tag','FileMenu');

      %
      % Objects on File menu
      %

      crest.viewdata = uimenu('Parent',filemenu, ...
         'Label','View &Data...', ...
         'Callback','cdlg viewdata', ...
         'Enable','on', ...
         'Tag','ViewData');

      add_sep = 'on';

      if (tm.cointvalue==1)&(tm.r>0)&(tm.r~=tm.n+tm.q1+tm.dcoint);
         crest.savecoint = uimenu('Parent',filemenu, ...
            'Label','Save &Cointegration Relations...', ...
            'Callback','cdlg savecoint', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','SaveResid');

         add_sep = 'off';

      end;

      crest.savetc = uimenu('Parent',filemenu, ...
         'Label','Save &Transitory Components...', ...
         'Callback','cdlg savetc', ...
         'Accelerator','T', ...
         'Separator',add_sep, ...
         'Enable',mypermcompenable, ...
         'Tag','SaveTC');

      crest.saveresid = uimenu('Parent',filemenu, ...
         'Label','Save &Residuals...', ...
         'Callback','cdlg saveresid', ...
         'Accelerator','D', ...
         'Enable','on', ...
         'Tag','SaveCoint');

      uimenu('Parent',filemenu, ...
         'Label','Center &Window', ...
         'Separator','on', ...
         'Accelerator','W', ...
         'Enable','on', ...
         'Callback','center_figure_fcn', ...
         'Tag','CenterMenu');

      crest.menuuse = uimenu('Parent',filemenu, ...
         'Label','&Use New C', ...
         'Callback','cdlg use', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuFileUse');

      crest.menuskip = uimenu('Parent',filemenu, ...
         'Label','R&ecycle New C', ...
         'Callback','cdlg skip', ...
         'Enable','off', ...
         'Tag','MenuFileSkip');

      crest.menucontinue = uimenu('Parent',filemenu, ...
         'Label','C&ontinue', ...
         'Separator','on', ...
         'Callback','cdlg done', ...
         'Accelerator','C', ...
         'Enable','on', ...
         'Tag','MenuFileContinue');


      %
      % Tools menu
      %

      testmenu = uimenu('Parent',cdlggui, ...
         'Label','&Tools', ...
         'Tag','TestMenu');

      %
      % Objects on Tools menu
      %

      crest.corrtest = uimenu('Parent',testmenu, ...
         'Label','&Serial Correlation...', ...
         'Callback','cdlg corrtest', ...
         'Enable','on', ...
         'Accelerator','S', ...
         'Tag','CorrTest');

      crest.archtest = uimenu('Parent',testmenu, ...
         'Label','&ARCH...', ...
         'Callback','cdlg archtest', ...
         'Accelerator','A', ...
         'Enable','on', ...
         'Tag','ArchTest');

      crest.normalitytest = uimenu('Parent',testmenu, ...
         'Label','&Normality...', ...
         'Callback','cdlg normalitytest', ...
         'Accelerator','N', ...
         'Enable','on', ...
         'Tag','NormalityTest');

      crest.constancytest = uimenu('Parent',testmenu, ...
         'Label','&Parameter Constancy...', ...
         'Callback','cdlg constancytest', ...
         'Accelerator','P', ...
         'Enable','on', ...
         'Tag','ConstancyTest');

      crest.lagorder = uimenu('Parent',testmenu, ...
         'Label','&Lag Order...', ...
         'Callback','cdlg lagorder', ...
         'Separator','on', ...
         'Accelerator','L', ...
         'Enable',lagenable, ...
         'Tag','LagOrder');

      crest.weakexo = uimenu('Parent',testmenu, ...
         'Label','&Weak Exogeneity...', ...
         'Callback','cdlg weakexo', ...
         'Enable',weakexoenable, ...
         'Tag','WeakExogeneity');

      crest.gctest = uimenu('Parent',testmenu, ...
         'Label','&Granger Causality...', ...
         'Callback','cdlg gctest', ...
         'Accelerator','G', ...
         'Enable',gcenable, ...
         'Tag','GrangerCausality');

      if (tm.n>2)&((tm.p>1)|(tm.r>0));
         crest.multigctest = uimenu('Parent',testmenu, ...
            'Label','M&ulti-Step Granger Causality...', ...
            'Callback','cdlg multigctest', ...
            'Enable','on', ...
            'Tag','MultiGrangerCausality');
      end;

      if ((tm.p>1)&(tm.n>2))|(tm.q0>0)|((tm.q1>0)&(tm.pe>0));
         crest.exclude = uimenu('Parent',testmenu, ...
            'Label','&Exclusion', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','ExclusionMenu');
         
         crest.firstdiffs = uimenu('Parent',crest.exclude, ...
            'Label','&First Differences', ...
            'Callback','cdlg firstdiffs', ...
            'Enable','on', ...
            'Tag','FirstDiffMenu');
      end;

      if (tm.p>1)&(tm.r<tm.n);
         crest.common_cycles = uimenu('Parent',testmenu, ...
            'Label','Co&mmon Cycles...', ...
            'Callback','cdlg common_cycles', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','ShowCommonCycles');
      end;

      if tm.datad>0;
         crest.dummy = uimenu('Parent',testmenu, ...
            'Label','&Deterministic Variables', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','DummyVarMenu');

         crest.dummyexclusion = uimenu('Parent',crest.dummy, ...
            'Label','&Exclusion...', ...
            'Callback','cdlg dummyexclusion', ...
            'Enable','on', ...
            'Tag','DummyExclusionMenu');

         crest.dummylongrun = uimenu('Parent',crest.dummy, ...
            'Label','&Long Run...', ...
            'Callback','cdlg dummylongrun', ...
            'Enable','off', ...
            'Tag','DummyLongRunMenu');

         if (tm.r<tm.n)&(tm.r>0);
            set(crest.dummylongrun,'Enable','on');
         end;

         if tm.datad>1;
            crest.propdummy = uimenu('Parent',crest.dummy, ...
               'Label','&Proportionality...', ...
               'Callback','cdlg dummyprop', ...
               'Enable','on', ...
               'Tag','DummyPropMenu');
         end;

      end;

      crest.cmat = uimenu('Parent',testmenu, ...
         'Label','&C-Matrix...', ...
         'Callback','cdlg cmat', ...
         'Separator','on', ...
         'Accelerator','M', ...
         'Enable',cmatenable, ...
         'Tag','CMatrix');

      crest.longrun_gir = uimenu('Parent',testmenu, ...
         'Label','L&ong Run GIRs...', ...
         'Callback','cdlg longrun_gir', ...
         'Separator','off', ...
         'Enable',cmatenable, ...
         'Tag','LongRunGIRsMenu');

      crest.residanalysis = uimenu('Parent',testmenu, ...
         'Label','&Residual Analysis...', ...
         'Callback','cdlg residanalysis', ...
         'Enable','on', ...
         'Accelerator','R', ...
         'Tag','ResidAnalysis');

      crest.shortdyn = uimenu('Parent',testmenu, ...
         'Label','S&hort Run Dynamics...', ...
         'Callback','cdlg shortdyn', ...
         'Accelerator','Y', ...
         'Enable','on', ...
         'Tag','ShortDyn');

      crest.distribution = uimenu('Parent',testmenu, ...
         'Label','Distri&bution...', ...
         'Callback','cdlg distribution', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','DistributionMenu');

      %
      % Graphics menu
      %

      graphmenu = uimenu('Parent',cdlggui, ...
         'Label','&Graphics', ...
         'Tag','GraphicsMenu');

      if tm.r>0;
         crest.menugraph = uimenu('Parent',graphmenu, ...
            'Label','&Cointegration Relations...', ...
            'Callback','cdlg graphs', ...
            'Enable','on', ...
            'Tag','MenuGraph');
      end;

      crest.menupermcomp = uimenu('Parent',graphmenu, ...
         'Label','&Permanent and Transitory Components...', ...
         'Callback','cdlg permcomp', ...
         'Enable',mypermcompenable, ...
         'Accelerator','E', ...
         'Tag','MenuPermComp');

      crest.menuresiduals = uimenu('Parent',graphmenu, ...
         'Label','&Residuals...', ...
         'Callback','cdlg residuals', ...
         'Enable','on', ...
         'Accelerator','I', ...
         'Tag','MenuResiduals');

      crest.menufitted = uimenu('Parent',graphmenu, ...
         'Label','Fitted &Terms...', ...
         'Callback','cdlg fitted', ...
         'Enable','on', ...
         'Tag','MenuFitted');

      crest.forecast = uimenu('Parent',graphmenu, ...
         'Label','Forec&asting...', ...
         'Callback','cdlg forecast', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuForecast');

      crest.varforecast = uimenu('Parent',graphmenu, ...
         'Label','Forecast &Variable Transformations...', ...
         'Callback','cdlg varforecast', ...
         'Enable','on', ...
         'Tag','MenuVARForecast');

      if tm.q1+tm.q0>0;
         set(crest.forecast,'Enable','off');
         set(crest.varforecast,'Enable','off');
      end;

      crest.menuchow = uimenu('Parent',graphmenu, ...
         'Label','Recursive Ch&ow Tests', ...
         'Callback','cdlg chow', ...
         'Separator','on', ...
         'Accelerator','O', ...
         'Enable','on', ...
         'Tag','MenuChow');

      crest.menuploberger = uimenu('Parent',graphmenu, ...
         'Label','Recursive &Fluctuation Test', ...
         'Callback','cdlg ploberger', ...
         'Accelerator','F', ...
         'Enable','on', ...
         'Tag','MenuPloberger');

      crest.genimpulse = uimenu('Parent',graphmenu, ...
         'Label','&Generalized Impulse Responses...', ...
         'Callback','cdlg genimpulse', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuGeneralizedImpulse');


%      if (isempty(tm.isdummy)==0)&(max(tm.isdummy)==1);
      if tm.datad>0;
         crest.dummyimpulse = uimenu('Parent',graphmenu, ...
            'Label','&Dummy Impulse Responses...', ...
            'Callback','cdlg dummyimpulse', ...
            'Enable','on', ...
            'Tag','MenuDummyImpulse');
      end;


      crest.companion = uimenu('Parent',graphmenu, ...
         'Label','Eigenvalues of Companion &Matrix', ...
         'Callback','cdlg companion', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuCompanion');

      %
      % Help menu
      %

      helpmenu = uimenu('Parent',cdlggui, ...
         'Label','&Help', ...
         'Tag','HelpMenu');

      crest.c_help = uimenu('Parent',helpmenu, ...
            'Label','&Restrictions on C', ...
            'Callback','help_fcn(''RestrictionsonC'');', ...
            'Accelerator','H', ...
            'Separator','off', ...
            'Enable','on', ...
            'Tag','MenuCHelp');

      %
      % Horizontal lines for toolbar
      %

      xline([1 434 (scrsz(3)/2)+100],[0.5 1]);

      crest.toolbar = uitoolbar('Parent',cdlggui, ...
                        'HandleVisibility','off', ...
                        'Interruptible','on', ...
                        'Clipping','on');

      %
      %  Use button
      %

      crest.use = uipushtool('Parent',crest.toolbar, ...
                         'TooltipString','Use New C', ...
                         'CData',button_images.pickc, ...
                         'ClickedCallback','cdlg use', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','UseButton');

      crest.skip = uipushtool('Parent',crest.toolbar, ...
                         'TooltipString','Recycle New C', ...
                         'CData',button_images.skipc, ...
                         'ClickedCallback','cdlg skip', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','off', ...
                         'Tag','SkipButton');

      %
      %  Done button
      %

      crest.done = uipushtool('Parent',crest.toolbar, ...
                         'TooltipString','Continue', ...
                         'CData',button_images.continue, ...
                         'ClickedCallback','cdlg done', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData','waiting', ...
                         'Enable','on', ...
                         'Tag','DoneButton');

      %
      % Graphics button
      %
      separator_str = 'on';

      if tm.r>0;
         crest.graphs = uipushtool('Parent',crest.toolbar, ...
                            'TooltipString','Cointegration Relations', ...
                            'CData',button_images.graph, ...
                            'ClickedCallback','cdlg graphs', ...
                            'Separator','on', ...
                            'Interruptible','on', ...
                            'Clipping','on', ...
                            'UserData','working', ...
                            'Enable','on', ...
                            'Tag','Graphics');
         separator_str = 'off';
      end;

      crest.permcomp = uipushtool('Parent',crest.toolbar, ...
                         'TooltipString','Permanent and Transitory Components', ...
                         'CData',button_images.permcomp, ...
                         'ClickedCallback','cdlg permcomp', ...
                         'Separator',separator_str, ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable',mypermcompenable, ...
                         'Tag','PermCompGraphics');

      crest.residgraph = uipushtool('Parent',crest.toolbar, ...
                         'TooltipString','Residuals', ...
                         'CData',button_images.histogram, ...
                         'ClickedCallback','cdlg residuals', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','ResidGraphics');

      %
      % Forecasting button
      %

      crest.forecast_button = uipushtool('Parent',crest.toolbar, ...
                         'TooltipString','Forecasting', ...
                         'CData',button_images.forecast, ...
                         'ClickedCallback','cdlg forecast', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','ForecastGraph');

      if tm.q1+tm.q0>0;
         set(crest.forecast_button,'Enable','off');
      end;

      %
      % Chow test button
      %

      crest.chow = uipushtool('Parent',crest.toolbar, ...
                         'TooltipString','Recursive Chow Tests', ...
                         'CData',button_images.chow, ...
                         'ClickedCallback','cdlg chow', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','RecChowGraph');

      crest.ploberger = uipushtool('Parent',crest.toolbar, ...
                         'TooltipString','Recursive Fluctuation Test', ...
                         'CData',button_images.fluctest, ...
                         'ClickedCallback','cdlg ploberger', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','RecPlobGraph');

      %
      % Outer axis
      %

      axesbox([2 2 (scrsz(3)/2)+98 420],'Maximum Likelihood Results',135,[0.5 1]);

      %
      % Outer axis for showing beta
      %

      axesbox([14 143 scrsz(3)/2+72 262],'Current Beta'' and C Matrix',125,[0.5 1]);

      %
      % Outer axis for additional info
      %

      axesbox([14 14 scrsz(3)/2+72 115],'Estimation Information',105,[0.5 1]);

      %
      % Listbox for showing estimated beta
      %

      crest.betamat = uicontrol('Units','pixels', ...
         'BackgroundColor','white', ...
         'Position',[30 280 scrsz(3)/2+40 110], ...
         'Style','listbox', ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'TooltipString','Beta estimates', ...
         'Max',2, ...
         'Value',[], ...
         'String',currbeta, ...
         'Enable','inactive', ...
         'UserData',myuserestrict, ...
         'HorizontalAlignment','center', ...
         'Tag','AlphaListbox');

      %
      % Listbox for showing estimated conditional standard
      % errors for beta
      %

      crest.alphamat = uicontrol('Units','pixels', ...
         'BackgroundColor','white', ...
         'Position',[30 159 scrsz(3)/2+40 110], ...
         'Style','listbox', ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'TooltipString','Restricted C estimates with standard errors within parentheses', ...
         'Max',2, ...
         'Value',[], ...
         'String',curralpha, ...
         'Enable','inactive', ...
         'UserData','working', ...
         'HorizontalAlignment','center', ...
         'Tag','AlphaStdListbox');

      %
      % Show output text
      %

      crest.model_header = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 30 scrsz(3)/2+40 83], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'ForegroundColor',curr_ini.gui_foreground_color, ...
         'String',tm.model_info.header, ...
         'Tag','OutputText');

      head_ext = get(crest.model_header,'Extent');
      if head_ext(1,4)<=83;
         new_pos = [30 30+(83-head_ext(1,4))/2 head_ext(1,3:4)];
      else;
         new_pos = [30 30 head_ext(1,3) 83];
      end;
      set(crest.model_header,'Position',new_pos);

      crest.model_info = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[35+new_pos(1,3) new_pos(1,2) scrsz(3)/2+35-new_pos(1,3) new_pos(1,4)], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',tm.model_info.info, ...
         'ForegroundColor',curr_ini.gui_foreground_color, ...
         'HorizontalAlignment','left', ...
         'Tag','ModelInfoText');

      set(cdlggui,'UserData',crest,...
         'HandleVisibility','callback');

   case 'fitted'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         thisgraph = fittedterms('init',tm,[tm.Psi tm.alpha],tm.beta);
         fitted = get(thisgraph,'UserData');
         set(fitted.show,'UserData','done');
         killguipid(thisgraph);
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'companion'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         show_companion(tm.beta,tm.alpha,tm.Psi,tm,curr_ini,ind,0);
         tm.myrun = 0;
      end;
   case 'distribution'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         %
         thisgraph = distdlg('init',tm.cbs);
         killguipid(thisgraph);
         %
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'common_cycles'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         show_common_cycles(tm,curr_ini,tm.alpha,tm.Psi,tm.beta);
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'dummyprop'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         proportionality_dummies_dlg('init',tm,curr_ini);
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'dummylongrun'
      if tm.myrun==0;
         tm.myrun = 1;
         show_longrun_dummies(cdlggui,tm,curr_ini);
         tm.myrun = 0;
      end;
   case 'dummyexclusion'
      if tm.myrun==0;
         tm.myrun = 1;
         show_exclude_dummies(cdlggui,tm,curr_ini);
         tm.myrun = 0;
      end;
   case 'dummyimpulse'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         %
         Omega = (1/tm.T)*tm.epsilon*tm.epsilon';
         alpha = tm.alpha;
         beta = tm.beta;
         Psi = tm.Psi;
         Sigma_theta = tm.Sigma_theta;
         %
         txt = 'Would you like to impose restrictions on the dummy variables?';
         if (tm.datad>1)|((tm.r<tm.n)&(tm.r>0));
            answer = query(txt,'question',120,'Structural VAR - Dummy Restrictions',300,'no');
         else;
            answer = 'No';
         end;
         if strcmp(answer,'Yes')==1;
            %
            % open a select restrictions dialog
            %
            cm = restrict_dummies_dlg('init',tm,curr_ini);
            if strcmp(cm.action,'cancel')==0;
               %
               % check if we should use new estimated
               %
               if (isfield(cm,'applied')==1)&(isempty(cm.applied)==0)&(ischar(cm.applied)==1)&(strcmp(cm.applied,'applied')==1);
                  if isfield(cm,'dm')==1;
                     txt = 'Are you sure you want to use the restricted estimates for the dummy impulse responses?';
                     answer = query(txt,'question',120,'Structural VAR - Dummy Restrictions',300,'yes');
                     if strcmp(answer,'Yes')==1;
                        alpha = cm.dm.alpha;
                        beta = cm.dm.beta;
                        Psi = cm.dm.Psi;
                        Omega = cm.dm.Omega;
                        Sigma_theta = cm.dm.Sigma_theta;
                     end;
                  end;
               end;
            else;
               %
               % no dummy impulses
               %
               set(cdlggui,'Visible','on');
               drawnow;
               return;
            end;
         end;
         %
         setlogtext(strvcat(['Computing impulse responses for dummies up to period ' num2str(tm.h)],['with asymptotic ' num2str(tm.c) ' % confidence bands...']));
         abort = '';
         %
         [tm.duir,tm.duir_std,tm.buir,tm.buir_std,abort] = dummy_impulses_fcn(tm,alpha,beta,Psi,Omega,Sigma_theta,1);
         %
         if strcmp(abort,'cancel')==0;
            %
            % get the value from the normal distribution corresponding to the
            % chosen levels of significance c
            %
            c_val = norm_inv((100+tm.c)/200);
            for i=1:size(tm.duir,2);
               for j=1:tm.n*tm.datad;
                  if abs(tm.duir_std(j,i))<1.0e-008;
                     tm.duir_std(j,i) = 0;
                  end;
               end;
            end;
            tm.duir_upper = tm.duir+(c_val*tm.duir_std);
            tm.duir_lower = tm.duir-(c_val*tm.duir_std);
            %
            if isempty(tm.buir)==0;
               for i=1:size(tm.buir,2);
                  for j=1:tm.r*tm.datad;
                     if abs(tm.buir_std(j,i))<1.0e-008;
                        tm.buir_std(j,i) = 0;
                     end;
                  end;
               end;
               tm.buir_upper = tm.buir+(c_val*tm.buir_std);
               tm.buir_lower = tm.buir-(c_val*tm.buir_std);
            else;
               tm.buir_upper = [];
               tm.buir_lower = [];
            end;
            %
            thisgraph = duir_graphdlg('init',tm);
            ir_graph = get(thisgraph,'UserData');
            set(ir_graph.show,'UserData','done');
            killguipid(thisgraph);
         end;
         %
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'genimpulse'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         setlogtext(strvcat(['Computing generalized impulse responses up to period ' num2str(tm.h)],['with asymptotic ' num2str(tm.c) ' % confidence bands...']));
         abort = '';
         %
         Omega = (1/tm.T)*tm.epsilon*tm.epsilon';
         [tm.gir,tm.gir_std,tm.bir,tm.bir_std,abort] = generalized_impulses_fcn(tm,tm.alpha,tm.beta,tm.Psi,Omega,tm.Sigma_theta,1);
         if strcmp(abort,'cancel')==0;
            %
            % get the value from the normal distribution corresponding to the
            % chosen levels of significance c
            %
            c_val = norm_inv((100+tm.c)/200);
            for i=1:size(tm.gir,2);
               for j=1:tm.n*tm.n;
                  if abs(tm.gir_std(j,i))<1.0e-008;
                     tm.gir_std(j,i) = 0;
                  end;
               end;
            end;
            tm.gir_upper = tm.gir+(c_val*tm.gir_std);
            tm.gir_lower = tm.gir-(c_val*tm.gir_std);
            if isempty(tm.bir)==0;
               for i=1:size(tm.bir,2);
                  for j=1:tm.r*tm.n;
                     if abs(tm.bir_std(j,i))<1.0e-008;
                        tm.bir_std(j,i) = 0;
                     end;
                  end;
               end;
               tm.bir_upper = tm.bir+(c_val*tm.bir_std);
               tm.bir_lower = tm.bir-(c_val*tm.bir_std);
            else;
               tm.bir_upper = [];
               tm.bir_lower = [];
            end;
            thisgraph = gir_graphdlg('init',tm);
            ir_graph = get(thisgraph,'UserData');
            set(ir_graph.show,'UserData','done');
            killguipid(thisgraph);
         end;
         %
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'residanalysis'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         show_residual_analysis(cdlggui,tm,ind);
         tm.myrun = 0;
      end;
   case 'corrtest'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         show_corrtest(cdlggui,tm,curr_ini,ind);
         tm.myrun = 0;
      end;
   case 'archtest'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         show_archtest(cdlggui,tm,curr_ini,ind);
         tm.myrun = 0;
      end;
   case 'normalitytest'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         show_normalitytest(cdlggui,tm,curr_ini,ind);
         tm.myrun = 0;
      end;
   case 'constancytest'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         rind = 0;
         if (tm.r==0)|(tm.dfbeta==0);
            ind = 0;
            rind = 1;
         end;
         aind = 0;
         if tm.restrict(1,1)==1;
            aind = 1;
         end;
         show_constancytest(cdlggui,tm.beta,tm,curr_ini,ind,aind,rind);
         tm.myrun = 0;
      end;
   case 'weakexo'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.restrict(1,2)==0;
            ind = 0;
         end;
         aind = 0;
         if tm.restrict(1,1)==1;
            aind = 1;
         end;
         show_weakexo(cdlggui,tm.beta,tm.alpha,tm.Sigma_theta,tm,curr_ini,ind,aind);
         tm.myrun = 0;
      end;
   case 'longrun_gir'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         %
         Omega = (1/tm.T)*tm.epsilon*tm.epsilon';
         [lr_gir,lr_gir_std] = longrun_gir_fcn(tm.alpha,tm.beta,tm.Psi,Omega,tm.Sigma_theta,tm.epsilon,tm.Z1,tm.Z2,tm.p,tm.d);
         %
         for i=1:tm.n;
            for j=1:tm.n;
               if abs(lr_gir_std(i,j))<1.0e-005;
                  lr_gir_std(i,j) = 0;
                  lr_gir(i,j) = mystr2num(strtrim(num2str(lr_gir(i,j),['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'])));
               end;
            end;
         end;
         %
         addrest = ' restricted';
         if tm.dfbeta==0;
            addrest = '';
         end;
         ctext = strvcat(' ',['LONG RUN GENERALIZED IMPULSE RESPONSES (for' addrest ' rank=' num2str(tm.r,'%1.0f') ')']);
         numchar = size(ctext,2);
         addcorr = '';
         for i=1:numchar;
            addcorr = strcat(addcorr,'=');
         end;
         ctext = strvcat(ctext,addcorr,' ');
         ctext = strvcat(ctext,showc(lr_gir,tm.myvars),' ');
         ctext = strvcat(ctext,'Estimated asymptotic standard errors:',' ',showc(lr_gir_std,tm.myvars),' ','t-ratios:',' ',showc(inf2nan(lr_gir./lr_gir_std),tm.myvars),' ');
         drawnow;
         textgui(ctext,'Long Run GIRs',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         %
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'cmat'
      if tm.myrun==0;
         tm.myrun = 1;
         addrest = ' restricted';
         if tm.dfbeta==0;
            addrest = '';
         end;
         std_C = vecinv(sqrt(abs(diag(tm.Sigma_C))),tm.n);
         ctext = strvcat(' ',['THE C-MATRIX (for' addrest ' rank=' num2str(tm.r,'%1.0f') ')']);
         numchar = size(ctext,2);
         addcorr = '';
         for i=1:numchar;
            addcorr = strcat(addcorr,'=');
         end;
         C_t = tm.C;
         for i=1:tm.n;
            for j=1:tm.n;
               if abs(std_C(i,j))<1.0e-005;
                  std_C(i,j) = 0;
                  C_t(i,j) = mystr2num(strtrim(num2str(C_t(i,j),['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'])));
               end;
            end;
         end;
         %
         ctext = strvcat(ctext,addcorr,' ');
         ctext = strvcat(ctext,showc(C_t,tm.myvars),' ');
         ctext = strvcat(ctext,'Estimated asymptotic standard errors:',' ',showc(std_C,tm.myvars),' ','t-ratios:',' ',showc(inf2nan(C_t./std_C),tm.myvars),' ');
         set(cdlggui,'Visible','off');
         drawnow;
         textgui(ctext,'C-Matrix',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'lagorder'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         aind = 0;
         if tm.restrict(1,1)==1;
            aind = 1;
         end;
         show_lagordertest(cdlggui,tm.beta,tm.alpha,tm.Psi,tm.Sigma_theta,tm,curr_ini,ind,aind);
         tm.myrun = 0;
      end;
   case 'firstdiffs'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         show_firstdiffstests(cdlggui,tm.beta,tm.alpha,tm.Psi,tm.Sigma_theta,tm,curr_ini,ind,1);
         tm.myrun = 0;
      end;
   case 'gctest'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         aind = 0;
         if tm.restrict(1,1)==1;
            aind = 1;
         end;
         show_gctest(cdlggui,tm.beta,tm.alpha,tm.Psi,tm.Sigma_theta,tm,curr_ini,ind,aind);
         tm.myrun = 0;
      end;
   case 'multigctest'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         ind = 1;
         if tm.dfbeta==0;
            ind = 0;
         end;
         this_graph = select_mgc('init',tm,tm.beta,tm.Sigma_theta,tm.restrict(1,1),ind,1);
         mgc = get(this_graph,'UserData');
         set(mgc.show,'UserData','done');
         killguipid(this_graph);
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'shortdyn'
      if tm.myrun==0;
         tm.myrun = 1;
         Omega = (1/tm.T)*tm.epsilon*tm.epsilon';
         shortstr = shortrun(tm.alpha,tm.Psi,Omega,tm.beta,tm.Z1,tm.Z2,tm.p,tm.pe,tm.d,tm.deterselect,tm.q0,tm.q1,tm.evars,tm.xvars0,tm.xvars1,tm.deter,tm.restvec,1,tm.outputfile,1,tm.Sigma_theta);
         if tm.dfbeta>0;
            myshortstr = shortstr(2,:);
            oldsize = size(myshortstr,2);
            myshortstr = strrep(myshortstr,'for rank=','for restricted rank=');
            newsize = size(myshortstr,2);
            myshortstr = myshortstr(1,1:newsize-11);
            kind = findstr(myshortstr,')');
            if isempty(kind)==0;
               shortstr(2,:) = myshortstr;
               shortstr(3,:) = strrep(shortstr(3,:),'=           ','============');
            end;
         end;
         set(cdlggui,'Visible','off');
         drawnow;
         textgui(shortstr,'Short Run Dynamics',[(realscrsz(3)-800)/2 (realscrsz(4)-500)/2 800 500],'Results',38);
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'graphs'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 0;
         if tm.dfbeta>0;
            ind = 1;
         end;
         set(cdlggui,'Visible','off');
         drawnow;
         thisgraph = cointgraphdlg('init',tm,tm.newbZ1t,tm.newbR1t,ind);
         graph = get(thisgraph,'UserData');
         set(graph.show,'UserData','done');
         killguipid(thisgraph);
         set(cdlggui,'Visible','on');
         drawnow;
         set(cdlggui,'UserData',crest);
         tm.myrun = 0;
      end;
   case 'ploberger'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 0;
         if tm.dfbeta>0;
            ind = 1;
         end;
         show_ploberger(cdlggui,tm.beta,tm,curr_ini,ind,1);
         tm.myrun = 0;
      end;
   case 'chow'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 0;
         if tm.dfbeta>0;
            ind = 1;
         end;
         aind = 0;
         if tm.dfalpha>0;
            aind = 1;
         end;
         show_chow(cdlggui,tm.beta,tm,curr_ini,ind,aind);
         tm.myrun = 0;
      end;
   case 'permcomp'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 0;
         if tm.dfbeta>0;
            ind = 1;
         end;
         [y_t,perm_y,trans_y] = permcomp(tm.Psi,tm.alpha,tm.beta,tm.epsilon,tm.x0,tm.x1,tm.x1_r,tm.x2,tm.d,tm.dcoint,tm.p,tm.pe,tm.q1,tm.q0);
         set(cdlggui,'Visible','off');
         drawnow;
         thisgraph = pctcdlg('init',tm,y_t,perm_y,trans_y,ind);
         pctc = get(thisgraph,'UserData');
         set(pctc.show,'UserData','done');
         killguipid(thisgraph);
         set(cdlggui,'Visible','on');
         drawnow;
         set(cdlggui,'UserData',crest);
         tm.myrun = 0;
      end;
   case 'varforecast'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         old_tm = tm;
         tm.beta = tm.beta;
         tm.alpha = tm.alpha;
         tm.Psi = tm.Psi;
         tm.Omega = (1/tm.T)*tm.epsilon*tm.epsilon';
         %
         thisgraph = var_forecastdlg('init',tm,tm.restrict);
         f_cast = get(thisgraph,'UserData');
         set(f_cast.show,'UserData','done');
         killguipid(thisgraph);
         tm = old_tm;
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'forecast'
      if tm.myrun==0;
         tm.myrun = 1;
         set(cdlggui,'Visible','off');
         drawnow;
         old_tm = tm;
         tm.beta = tm.beta;
         tm.alpha = tm.alpha;
         tm.Psi = tm.Psi;
         tm.Omega = (1/tm.T)*tm.epsilon*tm.epsilon';
         %
         thisgraph = forecastdlg('init',tm,tm.restrict);
         f_cast = get(thisgraph,'UserData');
         set(f_cast.show,'UserData','done');
         killguipid(thisgraph);
         tm = old_tm;
         set(cdlggui,'Visible','on');
         drawnow;
         tm.myrun = 0;
      end;
   case 'residuals'
      if tm.myrun==0;
         tm.myrun = 1;
         ind = 0;
         if tm.dfbeta>0;
            ind = 1;
         end;
         set(cdlggui,'Visible','off');
         drawnow;
         thisgraph = epsilondlg('init',tm,ind);
         epsi = get(thisgraph,'UserData');
         set(epsi.show,'UserData','done');
         killguipid(thisgraph);
         set(cdlggui,'Visible','on');
         drawnow;
         set(cdlggui,'UserData',crest);
         tm.myrun = 0;
      end;
   case 'viewdata'
      if tm.myrun==0;
         tm.myrun = 1;
         if matlabnumber>=6.0;
            myfile = {'*.txt;*.data;*.wk1;*.xls;*.mat;*.csv', 'Data Files (*.txt, *.data, *.wk1, *.xls, *.mat, *.csv)'; ...
                    '*.txt;*.data', 'Text Files (*.txt, *.data)'; ...
                    '*.mat', 'MatLab MAT Files (*.mat)'; ...
                    '*.wk1', 'Lotus 1-2-3 Spreadsheets (*.wk1)'; ...
                    '*.xls', 'Excel Spreadsheets (*.xls)'; ...
                    '*.csv', 'Comma Separated Value Files (*.csv)'; ...
                    '*.*', 'All Files (*.*)'};
         else;
            if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1);
               fix_fig = findobj('Tag','StructuralVAR');
               model = get(fix_fig,'UserData');
               afile = get(model.endogenous,'String');
               basedir = getpath(strtrim(afile));
               file_fltr = 'Data Files (*.txt, *.data, *.wk1, *.mat, *.csv)|*.txt;*.data;*.wk1;*.mat;*.csv|Text Files (*.txt, *.data)|*.txt;*.data|MatLab MAT Files (*.mat)|*.mat|Lotus 1-2-3 Spreadsheets (*.wk1)|*.wk1|Comma Separated Value Files (*.csv)|*.csv|All Files (*.*)|*.*|';
               best_pos = 1;
               title_txt = 'Open';
            else;
               fix_fig = findobj('Tag','StructuralVAR');
               model = get(fix_fig,'UserData');
               afile = get(model.endogenous,'String');
               afile = strtrim(afile);
               if fileexist(afile)==1;
                  adir = getpath(afile);
                  if strcmp(computer,'PCWIN')==1;
                     if strcmp(adir(1,size(adir,2)),'\')==0;
                        adir = [adir '\'];
                     end;
                  else;
                     if strcmp(adir(1,size(adir,2)),'/')==0;
                        adir = [adir '/'];
                     end;
                  end;
               else;
                  adir = tm.outputdir;
               end;
               myfile = [adir '*.txt;*.data;*.wk1;*.mat;*.csv'];
            end;
         end;
         if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1)&(matlabnumber<6.0);
            [openfile,openpath] = fileopen(basedir,file_fltr,best_pos,title_txt);
         else;
            if matlabnumber>=6;
               [openfile,openpath] = uigetfile(myfile,'Open',getpath(tm.endofile));
            else;
               [openfile,openpath] = uigetfile(myfile,'Open');
            end;
         end;
         drawnow;
         if isnumeric(openfile)==0;
            openfile = strcat(openpath,openfile);
            set(cdlggui,'Visible','off');
            drawnow;
            textgui(openfile,['View Data - ' openfile],[(realscrsz(3)-(realscrsz(4)-80))/2 1 realscrsz(4)-80 realscrsz(4)-54],'Data',25);
            set(cdlggui,'Visible','on');
            drawnow;
         end;
         tm.myrun = 0;
      end;
   case 'savecoint'
      if tm.myrun==0;
         tm.myrun = 1;
         addrest = 'rest-';
         if tm.dfbeta==0;
            addrest = '';
         end;
         myfile = [tm.outputdir 'coint-' addrest num2str(tm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myR myT] = size(tm.newbZ1t);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(tm.samplebeg,tm.sampleend,tm.freq,i/myT));
            end;
            setlogtext(['Saving cointegration relations to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myR;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '               ci_' num2str(i,'%1.0f')];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),tm.newbZ1t(:,i)');
            end;
            fclose(fid);
         end;
         tm.myrun = 0;
      end;
   case 'savetc'
      if tm.myrun==0;
         tm.myrun = 1;
         addrest = 'rest-';
         if tm.dfbeta==0;
            addrest = '';
         end;
         [y_t,perm_y,trans_y] = permcomp(tm.Psi,tm.alpha,tm.beta,tm.epsilon,tm.x0,tm.x1,tm.x1_r,tm.x2,tm.d,tm.dcoint,tm.p,tm.pe,tm.q1,tm.q0);
         myfile = [tm.outputdir 'trans.comp-' addrest num2str(tm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myN myT] = size(trans_y);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(tm.samplebeg,tm.sampleend,tm.freq,i/myT));
            end;
            setlogtext(['Saving transitory components to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myN;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '              tc_' tm.myvars(i,:)];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),trans_y(:,i)');
            end;
            fclose(fid);
         end;
         tm.myrun = 0;
      end;
   case 'saveresid'
      if tm.myrun==0;
         tm.myrun = 1;
         addrest = 'rest-';
         if tm.dfbeta==0;
            addrest = '';
         end;
         myfile = [tm.outputdir 'eps-' addrest num2str(tm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myN myT] = size(tm.epsilon);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(tm.samplebeg,tm.sampleend,tm.freq,i/myT));
            end;
            setlogtext(['Saving residuals to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myN;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '             eps_' tm.myvars(i,:)];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),tm.epsilon(:,i)');
            end;
            fclose(fid);
         end;
         tm.myrun = 0;
      end;
   case 'use'
      if tm.myrun==0;
         tm.myrun = 1;
         set(crest.use,'Enable','off');
         set(crest.menuuse,'Enable','off');
         set(crest.skip,'Enable','on');
         set(crest.menuskip,'Enable','on');
         set(crest.betamat,'UserData',1);
         set(cdlggui,'UserData',crest);
         tm.myrun = 0;
      end;
   case 'skip'
      if tm.myrun==0;
         tm.myrun = 1;
         set(crest.use,'Enable','on');
         set(crest.menuuse,'Enable','on');
         set(crest.skip,'Enable','off');
         set(crest.menuskip,'Enable','off');
         set(crest.betamat,'UserData',0);
         set(cdlggui,'UserData',crest);
         tm.myrun = 0;
      end;
   case 'done'
      if tm.myrun==0;
         tm.myrun = 1;
         set(crest.done,'UserData','done');
         set(cdlggui,'UserData',crest);
         tm.myrun = 0;
      end;
   case 'wayout'
      set(crest.done,'UserData','done');
      if strcmp(get(crest.alphamat,'UserData'),'done')==1;
         delete(cdlggui);
      end;
end;

if strcmp(selector,'init')==1;
   set(cdlggui,'Visible','on');
end;

if strcmp(selector,'wayout')==0;
   curr_ini.images = button_images;
   curr_ini.models = tm;
   set(crest.viewdata,'UserData',curr_ini);
end;

if nargout==1;
   h = cdlggui;
end;

if thisexit==0;
   waitfor(crest.done,'UserData','done');
end;

%
% end of cdlg.m
%
