function [tests,deg_free,pvals,params] = coint_stationarity(Z0,Z1,Z2,epsilon,r,q1,dcoint,df,sscorrect)

%
% compute log-likelihood value under the alternative
%
lnL_alt = loglike(epsilon);
%
% determine sample parameters
%
[n T] = size(Z0);
%
% initialize output
%
tests = zeros(n,1);
deg_free = zeros(n,1+sscorrect);
pvals = zeros(n,1);
%
% Compute product moment matrices M_ij
%

M_00 = (1/T)*Z0*Z0';
M_01 = (1/T)*Z0*Z1';
if isempty(Z2)==0;
   M_02 = (1/T)*Z0*Z2';
   M_12 = (1/T)*Z1*Z2';
   M_22 = (1/T)*Z2*Z2';
else;
   M_02 = [];
   M_12 = [];
   M_22 = [];
end;
M_10 = M_01';
M_20 = M_02';
M_11 = (1/T)*Z1*Z1';
M_21 = M_12';
%
% Compute product moment matrice S_ij
%
if isempty(Z2)==0;
   S_00 = M_00-(M_02/M_22)*M_20;
   S_11 = M_11-(M_12/M_22)*M_21;
   S_01 = M_01-(M_02/M_22)*M_21;
   S_10 = M_10-(M_12/M_22)*M_20;
else;
   S_00 = M_00;
   S_11 = M_11;
   S_01 = M_01;
   S_10 = M_10;
end;
%
% setup the base H matrix
%
H_base = eye(n);
if q1+dcoint>0;
   add_base = zeros(q1+dcoint,1);
else;
   add_base = [];
end;
%
% compute the tests
%
for i=1:n;
   h = [H_base(:,i);add_base];
   h_orth = null(h','r');
   %
   % compute S(ij.h)
   %
   S_00_h = S_00-(S_01*h*inverse(h'*S_11*h)*h'*S_10);
   S_01_h = S_01-(S_01*h*inverse(h'*S_11*h)*h'*S_11);
   S_11_h = S_11-(S_11*h*inverse(h'*S_11*h)*h'*S_11);
   S_10_h = S_10-(S_11*h*inverse(h'*S_11*h)*h'*S_10);
   %
   % Step 1: h_orth'*S_11_h*h_orth
   %
   HS_11 = h_orth'*S_11_h*h_orth;
   HS_symm = symm(HS_11);
   HS_adj = inverse(HS_symm)*h_orth'*S_10_h*inverse(S_00_h)*S_01_h*h_orth*inverse(HS_symm);
   %
   % Step 2: solve the eigenvalue problem
   %
   [U,lambda] = eig(HS_adj);
   %
   lambda = diag(lambda,0);
   [lambda2,AscInd] = sort(abs(lambda));
   %
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   for j=1:nr;
      DescInd(j)=AscInd(nr+1-j);
   end;
   %
   % Resort eigenvalues and eigenvectors
   % in descending order
   %
   lambda = abs(lambda(DescInd));
   U = U(:,DescInd);
   %
   % Renormalize U such that V'*h_orth'*S_11_h*h_orth*V=I
   %
   V = inverse(HS_symm)*U;
   %
   psi = V(:,1:r-1);
   beta = [h h_orth*psi];
   %
   % now we compute the remaining parameters
   %
   alpha = (S_01*beta)*inverse(beta'*S_11*beta);
   if isempty(Z2)==0;
      Psi = (M_02*inverse(M_22))-(alpha*beta'*M_12*inverse(M_22));
      epsilon_n = Z0-(alpha*beta'*Z1)-(Psi*Z2);
   else;
      epsilon_n = Z0-(alpha*beta'*Z1);
   end;
   lnL_new = loglike(epsilon_n);
   LR = 2*(lnL_alt-lnL_new);
   %
   if nargout>3;
      params.variable(i).beta = beta;
      params.variable(i).alpha = alpha;
      params.variable(i).Psi = Psi;
      params.variable(i).epsilon = epsilon_n;
      params.variable(i).Omega = (1/T)*epsilon_n*epsilon_n';
   end;
   %
   %
   deg_free(i,1) = n+q1+dcoint-r;
   if sscorrect==0;
      pvals(i,1) = 1-chis_cdf(LR,deg_free(i,1));
   else;
      dfc = round(((n+q1+dcoint-r)*r)/n);
      df_adj = round(((n+q1+dcoint-r-1)*(r-1))/n);
      df_adj = df-dfc+df_adj;
      deg_free(i,2) = T-df_adj-1;
      LR = ((T-df_adj-1)/((T-1)*deg_free(i,1)))*LR;
      pvals(i,1) = 1-fdis_cdf(LR,deg_free(i,1),deg_free(i,2));
   end;
   tests(i,1) = LR;
end;

%
% end of coint_stationarity.m
%
