function [tests,deg_free,pvals,params] = coint_stationarity_dcoint(Z0,Z1,Z2,myvars,epsilon,beta,r,q1,dcoint,df,sscorrect,convcrit,betagrad,maxit,showwait)

%
% determine the convergence criterion
%
if convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(convcrit)];
end;
ccrit = mystr2num(ch_convcrit);
%
% compute log-likelihood value under the alternative
%
lnL_alt = loglike(epsilon);
%
% determine sample parameters
%
[n T] = size(Z0);
%
% initialize output
%
tests = zeros(n,1);
deg_free = zeros(n,1+sscorrect);
pvals = zeros(n,1);
%
% Compute product moment matrices M_ij
%
M_00 = (1/T)*Z0*Z0';
M_01 = (1/T)*Z0*Z1';
if isempty(Z2)==0;
   M_02 = (1/T)*Z0*Z2';
   M_12 = (1/T)*Z1*Z2';
   M_22 = (1/T)*Z2*Z2';
else;
   M_02 = [];
   M_12 = [];
   M_22 = [];
end;
M_10 = M_01';
M_20 = M_02';
M_11 = (1/T)*Z1*Z1';
M_21 = M_12';
%
% Compute product moment matrice S_ij
%
if isempty(Z2)==0;
   S_00 = M_00-(M_02/M_22)*M_20;
   S_11 = M_11-(M_12/M_22)*M_21;
   S_01 = M_01-(M_02/M_22)*M_21;
   S_10 = M_10-(M_12/M_22)*M_20;
else;
   S_00 = M_00;
   S_11 = M_11;
   S_01 = M_01;
   S_10 = M_10;
end;
%
% setup the base H matrix
%
H_base = eye(n);
if q1+dcoint>0;
   add_base = zeros(q1+dcoint,1);
else;
   add_base = [];
end;
%
% compute the tests
%
for i=1:n;
   if showwait==1;
      txt = strvcat(' ',['Computing stationarity tests for: ' strtrim(myvars(i,:))],'Please hold!');
      if i==1;
         mywait = waitdlg(txt,'information');
         waitf = get(mywait,'UserData');
      else;
         set(waitf.text,'String',txt);
      end;
   end;
   h = [H_base(:,i);add_base];
   h_orth = null(h','r');
   h = [h [zeros(size(h,1)-1,1);1]];
   %
   H = h;
   if r>1;
      H_2 = kron(eye(r-1),h_orth);
      H = [[H zeros(size(H,1),size(H_2,2))];[zeros(size(H_2,1),size(H,2)) H_2]];
   end;
   %
   % initialize beta_1 by solving the eigenvalue problem:
   % for lambda*beta'*beta - beta'*h*inverse(h'*h)*h'*beta
   %
   S_bb = beta'*beta;
   S_bb = symm(S_bb);
   newbeta = zeros(size(beta));
   S_mat = inverse(S_bb)*beta'*h*inverse(h'*h)*h'*beta*inverse(S_bb);
   %
   [U,lambda] = eig(S_mat);
   %
   lambda = diag(lambda,0);
   [lambda2,AscInd] = sort(abs(lambda));
   %
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   for j1=1:nr;
      DescInd(j1)=AscInd(nr+1-j1);
   end;
   %
   % Resort eigenvalues and eigenvectors
   % in descending order
   %
   lambda = abs(lambda(DescInd));
   U = U(:,DescInd);
   %
   % Renormalize U such that V'*h_orth'*S_11_h*h_orth*V=I
   %
   V = inverse(S_bb)*U;
   beta_1 = beta*V(:,1);
   %
   % initialize beta_2
   %
   if r>1;
      S_mat = inverse(S_bb)*beta'*h_orth*inverse(h_orth'*h_orth)*h_orth'*beta*inverse(S_bb);
      %
      [U,lambda] = eig(S_mat);
      %
      lambda = diag(lambda,0);
      [lambda2,AscInd] = sort(abs(lambda));
      %
      nr = size(AscInd,1);
      DescInd = zeros(nr,1);
      for j1=1:nr;
         DescInd(j1)=AscInd(nr+1-j1);
      end;
      %
      % Resort eigenvalues and eigenvectors
      % in descending order
      %
      lambda = abs(lambda(DescInd));
      U = U(:,DescInd);
      %
      % Renormalize U such that V'*h_orth'*S_11_h*h_orth*V=I
      %
      V = inverse(S_bb)*U;
      beta_2 = beta*V(:,1:r-1);
   else;
      beta_2 = [];
   end;
   newbeta = [beta_1 beta_2];
   %
   % we're now ready to begin the iterations
   %
   it = 1;
   grad = 1;
   lnL_old = lnL_alt;
   if betagrad==1;
      %
      % compute a reference value for the gradient
      %
      alpha = (S_01*newbeta)*inverse(newbeta'*S_11*newbeta);
      if isempty(Z2)==0;
         Psi = (M_02*inverse(M_22))-(alpha*newbeta'*M_12*inverse(M_22));
         epsilon_n = Z0-(alpha*newbeta'*Z1)-(Psi*Z2);
      else;
         epsilon_n = Z0-(alpha*newbeta'*Z1);
      end;
      Omega = (1/T)*epsilon_n*epsilon_n';
      ref_grad = max(abs(T*vec(alpha)'*kron(eye(r),inverse(Omega)*(S_01-(alpha*newbeta'*S_11)))*H));
   end;
   while ((it<=maxit)&(grad>ccrit));
      %
      % estimate restricted cointegration vector conditional on the others
      % Step1: beta_1
      %
      if r>1;
         S_00_beta2 = S_00-(S_01*beta_2*inverse(beta_2'*S_11*beta_2)*beta_2'*S_10);
         S_01_beta2 = S_01-(S_01*beta_2*inverse(beta_2'*S_11*beta_2)*beta_2'*S_11);
         S_11_beta2 = S_11-(S_11*beta_2*inverse(beta_2'*S_11*beta_2)*beta_2'*S_11);
         S_10_beta2 = S_10-(S_11*beta_2*inverse(beta_2'*S_11*beta_2)*beta_2'*S_10);
         %
      else;
         S_00_beta2 = S_00;
         S_01_beta2 = S_01;
         S_11_beta2 = S_11;
         S_10_beta2 = S_10;
      end;
      %
      S_hh_1 = h'*S_11_beta2*h;
      HS_1 = symm(S_hh_1);
      %
      % solve the eigenvalue problem
      %
      HS_adj = inverse(HS_1)*h'*S_10_beta2*inverse(S_00_beta2)*S_01_beta2*h*inverse(HS_1);
      %
      % Step 2: solve the eigenvalue problem
      %
      [U,lambda] = eig(HS_adj);
      %
      lambda = diag(lambda,0);
      [lambda2,AscInd] = sort(abs(lambda));
      %
      nr = size(AscInd,1);
      DescInd = zeros(nr,1);
      for j=1:nr;
         DescInd(j)=AscInd(nr+1-j);
      end;
      %
      % Resort eigenvalues and eigenvectors
      % in descending order
      %
      lambda = abs(lambda(DescInd));
      U = U(:,DescInd);
      %
      % Renormalize U such that V'*h_orth'*S_11_h*h_orth*V=I
      %
      psi = inverse(HS_1)*U;
      %
      beta_1 = h*psi(:,1);
      %
      % Step 2: beta_2
      %
      if r>0;
         S_00_beta1 = S_00-(S_01*beta_1*inverse(beta_1'*S_11*beta_1)*beta_1'*S_10);
         S_01_beta1 = S_01-(S_01*beta_1*inverse(beta_1'*S_11*beta_1)*beta_1'*S_11);
         S_11_beta1 = S_11-(S_11*beta_1*inverse(beta_1'*S_11*beta_1)*beta_1'*S_11);
         S_10_beta1 = S_10-(S_11*beta_1*inverse(beta_1'*S_11*beta_1)*beta_1'*S_10);
         %
         %
         S_hh_2 = h_orth'*S_11_beta1*h_orth;
         HS_2 = symm(S_hh_2);
         %
         % solve the eigenvalue problem
         %
         HS_adj = inverse(HS_2)*h_orth'*S_10_beta1*inverse(S_00_beta1)*S_01_beta1*h_orth*inverse(HS_2);
         %
         % Step 2: solve the eigenvalue problem
         %
         [U,lambda] = eig(HS_adj);
         %
         lambda = diag(lambda,0);
         [lambda2,AscInd] = sort(abs(lambda));
         %
         nr = size(AscInd,1);
         DescInd = zeros(nr,1);
         for j=1:nr;
            DescInd(j)=AscInd(nr+1-j);
         end;
         %
         % Resort eigenvalues and eigenvectors
         % in descending order
         %
         lambda = abs(lambda(DescInd));
         U = U(:,DescInd);
         %
         % Renormalize U such that V'*h_orth'*S_11_h*h_orth*V=I
         %
         psi = inverse(HS_2)*U;
         %
         beta_2 = h_orth*psi(:,1:r-1);
      end;
      newbeta = [beta_1 beta_2];
      %
      % compute new values for the other coefficients
      %
      alpha = (S_01*newbeta)*inverse(newbeta'*S_11*newbeta);
      if isempty(Z2)==0;
         Psi = (M_02*inverse(M_22))-(alpha*newbeta'*M_12*inverse(M_22));
         epsilon_n = Z0-(alpha*newbeta'*Z1)-(Psi*Z2);
      else;
         epsilon_n = Z0-(alpha*newbeta'*Z1);
      end;
      lnL_new = loglike(epsilon_n);
      Omega_n = (1/T)*epsilon_n*epsilon_n';
      %
      % check if we have convergence
      %
      if betagrad==0;
         grad = abs(lnL_new-lnL_old)/lnL_old;
      else;
         new_grad = max(abs(T*vec(alpha)'*kron(eye(r),inverse(Omega_n)*(S_01-(alpha*newbeta'*S_11)))*H));
         grad = abs(new_grad-ref_grad);
         ref_grad = new_grad;
      end;
      lnL_old = lnL_new;
      it = it+1;
   end;
   %
   % Compute the test
   %
   LR = 2*(lnL_alt-lnL_new);
   deg_free(i,1) = n+q1+dcoint-r-1;
   %
   % store parameters if asked to do so
   %
   if nargout>3;
      params.variable(i).beta = newbeta;
      params.variable(i).alpha = alpha;
      params.variable(i).Psi = Psi;
      params.variable(i).epsilon = epsilon_n;
      params.variable(i).Omega = (1/T)*epsilon_n*epsilon_n';
   end;
   %
   if sscorrect==0;
      pvals(i,1) = 1-chis_cdf(LR,deg_free(i,1));
   else;
      dfc = round(((n+q1+dcoint-r)*r)/n);
      df_adj = round((((n+q1+dcoint-r-1)*(r-1))+1)/n);
      df_adj = df-dfc+df_adj;
      deg_free(i,2) = T-df_adj-1;
      LR = ((T-df_adj-1)/((T-1)*deg_free(i,1)))*LR;
      pvals(i,1) = 1-fdis_cdf(LR,deg_free(i,1),deg_free(i,2));;
   end;
   tests(i,1) = LR;
end;

if showwait==1;
   set(waitf.text,'UserData','done');
   delete(mywait);
   drawnow;
end;

%
% end of coint_stationarity_dcoint.m
%
