function copy_subplot_fcn

%
% copies all lines, patches, surfaces, images, lights and text objects into new figure
%

this_axes = gca;
oldname = get(gcf,'Name');
oldtag = get(gcf,'Tag');
fig_renderer = get(gcf,'Renderer');

%
% determine number of times graph has been copied
% this determines how we order objects for legends
%
%  odd  - descending
%  even - ascending
%

copied_times = findstr(oldtag,'var-copied-');
if isempty(copied_times)==0;
   new_tag = oldtag(1,(copied_times(1,1)+11):size(oldtag,2));
   this_num = mystr2num(new_tag);
   if isempty(this_num)==1;
      this_num = 0;
   else;
      this_num = this_num+1;
   end;
else;
   this_num = 0;
end;

if iscompiled==1;
   toolbar = get(gcf,'UserData');
   myaxes = get(toolbar.print,'UserData');
elseif nargin==0;
   these_axes = findobj(gcf,'Type','axes')';
   myaxes = [];
   for i=1:size(these_axes,2);
      if strcmp(get(these_axes(1,i),'Tag'),'legend')==0;
         myaxes = [myaxes these_axes(1,i)];
      end;
   end;
   myaxes = sort(myaxes);
end;

pos = get(gcf,'Position');
realscrsz = get(0,'ScreenSize');
for i=1:size(myaxes,2);
   if this_axes==myaxes(1,i);
      num = i;
      break;
   end;
end;
if this_num==0;
   newname = [oldname ' - Plot ' num2str(num,'%0.0f')];
else;
   newname = oldname;
   plot_pos = findstr(oldname,' - Plot ');
   if isempty(plot_pos)==0;
      newname = [oldname(1,1:plot_pos(1)-1) ' - Plot ' num2str(num,'%0.0f')];
   end;
end;

newpos = [pos(1)+22 pos(2)-22 512 384];
if ((pos(1)+534)>realscrsz(3))|(pos(2)<22);
   newpos(1) = 0;
   newpos(2) = realscrsz(4)-384;
end;

curr_ini = find_inisettings;

%
% set paper orientation and colors
%

if curr_ini.landscape==1;
   orientstr = 'landscape';
else;
   orientstr = 'portrait';
end;

if curr_ini.invert_hardcopy==1;
   inv_hardcopy = 'on';
else;
   inv_hardcopy = 'off';
end;

%
% collecting information about axes
%

ax_box = get(this_axes,'Box');
ax_xgrid = get(this_axes,'XGrid');
ax_ygrid = get(this_axes,'YGrid');
ax_zgrid = get(this_axes,'ZGrid');
ax_fontangle = get(this_axes,'FontAngle');
ax_fontname = get(this_axes,'FontName');
ax_fontsize = get(this_axes,'FontSize');
ax_fontunits = get(this_axes,'FontUnits');
ax_fontweight = get(this_axes,'FontWeight');
ax_xlim = get(this_axes,'XLim');
ax_ylim = get(this_axes,'YLim');
ax_zlim = get(this_axes,'ZLim');
ax_xscale = get(this_axes,'XScale');
ax_yscale = get(this_axes,'YScale');
ax_zscale = get(this_axes,'ZScale');
ax_xtick = get(this_axes,'XTick');
ax_ytick = get(this_axes,'YTick');
ax_ztick = get(this_axes,'ZTick');
ax_xtickmode = get(this_axes,'XTickMode');
ax_ytickmode = get(this_axes,'YTickMode');
ax_ztickmode = get(this_axes,'ZTickMode');
ax_color = get(this_axes,'Color');
ax_xcolor = get(this_axes,'XColor');
ax_ycolor = get(this_axes,'YColor');
ax_zcolor = get(this_axes,'ZColor');
ax_layer = get(this_axes,'Layer');
ax_ambientlightcolor = get(this_axes,'AmbientLightColor');
ax_clim = get(this_axes,'CLim');
ax_climmode = get(this_axes,'CLimMode');
ax_colororder = get(this_axes,'ColorOrder');
ax_xaxislocation = get(this_axes,'XAxisLocation');
ax_yaxislocation = get(this_axes,'YAxisLocation');
ax_view = get(this_axes,'View');
ax_nextplot = get(this_axes,'NextPlot');
ax_cameraposition = get(this_axes,'CameraPosition');
ax_camerapositionmode = get(this_axes,'CameraPositionMode');
ax_cameratarget = get(this_axes,'CameraTarget');
ax_cameratargetmode = get(this_axes,'CameraTargetMode');
ax_cameraupvector = get(this_axes,'CameraUpVector');
ax_cameraupvectormode = get(this_axes,'CameraUpVectorMode');
ax_cameraviewangle = get(this_axes,'CameraViewAngle');
ax_cameraviewanglemode = get(this_axes,'CameraViewAngleMode');
ax_projection = get(this_axes,'Projection');
ax_xlimmode = get(this_axes,'XLimMode');
ax_ylimmode = get(this_axes,'YLimMode');
ax_zlimmode = get(this_axes,'ZLimMode');
ax_dataaspectratio = get(this_axes,'DataAspectRatio');
ax_dataaspectratiomode = get(this_axes,'DataAspectRatioMode');
ax_plotboxaspectratio = get(this_axes,'PlotBoxAspectRatio');
ax_plotboxaspectratiomode = get(this_axes,'PlotBoxAspectRatioMode');
ax_tickdir = get(this_axes,'TickDir');
ax_gridlinestyle = get(this_axes,'GridLineStyle');

%
% collecting info about title
%

my_title = get(this_axes,'Title');
title_text = get(my_title,'String');
title_color = get(my_title,'Color');
title_fontangle = get(my_title,'FontAngle');
title_fontname = get(my_title,'FontName');
title_fontsize = get(my_title,'FontSize');
title_fontunits = get(my_title,'FontUnits');
title_fontweight = get(my_title,'FontWeight');
title_units = get(my_title,'Units');
title_position = get(my_title,'Position');
title_rotation = get(my_title,'Rotation');
title_horizontalalignment = get(my_title,'HorizontalAlignment');
title_verticalalignment = get(my_title,'VerticalAlignment');

%
% collecting info about the xlabel, ylabel, zlabel
%

my_xlabel = get(this_axes,'XLabel');
xlabel_text = get(my_xlabel,'String');
xlabel_color = get(my_xlabel,'Color');
xlabel_fontangle = get(my_xlabel,'FontAngle');
xlabel_fontname = get(my_xlabel,'FontName');
xlabel_fontsize = get(my_xlabel,'FontSize');
xlabel_fontunits = get(my_xlabel,'FontUnits');
xlabel_fontweight = get(my_xlabel,'FontWeight');
xlabel_units = get(my_xlabel,'Units');
xlabel_position = get(my_xlabel,'Position');
xlabel_rotation = get(my_xlabel,'Rotation');
xlabel_horizontalalignment = get(my_xlabel,'HorizontalAlignment');
xlabel_verticalalignment = get(my_xlabel,'VerticalAlignment');

my_ylabel = get(this_axes,'YLabel');
ylabel_text = get(my_ylabel,'String');
ylabel_color = get(my_ylabel,'Color');
ylabel_fontangle = get(my_ylabel,'FontAngle');
ylabel_fontname = get(my_ylabel,'FontName');
ylabel_fontsize = get(my_ylabel,'FontSize');
ylabel_fontunits = get(my_ylabel,'FontUnits');
ylabel_fontweight = get(my_ylabel,'FontWeight');
ylabel_units = get(my_ylabel,'Units');
ylabel_position = get(my_ylabel,'Position');
ylabel_rotation = get(my_ylabel,'Rotation');
ylabel_horizontalalignment = get(my_ylabel,'HorizontalAlignment');
ylabel_verticalalignment = get(my_ylabel,'VerticalAlignment');

my_zlabel = get(this_axes,'ZLabel');
zlabel_text = get(my_zlabel,'String');
zlabel_color = get(my_zlabel,'Color');
zlabel_fontangle = get(my_zlabel,'FontAngle');
zlabel_fontname = get(my_zlabel,'FontName');
zlabel_fontsize = get(my_zlabel,'FontSize');
zlabel_fontunits = get(my_zlabel,'FontUnits');
zlabel_fontweight = get(my_zlabel,'FontWeight');
zlabel_units = get(my_zlabel,'Units');
zlabel_position = get(my_zlabel,'Position');
zlabel_rotation = get(my_zlabel,'Rotation');
zlabel_horizontalalignment = get(my_zlabel,'HorizontalAlignment');
zlabel_verticalalignment = get(my_zlabel,'VerticalAlignment');

%
% create new figure
%

newfig = figure('Units','pixels', ...
   'Position',newpos, ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Menubar','none', ...
   'Toolbar','none', ...
   'Renderer',fig_renderer, ...
   'NumberTitle','off', ...
   'Visible','off', ...
   'Tag',['var-copied-' num2str(this_num,'%0.0f')], ...
   'PaperUnits',curr_ini.paperunits, ...
   'PaperOrientation',orientstr, ...
   'PaperPositionMode',curr_ini.paperposmode, ...
   'PaperType',curr_ini.papertype, ...
   'InvertHardcopy',inv_hardcopy, ...
   'Colormap',get(get(this_axes,'Parent'),'Colormap'), ...
   'Name',newname);

if matlabnumber>=7.0;
   set(newfig,'DockControl','off');
end;

%
% obey centering and filling defaults
%
center_fill_figure(newfig,curr_ini.fill_value,curr_ini.center_value);
%

if iscompiled==1;
   set(newfig,'Menubar','none','Toolbar','none');
end;

new_axes = axes('Units','normalized', ...
   'Position',[0.1300 0.1100 0.7750 0.8150], ...
   'Box',ax_box, ...
   'AmbientLightColor',ax_ambientlightcolor, ...
   'CLim',ax_clim, ...
   'CLimMode',ax_climmode, ...
   'ColorOrder',ax_colororder, ...
   'Layer',ax_layer, ...
   'XAxisLocation',ax_xaxislocation, ...
   'YAxisLocation',ax_yaxislocation, ...
   'GridLineStyle',ax_gridlinestyle, ...
   'XGrid',ax_xgrid, ...
   'YGrid',ax_ygrid, ...
   'ZGrid',ax_zgrid, ...
   'TickDir',ax_tickdir, ...
   'NextPlot',ax_nextplot, ...
   'CameraPosition',ax_cameraposition, ...
   'CameraPositionMode',ax_camerapositionmode, ...
   'CameraTarget',ax_cameratarget, ...
   'CameraTargetMode',ax_cameratargetmode, ...
   'CameraUpVector',ax_cameraupvector, ...
   'CameraUpVectorMode',ax_cameraupvectormode, ...
   'CameraViewAngle',ax_cameraviewangle, ...
   'CameraViewAngleMode',ax_cameraviewanglemode, ...
   'Projection',ax_projection, ...
   'LineWidth',get(this_axes,'LineWidth'), ...
   'DataAspectRatio',ax_dataaspectratio, ...
   'DataAspectRatioMode',ax_dataaspectratiomode, ...
   'PlotBoxAspectRatio',ax_plotboxaspectratio, ...
   'PlotBoxAspectRatioMode',ax_plotboxaspectratiomode, ...
   'FontAngle',ax_fontangle, ...
   'FontName',ax_fontname, ...
   'FontSize',ax_fontsize, ...
   'FontUnits',ax_fontunits, ...
   'FontWeight',ax_fontweight, ...
   'XLim',ax_xlim, ...
   'YLim',ax_ylim, ...
   'ZLim',ax_zlim, ...
   'XLimMode',ax_xlimmode, ...
   'YLimMode',ax_ylimmode, ...
   'ZLimMode',ax_zlimmode, ...
   'XScale',ax_xscale, ...
   'YScale',ax_yscale, ...
   'ZScale',ax_zscale, ...
   'XTick',ax_xtick, ...
   'YTick',ax_ytick, ...
   'ZTick',ax_ztick, ...
   'XTickMode',ax_xtickmode, ...
   'YTickMode',ax_ytickmode, ...
   'ZTickMode',ax_ztickmode, ...
   'Color',ax_color, ...
   'View',ax_view, ...
   'XColor',ax_xcolor, ...
   'YColor',ax_ycolor, ...
   'ZColor',ax_zcolor);

%
% setting up the title
%

new_title = title(title_text, ...
   'Color',title_color, ...
   'Units',title_units, ...
   'Rotation',title_rotation, ...
   'HorizontalAlignment',title_horizontalalignment, ...
   'VerticalAlignment',title_verticalalignment, ...
   'FontAngle',title_fontangle, ...
   'FontName',title_fontname, ...
   'FontSize',title_fontsize, ...
   'FontUnits',title_fontunits, ...
   'FontWeight',title_fontweight);

if isempty(copied_times)==1;
   if (isempty(findstr(title_text,'_'))==0)|(isempty(findstr(title_text,'^'))==0);
      set(new_axes,'Units','pixels');
      set(new_axes,'Position',get(new_axes,'Position')-[0 0 0 10]);
      set(new_axes,'Units','normalized');
   end;
end;

%   'Position',title_position, ...


%
% setting up the xlabel, ylabel, zlabel
%

new_xlabel = xlabel(xlabel_text, ...
   'Color',xlabel_color, ...
   'Units',xlabel_units, ...
   'Rotation',xlabel_rotation, ...
   'HorizontalAlignment',xlabel_horizontalalignment, ...
   'VerticalAlignment',xlabel_verticalalignment, ...
   'FontAngle',xlabel_fontangle, ...
   'FontName',xlabel_fontname, ...
   'FontSize',xlabel_fontsize, ...
   'FontUnits',xlabel_fontunits, ...
   'FontWeight',xlabel_fontweight);

%    'Position',xlabel_position, ...

new_ylabel = ylabel(ylabel_text, ...
   'Color',ylabel_color, ...
   'Units',ylabel_units, ...
   'Rotation',ylabel_rotation, ...
   'HorizontalAlignment',ylabel_horizontalalignment, ...
   'VerticalAlignment',ylabel_verticalalignment, ...
   'FontAngle',ylabel_fontangle, ...
   'FontName',ylabel_fontname, ...
   'FontSize',ylabel_fontsize, ...
   'FontUnits',ylabel_fontunits, ...
   'FontWeight',ylabel_fontweight);

%   'Position',ylabel_position, ...

new_zlabel = zlabel(zlabel_text, ...
   'Color',zlabel_color, ...
   'Units',zlabel_units, ...
   'Rotation',zlabel_rotation, ...
   'HorizontalAlignment',zlabel_horizontalalignment, ...
   'VerticalAlignment',zlabel_verticalalignment, ...
   'FontAngle',zlabel_fontangle, ...
   'FontName',zlabel_fontname, ...
   'FontSize',zlabel_fontsize, ...
   'FontUnits',zlabel_fontunits, ...
   'FontWeight',zlabel_fontweight);

%   'Position',zlabel_position, ...

%
% recreate lines, patches, surfaces, images, and lights
%

leg_objects = [];
linehandles = sort(findobj(this_axes,'Type','line')');
patchhandles = sort(findobj(this_axes,'Type','patch')');
surfhandles = sort(findobj(this_axes,'Type','surface')');
imagehandles = sort(findobj(this_axes,'Type','image')');
lighthandles = sort(findobj(this_axes,'Type','light')');

if isempty(surfhandles)==0;
   patchhandles = [];
   imagehandles = [];
end;
%
% remove lines which are representing lights.
%
fixhandles = [];
show_light = 'off';
if isempty(linehandles)==0;
   for i=1:size(linehandles,2);
      if strcmp(get(linehandles(1,i),'Tag'),'light')==1;
         fixhandles = [fixhandles 0];
         show_light = get(linehandles(1,i),'Visible');
      else;
         fixhandles = [fixhandles 1];
      end;
   end;
   linehandles = linehandles(fixhandles==1);
end;
old_objects = sort([linehandles patchhandles surfhandles lighthandles imagehandles]);
if isempty(old_objects)==0;
   num_lines = 0;
   num_patches = 0;
   num_surfaces = 0;
   num_images = 0;
   num_lights = 0;
   light_ud = [];
   for i=1:size(old_objects,2);
      if strcmp(get(old_objects(1,i),'Type'),'line')==1;
         num_lines = num_lines+1;
         this_line = line('XData',get(linehandles(1,num_lines),'XData'), ...
            'YData',get(linehandles(1,num_lines),'YData'), ...
            'Zdata',get(linehandles(1,num_lines),'ZData'), ...
            'Color',get(linehandles(1,num_lines),'Color'), ...
            'Visible',get(linehandles(1,num_lines),'Visible'), ...
            'LineStyle',get(linehandles(1,num_lines),'LineStyle'), ...
            'LineWidth',get(linehandles(1,num_lines),'LineWidth'), ...
            'Marker',get(linehandles(1,num_lines),'Marker'), ...
            'MarkerSize',get(linehandles(1,num_lines),'MarkerSize'), ...
            'MarkerEdgeColor',get(linehandles(1,num_lines),'MarkerEdgeColor'), ...
            'MarkerFaceColor',get(linehandles(1,num_lines),'MarkerFaceColor'), ...
            'Tag',get(linehandles(1,num_lines),'Tag'));
         if isempty(get(linehandles(1,num_lines),'UserData'))==0;
            this_tag = get(linehandles(1,num_lines),'Tag');
            if size(this_tag,2)>8;
               this_tag = this_tag(1,1:8);
            end;
            if (strcmp(this_tag,'3dlines-')==1)|(strcmp(this_tag,'light')==1);
               if strcmp(this_tag,'light')==1;
                  set(this_line,'UserData','light');
               end;
            else;
               leg_objects = [leg_objects this_line];
            end;
         end;
      elseif strcmp(get(old_objects(1,i),'Type'),'patch')==1;
         num_patches = num_patches+1;
         this_patch = patch('XData',get(patchhandles(1,num_patches),'XData'), ...
            'YData',get(patchhandles(1,num_patches),'YData'), ...
            'Zdata',get(patchhandles(1,num_patches),'ZData'), ...
            'CData',get(patchhandles(1,num_patches),'CData'), ...
            'CDataMapping',get(patchhandles(1,num_patches),'CDataMapping'), ...
            'AmbientStrength',get(patchhandles(1,num_patches),'AmbientStrength'), ...
            'BackFaceLighting',get(patchhandles(1,num_patches),'BackFaceLighting'), ...
            'DiffuseStrength',get(patchhandles(1,num_patches),'DiffuseStrength'), ...
            'EdgeLighting',get(patchhandles(1,num_patches),'EdgeLighting'), ...
            'FaceLighting',get(patchhandles(1,num_patches),'FaceLighting'), ...
            'SpecularColorReflectance',get(patchhandles(1,num_patches),'SpecularColorReflectance'), ...
            'SpecularExponent',get(patchhandles(1,num_patches),'SpecularExponent'), ...
            'SpecularStrength',get(patchhandles(1,num_patches),'SpecularStrength'), ...
            'Clipping',get(patchhandles(1,num_patches),'Clipping'), ...
            'FaceColor',get(patchhandles(1,num_patches),'FaceColor'), ...
            'EdgeColor',get(patchhandles(1,num_patches),'EdgeColor'), ...
            'LineStyle',get(patchhandles(1,num_patches),'LineStyle'), ...
            'LineWidth',get(patchhandles(1,num_patches),'LineWidth'), ...
            'Marker',get(patchhandles(1,num_patches),'Marker'), ...
            'MarkerSize',get(patchhandles(1,num_patches),'MarkerSize'), ...
            'MarkerFaceColor',get(patchhandles(1,num_patches),'MarkerFaceColor'), ...
            'MarkerEdgeColor',get(patchhandles(1,num_patches),'MarkerEdgeColor'), ...
            'Tag',get(patchhandles(1,num_patches),'Tag'));
         if isempty(get(patchhandles(1,num_patches),'UserData'))==0;
            leg_objects = [leg_objects this_patch];
         end;
      elseif strcmp(get(old_objects(1,i),'Type'),'surface')==1;
         num_surfaces = num_surfaces+1;
         this_surface = surface('Xdata',get(surfhandles(1,num_surfaces),'XData'), ...
            'YData',get(surfhandles(1,num_surfaces),'YData'), ...
            'ZData',get(surfhandles(1,num_surfaces),'ZData'), ...
            'UserData',get(surfhandles(1,num_surfaces),'UserData'), ...
            'Tag',get(surfhandles(1,num_surfaces),'Tag'), ...
            'Parent',new_axes, ...
            'CData',get(surfhandles(1,num_surfaces),'CData'), ...
            'CDataMapping',get(surfhandles(1,num_surfaces),'CDataMapping'), ...
            'EdgeColor',get(surfhandles(1,num_surfaces),'EdgeColor'), ...
            'FaceColor',get(surfhandles(1,num_surfaces),'FaceColor'), ...
            'AmbientStrength',get(surfhandles(1,num_surfaces),'AmbientStrength'), ...
            'BackFaceLighting',get(surfhandles(1,num_surfaces),'BackFaceLighting'), ...
            'DiffuseStrength',get(surfhandles(1,num_surfaces),'DiffuseStrength'), ...
            'EdgeLighting',get(surfhandles(1,num_surfaces),'EdgeLighting'), ...
            'FaceLighting',get(surfhandles(1,num_surfaces),'FaceLighting'), ...
            'SpecularColorReflectance',get(surfhandles(1,num_surfaces),'SpecularColorReflectance'), ...
            'SpecularExponent',get(surfhandles(1,num_surfaces),'SpecularExponent'), ...
            'SpecularStrength',get(surfhandles(1,num_surfaces),'SpecularStrength'), ...
            'LineStyle',get(surfhandles(1,num_surfaces),'LineStyle'), ...
            'LineWidth',get(surfhandles(1,num_surfaces),'LineWidth'), ...
            'Marker',get(surfhandles(1,num_surfaces),'Marker'), ...
            'MarkerSize',get(surfhandles(1,num_surfaces),'MarkerSize'), ...
            'MarkerFaceColor',get(surfhandles(1,num_surfaces),'MarkerFaceColor'), ...
            'MarkerEdgeColor',get(surfhandles(1,num_surfaces),'MarkerEdgeColor'));

         %
         % fix UIContextMenu
         %
         surf_context = uicontextmenu('Callback','set(gcf,''Pointer'',''arrow'')');
         old_context = get(surfhandles(1,num_surfaces),'UIContextMenu');
         kids = sort(get(old_context,'Children')');
         if isempty(light_ud)==1;
            ui_ud = this_surface;
            old_axunits = get(new_axes,'Units');
            set(new_axes,'Units','pixels');
            axpos = get(new_axes,'Position');
            set(new_axes,'Position',axpos-[0 0 0 10]);
            set(new_axes,'Units',old_axunits);
         end;
         for j=1:size(kids,2);
            if isempty(get(kids(1,j),'Children'))==1;
               uimenu('Parent',surf_context,'Label',get(kids(1,j),'Label'),'UserData',ui_ud,'Separator',get(kids(1,j),'Separator'),'Checked',get(kids(1,j),'Checked'),'Callback',get(kids(1,j),'Callback'));
            else;
               main_context = uimenu('Parent',surf_context,'Label',get(kids(1,j),'Label'),'UserData',ui_ud,'Separator',get(kids(1,j),'Separator'));
               gkids = sort(get(kids(1,j),'Children')');
               for j1=1:size(gkids,2);
                  uimenu('Parent',main_context,'Label',get(gkids(1,j1),'Label'),'UserData',ui_ud,'Separator',get(gkids(1,j1),'Separator'),'Checked',get(gkids(1,j1),'Checked'),'Callback',get(gkids(1,j1),'Callback'));
               end;
            end;
         end;
         set(this_surface,'UIContextMenu',surf_context);
         %
         % fix last_light user data
         %
         if isempty(light_ud)==1;
            light_ud = this_surface;
         end;
         %
         if isempty(get(surfhandles(1,num_surfaces),'UserData'))==0;
            leg_objects = [leg_objects this_surface];
         end;
      elseif strcmp(get(old_objects(1,i),'Type'),'image')==1;
         num_images = num_images+1;
         image('CData',get(imagehandles(1,num_images),'CData'), ...
            'CDataMapping',get(imagehandles(1,num_images),'CDataMapping'), ...
            'UserData',imagehandles(1,num_images), ...
            'Parent',new_axes, ...
            'XData',get(imagehandles(1,num_images),'XData'), ...
            'YData',get(imagehandles(1,num_images),'YData'));
      elseif strcmp(get(old_objects(1,i),'Type'),'light')==1;
         num_lights = num_lights+1;
         last_light = light('Color',get(lighthandles(1,num_lights),'Color'), ...
            'Position',get(lighthandles(1,num_lights),'Position'), ...
            'UserData',light_ud, ...
            'Parent',new_axes, ...
            'Style',get(lighthandles(1,num_lights),'Style'));
         light_ud = [];
      end;
      hold('on');
   end;
end;

%
% recreate text
%

texthandles = sort(findobj(this_axes,'Type','text')');
if isempty(texthandles)==0;
   for j=1:size(texthandles,2);
      old_text_units = get(texthandles(1,j),'Units');
      set(texthandles(1,j),'Units','data');
      this_text = text('Units',get(texthandles(1,j),'Units'), ...
         'Position',get(texthandles(1,j),'Position'), ...
         'String',get(texthandles(1,j),'String'), ...
         'Editing',get(texthandles(1,j),'Editing'), ...
         'HorizontalAlignment',get(texthandles(1,j),'HorizontalAlignment'), ...
         'VerticalAlignment',get(texthandles(1,j),'VerticalAlignment'), ...
         'Color',get(texthandles(1,j),'Color'), ...
         'FontName',get(texthandles(1,j),'FontName'), ...
         'FontSize',get(texthandles(1,j),'FontSize'), ...
         'FontWeight',get(texthandles(1,j),'FontWeight'), ...
         'FontUnits',get(texthandles(1,j),'FontUnits'), ...
         'FontAngle',get(texthandles(1,j),'FontAngle'), ...
         'UserData',get(texthandles(1,j),'UserData'), ...
         'Tag',get(texthandles(1,j),'Tag'));
      set(texthandles(1,j),'Units',old_text_units);
      if iscompiled==1;
         text_context_menu = uicontextmenu('Callback','set(gcf,''Pointer'',''arrow'')');
         uimenu('Parent',text_context_menu,'Label','Edit','Callback','set(gco,''Editing'',''on'')');
         uimenu('Parent',text_context_menu,'Label','Cut','Callback','delete(gco); empty_axes');
         uimenu('Parent',text_context_menu,'Separator','on','Label','Font','Callback','change_text_font');
         uimenu('Parent',text_context_menu,'Label','Font Color','Callback','change_text_color');
         set(this_text,'UIContextMenu',text_context_menu,'ButtonDownFcn','check_text_fcn');
      end;
      hold('on');
   end;
end;

%
% legends
%

if isempty(get(this_axes,'UserData'))==0;
   if ishandle(get(this_axes,'UserData'))==1;
      old_leg = get(this_axes,'UserData');
   else;
      old_leg = find_legend(this_axes);
   end;
   leg_data = get(old_leg,'UserData');
   %
   % original legend position
   %
   old_pos = leg_data.legendpos;
   if length(old_pos)>1;
      old_ax_units = get(this_axes,'Units');
      set(this_axes,'Units','pixels');
      ax_pos = get(this_axes,'Position');
      set(this_axes,'Units',old_ax_units);
      old_leg_units = get(old_leg,'Units');
      set(old_leg,'Units','pixels');
      new_legpos = get(old_leg,'Position');
      set(old_leg,'Units',old_leg_units);
      %
      % determine which corner the legend is located in
      %
      lower_c = 0;
      left = 0;
      outside = 0;
      %
      % checking left/right
      %
      if (new_legpos(1)<ax_pos(1))|(new_legpos(1)+new_legpos(3)>ax_pos(1)+ax_pos(3));
         outside = 1;
      elseif new_legpos(1)-ax_pos(1)<(ax_pos(1)+ax_pos(3))-(new_legpos(1)+new_legpos(3));
         left = 1;
      end;
      if outside==0;
         if (new_legpos(2)<ax_pos(2))|(new_legpos(2)+new_legpos(4)>ax_pos(2)+ax_pos(4));
            outside = 1;
         elseif new_legpos(2)-ax_pos(2)<(ax_pos(2)+ax_pos(4))-(new_legpos(2)+new_legpos(4));
            lower_c = 1;
         end;
      end;
      %
      % outside or lower-left quadrant
      %
      if (outside==1)|((lower_c==1)&(left==1));
         old_pos = 3;
      %
      % lower-right quandrant
      %
      elseif ((lower_c==1)&(left==0));
         old_pos = 4;
      %
      % upper-left quadrant
      %
      elseif ((lower_c==0)&(left==1));
         old_pos = 2;
      %
      % upper-right quandrant
      %
      else;
         old_pos = 1;
      end;
   end;
   legstringhandles = sort(findobj(old_leg,'Type','text')');
   if isempty(legstringhandles)==0;
      old_ud = get(old_leg,'UserData');
      %
      % make new legend
      %
      lstrings = old_ud.lstrings;
      %
      % need to set leg_objects in the original order
      %
      if size(old_ud.handles,1)>1;
         old_ud.handles = old_ud.handles';
      end;
      [sort_h,h_ind] = sort(old_ud.handles);
      h_leg_objects = zeros(size(h_ind));
      for i1=1:size(h_ind,2);
         h_leg_objects(1,h_ind(1,i1)) = leg_objects(1,i1);
      end;
      leg_objects = h_leg_objects;
      %
      new_leg = legend_fcn(leg_objects,lstrings,old_pos);
      set(new_leg,'Color',get(old_leg,'Color'));
      set(new_leg,'XColor',get(old_leg,'XColor'));
      set(new_leg,'YColor',get(old_leg,'YColor'));
      %
      % fix font and font color with new text objects
      %
      new_legstringhandles = findobj(new_leg,'Type','text')';
      if isempty(new_legstringhandles)==0;
         if size(new_legstringhandles)==size(legstringhandles);
            for i=1:size(new_legstringhandles,2);
               set(new_legstringhandles(1,i),'Color',get(legstringhandles(1,i),'Color'));
               set(new_legstringhandles(1,i),'FontAngle',get(legstringhandles(1,i),'FontAngle'));
               set(new_legstringhandles(1,i),'FontName',get(legstringhandles(1,i),'FontName'));
               set(new_legstringhandles(1,i),'FontSize',get(legstringhandles(1,i),'FontSize'));
               set(new_legstringhandles(1,i),'FontWeight',get(legstringhandles(1,i),'FontWeight'));
               set(new_legstringhandles(1,i),'FontUnits',get(legstringhandles(1,i),'FontUnits'));
            end;
         end;
      end;
      %
      % resize legend
      %
      new_leg = resize_legend_fcn(new_leg,this_axes);
      %
      % make sure that lines and patches have user data equal to the
      % correct line or patch in the legend (should work directly from
      % toolbar_fcn but doesn't seem to
      %
      if iscompiled==1;
         new_leg_lines = findobj(new_leg,'Type','line')';
         new_leg_patch = findobj(new_leg,'Type','patch')';
         new_leg_objects = sort([new_leg_lines new_leg_patch]);
         sort_objects = sort(leg_objects);
         for i=1:size(sort_objects,2);
            if strcmp(get(sort_objects(1,i),'Type'),'surface')==0;
               set(sort_objects(1,i),'UserData',new_leg_objects(1,i));
            end;
         end;
      end;
      set(new_axes,'UserData',new_leg);
   end;
end;

hold('off');

%
% May have to change kids around if there are patches with the Tag 'rectpatch'
% or 'confpatch' and if there are lines or patches from imported data
%

r_p = sort([findobj(new_axes,'Tag','rectpatch');findobj(new_axes,'Tag','rectpatch-new');findobj(new_axes,'Tag','RECTPATCH');findobj(new_axes,'Tag','confpatch');findobj(new_axes,'Tag','confpatch-new')]);
r_p = [sort([findobj(new_axes,'Type','patch','Tag','import-data');findobj(new_axes,'Type','patch','Tag','import-data-new')]);r_p];
if isempty(r_p)==0;
   kids = get(new_axes,'Children');
   new_kids = [];
   for i=1:size(kids,1);
      del_kid = 0;
      for j=1:size(r_p,1);
         if kids(i,1)==r_p(j,1);
            del_kid = 1;
            break;
         end;
      end;
      if del_kid==0;
         new_kids = [new_kids;kids(i,1)];
      end;
   end;
   set(new_axes,'Children',[new_kids;r_p]);
end;

toolbar_fcn(new_axes,show_light);
set(newfig,'Renderer',fig_renderer,'Visible','on');

%
% end of copy_subplot_fcn.m
%
