function create_axes_contextmenu_fcn(this_axes)

%
% sets up a context menu for this_axes
%

if nargin==0;
   this_axes = gca;
end;


h = get(0,'CurrentFigure');
toolbar = get(h,'UserData');
myaxes = get(toolbar.print,'UserData');
this_colorbar = findobj(h,'Type','axes','Tag','colorbar')';
this_label = get(toolbar.lock,'Label');

%
% Collecting info about the axes
%

surfhandles = findobj(this_axes,'Type','surface')';
if isempty(surfhandles)==0;
   use_surf = [];
   for i=1:size(surfhandles,2);
      if ischar(get(surfhandles(1,i),'UserData'))==1;
         use_surf = [use_surf surfhandles(1,i)];
      end;
   end;
   surfhandles = use_surf;
end;

x_zeros = 0;
y_zeros = 0;
linehandles = findobj(this_axes,'Type','line')';
if isempty(linehandles)==0;
   fixhandles = [];
   for j=1:size(linehandles,2);
      checkvec = get(linehandles(1,j),'YData');
      checkvec2 = sum(get(linehandles(1,j),'XData'));
      checkstyle = get(linehandles(1,j),'LineStyle');
      if ((checkvec==zeros(size(checkvec,1),1))|(checkvec2==zeros(size(checkvec2,1),1)))&((strcmp(checkstyle,'none')==0)|(isempty(get(linehandles(1,j),'UserData'))==0))|(strcmp(get(linehandles(1,j),'Tag'),'light')==1);
         fixhandles = [fixhandles 0];
      else;
         fixhandles = [fixhandles 1];
      end;
      if checkvec==zeros(size(checkvec,1),1);
         x_zeros = 1;
      end;
      if max(get(linehandles(1,j),'LineStyle'))==min(get(linehandles(1,j),'LineStyle'));
         y_zeros = 1;
      end;
   end;
   linehandles = linehandles(fixhandles==1);
   if (size(linehandles,1)==0)|(size(linehandles,2)==0);
      linehandles = [];
   end;
end;

patchhandles = findobj(this_axes,'Type','patch')';
if isempty(patchhandles)==0;
   fixhandles = [];
   for j=1:size(patchhandles,2);
      checktag = get(patchhandles(1,j),'Tag');
      if size(checktag,2)>9;
         checktag = checktag(1,1:9);
      end;
      if strcmp(checktag,'varpatch-')==1;
         fixhandles = [fixhandles 0];
      else;
         fixhandles = [fixhandles 1];
      end;
   end;
   patchhandles = patchhandles(fixhandles==1);
end;

%
% create a context menu for axes objects
%

axes_context_menu = uicontextmenu('Callback','set(gcf,''Pointer'',''arrow'')');
   uimenu('Parent',axes_context_menu,'Label','Font','Callback','change_axes_font');
   uimenu('Parent',axes_context_menu,'Separator','on','Label','Background Color','Callback','change_axes_color');
axes_color_menu = uimenu('Parent',axes_context_menu,'Label','Axes Color');
   uimenu('Parent',axes_color_menu,'Label','Axes Color','Callback','change_allaxes_color');
   uimenu('Parent',axes_color_menu,'Label','X Axis Color','Callback','change_xaxes_color');
   uimenu('Parent',axes_color_menu,'Label','Y Axis Color','Callback','change_yaxes_color');
axes_width_menu = uimenu('Parent',axes_context_menu,'Label','Axes Width');
   pt0 = 'off';
   pt1 = 'off';
   pt2 = 'off';
   pt3 = 'off';
   pt4 = 'off';
   pt5 = 'off';
   pt6 = 'off';
   pt7 = 'off';
   pt8 = 'off';
   pt9 = 'off';
   pt10 = 'off';
   if get(this_axes,'LineWidth')==0.5;
      pt0 = 'on';
   elseif get(this_axes,'LineWidth')==1;
      pt1 = 'on';
   elseif get(this_axes,'LineWidth')==2;
      pt2 = 'on';
   elseif get(this_axes,'LineWidth')==3;
      pt3 = 'on';
   elseif get(this_axes,'LineWidth')==4;
      pt4 = 'on';
   elseif get(this_axes,'LineWidth')==5;
      pt5 = 'on';
   elseif get(this_axes,'LineWidth')==6;
      pt6 = 'on';
   elseif get(this_axes,'LineWidth')==7;
      pt7 = 'on';
   elseif get(this_axes,'LineWidth')==8;
      pt8 = 'on';
   elseif get(this_axes,'LineWidth')==9;
      pt9 = 'on';
   elseif get(this_axes,'LineWidth')==10;
      pt10 = 'on';
   end;
   uimenu('Parent',axes_width_menu,'Label','0.5 pt','Checked',pt0,'Callback','set(gca,''LineWidth'',0.5); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','1 pt','Checked',pt1,'Callback','set(gca,''LineWidth'',1); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','2 pts','Checked',pt2,'Callback','set(gca,''LineWidth'',2); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','3 pts','Checked',pt3,'Callback','set(gca,''LineWidth'',3); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','4 pts','Checked',pt4,'Callback','set(gca,''LineWidth'',4); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','5 pts','Checked',pt5,'Callback','set(gca,''LineWidth'',5); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','6 pts','Checked',pt6,'Callback','set(gca,''LineWidth'',6); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','7 pts','Checked',pt7,'Callback','set(gca,''LineWidth'',7); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','8 pts','Checked',pt8,'Callback','set(gca,''LineWidth'',8); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','9 pts','Checked',pt9,'Callback','set(gca,''LineWidth'',9); check_axes_lines; uimenu_check_fcn');
   uimenu('Parent',axes_width_menu,'Label','10 pts','Checked',pt10,'Callback','set(gca,''LineWidth'',10); check_axes_lines; uimenu_check_fcn');
axes_grid_menu = uimenu('Parent',axes_context_menu,'Label','Grid Style');
   pt1 = 'off';
   pt2 = 'off';
   pt3 = 'off';
   pt4 = 'off';
   pt5 = 'off';
   if strcmp(get(this_axes,'GridLineStyle'),'-')==1;
      pt1 = 'on';
   elseif strcmp(get(this_axes,'GridLineStyle'),'--')==1;
      pt2 = 'on';
   elseif strcmp(get(this_axes,'GridLineStyle'),':')==1;
      pt3 = 'on';
   elseif strcmp(get(this_axes,'GridLineStyle'),'-.')==1;
      pt4 = 'on';
   elseif strcmp(get(this_axes,'GridLineStyle'),'none')==1;
      pt5 = 'on';
   end;
   uimenu('Parent',axes_grid_menu,'Label','Solid','Checked',pt1,'Callback','set(gca,''GridLineStyle'',''-''); uimenu_check_fcn');
   uimenu('Parent',axes_grid_menu,'Label','Dash','Checked',pt2,'Callback','set(gca,''GridLineStyle'',''--''); uimenu_check_fcn');
   uimenu('Parent',axes_grid_menu,'Label','Dot','Checked',pt3,'Callback','set(gca,''GridLineStyle'','':''); uimenu_check_fcn');
   uimenu('Parent',axes_grid_menu,'Label','Dash-Dot','Checked',pt4,'Callback','set(gca,''GridLineStyle'',''-.''); uimenu_check_fcn');
   uimenu('Parent',axes_grid_menu,'Label','None','Checked',pt5,'Callback','set(gca,''GridLineStyle'',''none''); uimenu_check_fcn');
axes_tick_menu = uimenu('Parent',axes_context_menu,'Label','Tick Direction');
   if strcmp(get(this_axes,'TickDir'),'in')==1;
      uimenu('Parent',axes_tick_menu,'Label','In','Checked','on','Callback','set(gca,''TickDir'',''in''); uimenu_check_fcn');
      uimenu('Parent',axes_tick_menu,'Label','Out','Checked','off','Callback','set(gca,''TickDir'',''out''); uimenu_check_fcn');
   else;
      uimenu('Parent',axes_tick_menu,'Label','In','Checked','off','Callback','set(gca,''TickDir'',''in''); uimenu_check_fcn');
      uimenu('Parent',axes_tick_menu,'Label','Out','Checked','on','Callback','set(gca,''TickDir'',''out''); uimenu_check_fcn');
   end;
axes_layer_menu = uimenu('Parent',axes_context_menu,'Label','Layer');
   if strcmp(get(this_axes,'Layer'),'bottom')==1;
      uimenu('Parent',axes_layer_menu,'Label','Bottom','Checked','on','Callback','set(gca,''Layer'',''bottom''); uimenu_check_fcn');
      uimenu('Parent',axes_layer_menu,'Label','Top','Checked','off','Callback','set(gca,''Layer'',''top''); uimenu_check_fcn');
   else;
      uimenu('Parent',axes_layer_menu,'Label','Bottom','Checked','off','Callback','set(gca,''Layer'',''bottom''); uimenu_check_fcn');
      uimenu('Parent',axes_layer_menu,'Label','Top','Checked','on','Callback','set(gca,''Layer'',''top''); uimenu_check_fcn');
   end;
%
if isempty(surfhandles)==0;
   uimenu('Parent',axes_color_menu,'Label','Z Axis Color','Callback','change_zaxes_color');
   axes_projection_menu = uimenu('Parent',axes_context_menu,'Label','Projection');
   if strcmp(get(this_axes,'Projection'),'orthographic')==1;
      uimenu('Parent',axes_projection_menu,'Label','Orthographic','Checked','on','Callback','change_axes_projection');
      uimenu('Parent',axes_projection_menu,'Label','Perspective','Checked','off','Callback','change_axes_projection');
   else;
      uimenu('Parent',axes_projection_menu,'Label','Orthographic','Checked','off','Callback','change_axes_projection');
      uimenu('Parent',axes_projection_menu,'Label','Perspective','Checked','on','Callback','change_axes_projection');
   end;
   axes_motion_menu = uimenu('Parent',axes_context_menu,'Label','Motion');
      uimenu('Parent',axes_motion_menu,'Label','Horizontal 360','Interruptible','off','Callback','axes_motion(1)');
      uimenu('Parent',axes_motion_menu,'Label','Vertical 360','Interruptible','off','Callback','axes_motion(0)');
      uimenu('Parent',axes_motion_menu,'Label','Joint 360','Interruptible','off','Callback','axes_motion(2)');
   uimenu('Parent',axes_context_menu,'Label','Ambient Light Color','Separator','on','Callback','change_ambientlight_color');
   uimenu('Parent',axes_context_menu,'Label','Light Color','Callback','change_light_color');
   axes_lightstyle_menu = uimenu('Parent',axes_context_menu,'Label','Light Style');
   my_light = findobj(this_axes,'Type','light')';
   if strcmp(get(my_light,'Style'),'infinite')==1;
      uimenu('Parent',axes_lightstyle_menu,'Label','Infinite','Checked','on','Callback','change_light_style');
      uimenu('Parent',axes_lightstyle_menu,'Label','Local','Checked','off','Callback','change_light_style');
   else;
      uimenu('Parent',axes_lightstyle_menu,'Label','Infinite','Checked','off','Callback','change_light_style');
      uimenu('Parent',axes_lightstyle_menu,'Label','Local','Checked','on','Callback','change_light_style');
   end;
   light_line = findobj(this_axes,'Type','line','UserData','light')';
   light_pos = get(my_light,'Position');
   axes_light_menu = uimenu('Parent',axes_context_menu,'Label','Light Visibility');
   if strcmp(get(light_line,'Visible'),'on')==1;
      uimenu('Parent',axes_light_menu,'Label','Hide','Checked','off','Callback','show_hide_light');
      uimenu('Parent',axes_light_menu,'Label','Show','Checked','on','Callback','show_hide_light');
   else;
      uimenu('Parent',axes_light_menu,'Label','Hide','Checked','on','Callback','show_hide_light');
      uimenu('Parent',axes_light_menu,'Label','Show','Checked','off','Callback','show_hide_light');
   end;
   %
   % determine current light position relative to surface
   %
   light_surf = get(my_light,'UserData');
   surf_pos1 = 'off';
   surf_pos2 = 'off';
   surf_pos3 = 'off';
   surf_pos4 = 'off';
   surf_pos5 = 'off';
   surf_pos6 = 'off';
   surf_pos7 = 'off';
   surf_pos8 = 'off';
   if [min(min(get(light_surf,'XData'))) min(min(get(light_surf,'YData'))) min(min(get(light_surf,'ZData')))]==light_pos;
      surf_pos1 = 'on';
   elseif [min(min(get(light_surf,'XData'))) min(min(get(light_surf,'YData'))) max(max(get(light_surf,'ZData')))]==light_pos;
      surf_pos2 = 'on';
   elseif [min(min(get(light_surf,'XData'))) max(max(get(light_surf,'YData'))) min(min(get(light_surf,'ZData')))]==light_pos;
      surf_pos3 = 'on';
   elseif [min(min(get(light_surf,'XData'))) max(max(get(light_surf,'YData'))) max(max(get(light_surf,'ZData')))]==light_pos;
      surf_pos4 = 'on';
   elseif [max(max(get(light_surf,'XData'))) min(min(get(light_surf,'YData'))) min(min(get(light_surf,'ZData')))]==light_pos;
      surf_pos5 = 'on';
   elseif [max(max(get(light_surf,'XData'))) min(min(get(light_surf,'YData'))) max(max(get(light_surf,'ZData')))]==light_pos;
      surf_pos6 = 'on';
   elseif [max(max(get(light_surf,'XData'))) max(max(get(light_surf,'YData'))) min(min(get(light_surf,'ZData')))]==light_pos;
      surf_pos7 = 'on';
   elseif [max(max(get(light_surf,'XData'))) max(max(get(light_surf,'YData'))) max(max(get(light_surf,'ZData')))]==light_pos;
      surf_pos8 = 'on';
   end;
   axes_lightposition_menu = uimenu('Parent',axes_context_menu,'Label','Light Position');
      uimenu('Parent',axes_lightposition_menu,'Label','min(X) max(Y) min(Z)','Checked',surf_pos3,'Callback','change_light_position');
      uimenu('Parent',axes_lightposition_menu,'Label','min(X) max(Y) max(Z)','Checked',surf_pos4,'Callback','change_light_position');
      uimenu('Parent',axes_lightposition_menu,'Label','min(X) min(Y) min(Z)','Checked',surf_pos1,'Callback','change_light_position');
      uimenu('Parent',axes_lightposition_menu,'Label','min(X) min(Y) max(Z)','Checked',surf_pos2,'Callback','change_light_position');
      uimenu('Parent',axes_lightposition_menu,'Label','max(X) min(Y) min(Z)','Checked',surf_pos5,'Callback','change_light_position');
      uimenu('Parent',axes_lightposition_menu,'Label','max(X) min(Y) max(Z)','Checked',surf_pos6,'Callback','change_light_position');
      uimenu('Parent',axes_lightposition_menu,'Label','max(X) max(Y) min(Z)','Checked',surf_pos7,'Callback','change_light_position');
      uimenu('Parent',axes_lightposition_menu,'Label','max(X) max(Y) max(Z)','Checked',surf_pos8,'Callback','change_light_position');
else;
   axes_stretch_menu = uimenu('Parent',axes_context_menu,'Label','Stretch');
      uimenu('Parent',axes_stretch_menu,'Label','Horizontal Lines','Callback','stretch_lines(''h'')');
      uimenu('Parent',axes_stretch_menu,'Label','Vertical Lines','Callback','stretch_lines(''v'')');
      uimenu('Parent',axes_stretch_menu,'Label','45 Degree Lines','Callback','stretch_lines(''d'')');
      if isempty(surfhandles)==1;
         uimenu('Parent',axes_stretch_menu,'Label','Rectangles Horizontally','Separator','on','Callback','stretch_patch(''h'')');
         uimenu('Parent',axes_stretch_menu,'Label','Rectangles Vertically','Callback','stretch_patch(''v'')');
      end;
   uimenu('Parent',axes_context_menu,'Separator','on','Label','Import Data','Callback','import_data_fcn');
   uimenu('Parent',axes_context_menu,'Label','Export Data','Callback','export_data_fcn');
   if isempty(get(this_axes,'Children'))==0;
      uimenu('Parent',axes_context_menu,'Separator','on','Enable','on','Label','Statistics','Callback','axes_statistics_fcn');
   else;
      uimenu('Parent',axes_context_menu,'Separator','on','Enable','off','Label','Statistics','Callback','axes_statistics_fcn');
   end;
end;
%
% Deal with X and Y labels
%
if isempty(surfhandles)==1;
   %
   % check if we need to add Match Data Units functions
   %
   fix_y = 0;
   fix_x = 0;
   ytick = get(this_axes,'YTick');
   ylabels = get(this_axes,'YTickLabel');
   if isempty(ylabels)==0;
      if ((isalmostequal(mystr2num(strtrim(ylabels(1,:))),ytick(1,1))==0)|(isalmostequal(mystr2num(strtrim(ylabels(size(ylabels,1),:))),ytick(1,size(ytick,2)))==0))&(size(ytick,2)>=2);
         fix_y = 1;
      end;
   end;
   xtick = get(this_axes,'XTick');
   xlabels = get(this_axes,'XTickLabel');
   if isempty(xlabels)==0;
      if ((isalmostequal(mystr2num(strtrim(xlabels(1,:))),xtick(1,1))==0)|(isalmostequal(mystr2num(strtrim(xlabels(size(xlabels,1),:))),xtick(1,size(xtick,2)))==0))&(size(xtick,2)>=2);
         fix_x = 1;
      end;
   end;
   %
   % add controls if necessary
   %
   if (fix_x==1)|(fix_y==1)|(strcmp(get(this_axes,'XTickLabelMode'),'manual')==1)|(strcmp(get(this_axes,'YTickLabelMode'),'manual')==1);
      axes_tick_menu = uimenu('Parent',axes_context_menu,'Separator','on','label','Match Data Units');
      paste_sep = 'off';
   else;
      paste_sep = 'on';
   end;
   if fix_x==1;
      uimenu('Parent',axes_tick_menu,'Label','Match X Axis','Callback','fix_xtix_fcn');
   elseif strcmp(get(this_axes,'XTickLabelMode'),'manual')==1;
      uimenu('Parent',axes_tick_menu,'Label','Automatic X Axis Labels','Callback','fix_xtix_fcn');
   end;
   if fix_y==1;
      uimenu('Parent',axes_tick_menu,'Label','Match Y Axis','Callback','fix_ytix_fcn');
   elseif strcmp(get(this_axes,'YTickLabelMode'),'manual')==1;
      uimenu('Parent',axes_tick_menu,'Label','Automatic Y Axis Labels','Callback','fix_ytix_fcn');
   end;
else;
   paste_sep = 'on';
end;
%
% check for model.run userdata
%
fix_fig = findobj('Tag','StructuralVAR');
model = get(fix_fig,'UserData');
copy = get(model.run,'UserData');
line_copy_enable = 'off';
patch_copy_enable = 'off';
surface_copy_enable = 'off';
if isfield(copy,'line_properties')==1;
   line_copy_enable = 'on';
end;
if isfield(copy,'patch_properties')==1;
   patch_copy_enable = 'on';
end;
if isfield(copy,'surface_properties')==1;
   surface_copy_enable = 'on';
end;
%
curr_colorbar = [];
if isempty(this_colorbar)==0;
   for j=1:size(this_colorbar,2);
      if get(this_colorbar(1,j),'UserData')==this_axes;
         curr_colorbar = this_colorbar(1,j);
         break;
      end;
   end;
end;
%
axes_copy_menu = uimenu('Parent',axes_context_menu,'Separator',paste_sep,'label','Paste');
   uimenu('Parent',axes_copy_menu,'Label','Line','Enable',line_copy_enable,'Callback','paste_line_fcn');
   uimenu('Parent',axes_copy_menu,'Label','Patch','Enable',patch_copy_enable,'Callback','paste_patch_fcn');
   uimenu('Parent',axes_copy_menu,'Label','Surface','Enable',surface_copy_enable,'Callback','paste_surface_fcn');
if (size(myaxes,2)>1)|((isempty(curr_colorbar)==0)&(ishandle(curr_colorbar)==1)&(strcmp(get(curr_colorbar,'Visible'),'on')==1));
   uimenu('Parent',axes_context_menu,'Label','Copy Subplot','Callback','copy_subplot_fcn');
end;
if (isempty(surfhandles)==1)&(x_zeros==0);
   uimenu('Parent',axes_context_menu,'Label','Y Axis Zero Line','Callback','x_zeros_fcn');
end;
if (isempty(surfhandles)==1)&(y_zeros==0);
   uimenu('Parent',axes_context_menu,'Label','X Axis Zero Line','Callback','y_zeros_fcn');
end;
%
% check for legend
%
if (isempty(get(this_axes,'UserData'))==1)&((isempty(linehandles)==0)|(isempty(patchhandles)==0)|(isempty(surfhandles)==0));
   uimenu('Parent',axes_context_menu,'Separator','on','Label','Add Legend','Callback','add_legend_fcn');
   if size(myaxes,2)>1;
      uimenu('Parent',axes_context_menu,'Label','Delete Axes','Callback','delete_axes_fcn');
   end;
elseif (isempty(get(this_axes,'UserData'))==0)&((isempty(linehandles)==0)|(isempty(patchhandles)==0)|(isempty(surfhandles)==0));
   uimenu('Parent',axes_context_menu,'Separator','on','Label','Delete Legend','Callback','delete_legend_fcn');
   if size(myaxes,2)>1;
      uimenu('Parent',axes_context_menu,'Label','Delete Axes','Callback','delete_axes_fcn');
   end;
else
   if size(myaxes,2)>1;
      uimenu('Parent',axes_context_menu,'Label','Delete Axes','Separator','on','Callback','delete_axes_fcn');
   end;
end;
if strcmp(this_label,'Loc&k Axes')==1;
   labeltxt = 'Lock Axes';
else;
   labeltxt = 'Unlock Axes';
end;
uimenu('Parent',axes_context_menu,'Separator','on','Label',labeltxt,'Callback','toolbar_lock_fcn');
set(this_axes,'UIContextMenu',axes_context_menu);

%
% end of create_axes_contextmenu_fcn.m
%

