%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying graphs of data       %
%                                                                %
%          Function: epsilondlg                                  %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       c_mod: structure with model info         %
%                       restricted: 0-1 integer indicating if    %
%                           the beta vectors are restricted      %
%                                                                %
%          Output:                                               %
%                       h: handle                                %
%                                                                %
%          This version:        June 13, 2006                    %
%          Written by:          Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = epsilondlg(selector,c_mod,restricted)

if strcmp(selector,'init')==1;
   t_m = c_mod;
   t_m.restricted = restricted;
   t_m.Z0est = t_m.Z0-t_m.epsilon;
   t_m.normepsilon = normresid(t_m.epsilon);
   %
   % setting up the actual sample period
   %
   [t_m.dataindex,t_m.samplebeg,t_m.sampleend] = make_dataindex(t_m.freq,t_m.samplebeg,t_m.sampleend);
   checkyear = floor(t_m.samplebeg);
   checkperiod = t_m.samplebeg-checkyear;
   t_m.subbeginyear = num2str(checkyear);
   if strcmp(t_m.freq,'q')==1;
      checkperiod = 4*(checkperiod+(1/8));
      t_m.subbeginperiod = num2str(round(checkperiod));
   elseif strcmp(t_m.freq,'m')==1;
      checkperiod = 12*(checkperiod+(1/24));
      t_m.subbeginperiod = num2str(round(checkperiod));
   end;
   %
   % check if t_m.evars is empty and if so generate
   % generic names
   %
   if strcmp(t_m.evars,'')==1;
      for i=1:t_m.n;
         if i==1;
            t_m.evars = 'y_1';
         else;
            t_m.evars = [t_m.evars ' y_' num2str(i)];
         end;
      end;
   end;
   t_m.namevector = vecstr(t_m.evars);
   thisexit = 0;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
else;
   thisexit = 1;
   epsilongui = get(0,'CurrentFigure');
   epsi = get(epsilongui,'UserData');
   curr_ini = get(epsi.popup,'UserData');
   t_m = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      epsilongui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','epsilondlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-341)/2 (realscrsz(4)-140)/2 341 140], ...
         'Tag','FigParent', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Graph Residuals', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','epsilondlg done', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(epsilongui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 339 130],'Graphics',45,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 80 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.histogrambig_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      epsi.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 200 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',['Estimation Sample:  ' t_m.subbeginyear ':' t_m.subbeginperiod ' - ' t_m.subendyear ':' t_m.subendperiod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');

      %
      % Select variable
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 80 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Select Variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');


      epsi.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[120 20 80 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',t_m.namevector, ...
         'HorizontalAlignment','center', ...
         'Tag','GraphListbox',...
         'Value',1);

      %
      % Create graph button
      %

      epsi.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 55 90 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','epsilondlg showgraph', ...
         'Enable','on', ...
         'UserData','working', ...
         'Tag','Done');

      %
      %  Done button
      %

      epsi.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 20 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','epsilondlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(epsilongui,'UserData',epsi,...
         'HandleVisibility','callback');

   case 'showgraph'
      series = get(epsi.popup,'Value');
      newvalue = series+1;
      if newvalue>t_m.n;
         set(epsi.popup,'Value',1);
      else;
         set(epsi.popup,'Value',newvalue);
      end;
      if curr_ini.colors==1;
         colorstr = 'b';
         colorstr2 = 'r';
         colorstr3 = (1/255)*[255 255 177];
         typestr = '-';
      else;
         colorstr = 'k';
         colorstr2 = [0.5 0.5 0.5];
         colorstr3 = [0.6 0.6 0.6];
         typestr = '--';
      end;
      z0 = t_m.Z0(series,:)';
      z0est = t_m.Z0est(series,:)';
      resid = t_m.normepsilon(series,:)';
      resepsilon = t_m.epsilon(series,:);
      variable = t_m.namevector(series,:);
      variable = strrep(variable,' ','');
      myplot = figure('Units','pixels', ...
         'Position',[(realscrsz(3)-612)/2 (realscrsz(4)-484)/2 612 484], ...
         'Menubar','none', ...
         'Toolbar','none', ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'Name',['Residuals for ' variable]);
      %
      % default figure properties
      %
      setfigureproperties(myplot);
      %
      %
      % Setting up the plots
      %
      subplot(2,2,1);
      %
      % check if we should add a zero line
      %
      if ((max(z0)>0)|(max(z0est)>0))&((min(z0)<0)|(min(z0est)<0));
         plot([min(t_m.dataindex) max(t_m.dataindex)],zeros(2,1),'-','LineWidth',1,'Color',[0 0 0]);
         hold('on');
      end;
      first_line = plot(t_m.dataindex,z0,'-','LineWidth',1,'Color',colorstr,'Tag',['d' variable]);
      hold('on');
      second_line = plot(t_m.dataindex,z0est,typestr,'LineWidth',1,'Color',colorstr2,'Tag',['est-d' variable]);
      set(gca,'XLim',[t_m.dataindex(1,1) t_m.dataindex(size(t_m.dataindex,1),1)]);
      %
      setaxesfonts('Title',['Actual and estimated data on \Delta' variable ]);
      myleg = legend_fcn([first_line second_line],{'Actual' 'Estimated'},1);
      setaxesfonts(myleg);
      set(gca,'UserData',myleg);
      axeshandles = gca;
      hold('off');
      %
      subplot(2,2,2);
      if max(abs(resid))>=2.5;
         %
         % plot upper and lower 2.5 bounds
         %
         plot([min(t_m.dataindex) max(t_m.dataindex)],[2.5 2.5],'-','LineWidth',1,'Color','k','Tag','nolegend-upper');
         hold('on');
         plot([min(t_m.dataindex) max(t_m.dataindex)],[-2.5 -2.5],'-','LineWidth',1,'Color','k','Tag','nolegend-lower');
         hold('on');
      end;
      plot(t_m.dataindex,resid,'-','LineWidth',1,'Color',colorstr,'Tag',['norm-resid-d' variable]);
      hold('on');
      plot(t_m.dataindex,zeros(t_m.T,1),'-','LineWidth',1,'Color','k');
      set(gca,'XLim',[t_m.dataindex(1,1) t_m.dataindex(size(t_m.dataindex,1),1)]);
      setaxesfonts('Title',['Normalized residuals for \Delta' variable]);
      axeshandles = [axeshandles gca];
      hold('off');
      %
      subplot(2,2,3);
      resmax = max(resid)+0.5;
      resmin = min(resid)-0.5;
      numcells = ceil(t_m.T/6);
      [n1,n2] = hist(resid,numcells);
      n1 = n1/t_m.T;
      n2i = n2(1,2)-n2(1,1);
      n2smin = min(n2);
      n2smax = max(n2);
      if n2smax+n2smin>=0;
         n2s = -n2smax-n2i:n2i:n2smax+n2i;
      else;
         n2s = n2smin-n2i:n2i:-n2smin+n2i;
      end;
      n4 = [];
      sizen2s = size(n2s,2);
      for i=1:sizen2s-1;
         n4i = norm_cdf(n2s(1,i+1),0,1)-norm_cdf(n2s(1,i),0,1);
         n4 = [n4 n4i];
      end;
      n4 = [n4 (norm_cdf(sizen2s+n2i,0,1)-norm_cdf(sizen2s,0,1))];
      n4 = (1/sum(n4))*n4;
      n5 = n2s-n2i/3;
      n6 = bar(n5,n4,0.75);
      hold('on');
      n3 = bar(n2,n1,0.75);
      set(gca,'XLim',[min(n2s)-2*n2i max(n2s)+2*n2i]);
      set(gca,'YLim',[0 max(max([n1 n4]))+0.01]);
      if curr_ini.colors==1;
         set(n6,'FaceColor',[0 0 0.4],'EdgeColor','k','LineWidth',1,'Tag','distpatch');
         set(n3,'FaceColor',[0 0 0.8],'EdgeColor','k','LineWidth',1,'Tag','distpatch');
      else;
         set(n6,'FaceColor',[0.2 0.2 0.2],'EdgeColor','k','LineWidth',1,'Tag','distpatch');
         set(n3,'FaceColor',[0.5 0.5 0.5],'EdgeColor','k','LineWidth',1,'Tag','distpatch');
      end;
      hold('off');
      setaxesfonts('Title',['Histogram for normalized residuals of \Delta' variable]);
      myleg = legend_fcn('N(0,1)',['\Delta' variable],1);
      setaxesfonts(myleg);
      set(gca,'UserData',myleg);
      axeshandles = [axeshandles gca];
      %
      subplot(2,2,4);
      [correps,stdeps] = correlate(resepsilon);
      upper_c = correps+stdeps;
      lower_c = correps-stdeps;
      myx = 1:size(correps,2);
      myxtra = 0:size(correps,2)+1;
      %
      c_val = norm_inv((100+t_m.c)/200);
      upper_p = c_val*stdeps';
      lower_p = -c_val*stdeps';
      %
      % Using a patch to display the confidence region
      %
      if curr_ini.colors==1;
         colorstr2 = (1/255)*[255 255 177];
      else;
         colorstr2 = [0.6 0.6 0.6];
      end;
      my_p = patch([myx reverse(myx)],[upper_p' reverse(lower_p')],colorstr2);
      set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
      hold('on');
      %
      % Plot a zero line
      %
      plot(myxtra',zeros(size(myxtra,2),1),'-','LineWidth',1,'Color','k');
      hold('on');
      m1 = bar(myx',correps',0.75);
      hold('on');
      %
      if curr_ini.colors==1;
         set(m1,'FaceColor',[0 0 0.8],'EdgeColor','k','LineWidth',1,'Tag','autopatch');
      else;
         set(m1,'FaceColor',[0.5 0.5 0.5],'EdgeColor','k','LineWidth',1,'Tag','autopatch');
      end;
      mini = min([lower_c -c_val*stdeps]);
      maxi = max([upper_c c_val*stdeps]);
      if maxi+mini>=0;
         mini = -maxi;
      else;
         maxi = abs(mini);
      end;
      set(gca,'YLim',[mini-0.1 maxi+0.1]);
      set(gca,'XLim',[0 size(correps,2)+1],'Layer','top');
      setaxesfonts('Title',['Partial autocorrelation function for residuals of \Delta' variable]);
      axeshandles = [axeshandles gca];
      toolbar_fcn(axeshandles);
      %
      % create a cross-plot for actual and fitted values
      %
      next_plot = figure('Units','pixels', ...
         'Position',[((realscrsz(3)-612)/2)+22 ((realscrsz(4)-484)/2)+78 512 384], ...
         'Menubar','none', ...
         'Toolbar','none', ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'Name',['Scatter plot of actual and fitted values for ' variable]);
      %
      % default figure properties
      %
      setfigureproperties(next_plot);
      %
      if curr_ini.colors==1;
         colorstr1 = [1 0 0];
      else;
         colorstr1 = [0.3 0.3 0.3];
      end;
      %
      x_var = z0;
      y_var = z0est;
      new_line = plot(x_var,y_var,'-');
      set(new_line,'Color',get(gca,'Color'), ...
         'LineWidth',1, ...
         'Marker','+', ...
         'MarkerSize',6, ...
         'MarkerFaceColor',get(gca,'Color'), ...
         'MarkerEdgeColor',colorstr1, ...
         'Tag',['scatter-' strrep(variable,' ','')]);
      %
      hold('on');
      %
      % check if lines should be added below the scatter plot
      %
      y_min = min(y_var);
      y_max = max(y_var);
      if y_min<0;
         y_min = 1.01*y_min;
      else;
         y_min = 0.99*y_min;
      end;
      if y_max>0;
         y_max = 1.01*y_max;
      else;
         y_max = 0.99*y_max;
      end;
      %
      x_min = min(x_var);
      x_max = max(x_var);
      if x_min<0;
         x_min = 1.01*x_min;
      else;
         x_min = 0.99*x_min;
      end;
      if x_max>0;
         x_max = 1.01*x_max;
      else;
         x_max = 0.99*x_max;
      end;
      if (x_min<0)&(x_max>0);
         zero_line_x = plot([0 0],[min(y_min,x_min) max(y_max,x_max)],'-','LineWidth',1,'Color',[0 0 0],'Tag','nolegend-x');
         hold('on');
      end;
      if (y_min<0)&(y_max>0);
         zero_line_y = plot([min(y_min,x_min) max(y_max,x_max)],[0 0],'-','LineWidth',1,'Color',[0 0 0],'Tag','nolegend-y');
         hold('on');
      end;
      %
      % make the limits symmetric
      %
      fix_y = max(abs(y_max-max(y_var)),abs(min(y_var)-y_min));
      y_lim = [min(y_var)-fix_y max(y_var)+fix_y];
      %
      % make the limits symmetric
      %
      fix_x = max(abs(x_max-max(x_var)),abs(min(x_var)-x_min));
      x_lim = [min(x_var)-fix_x max(x_var)+fix_x];
      %
      my_lim = [min(x_lim(1),y_lim(1)) max(x_lim(2),y_lim(2))];
      %
      % create 45 degree line
      %
      forty_five_line = plot(my_lim,my_lim,'-','LineWidth',1,'Color',[0 0 0],'Tag','nolegend-xy');
      hold('on');
      %
      set(gca,'XLim',my_lim,'YLim',my_lim,'Layer','top');
      %
      if curr_ini.ytick==1;
         fixytix(gca);
      end;
      if curr_ini.xtick==1;
         fixxtix(gca);
      end;
      hold('off');
      setaxesfonts('Title',['Scatter plot of actual and fitted values of \Delta' variable],'XLabel','Actual','YLabel','Fitted');
      toolbar_fcn(gca);
      if curr_ini.external==0;
         set([myplot next_plot],'Visible','on');
      else;
         useres = '';
         if t_m.restricted==1;
            useres = 'rest-';
         end
         file = [t_m.outputdir useres 'residual-' variable];
         file_1 = [t_m.outputdir useres 'scatter-' variable];
         set(0,'CurrentFigure',myplot);
         mstat = autosavefigure(myplot,file);
         set(0,'CurrentFigure',next_plot);
         mstat_1 = autosavefigure(next_plot,file_1);
         if mstat==0;
            autoshowfigure(file);
         end;
         if mstat_1==0;
            autoshowfigure(file_1);
         end;
      end;
      set(epsilongui,'UserData',epsi);
   case 'done'
      set(epsi.done,'UserData','done');
end;

if nargout==1;
   h = epsilongui;
end;

if strcmp(selector,'done')==0;
   curr_ini.models = t_m;
   set(epsi.popup,'UserData',curr_ini);
   set(epsilongui,'UserData',epsi);
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(epsilongui,'Name') ' ' fix_filepath([pwd '\icons\residual.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(epsi.icon,'UserData',pid);
      end;
   end;
   %
   set(epsilongui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(epsilongui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(epsi.done);
   end;
   waitfor(epsi.done,'UserData','done');
end;

%
% end of epsilondlg.m
%
