function [alpha,beta,Psi,phi,kappa,epsilon_n,info_mat,k_orth,abort] = estimate_proportionality(Z0,Z1,Z2,D,epsilon,p,pe,d,q1,q0,dcoint,alpha,beta,Psi,modelsdir,modelfile,convcrit,maxit,reestbeta,restrict,showwaitbar)

%
% The model is: Z0 = phi*kappa'*D + alpha*beta'*Z1 + Psi*Z2 + epsilon
%
if size(restrict,2)==2;
   restrict = [restrict 0];
end;

if convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(convcrit)];
end;
ccrit = mystr2num(ch_convcrit);

[n T] = size(Z0);
r = size(beta,2);
%
% setting up reduced rank matrices for estimation of phi and kappa
%
if r>0;
   Z1t = [beta'*Z1;Z2];
else;
   Z1t = Z2;
end;
%
M00 = (1/T)*Z0*Z0';
M01t = (1/T)*Z0*Z1t';
M11t = (1/T)*Z1t*Z1t';
M10t = M01t';
M0d = (1/T)*Z0*D';
M1dt = (1/T)*Z1t*D';
Md1t = M1dt';
Md0 = M0d';
Mdd = (1/T)*D*D';
%
if isempty(Z1t)==0;
   S00t = M00-(M01t*inverse(M11t)*M10t);
   S11t = Mdd-(Md1t*inverse(M11t)*M1dt);
   S01t = M0d-(M01t*inverse(M11t)*M1dt);
   S10t = S01t';
else;
   S00t = M00;
   S11t = Mdd;
   S01t = M0d;
   S10t = M0d';
end;
Md1 = (1/T)*D*Z1';
M11 = (1/T)*Z1*Z1';
if isempty(Z2)==0;
   Md2 = (1/T)*D*Z2';
   M22 = (1/T)*Z2*Z2';
   M12 = (1/T)*Z1*Z2';
   M21 = M12';
else;
   Md2 = [];
   M22 = [];
end;

%
% rewrite the eigenvalue problem
%
A_S11t = symm(S11t);
S_adj = inverse(A_S11t)*S10t*inverse(S00t)*S01t*inverse(A_S11t);
%
% compute the eigenvalues
%
[U,lambda] = eig(S_adj);
lambda = diag(lambda,0);
[lambda2,AscInd] = sort(abs(lambda));
%
nr = size(AscInd,1);
DescInd = zeros(nr,1);
for j=1:nr;
   DescInd(j)=AscInd(nr+1-j);
end;
%
% Resort eigenvalues and eigenvectors
% in descending order
%
lambda = abs(lambda(DescInd));
U = U(:,DescInd);
V = inverse(S_adj)*U;
kappa = V(:,1);
[norm_par,norm_index] = max(abs(kappa));
kappa = (1/kappa(norm_index,1))*kappa;
phi = S01t*kappa*inverse(kappa'*S11t*kappa);
%
% restrictions matrices for kappa
%
k_mat = zeros(size(kappa,1),1);
k_mat(norm_index,1) = 1;
k_orth = null(k_mat','r');
%
% collect all restrictions on alpha, Psi and beta
%
if restrict(1,2)==1;
   for i=1:r;
      file = strcat(modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.rest');
      if fileexist(file)==1;
         h = loaddata(file);
         if i==1;
            bH = h;
         else;
            bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
         end;
      end;
   end;
elseif (restrict(1,2)==0)&(r>0);
   bH = eye((n+q1+dcoint)*r);
end;
%
if restrict(1,1)==1;
   file = strcat(modelsdir,'G-',int2str(n),'x',int2str(r),'-',modelfile,'.rest');
   G = loaddata(file);
   [Rmat,rvec] = rmatrix(G,n,r);
   dfalpha = size(G,1);
   if isempty(Z2)==0;
      Rmat = [zeros(dfalpha,size(Z2,1)*n) Rmat];
   end;
   Rmat_f = [zeros(dfalpha,n) Rmat];
else;
   Rmat = [];
end;
%
R_mat = Rmat;
%
if (size(restrict,2)>2)&(restrict(1,3)==1);
   %
   % collect C restrictions
   %
   file = strcat(modelsdir,'C-',int2str(n),'x',int2str(r),'-',modelfile,'.rest');
   G = loaddata(file);
   [Rmat_C,rvec_C] = rmatrix(G,n,n);
   %
   % linearize them wrt Psi and alpha
   %
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,0);
   else;
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,p,pe,r,d,q1,q0);
   R_mat = [R_mat;Rmat_C*kron(xi',C)];
else;
   Rmat_C = [];
end;
%
if isempty(R_mat)==0;
   R_orth = null(R_mat,'r');
else;
   R_orth = eye((size(Z2,1)+r)*n);
end;
%
% check the gradient
%
epsilon_n = Z0-(phi*kappa'*D);
if r>0;
   epsilon_n = epsilon_n-(alpha*beta'*Z1);
end;
if isempty(Z2)==0;
   epsilon_n = epsilon_n-(Psi*Z2);
end;
Omega = (1/T)*epsilon_n*epsilon_n';

grad = [];
if r>0;
   M_e1 = (1/T)*epsilon_n*Z1';
   %
   % partials wrt beta and alpha (requires additional matrices if there are restrictions)
   %
   grad = T*(vec(alpha))'*kron(eye(r),inverse(Omega)*M_e1)*bH;
   grad_alpha = T*(vec(eye(n)))'*kron(M_e1*beta,inverse(Omega));
else;
   grad_alpha = [];
end;
if isempty(Z2)==0;
   M_e2 = (1/T)*epsilon_n*Z2';
   grad_alpha = [T*vec(eye(n))'*kron(M_e2,inverse(Omega)) grad_alpha]*R_orth;
   grad = [grad grad_alpha];
else;
   if r>0;
      grad_alpha = grad_alpha*R_orth;
      grad = [grad grad_alpha];
   end;
end;
M_ed = (1/T)*epsilon_n*D';
grad = [grad T*phi'*inverse(Omega)*M_ed*k_orth T*kappa'*M_ed'*inverse(Omega)];
grad = max(abs(grad));
old_grad = grad;
lnL_old = loglike(epsilon_n);

if (showwaitbar==1)&(ccrit<grad);
   curr_ini = find_inisettings;
   shown_waitbar = 1;
   input_var.title = ['Progress for maximum of ' int2str(maxit) ' iterations'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.stop = 1;
   input_var.clock = 0;
   input_var.name = 'ML Estimation of Proportionality';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','ML Estimation of Proportionality');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   shown_waitbar = 0;
end;

abort = '';
it = 1;
while (ccrit<grad)&(it<=maxit);
   if showwaitbar==1;
      abort = get(mywait,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         break;
      else;
         progressdlg([it/maxit grad]);
      end;
   end;
   %
   % start estimation from scratch
   %
   Z2_new = [kappa'*D;Z2];
   if r>0;
      [S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,lambda,m_trace] = mlcointbasic(Z0,Z1,Z2_new,r);
      if restrict(1,2)==1;
         [beta,newlambda,newdfbeta,stdbeta,outputtxt,new_abort] = betarestrict(S_00,S_11,S_01,S_10,beta,T,modelsdir,modelfile,0,dcoint);
      end;
      alpha = (S_01*beta)*inverse(beta'*S_11*beta);
      Psi = (M_02*inverse(M_22))-(alpha*beta'*M_12*inverse(M_22));
      epsilon_n = Z0-(alpha*beta'*Z1)-(Psi*Z2_new);
      Z3 = [Z2_new;beta'*Z1];
      Theta = [Psi alpha];
      %
      % under alpha restrictions
      %
      if restrict(1,1)==1;
         if reestbeta==0;
            [theta,Sigma_theta,infomat,newabort] = linrest(Z0,Z3,Theta,Rmat_f,rvec,0);
         else;
            [theta,Sigma_theta,beta,newstdbeta,Z3,infomat,newabort] = alpha_beta_restrict(Z0,Z1,Z2_new,Z3,beta,Theta,Rmat_f,rvec,dcoint,q1,modelsdir,modelfile,0);
         end;
         g2 = size(theta,2);
         alpha = theta(:,g2-r+1:g2);
         Psi = theta(:,1:g2-r);
         epsilon_n = Z0-(alpha*beta'*Z1)-(Psi*Z2_new);
      end;
   else;
      M_02 = (1/T)*Z0*Z2_new';
      M_22 = (1/T)*Z2_new*Z2_new';
      Psi = M_02*inverse(M_22);
      alpha = [];
      beta = [];
      epsilon_n = Z0-(Psi*Z2_new);
   end;
   %
   % under C restrictions
   %
   if (size(restrict,2)>2)&(restrict(1,3)==1);
      [theta,Sigma_theta,beta,stdbeta,C_n,Sigma_C,num_c_rest,num_a_rest,infomat,newabort] = c_restrict(Z0,Z1,Z2_new,restrict,beta,[Psi alpha],Rmat_C,rvec_C,p,pe,d+1,dcoint,q0,q1,modelsdir,modelfile,0);
      g2 = size(theta,2);
      if r>0;
         alpha = theta(:,g2-r+1:g2);
         Psi = theta(:,1:g2-r);
         epsilon_n = Z0-(alpha*beta'*Z1)-(Psi*Z2_new);
      else;
         Psi = theta;
         epsilon_n = Z0-(Psi*Z2_new);
      end;
   end;
   phi = Psi(:,1);
   if size(Psi,2)>1;
      Psi = Psi(:,2:size(Psi,2));
   else;
      Psi = [];
   end;
   %
   % now we have estimates conditional on the previous kappa. use these to estimate kappa from the
   % first order conditions of the log-likelihood function wrt kappa
   %
   Omega_n = (1/T)*epsilon_n*epsilon_n';
   Md_fix = Md0;
   if r>0;
      Md_fix = Md_fix-(Md1*beta*alpha');
   end;
   if isempty(Md2)==0;
      Md_fix = Md_fix-(Md2*Psi');
   end;
   %
   % compute kappa and phi that satisfy the first order conditions
   %
   it_in = 1;
   in_grad = 1;
   while (in_grad>ccrit)&(it_in<=maxit);
      Md_new = Md_fix-(Mdd*k_mat*phi');
      psi = inverse(k_orth'*Mdd*k_orth)*k_orth'*Md_new*inverse(Omega_n)*phi*inverse(phi'*inverse(Omega_n)*phi);
      kappa = k_mat+(k_orth*psi);
      phi = Md_fix'*kappa*inverse(kappa'*Mdd*kappa);
      %
      epsilon_n = Z0-(phi*kappa'*D);
      if r>0;
         epsilon_n = epsilon_n-(alpha*beta'*Z1);
      end;
      if isempty(Z2)==0;
         epsilon_n = epsilon_n-(Psi*Z2);
      end;
      Omega = (1/T)*epsilon_n*epsilon_n';
      Omega_n = Omega;
      M_ed = (1/T)*epsilon_n*D';
      in_grad = [T*phi'*inverse(Omega)*M_ed*k_orth T*kappa'*M_ed'*inverse(Omega)];
      in_grad = max(abs(in_grad));
      it_in = it_in+1;
   end;
   %
   % compute the gradient
   %
%   R_mat = Rmat;
%   if (size(restrict,2)>2)&(restrict(1,3)==1);
      %
      % linearize them wrt Psi and alpha
      %
%      if r<n;
%         C = cmat(Psi,alpha,beta,epsilon_n,Z1,Z2,p,d,0);
%      else;
%         C = cmat_stat(Psi,alpha,beta,epsilon_n,Z1,Z2,p,d,0,[]);
%      end;
%      xi = xi_mat(C,Psi,alpha,p,pe,r,d,q1,q0);
%      R_mat = [R_mat;Rmat_C*kron(xi',C)];
%   end;
%   if isempty(R_mat)==0;
%      R_orth = null(R_mat,'r');
%   else;
%      R_orth = eye((size(Z2,1)+r)*n);
%   end;
%   grad = [];
%   if r>0;
%      M_e1 = (1/T)*epsilon_n*Z1';
      %
      % partials wrt beta and alpha (requires additional matrices if there are restrictions)
      %
%      grad = T*(vec(alpha))'*kron(eye(r),inverse(Omega)*M_e1)*bH;
%      grad_alpha = T*(vec(eye(n)))'*kron(M_e1*beta,inverse(Omega));
%   else;
%      grad_alpha = [];
%   end;
%   if isempty(Z2)==0;
%      M_e2 = (1/T)*epsilon_n*Z2';
%      grad_alpha = [T*vec(eye(n))'*kron(M_e2,inverse(Omega)) grad_alpha]*R_orth;
%      grad = [grad grad_alpha];
%   else;
%      if r>0;
%         grad_alpha = grad_alpha*R_orth;
%         grad = [grad grad_alpha];
%      end;
%   end;
%   M_ed = (1/T)*epsilon_n*D';
%   grad = [grad T*phi'*inverse(Omega)*M_ed*k_orth T*kappa'*M_ed'*inverse(Omega)];
%   grad = max(abs(grad));
   %
   lnL_new = loglike(epsilon_n);
   if (restrict(1,1)==0)&(restrict(1,3)==0);
      grad = abs(T*T*lnL_new*(lnL_new-lnL_old));
   else;
      grad = abs(lnL_new-lnL_old);
   end;
   lnL_old = lnL_new;
   it = it+1;
end;

if shown_waitbar==1;
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;

if strcmp(abort,'cancel')==0;
   %
   % compute the information matrix in terms of phi, kappa, Psi, alpha,
   %
   M_phi_phi = T*kron(kappa'*Mdd*kappa,inverse(Omega));
   M_phi_kappa = T*kron(kappa'*Mdd*k_orth,inverse(Omega)*phi);
   M_kappa_phi = M_phi_kappa';
   M_kappa_kappa = T*kron(k_orth'*Mdd*k_orth,phi'*inverse(Omega)*phi);
   if isempty(Z2)==0;
      M_phi_Psi = T*kron(kappa'*Md2,inverse(Omega));
      M_kappa_Psi = T*kron(k_orth'*Md2,phi'*inverse(Omega));
      M_Psi_phi = M_phi_Psi';
      M_Psi_kappa = M_kappa_Psi';
      M_Psi_Psi = T*kron(M22,inverse(Omega));
      if r>0;
         M_Psi_alpha = T*kron(M21*beta,inverse(Omega));
         M_alpha_Psi = M_Psi_alpha';
      else;
         M_Psi_alpha = [];
         M_alpha_Psi = [];
      end;
   else;
      M_phi_Psi = [];
      M_kappa_Psi = [];
      M_Psi_phi = [];
      M_Psi_kappa = [];
      M_Psi_Psi = [];
      M_Psi_alpha = [];
      M_alpha_Psi = [];
   end;
   if r>0;
      M_phi_alpha = T*kron(kappa'*Md1*beta,inverse(Omega));
      M_kappa_alpha = T*kron(k_orth'*Md1*beta,phi'*inverse(Omega));
      M_alpha_alpha = T*kron(beta'*M11*beta,inverse(Omega));
      M_alpha_phi = M_phi_alpha';
      M_alpha_kappa = M_kappa_alpha';
   else;
      M_phi_alpha = [];
      M_kappa_alpha = [];
      M_alpha_alpha = [];
      M_alpha_phi = [];
      M_alpha_kappa = [];
   end;
   %
   my_hess = [[M_phi_phi M_phi_kappa M_phi_Psi M_phi_alpha];[M_kappa_phi M_kappa_kappa M_kappa_Psi M_kappa_alpha];[M_Psi_phi M_Psi_kappa M_Psi_Psi M_Psi_alpha];[M_alpha_phi M_alpha_kappa M_alpha_Psi M_alpha_alpha]];
   %
   % make sure that restrictions on Psi, alpha are handled
   %
   if isempty(Rmat)==0;
      R_mat = [zeros(size(Rmat,1),size(phi,1)+size(kappa,1)-1) Rmat];
   else;
      R_mat = [];
   end;
   if (size(restrict,2)>2)&(restrict(1,3)==1);
      %
      % linearize them wrt Psi and alpha
      %
      if r<n;
         C = cmat(Psi,alpha,beta,epsilon_n,Z1,Z2,p,d,0);
      else;
         C = cmat_stat(Psi,alpha,beta,epsilon_n,Z1,Z2,p,d,0,[]);
      end;
      xi = xi_mat(C,Psi,alpha,p,pe,r,d,q1,q0);
      R_mat_C = Rmat_C*kron(xi',C);
      R_mat_C = [zeros(size(R_mat_C,1),size(phi,1)+size(kappa,1)-1) R_mat_C];
      R_mat = [R_mat;R_mat_C];
   end;
   if isempty(R_mat)==0;
      R_orth = null(R_mat,'r');
   else;
      R_orth = eye(size(my_hess,1));
   end;
   %
   info_mat = R_orth*inverse(R_orth'*my_hess*R_orth)*R_orth';
else;
   info_mat = [];
end;

%
% end of estimate_proportionality.m
%
