function [f_err,f_err_decomp,abort] = hist_decomp(mm,fh,alpha,beta,Psi,B,epsilon,waitbar)

if nargin<8;
   waitbar = 0;
end;
r = size(alpha,2);

shocks = inverse(B)*epsilon;

%
% constructing matrices for determining the within sample forecast errors
%

if r>0;
   Pi = alpha*beta(1:mm.n,:)';
else;
   Pi = zeros(mm.n,mm.n);
end;
if mm.p>=2;
   Phi = Psi(:,mm.d+1:mm.d+mm.n*(mm.p-1));
end;

ip = zeros(mm.p,mm.p);
for i=1:mm.p;
   for j=1:mm.p;
      if j>=i;
         ip(j,i)=1;
      end;
   end;
end;

%
% Computing levels VAR coefficients from
% VECM coefficients
%

ip = kron(ip,eye(mm.n));
J = eye(mm.n);
if mm.p>=2;
   M = [(Pi+eye(mm.n)) -Phi];
   M = M/ip;
   M = [M;[eye(mm.n*(mm.p-1)) zeros(mm.n*(mm.p-1),mm.n)]];
   J = [J zeros(mm.n,mm.n*(mm.p-1))];
else;
   M = (Pi+eye(mm.n))/ip;
end;

%
% computing the forecast error matrices
%

for i=1:fh;
   if i==1;
      M_i = eye(size(M,1));
   else;
      M_i = M_i*M;
   end;
   fe.lag(i).B = J*M_i*J';
end;

%
% setting up a progress dialog
%

if waitbar==1;
   curr_ini = find_inisettings;
   realscrsz = get(0,'ScreenSize');
   input_var.title = ['Progress for ' int2str(mm.T-fh+1) ' forecast error decompositions'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.label = 'Iteration number:';
   input_var.stop = 0;
   input_var.clock = 0;
   input_var.name = 'Historical Decompositions';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','Historical Decompositions');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;


%
% starting the forecast error computations
%

f_err = zeros(mm.n,mm.T-fh+1);
f_err_decomp = zeros(mm.n*mm.n,mm.T-fh+1);
abort = '';
shock_mat = eye(mm.n);
for t=fh:mm.T;
   if waitbar==1;
      if t==fh;
         drawnow;
         progressdlg([(1/mm.h) 1]);
      else;
         abort = get(mywait,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            progressdlg([((t-fh+1)/(mm.T-fh+1)) t-fh+1]);
         end;
      end;
   end;
   %
   f_err_t = f_err(:,t-fh+1);
   f_err_decomp_t = zeros(mm.n*mm.n,1);
   for i=1:fh;
      f_err_t = f_err_t+(fe.lag(i).B*epsilon(:,t-i+1));
      %
      % individual shock j
      %
      f_err_help = [];
      for j=1:mm.n;
         shock_vec = fe.lag(i).B*B*shock_mat(:,j)*shocks(j,t-i+1);
         f_err_help = [f_err_help;shock_vec];
      end;
      f_err_decomp_t = f_err_decomp_t+f_err_help;
   end;
   f_err(:,t-fh+1) = f_err_t;
   f_err_decomp(:,t-fh+1) = f_err_decomp_t;
end;

%
% premultiplying f_err_decomp with [eye(mm.n) ... eye(mm.n)] (mm.n x mm.x^2) yields f_err
% verifying that the computations are correct
%

if waitbar==1;
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;

%
% end of hist_decomp.m
%
