%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for selecting files from list       %
%                                                                %
%          Function: list_files_dlg                              %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       files: string matrix                     %
%                       name: figure title                       %
%                       axlist: axesbox title                    %
%                       axlength: width of axesbox title         %
%                       this_text: string with header text for   %
%                           listbox control                      %
%                       this_image: field in structure giving    %
%                           image data to display                %
%                                                                %
%          Output:                                               %
%                       h = handle                               %
%                                                                %
%          This version:        May 3, 2006                      %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = list_files_dlg(selector,files,name,axlist,axlength,this_text,this_image)

if strcmp(selector,'init')==1;
   thisexit = 0;
   %
   % button icons
   %
   mycons = getstoredimages;
   cu_ini = find_inisettings;
   if cu_ini.use_toolbar_bgcolor==1;
      bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
      bgcolor(1) = max(0,bgcolor(1));
      bgcolor(2) = max(0,bgcolor(2));
      bgcolor(3) = max(0,bgcolor(3));
   else;
      bgcolor = get(0,'defaultuicontrolbackgroundcolor');
   end;
   %
   % current model
   %
   tm.curr_modelfile = cu_ini.model.inputfile;
   %
   % Set screen size directly instead of using
   % get(0,'ScreenSize')!
   %
   realscrsz = get(0,'ScreenSize');
else;
   thisexit = 1;
   listgui = findobj('Type','figure','Tag','ListGUI');
   list_controls = get(listgui,'UserData');
   tm = get(list_controls.allfiles,'UserData');
end;

switch selector;
   case 'init';
      listgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','list_files_dlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-650)/2 (realscrsz(4)-500)/2 660 500], ...
         'Tag','ListGUI', ...
         'Resize','off', ...
         'Name',name, ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','list_files_dlg exit', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(listgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 656 486],axlist,axlength,[0.5 1]);

      %
      % Displaying a delete icon and the simulation directory name
      %

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[20 435 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      image('CData',mirrorimage(maketrans(getfield(mycons,this_image),[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      list_controls.icon = uicontrol('Style','Text', ...
         'String',this_text, ...
         'HorizontalAlignment','left', ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'Position',[20 407 620 20], ...
         'Tag','FilesText');

      %
      % Displaying the files
      %

      list_controls.allfiles = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[20 60 620 350], ...
         'FontSize',cu_ini.gui_fontsize+2, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'Style','listbox', ...
         'Enable','on', ...
         'String',files, ...
         'Max',size(files,1), ...
         'Min',0, ...
         'HorizontalAlignment','center', ...
         'Tag','DeleteFilesListbox',...
         'Value',1);

      %
      % OK and Cancel buttons
      %

      list_controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[235 14 90 20], ...
         'String','OK', ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'CallBack','list_files_dlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      list_controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[335 14 90 20], ...
         'String','Cancel', ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'CallBack','list_files_dlg cancel', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','CancelButton');

      list_controls.string = 'OK';

      set(listgui,'UserData',list_controls,...
         'HandleVisibility','callback');

   case 'cancel';
      list_controls.string = 'Cancel';
      set(list_controls.done,'UserData','done');
      set(listgui,'UserData',list_controls);
   case 'done';
      set(list_controls.done,'UserData','done');
      set(listgui,'UserData',list_controls);
   case 'exit';
      list_controls.string = 'Cancel';
      set(list_controls.done,'UserData','done');
      set(listgui,'UserData',list_controls);
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(cu_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(listgui,'Name') ' ' fix_filepath([pwd '\icons\contents.ico'])];
      dos(prog);
      if (strcmp(cu_ini.pstoolsdir,'')==0)&(appexist([cu_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(list_controls.icon,'UserData',pid);
      end;
   end;
   set(listgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(listgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(list_controls.cancel);
   end;
end;

if nargout==1;
   h = listgui;
end;

if strcmp(selector,'exit')==0;
   set(list_controls.allfiles,'UserData',tm);
end;

if thisexit==0;
   waitfor(list_controls.done,'UserData','done');
end;

%
% end of list_files_dlg.m
%
