%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for selecting restrictions on beta  %
%          from a selected file                                  %
%                                                                %
%          Function: openrestgui                                 %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       cm: structure with model info            %
%                                                                %
%          Output:                                               %
%                       h = handle                               %
%                                                                %
%          This version:        May 18, 2006                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = openrestgui(selector,cm)

if strcmp(selector,'init')==1;
   tm = cm;
   curr_ini = find_inisettings;
   thisexit = 0;
   mycons = getstoredimages;
   bgcolor = get(0,'defaultuicontrolbackgroundcolor');
   mybrowse = maketrans(mycons.browse_image,[160 160 164],bgcolor);
   tm.restfile = tm.modelsdir;
   normfile = strcat(tm.modelsdir,'normalize-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint),'-',tm.modelfile,'.norm');
   if fileexist(normfile)==1;
      tm.normvec = loaddata(normfile);
      n3 = size(tm.normvec,1);
      if n3==tm.r;
         normvar = tm.normvec(1,1);
      elseif n3>tm.r;
         normvar = tm.normvec(1,1);
         tm.normvec = tm.normvec(1:tm.r,1);
      else;
         normvar = 1;
         tm.normvec = [tm.normvec;ones(tm.r-n3,1)];
      end;
   else;
      normvar = 1;
      tm.normvec = ones(tm.r,1);
   end;
   tm.variables = strtrim([tm.evars ' ' strtrim([tm.xvars1 ' ' tm.restvec])]);
   tm.variables = vecstr(tm.variables);
   myvariables = strrep(tm.variables(1,:),' ','');
   for i=2:tm.n+tm.q1+tm.dcoint;
      myvariables = [myvariables '|' strrep(tm.variables(i,:),' ','')];
   end;
   myrest = '1';
   for i=2:tm.r;
      myrest = [myrest '|' int2str(i)];
   end;
else;
   thisexit = 1;
%   openrestdlg = get(0,'CurrentFigure');
   openrestdlg = findobj('Type','figure','Tag','OpenRestBeta');
   openrest = get(openrestdlg,'UserData');
   curr_ini = get(openrest.restfile,'UserData');
   tm = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

scrsz = [1 1 1024 768];
realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      openrestdlg = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','openrestgui.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-512)/2 (realscrsz(4)-460)/2 512 460], ...
         'Tag','OpenRestBeta', ...
         'Resize','off', ...
         'Name','Open Beta Restrictions', ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'CloseRequestFcn','openrestgui exit', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(openrestdlg,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 510 450],'Restrictions on Beta',100,[0.5 1]);

      %
      % GUI Header
      %

      mycons = getstoredimages;
      header_txt = 'Select Beta Restrictions File';
      desc_txt = 'Open a restrictions file (*.rest) by pressing on the "Browse" button and view its content in the display. Select the cointegration vector number, and the variable by which to normalize the vector. Press "Apply" to make setting permanent.';
      dialog_header_fcn([14 365 484 72],mirrorimage(maketrans(mycons.openbetarestbig_image,[160 160 164],[1 1 1])),header_txt,desc_txt);

      %
      % Outer axis for showing restrictions file
      %

      axesbox([14 166 484 182],'Transpose of Restrictions Matrix',155,[0.5 1]);

      %
      % Outer axis for restrictions file
      %

      axesbox([14 80 484 73],'Restrictions Selection',105,[0.5 1]);

      %
      % Outer axis for Normalizing control
      %

      axesbox([14 14 380 52],'Normalization Selection',115,[0.5 1]);

      %
      % Listbox to show restrictions on beta
      %

      openrest.hmat = uicontrol('Units','pixels', ...
         'BackgroundColor','white', ...
         'Position',[30 182 452 150], ...
         'Style','listbox', ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'Max',2, ...
         'Value',[], ...
         'String','', ...
         'Enable','inactive', ...
         'UserData','working', ...
         'HorizontalAlignment','center', ...
         'Tag','BetaListbox');

      %
      % Show restrictions file
      %

      openrest.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      	 'Position',[30 116 150 20], ...
      	 'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
      	 'Style','text', ...
         'HorizontalAlignment','left', ...
      	 'String','Restrictions file:', ...
      	 'Tag','ShowRest');

      openrest.restfile = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[30 96 200 22], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','edit', ...
         'HorizontalAlignment','left', ...
         'String',tm.modelsdir, ...
         'Callback','openrestgui checkrest', ...
         'Enable','on', ...
         'Tag','RestFile');

      %
      % Button for selecting restrictions file
      %

      openrest.restbutton = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[235 96 50 22], ...
         'TooltipString','Browse', ...
         'CData',mybrowse, ...
         'Callback','openrestgui openrest', ...
         'Enable','on', ...
         'Tag','RestButton');

      %
      % show restriction number
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      	 'Position',[330 91 100 20], ...
      	 'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
      	 'Style','text', ...
      	 'String','Restriction Number:', ...
      	 'Tag','RestText');

      %
      % Popup control for selecting restriction number
      %

      openrest.restnumber = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[434 96 50 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Value',1, ...
         'Enable','off', ...
         'String',myrest, ...
         'CallBack','openrestgui changerest', ...
         'Tag','RestPopup');


      %
      % Text control for normalizing beta
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 25 140 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'HorizontalAlignment','left', ...
         'String','Normalizing Beta by variable:', ...
         'Tag','NormalizeText');

      %
      % Popup control for selecting variable to normalize
      %

      openrest.norm = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[175 30 50 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Value',normvar, ...
         'String',myvariables, ...
         'Enable','off', ...
         'CallBack','openrestgui changenorm', ...
         'Tag','NormalizePopup');

      %
      % Use button
      %


      openrest.userest = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[408 46 90 20], ...
         'String','Apply', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','openrestgui userest', ...
         'Enable','off', ...
         'Tag','DoneButton');

      %
      %  Done button
      %

      openrest.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[408 14 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','openrestgui done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      set(openrestdlg,'UserData',openrest,...
         'HandleVisibility','callback');
   case 'openrest'
      if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1)&(matlabnumber<6.0);
         newrestfile = get(openrest.restfile,'String');
         basedir = getpath(newrestfile);
         file_fltr = 'VAR Restrictions Files (*.rest)|*.rest|';
         best_pos = 1;
         title_txt = 'Select file or enter filename';
         [newrestfile,restpath] = fileopen(basedir,file_fltr,best_pos,title_txt);
      else;
         newrestfile = get(openrest.restfile,'String');
         newrestfile = getpath(newrestfile);
         newrestfile = strcat(newrestfile,'*.rest');
         [newrestfile,restpath] = uigetfile(newrestfile, 'Select file or enter filename');
      end;
      drawnow;
      if isnumeric(newrestfile)==0;
         newrestfile = strcat(restpath,newrestfile);
         if fileexist(newrestfile)==1;
            hvar = loaddata(newrestfile);
            [hv1 hv2] = size(hvar);
            if hv1==tm.n+tm.q1+tm.dcoint;
               if hv2<=tm.n+tm.q1+tm.dcoint-tm.r+1;
                  tm.restfile = newrestfile;
                  set(openrest.restfile,'String',tm.restfile);
                  set(openrest.restnumber,'Enable','on');
                  set(openrest.norm,'Enable','on');
                  set(openrest.userest,'Enable','on');
                  hvar = showbeta(hvar,tm.evars,strtrim([tm.xvars1 ' ' tm.restvec]));
                  hv = size(hvar,1);
                  hvec = ' ';
                  for i=1:hv-1;
                     hvec = strvcat(hvec,' ');
                  end;
                  hvar = [hvec hvar];
                  set(openrest.hmat,'String',hvar);
               else;
                  txt = ['The file you selected, ' getname(newrestfile) ', contains too few restrictions.'];
                  about(txt,'information','Too few restrictions...',120,350);
               end;
            else;
               if hv1>tm.n+tm.q1+tm.dcoint;
                  txt = ['The file you selected, ' getname(newrestfile) ', assumes too many variables.'];
                  about(txt,'information','Too many variables...',120,350);
               else;
                  txt = ['The file you selected, ' getname(newrestfile) ', assumes too few variables.'];
                  about(txt,'information','Too few variables...',120,350);
               end;
            end;
         else;
            set(openrest.restfile,'String',tm.restfile);
         end;
      else;
         set(openrest.restfile,'String',tm.restfile);
      end;
      set(openrestdlg,'UserData',openrest);
   case 'checkrest'
      newrestfile = get(openrest.restfile,'String');
      if fileexist(newrestfile)==0;
         set(openrest.restfile,'String',tm.restfile);
         if fileexist(tm.restfile)==0;
            set(openrest.restnumber,'Enable','off');
            set(openrest.norm,'Enable','off');
            set(openrest.userest,'Enable','off');
         else;
            set(openrest.restnumber,'Enable','on');
            set(openrest.norm,'Enable','on');
            set(openrest.userest,'Enable','on');
         end;
      else;
         hvar = loaddata(newrestfile);
         [hv1 hv2] = size(hvar);
         if hv1==tm.n+tm.q1+tm.dcoint;
            if hv2<=tm.n+tm.q1+tm.dcoint-tm.r+1;
               tm.restfile = newrestfile;
               set(openrest.restfile,'String',tm.restfile);
               set(openrest.restnumber,'Enable','on');
               set(openrest.norm,'Enable','on');
               set(openrest.userest,'Enable','on');
               hvar = showbeta(hvar,tm.evars,strtrim([tm.xvars1 ' ' tm.restvec]));
               hv = size(hvar,1);
               hvec = ' ';
               for i=1:hv-1;
                  hvec = strvcat(hvec,' ');
               end;
               hvar = [hvec hvar];
               set(openrest.hmat,'String',hvar);
            else;
               txt = ['The file you selected, ' getname(newrestfile) ', contains too few restrictions.'];
               about(txt,'information','Too few restrictions...',120,350);
               set(openrest.restfile,'String',tm.restfile);
               if fileexist(tm.restfile)==0;
                  set(openrest.restnumber,'Enable','off');
                  set(openrest.norm,'Enable','off');
                  set(openrest.userest,'Enable','off');
               else;
                  set(openrest.restnumber,'Enable','on');
                  set(openrest.norm,'Enable','on');
                  set(openrest.userest,'Enable','on');
               end;
            end;
         else;
            if hv1>tm.n+tm.q1+tm.dcoint;
               txt = ['The file you selected, ' getname(newrestfile) ', assumes too many variables.'];
               about(txt,'information','Too many variables...',120,350);
            else;
               txt = ['The file you selected, ' getname(newrestfile) ', assumes too few variables.'];
               about(txt,'information','Too few variables...',120,350);
            end;
            set(openrest.restfile,'String',tm.restfile);
            if fileexist(tm.restfile)==0;
               set(openrest.restnumber,'Enable','off');
               set(openrest.norm,'Enable','off');
               set(openrest.userest,'Enable','off');
            else;
               set(openrest.restnumber,'Enable','on');
               set(openrest.norm,'Enable','on');
               set(openrest.userest,'Enable','on');
            end;
         end;
      end;
      set(openrestdlg,'UserData',openrest);
   case 'changerest'
      restval = get(openrest.restnumber,'Value');
      normvar = tm.normvec(restval,1);
      set(openrest.norm,'Value',normvar);
      set(openrest.userest,'Enable','on');
      set(openrestdlg,'UserData',openrest);
   case 'changenorm'
      set(openrest.userest,'Enable','on');
      set(openrestdlg,'UserData',openrest);
   case 'userest'
      newnormvec = tm.normvec;
      normvar = get(openrest.norm,'Value');
      restval = get(openrest.restnumber,'Value');
      newnormvec(restval,1) = normvar;
      hmat = loaddata(tm.restfile);
      checknorm = hmat(normvar,:)*hmat(normvar,:)';
      if abs(checknorm)<0.00000000000001;
         txt = ['You have chosen to normalize H(' int2str(restval) ') by ' strrep(tm.variables(normvar,:),' ','') '. This implies that the program should divide the coefficients of Beta(' int2str(restval) ') by 0. Please change your selection or cancel the operation.'];
         about(txt,'warning',['Restrictions on Beta(' int2str(restval) ') - Information'],120,500);
      else;
         if curr_ini.sillyq==0;
            txt = ['Are you sure you want to use these restrictions for beta(' int2str(restval) ')?'];
            answer = query(txt,'question',120,['Save Restrictions on Beta(' int2str(restval) ')...'],350);
         else;
            answer = 'Yes';
         end;
         if strcmp(answer,'Yes');
            tm.normvec = newnormvec;
            normfile = strcat(tm.modelsdir,'normalize-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.norm');
            fid = fopen(normfile,'wt');
            fprintf(fid,'%2.0f\n',tm.normvec');
            fclose(fid);
            newrestfile = strcat(tm.modelsdir,'H_',int2str(restval),'-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.rest');
            myprintstr = '%2.4f';
            for j=2:size(hmat,2);
               myprintstr = [myprintstr ' %2.4f'];
            end;
            myprintstr = [myprintstr '\n'];
            fid = fopen(newrestfile,'wt');
            fprintf(fid,myprintstr,hmat');
            fclose(fid);
            set(openrest.userest,'Enable','off');
            setlogtext(strvcat(' ',['You decided to use "' getfilename(tm.restfile) '"'],['   for restricting beta(' num2str(restval,'%0.0f') ') and to have'],['   ' strrep(tm.variables(normvar,:),' ','') ' as normalizing variable...']));
         end;
      end;
      set(openrestdlg,'UserData',openrest);
   case 'done'
      set(openrest.done,'UserData','done');
   case 'exit'
      set(openrest.done,'UserData','done');
%      if strcmp(get(openrest.hmat,'UserData'),'done')==1;
%         delete(openrestdlg);
%      end;
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(openrestdlg,'Name') ' ' fix_filepath([pwd '\icons\var_restfile.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(openrest.icon,'UserData',pid);
      end;
   end;
   set(openrestdlg,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(openrestdlg,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(openrest.done);
   end;
end;

if nargout==1;
   h = openrestdlg;
end;

if strcmp(selector,'exit')==0;
   curr_ini.models = tm;
   set(openrest.restfile,'UserData',curr_ini);
end;

if thisexit==0;
   waitfor(openrest.done,'UserData','done');
end;

%
% end of openrestgui.m
%
