function [pos,bplot] = parm_dist(pos,fig_tag,fig_name,p_title,est_parm,parm,stdparm,tm,curr_ini,my_structure,T,as_series,boot_series,fix_ax_pos,numcells,col,rep)

%
% est_parm: ML estimated parameter (usually the same as parm, but may be empty)
% parm: parameter
% stdparm: standard error
% as_series: asymptotic series
%

cdf_value = get(my_structure.cdf,'Value');
my_lim = 1-tm.cbs/100;
l_lim = my_lim/2;
u_lim = (tm.cbs/100)+l_lim;

if isempty(as_series)==0;
%   p_dist = parm+(boot_series*sqrt(tm.T/T)*stdparm);
   p_dist = sort(parm-(boot_series*sqrt(tm.T/T)*stdparm));
else;
   p_dist = boot_series;
end;

mean_val = (1/size(p_dist,1))*sum(p_dist);
med_val = p_dist(ceil(0.5*rep),1);
l_val = p_dist(ceil(l_lim*rep),1);
u_val = p_dist(ceil(u_lim*rep),1);

bplot = figure('Units','pixels', ...
   'Position',[pos 600 400], ...
   'NumberTitle','off', ...
   'Menubar','none', ...
   'Toolbar','none', ...
   'Tag',fig_tag, ...
   'Visible','off', ...
   'Name',fig_name);
%
pos = [pos(1)+22 pos(2)-22];
if pos(1,1)>(tm.realscrsz(3)-600);
   pos = [0 tm.realscrsz(4)-470];
end;
if pos(1,2)>(tm.realscrsz(4)-470);
   pos = [0 tm.realscrsz(4)-470];
elseif pos(1,2)<32;
   pos = [0 tm.realscrsz(4)-470];
end;
%
% obey centering and filling defaults
%
setfigureproperties(bplot);
%
[n1,n2] = hist(p_dist,numcells);
n1 = n1/rep;
if (strcmp(tm.interpolate,'nearest')==1)|(strcmp(tm.interpolate,'linear')==1)|(strcmp(tm.interpolate,'spline')==1)|(strcmp(tm.interpolate,'cubic')==1);
   x_v = [];
   xsep = (n2(2)-n2(1))/100;
   for i1=1:size(n2,2);
      x_v = [x_v n2(i1)];
      if i1<size(n2,2);
         for j=1:99;
            x_v = [x_v (n2(i1)+(j*xsep))];
         end;
      end;
   end;
   x_v = sort(x_v);
end;
%
if isempty(as_series)==0;
   b_series = parm+(as_series*sqrt(tm.T/T)*stdparm);
   [t1,t2] = hist(b_series,numcells);
   t1 = t1/10000;
   if (strcmp(tm.interpolate,'nearest')==1)|(strcmp(tm.interpolate,'linear')==1)|(strcmp(tm.interpolate,'spline')==1)|(strcmp(tm.interpolate,'cubic')==1);
      t_v = [];
      tsep = (t2(2)-t2(1))/100;
      for i1=1:size(t2,2);
         t_v = [t_v t2(i1)];
         if i1<size(t2,2);
            for j=1:99;
               t_v = [t_v (t2(i1)+(j*tsep))];
            end;
         end;
      end;
      t_v = sort(t_v);
      t3 = interp1(t2,t1,t_v,tm.interpolate);
      %
      % remove any NaN
      %
      cdf_t3 = cdf(t_v,t3);
      t3 = (1/max(cdf_t3))*t3;
      if cdf_value==1;
         t3 = cdf(t_v,t3);
      end;
   else;
      xmin = min(b_series);
      xmax = max(b_series);
      xrange = xmax-xmin;
      ax = xmin-xrange/4;
      bx = xmax+xrange/4;
      if strcmp(tm.interpolate,'knorm')==1;
         h = 0;
      else;
         h = hns(b_series,tm.interpolate);
      end;
      if get(my_structure.adaptive,'Value')==1;
         [t_v,xa,t3,h,hl] = kdadapt(b_series,tm.interpolate,512,h);
      else;
         [t_v,t3] = kdfft1(b_series,tm.interpolate,512,h,0,[ax,bx]);
      end;
      if cdf_value==1;
         t3 = cdf(t_v,t3);
      end;
   end;
end;
%
ylabelstr = 'Frequency';
leg_pos = 1;
cdf_value = get(my_structure.cdf,'Value');
if (gcbo==my_structure.histogram)|(gcbo==my_structure.histogrammenu);
   if cdf_value==1;
      n1 = cumsum(n1);
      set(bplot,'Name',strrep(fig_name,'Density','Distribution'));
   end;
   %
   n3 = bar(n2,n1,0.75);
   this_axes = gca;
   if curr_ini.colors==1;
      set(n3,'FaceColor',[0 0 0.8],'EdgeColor','k','LineWidth',1,'Tag','autopatch');
   else;
      set(n3,'FaceColor',[0.5 0.5 0.5],'EdgeColor','k','LineWidth',1,'Tag','autopatch');
   end;
   myleg = [];
   if cdf_value==1;
      set(gca,'YLim',[0 1.1],'XLim',[0 max(n2)+n2(1,2)-n2(1,1)]);
   end;
   p_title = strrep(p_title,['with ' num2str(tm.cbs) '% confidence interval '],'');
else;
   ylabelstr = 'Density';
   %
   if (strcmp(tm.interpolate,'nearest')==1)|(strcmp(tm.interpolate,'linear')==1)|(strcmp(tm.interpolate,'spline')==1)|(strcmp(tm.interpolate,'cubic')==1);
      n3 = interp1(n2,n1,x_v,tm.interpolate);
      cdf_n3 = cdf(x_v,n3);
      n3 = (1/max(cdf_n3))*n3;
   else;
      xmin = min(p_dist);
      xmax = max(p_dist);
      xrange = xmax-xmin;
      ax = xmin-xrange/4;
      bx = xmax+xrange/4;
      if strcmp(tm.interpolate,'knorm')==1;
         h = 0;
      else;
         h = hns(p_dist,tm.interpolate);
      end;
      if get(my_structure.adaptive,'Value')==1;
         [x_v,xa,n3,h,hl] = kdadapt(p_dist,tm.interpolate,512,h);
      else;
         [x_v,n3] = kdfft1(p_dist,tm.interpolate,512,h,0,[ax,bx]);
      end;
   end;
   if cdf_value==1;
      ylabelstr = 'Frequency';
      leg_pos = 4;
      n3 = cdf(x_v,n3);
      set(bplot,'Name',strrep(fig_name,'Density','Distribution'));
   end;
   %
   y_lim = [0 1.1*max(n3)];
   %
   my_lines = [];
   %
   % plot confidence band as patch
   %
%   my_p = patch([l_val*ones(1,3) u_val*ones(1,4) l_val],[y_lim(2) y_lim(1)*ones(1,4) y_lim(2)*ones(1,3)],[1 1 1]);
%   set(my_p,'FaceColor',col.vec_9,'EdgeColor',col.vec_9,'Tag','RECTPATCH');
%   hold('on');
   %
   if isempty(as_series)==0;
      if max(t3)>max(n3);
         y_lim = [0 1.1*max(t3)];
      end;
      my_lines = plot(t_v,t3,'LineWidth',1,'Color',col.vec_7,'LineStyle','--','Tag','asymp-dist');
      hold('on');
   end;
   %
   new_line = plot(x_v,n3,'LineWidth',1,'Color',col.vec_1,'Tag','boot-dist');
   my_lines = [my_lines new_line];
   hold('on');
   this_axes = gca;
   set(this_axes,'YLim',y_lim);
   set(this_axes,'XLim',[min(x_v) max(x_v)]);
   %
   if isempty(est_parm)==0;
      new_line = plot(parm*ones(1,2),y_lim,'Color',col.vec_5,'LineStyle','-','LineWidth',1);
      my_lines = [my_lines new_line];
      hold('on');
   end;
   %
   % plotting mean, median, and x percent confidence interval
   %
   new_line = plot(mean_val*ones(1,2),y_lim,'Color',col.vec_2,'LineStyle',':','LineWidth',1,'Tag','mean');
   my_lines = [my_lines new_line];
   hold('on');
   new_line = plot(med_val*ones(1,2),y_lim,'Color',col.vec_3,'LineStyle','-.','LineWidth',1,'Tag','median');
   my_lines = [my_lines new_line];
   hold('on');
   %
   % Find the mode
   %
   [n3_max,n3_ind] = max(n3);
   if size(x_v,1)>size(x_v,2);
      mode_val = x_v(n3_ind,1);
   else;
      mode_val = x_v(1,n3_ind);
   end;
   %
   % and plot it
   %
   new_line = plot(mode_val*ones(1,2),y_lim,'Color',col.vec_4,'LineStyle','--','LineWidth',1,'Tag','mode');
   my_lines = [my_lines new_line];
   hold('on');
   %
   % fix patch Y axis limits
   %
%   set(my_p,'YData',[y_lim(2) y_lim(1)*ones(1,4) y_lim(2)*ones(1,3)]);
   %
%   my_lines = [my_lines my_p];
   %
   % plotting edge line for confidence band
   %
   plot(l_val*ones(1,2),y_lim,'Color',col.vec_6,'LineStyle',':','LineWidth',1,'Tag','nolegend-lower');
   hold('on');
   plot(u_val*ones(1,2),y_lim,'Color',col.vec_6,'LineStyle',':','LineWidth',1,'Tag','nolegend-upper');
   x_lim = get(gca,'XLim');
   if (x_lim(1,1)<0)&(x_lim(1,2)>0);
      %
      % plot a vertical zero line
      %
      hold('on');
      plot(zeros(1,2),y_lim,'Color','k','LineStyle','-','LineWidth',1)
   end;
   if isempty(est_parm)==0;
      if isempty(as_series)==0;
%         myleg = legend_fcn(my_lines,'Asymptotic','Bootstrap','ML Estimate','Mean','Median','Confidence',leg_pos);
         myleg = legend_fcn(my_lines,'Asymptotic','Bootstrap','ML Estimate','Mean','Median','Mode',leg_pos);
      else;
%         myleg = legend_fcn(my_lines,'Bootstrap','ML Estimate','Mean','Median','Confidence',leg_pos);
         myleg = legend_fcn(my_lines,'Bootstrap','ML Estimate','Mean','Median','Mode',leg_pos);
      end;
   else;
      if isempty(as_series)==0;
%         myleg = legend_fcn(my_lines,'Asymptotic','Bootstrap','Mean','Median','Confidence',leg_pos);
         myleg = legend_fcn(my_lines,'Asymptotic','Bootstrap','Mean','Median','Mode',leg_pos);
      else;
%         myleg = legend_fcn(my_lines,'Bootstrap','Mean','Median','Confidence',leg_pos);
         myleg = legend_fcn(my_lines,'Bootstrap','Mean','Median','Mode',leg_pos);
      end;
   end;
end;
setaxesfonts('Title',p_title,'XLabel','Value','YLabel',ylabelstr);
if isempty(myleg)==0;
   setaxesfonts(myleg);
   myleg = update_legend(myleg);
end;
set(this_axes,'UserData',myleg,'Layer','top');
toolbar_fcn(this_axes);
hold('off');

%
% end of parm_dist.m
%
