function [names,status] = readnames(file,arg)

%
% try to read variable names from file and return
% status of data (1 is successful and 0 is unsuccessful)
%
% NOTE: file is assumed to be a file with numeric data
%       and possible 1 nonempty line with text containing
%       variable names. A line is considered nonempty if
%       it contains nonnumeric characters other than the
%       space character. The number of variable names must
%       match the lowest dimension of the created numeric
%       data matrix for status to report success.
%

status = 0;
names = '';
ext = getext(file);
if strcmp(ext,'.xls')==1;
   if matlabnumber>=6.0;
      if strcmp(arg,'')==1;
         [a,fileline,thisdate] = xlsread_fcn(file);
      else;
         [a,fileline,thisdate] = xlsread_fcn(file,arg);
      end;
      n = size(fileline,2);
      if n>0;
         names = fileline;
         status = 1;
         return;
      else;
         return;
      end;
   else;
      return;
   end;
elseif strcmp(ext,'.wk1')==1;
   [a,fileline,thisdate] = wk1read_fcn(file);
   n = size(fileline,2);
   if n>0;
      names = fileline;
      status = 1;
      return;
   else;
      return;
   end;
elseif strcmp(ext,'.mat')==1;
   if strcmp(arg,'')==1;
      [a,fileline,thisdate] = matread_fcn(file);
   else;
      [a,fileline,thisdate] = matread_fcn(file,arg);
   end;
   n = size(fileline,2);
   if n>0;
      names = fileline;
      status = 1;
      return;
   else;
      return;
   end;
end;

fid = fopen(file,'rt');
fileline = fgetl(fid);
if ischar(fileline)==1;
   fileline = strtrim(fileline);
end;
while isnumeric(fileline)==0;
   c_line = strrep(fileline,',',' ');
   c_line = strrep(fileline,':',' ');
   c_line = strrep(fileline,';',' ');
   c_line = strrep(fileline,'"',' ');
   if (isempty(mystr2num(strrep(c_line,' ','')))==1)&(strcmp(fileline,'')==0);
      status = 1;
      break;
   else;
      fileline = fgetl(fid);
      if ischar(fileline)==1;
         fileline = strtrim(fileline);
      end;
   end;
end;
fclose(fid);

%
% make sure there are not tabs
%

fileline = strrep(fileline,'	',' ');
fileline = strrep(fileline,';',' ');
fileline = strrep(fileline,',',' ');
fileline = strrep(fileline,':',' ');
fileline = strrep(fileline,'"',' ');

if isnumeric(fileline)==1;
   status = 0;
   return;
end;

if isempty(mystr2num(strrep([' ' fileline ' '],' i ',' @ ')))==0;
   status = 0;
   return;
end;
if size(strrep(fileline,' ',''),2)==0;
   status = 0;
   return;
end;

newchars = '';
namesmat = '';
for i=1:size(fileline,2);
   char = fileline(1,i);
   if strcmp(char,' ')==0;
      newchars = strcat(newchars,char);
   else;
      if strcmp(newchars,'')==0;
         namesmat = strvcat(namesmat,newchars);
         newchars = '';
      end;
   end;
   if i==size(fileline,2);
      if strcmp(char,' ')==0;
         namesmat = strvcat(namesmat,newchars);
      end;
   end;
end;

%
% check if number of names matches the number of variables
%

[myfile,stat,dtstr] = loaddata(file);

[T n] = size(myfile);

if n>T;
   t = T;
   T = n;
   n = t;
end;

checknames = '';
for i=1:size(namesmat,1);
    chname = strrep(namesmat(i,:),' ','');
    if strcmpi(chname,'DATE')==0;
       checknames = strvcat(checknames,chname);
    end;
end;

namesmat = checknames;
if abs(n-size(namesmat,1))>0;
   status = 0;
end;

if status==1;
   names = strrep(namesmat(1,:),' ','');
   for i=2:size(namesmat,1);
      names = [names ' ' strrep(namesmat(i,:),' ','')];
   end;
else;
   names = '';
end;

%
% end of readnames.m
%
