function [test,df,pval,params] = rest_const_trend(test_type,epsilon,r,Z0,Z1,Z2,degf,sscorrect)

[n T] = size(epsilon);
lnL_alt = loglike(epsilon);
%
% readjust Z1 and Z2
%
if strcmp(test_type,'uconst')==1;
   Z1 = [Z1;Z2(1,:)];
   if size(Z2,1)>1;
      Z2 = Z2(2:size(Z2,1),:);
   else;
      Z2 = [];
   end;
   d_f = n-r;
elseif strcmp(test_type,'rconst')==1;
   Z1 = Z1(1:size(Z1,1)-1,:);
   d_f = r;
elseif strcmp(test_type,'utrend')==1;
   Z1 = [Z1;Z2(2,:)];
   if size(Z2,1)>2;
      Z2 = [Z2(1,:);Z2(3:size(Z2,1),:)];
   else;
      Z2 = Z2(1,:);
   end;
   d_f = n-r;
elseif strcmp(test_type,'rtrend')==1;
   Z1 = Z1(1:size(Z1,1)-1,:);
   d_f = r;
end;

[S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,lambda,mytrace] = mlcointbasic(Z0,Z1,Z2,r);
%
alpha = (S_01*beta)*inverse(beta'*S_11*beta);
if isempty(Z2)==0;
   Psi = (M_02*inverse(M_22))-(alpha*beta'*M_12*inverse(M_22));
   epsilon_n = Z0-(alpha*beta'*Z1)-(Psi*Z2);
else;
   Psi = [];
   epsilon_n = Z0-(alpha*beta'*Z1);
end;
lnL_new = loglike(epsilon_n);
%
% compute the test statistic
%
LR = 2*(lnL_alt-lnL_new);
%
% save parameters if asked to do do
%
if nargout>3;
   %
   % need to take the model into account
   %
   if strcmp(test_type,'uconst')==1;
      params.Psi = [alpha*beta(size(beta,1),:)' Psi];
      params.beta = beta(1:size(beta,1)-1,:);
   elseif strcmp(test_type,'rconst')==1;
      params.beta = [beta;zeros(1,r)];
      params.Psi = Psi;
   elseif strcmp(test_type,'utrend')==1;
      if size(Z2,1)>1;
         params.Psi = [Psi(:,1) alpha*beta(size(beta,1),:)' Psi(:,2:size(Psi,2))];
      else;
         params.Psi = [Psi alpha*beta(size(beta,1),:)'];
      end;
      params.beta = beta(1:size(beta,1)-1,:);
   elseif strcmp(test_type,'rtrend')==1;
      params.beta = [beta;zeros(1,r)];
      params.Psi = Psi;
   end;
   params.alpha = alpha;
   params.epsilon = epsilon_n;
   params.Omega = (1/T)*epsilon_n*epsilon_n';
end;
%
if sscorrect==0;
   test = LR;
   df = d_f;
   pval = 1-chis_cdf(test,df);
else;
   df = [d_f T-degf+round((d_f)/n)];
   test = (df(1,2)/(T*df(1,1)))*LR;
   pval = 1-fdis_cdf(test,df(1,1),df(1,2));
end;

%
% end of rest_const_trend.m
%
