%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for showing spreadsheet choices     %
%                                                                %
%          Function: sheetdlg                                    %
%                                                                %
%          Input:                                                %
%                       mysheets: text string                    %
%                       filename: text string                    %
%                                                                %
%          Output:                                               %
%          	            action: OK or Cancel                     %
%                       thissheet: selected sheet                %
%                                                                %
%          This version:        May 3, 2006                      %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [action,thissheet] = sheetdlg(mysheets,filename)

scrsz = get(0,'ScreenSize');
curr_ini = find_inisettings;

sheetgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'FileName','sheetdlg.m', ...
   'MenuBar','none', ...
   'Units','pixels', ...
   'Position',[scrsz(3)/2-300 (scrsz(4)-130)/2 600 140], ...
   'Tag','FigSheetDlg', ...
   'Resize','off', ...
   'Visible','off', ...
   'Name',['Select sheet - ' filename], ...
   'NumberTitle','off', ...
   'WindowStyle','modal', ...
   'CloseRequestFcn','closegui', ...
   'ToolBar','none');

if matlabnumber>=7.0;
   set(sheetgui,'DockControl','off');
end;

axesbox([2 2 598 130],'Sheet Selection',78,[0.5 1]);

imageaxes = axes('Units','pixels', ...
   'Box','on', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[14 85 39 39], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','AxisLogo', ...
   'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

mycons = getstoredimages;
curr_ini = find_inisettings;
if curr_ini.use_toolbar_bgcolor==1;
   bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
   bgcolor(1) = max(0,bgcolor(1));
   bgcolor(2) = max(0,bgcolor(2));
   bgcolor(3) = max(0,bgcolor(3));
else;
   bgcolor = get(0,'defaultuicontrolbackgroundcolor');
end;
%
image('CData',mirrorimage(maketrans(getfield(mycons,[strrep(getext(filename),'.','') 'read_image']),[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[12 55 88 20], ...
   'FontSize',curr_ini.gui_fontsize, ...
   'FontName',curr_ini.gui_fontname, ...
   'FontWeight',curr_ini.gui_fontweight, ...
   'FontAngle',curr_ini.gui_fontangle, ...
   'Style','text', ...
   'HorizontalAlignment','left', ...
   'String','Select a sheet:', ...
   'Tag','Text');

sheets.select = uicontrol('Units','pixels', ...
   'BackgroundColor',[1 1 1], ...
   'Position',[100 60 488 20], ...
   'Value',1, ...
   'String',mysheets, ...
   'Style','popupmenu', ...
   'FontSize',curr_ini.gui_fontsize, ...
   'FontName',curr_ini.gui_fontname, ...
   'FontWeight',curr_ini.gui_fontweight, ...
   'FontAngle',curr_ini.gui_fontangle, ...
   'Enable','on', ...
   'Tag','SheetsPopup');

%
% Two buttons to select action
%

sheets.ok = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[222.5 20 75 25], ...
   'String','OK', ...
   'FontSize',curr_ini.gui_fontsize, ...
   'FontName',curr_ini.gui_fontname, ...
   'FontWeight',curr_ini.gui_fontweight, ...
   'FontAngle',curr_ini.gui_fontangle, ...
   'Enable','on', ...
   'Callback','closegui', ...
   'UserData','waiting', ...
   'Tag','ButtonOK');

sheets.cancel = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[302.5 20 75 25], ...
   'String','Cancel', ...
   'FontSize',curr_ini.gui_fontsize, ...
   'FontName',curr_ini.gui_fontname, ...
   'FontWeight',curr_ini.gui_fontweight, ...
   'FontAngle',curr_ini.gui_fontangle, ...
   'Enable','on', ...
   'Callback','closegui', ...
   'Tag','ButtonCancel');

if (iswinnt==1)&(iscompiled==1)&(curr_ini.noicon==1);
   set(sheetgui,'WindowStyle','normal');
   prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(sheetgui,'Name') ' NULL'];
   dos(prog);
   wait_fcn(curr_ini.delay);
   pos = get(sheetgui,'Position');
   set(sheetgui,'Position',pos);
end;
set(sheetgui,'Visible','on');
drawnow;
if (iswinnt==1)&(iscompiled==1);
   if curr_ini.noicon==1;
      drawnow;
      set(sheetgui,'WindowStyle','modal');
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(sheetgui,'Name') ' NULL'];
      dos(prog);
      wait_fcn(curr_ini.delay);
      set(sheetgui,'Position',pos);
   end;
   drawnow;
   if matlabnumber<7;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(sheetgui,'Name') ' 2'];
      dos(prog);
      drawnow;
   else;
      uicontrol(sheets.ok);
   end;
   if curr_ini.noicon==1;
      set(sheetgui,'Position',pos);
   end;
elseif matlabnumber>=7;
   uicontrol(sheets.ok);
end;
waitfor(sheetgui,'waitstatus','inactive');
%uiwait(sheetgui);

action = get(get(sheetgui,'CurrentObject'),'String');

if isempty(action)==1;
   action = 'Cancel';
end;

thissheet = get(sheets.select,'Value');

delete(sheetgui);
drawnow;

%
% end of sheetdlg.m
%
