function h = statusbar(varargin)

%
% Properties:  Position, (default is based on current window size)
%                   this property can't be changed with set command
%              String,
%              FontSize, (default is 7)
%              FontName, (system dependent)
%              FontUnits,
%              FontWeight,
%              FontAngle
%
%

fig = get(0,'CurrentFigure');
if isempty(fig)==1;
   fig = gcf;
   set(fig,'Color',get(0,'defaultuicontrolbackgroundcolor'));
end;
old_figunits = get(fig,'Units');
set(fig,'Units','pixels');
pos = get(fig,'Position');
pos = [1 1 pos(3)-1 16];
num = round(size(varargin,2)/2);
for i=1:num;
   k = 1+2*(i-1);
   if ischar(varargin{k})==1;
      if strcmp(lower(varargin{k}),'position')==1;
         pos = varargin{k+1};
         break;
      end;
   end;
end;

%
% create a box first
%

cu_ini = find_inisettings;

bg_color = get(0,'defaultuicontrolbackgroundcolor');
bg_color = mean(bg_color);
ax_handles = axesbox(pos,'',0,[0.5 bg_color bg_color 1],'on');
set([ax_handles(4) ax_handles(5)],'XColor',get(0,'defaultuicontrolbackgroundcolor'));
set([ax_handles(4) ax_handles(5)],'YColor',get(0,'defaultuicontrolbackgroundcolor'));
set([ax_handles(4) ax_handles(5)],'ZColor',get(0,'defaultuicontrolbackgroundcolor'));
%
for i=1:size(ax_handles,2);
   if strcmp(get(ax_handles(1,i),'Type'),'axes')==1;
      set(ax_handles(1,i),'Tag','gui');
   end;
end;
%
% fix positions of bottom X axes and right Y axis of ax_handles
%
set(ax_handles(1),'Position',get(ax_handles(1),'Position')+[-1 0 1 0]);
set(ax_handles(8),'Position',get(ax_handles(8),'Position')+[0 0 0 1]);

shadevec = 0.5*[1 1 1];
if isalmostequal(shadevec,cu_ini.gui_shade_color)==0;
   shadevec = cu_ini.gui_shade_color;
end;

ax_handle = axes('Units','pixels', ...
   'Position',[pos(3) pos(4)+1 2 1], ...
   'Tag','axes1', ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'XColor',shadevec, ...
   'YColor',shadevec, ...
   'Box','on', ...
   'Visible','on', ...
   'TickLength',[0 0], ...
   'XTickLabel',[], ...
   'YTickLabel',[], ...
   'XTick',[0 1], ...
   'YTick',[0 1]);

ax_handles = [ax_handles ax_handle];

%
% Create a text uicontrol
%

text_handle = uicontrol('Units','pixels', ...
   'Position',[pos(1)+4 pos(2)+1 pos(3)-8 pos(4)-2], ...
   'String','', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'HorizontalAlignment','left', ...
   'FontSize',cu_ini.gui_fontsize, ...
   'FontName',cu_ini.gui_fontname, ...
   'Style','text', ...
   'Visible','on', ...
   'Tag','Statusbar');
if matlabnumber>=7;
   set(text_handle,'Position',[pos(1) pos(2)-1 pos(3)-8 pos(4)-2]);
end;
%
% parse input
%

prop.string = '';
prop.fontsize = 7;
prop.fontname = get(text_handle,'FontName');
prop.fontunits = get(text_handle,'FontUnits');
prop.fontangle = get(text_handle,'FontAngle');
prop.fontweight = get(text_handle,'FontWeight');
prop.handles = ax_handles;
prop.extrapixel = 0;

for i=1:num;
   k = 1+2*(i-1);
   if ischar(varargin{k})==1;
      if strcmp(lower(varargin{k}),'string')==1;
         set(text_handle,'String',varargin{k+1});
         prop.string = varargin{k+1};
      elseif strcmp(lower(varargin{k}),'fontname')==1;
         set(text_handle,'FontName',varargin{k+1});
         prop.fontname = varargin{k+1};
      elseif strcmp(lower(varargin{k}),'fontsize')==1;
         set(text_handle,'FontSize',varargin{k+1});
         prop.fontsize = varargin{k+1};
      elseif strcmp(lower(varargin{k}),'fontunits')==1;
         set(text_handle,'FontUnits',varargin{k+1});
         prop.units = varargin{k+1};
      elseif strcmp(lower(varargin{k}),'fontangle')==1;
         set(text_handle,'FontAngle',varargin{k+1});
         prop.fontangle = varargin{k+1};
      elseif strcmp(lower(varargin{k}),'fontweight')==1;
         set(text_handle,'FontWeight',varargin{k+1});
         prop.fontweight = varargin{k+1};
      elseif strcmp(lower(varargin{k}),'extrapixel')==1;
         extrapixel = varargin{k+1};
         ax_pos = get(ax_handle,'Position');
         if isnumeric(extrapixel)==1;
            set(ax_handle,'Position',ax_pos+[0 extrapixel 0 0]);
         else;
            if strcmp(extrapixel,'off')==1;
               set(ax_handle,'Visible','off');
            end;
         end;
         prop.extrapixel = varargin{k+1};
      end;
   end;
end;

set(text_handle,'UserData',prop);

set(fig,'Units',old_figunits);

if nargout==1;
   h = text_handle;
end;

%
% end of statusbar.m
%
