function [status,num_res,tot_res,B_rand] = test_ident_m1(tm,type_var)

%
% status = 0 means valid identification, 1 means
% too few restrictions, 2 means that B is singular
%

B_rand = [];
status = 1;
curr_ini = find_inisettings;

if strcmp(type_var,'short_run')==1;
   file = strcat(tm.modelsdir,'F-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
   F = loaddata(file);
   num_res = size(F,1);
   tot_res = num_res;
   [Rmat_sr,rvec_sr] = rmatrix(F,tm.n,tm.n);
   if rank(Rmat_sr)==size(F,1);
      if size(Rmat_sr,1)<(tm.n*(tm.n-1))/2;
         return;
      end;
   else;
      return;
   end;
   Rmat = Rmat_sr;
   rvec = rvec_sr;
elseif strcmp(type_var,'long_run')==1;
   file = strcat(tm.modelsdir,'LF-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
   LF = loaddata(file);
   num_res = size(LF,1);
   tot_res = num_res;
   if (tm.r>0)&(tm.r<tm.n)&(tm.r-1>0);
      return;
   end;
   [Rmat_lr,rvec_lr] = rmatrix(LF,tm.n,tm.n);
   if tm.r<=1;
      C = cmat(tm.Psi,tm.alpha,tm.beta,tm.epsilon,tm.Z1,tm.Z2,tm.p,tm.d,0);
   else;
      C = cmat_stat(tm.Psi,tm.alpha,tm.beta,tm.epsilon,tm.Z1,tm.Z2,tm.p,tm.d,0,[]);
   end;
   %
   % check if restricted C makes any long-run restrictions redundant
   %
   if tm.restrict(1,3)==1;
      check_lr_rest = Rmat_lr*vec(C*randn(tm.n,tm.n))-rvec_lr;
      if curr_ini.convcrit<10;
         ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
      else;
         ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
      end;
      ccrit = mystr2num(ch_convcrit);
      k_rest = [];
      for i=1:size(check_lr_rest,1);
         if abs(check_lr_rest(i,1))>ccrit;
            k_rest = [k_rest;i];
         end;
      end;
      if isempty(k_rest)==0;
         Rmat_lr = Rmat_lr(k_rest,:);
         rvec_lr = rvec_lr(k_rest,1);
      else;
         Rmat_lr = zeros(1,size(Rmat_lr,2));
         rvec_lr = 0;
      end;
   end;
   Rmat_lr = Rmat_lr*kron(eye(tm.n),C);
   if rank(Rmat_lr)==size(LF,1);
      if size(Rmat_lr,1)<(tm.n*(tm.n-1))/2;
         return;
      end;
   else;
      return;
   end;
   Rmat = Rmat_lr;
   rvec = rvec_lr;
else;
   %
   % this case allows for tm.r>0 and tm.r<tm.n
   %
   file = strcat(tm.modelsdir,'F-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
   F = loaddata(file);
   [Rmat_sr,rvec_sr] = rmatrix(F,tm.n,tm.n);
   %
   file = strcat(tm.modelsdir,'LF-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
   LF = loaddata(file);
   [Rmat_lr,rvec_lr] = rmatrix(LF,tm.n,tm.n);
   %
   if tm.r<tm.n;
      C = cmat(tm.Psi,tm.alpha,tm.beta,tm.epsilon,tm.Z1,tm.Z2,tm.p,tm.d,0);
   else;
      C = cmat_stat(tm.Psi,tm.alpha,tm.beta,tm.epsilon,tm.Z1,tm.Z2,tm.p,tm.d,0,[]);
   end;
   %
   % check if restricted C makes any long-run restrictions redundant
   %
   if tm.restrict(1,3)==1;
      check_lr_rest = Rmat_lr*vec(C*randn(tm.n,tm.n))-rvec_lr;
      if curr_ini.convcrit<10;
         ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
      else;
         ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
      end;
      ccrit = mystr2num(ch_convcrit);
      k_rest = [];
      for i=1:size(check_lr_rest,1);
         if abs(check_lr_rest(i,1))>ccrit;
            k_rest = [k_rest;i];
         end;
      end;
      if isempty(k_rest)==0;
         Rmat_lr = Rmat_lr(k_rest,:);
         rvec_lr = rvec_lr(k_rest,1);
      else;
         Rmat_lr = zeros(1,size(Rmat_lr,2));
         rvec_lr = 0;
      end;
   end;
   %
   Rmat_lr = Rmat_lr*kron(eye(tm.n),C);
   %
   % need to check the number of restrictions
   %
   Rmat = [Rmat_sr;Rmat_lr];
   rvec = [rvec_sr;rvec_lr];
   num_res = rank(Rmat);
   tot_res = size(Rmat,1);
   if rank(Rmat)==size(Rmat,1);
      if rank(Rmat)<(tm.n*(tm.n-1))/2;
         return;
      end;
   else;
      status = 3;
      return;
   end;
end;

%
% create a random B which satisfies the restrictions and check its rank
%

S = null(Rmat,'r');
d = (Rmat'/(Rmat*Rmat'))*rvec;
vB = (S*randn(size(S,2),1))+d;
B = vecinv(vB,tm.n);
B_rand = B;
if rank(B)==tm.n;
   %
   % Need to check for identification using Rothenberg (1971, Theorem 2)
   %
   B_inv = B\eye(tm.n);
   Nn = ncommutation(tm.n);
   i_b = kron(eye(tm.n),B_inv);
   info_mat = 2*i_b'*Nn*i_b;
   test_mat = [info_mat;Rmat];
   if rank(test_mat)==floor(tm.n*tm.n);
      status = 0;
   end;
else;
   status = 2;
end;

%
% end of test_ident_m1.m
%
