function toolbar_paperposition_fcn

h = get(0,'CurrentFigure');
toolbar = get(h,'UserData');

mypaperunits = get(h,'PaperUnits');
mypaperposition = get(h,'PaperPosition');

scrsz = get(0,'ScreenSize');

ppdlg = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'FileName','toolbar_paperposition_fcn.m', ...
   'MenuBar','none', ...
   'Units','pixels', ...
   'Position',[scrsz(3)/2-200 (scrsz(4)-115)/2 400 115], ...
   'Tag','PaperPositionDlg', ...
   'Resize','off', ...
   'Name',['Set New Paper Position - ' mypaperunits], ...
   'NumberTitle','off', ...
   'WindowStyle','modal', ...
   'CloseRequestFcn','closegui', ...
   'ToolBar','none');

axesbox([2 2 398 108],'Select Paper Position',105,[0.5 0.9]);

%
% Display current paper position
%

uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[12 75 86.5 20], ...
   'FontSize',6, ...
   'Style','text', ...
   'HorizontalAlignment','center', ...
   'String','Left:', ...
   'Tag','TextLeft');

ppgui.left = uicontrol('Units','pixels', ...
   'BackgroundColor',[1 1 1], ...
   'Position',[12 50 86.5 20], ...
   'String',num2str(mypaperposition(1)), ...
   'Enable','on', ...
   'FontSize',6, ...
   'Style','edit', ...
   'Tag','EditLeft');

uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[108.5 75 86.5 20], ...
   'FontSize',6, ...
   'Style','text', ...
   'HorizontalAlignment','center', ...
   'String','Bottom:', ...
   'Tag','TextLeft');

ppgui.bottom = uicontrol('Units','pixels', ...
   'BackgroundColor',[1 1 1], ...
   'Position',[108.5 50 86.5 20], ...
   'String',num2str(mypaperposition(2)), ...
   'Enable','on', ...
   'FontSize',6, ...
   'Style','edit', ...
   'Tag','EditBottom');

uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[205 75 86.5 20], ...
   'FontSize',6, ...
   'Style','text', ...
   'HorizontalAlignment','center', ...
   'String','Width:', ...
   'Tag','TextWidth');

ppgui.width = uicontrol('Units','pixels', ...
   'BackgroundColor',[1 1 1], ...
   'Position',[205 50 86.5 20], ...
   'String',num2str(mypaperposition(3)), ...
   'Enable','on', ...
   'Style','edit', ...
   'FontSize',6, ...
   'Tag','EditWidth');

uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[301.5 75 86.5 20], ...
   'FontSize',6, ...
   'Style','text', ...
   'HorizontalAlignment','center', ...
   'String','Height:', ...
   'Tag','TextHeight');

ppgui.height = uicontrol('Units','pixels', ...
   'BackgroundColor',[1 1 1], ...
   'Position',[301.5 50 86.5 20], ...
   'String',num2str(mypaperposition(4)), ...
   'Enable','on', ...
   'Style','edit', ...
   'FontSize',6, ...
   'Tag','EditHeight');

%
% OK and Cancel buttons
%

ppgui.ok = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[148 12 50 25], ...
   'String','OK', ...
   'FontSize',6, ...
   'Enable','on', ...
   'Callback','closegui', ...
   'UserData','waiting', ...
   'Tag','Button1');

ppgui.cancel = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[202 12 50 25], ...
   'String','Cancel', ...
   'FontSize',6, ...
   'Enable','on', ...
   'Callback','closegui', ...
   'Tag','Button2');


uiwait(ppdlg);

answer = get(get(ppdlg,'CurrentObject'),'String');

text = '';
if strcmp(answer,'Cancel')==0;
   %
   % Check if paper position is valid
   %
   newleft = strrep(get(ppgui.left,'String'),' ','');
   newbottom = strrep(get(ppgui.bottom,'String'),' ','');
   newwidth = strrep(get(ppgui.width,'String'),' ','');
   newheight = strrep(get(ppgui.height,'String'),' ','');
   newpaperposition = [mystr2num(newleft) mystr2num(newbottom) mystr2num(newwidth) mystr2num(newheight)];
   if (isnumeric(newpaperposition)==1)&(size(newpaperposition,2)==4);
      if newpaperposition>zeros(1,4);
         set(h,'PaperPosition',newpaperposition);
         %
         % Check if too big by temporarily switching to normalized units
         %
         set(h,'PaperUnits','normalized');
         check = get(h,'PaperPosition');
         check = [check(1)+check(3) check(2)+check(4)];
         set(h,'PaperUnits',mypaperunits);
         if (check(1,1)>1.07)|(check(1,2)>1.07);
            text = 'The paper position you selected may cause the graph to be printed outside the selected paper.';
         end;
      else;
         delete(ppdlg);
         drawnow;
         text = ['The paper position you selected, (left=' newleft ',bottom=' newbottom ',width=' newwidth ',height=' newheight '), is not valid. Unable to change paper position.'];
         about(strvcat(' ',text),'information','Bad paper position...');
         drawnow;
         return;
      end;
   else;
      delete(ppdlg);
      drawnow;
      text = ['The paper position you selected, (left=' newleft ',bottom=' newbottom ',width=' newwidth ',height=' newheight '), is not valid. Unable to change paper position.'];
      about(strvcat(' ',text),'information','Bad paper position...');
      drawnow;
      return;
   end;
end;

delete(ppdlg);
drawnow;

if strcmp(text,'')==0;
   about(strvcat(' ',text),'warning','Warning - Large Paper Position');
   drawnow;
end;

%
% end of toolbar_paperposition_fcn.m
%
