%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for reordering model files          %
%                                                                %
%          Function: updowndlg                                   %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       files: string matrix                     %
%                       name: figure title                       %
%                       axlist: axesbox title                    %
%                       axlength: width of axesbox title         %
%                                                                %
%          Output:                                               %
%                       h = handle                               %
%                                                                %
%          This version:        May 18, 2006                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = updowndlg(selector,files,name,axlist,axlength)

if strcmp(selector,'init')==1;
   thisexit = 0;
   tm.modelfiles = files;
   if size(tm.modelfiles,1)>3;
      height = size(tm.modelfiles,1)*18+94;
   else;
      height = (size(tm.modelfiles,1)+2)*18+94;
   end;
   tm.oldmodelfiles = tm.modelfiles;
   %
   % button icons
   %
   mycons = getstoredimages;
   cu_ini = find_inisettings;
   if cu_ini.use_toolbar_bgcolor==1;
      bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
      bgcolor(1) = max(0,bgcolor(1));
      bgcolor(2) = max(0,bgcolor(2));
      bgcolor(3) = max(0,bgcolor(3));
   else;
      bgcolor = get(0,'defaultuicontrolbackgroundcolor');
   end;
   myup = maketrans(mycons.up_image,[160 160 164],bgcolor);
   mydown = maketrans(mycons.down_image,[160 160 164],bgcolor);
   mydel = maketrans(mycons.delete_image,[160 160 164],bgcolor);
   %
   % current model
   %
   tm.curr_modelfile = cu_ini.model.inputfile;
   %
   % Set screen size directly instead of using
   % get(0,'ScreenSize')!
   %
   realscrsz = get(0,'ScreenSize');
   if height>realscrsz(4)-100;
      height = realscrsz(4)-100;
   elseif height<152;
      height = 152;
   end;
else;
   thisexit = 1;
   updowngui = get(0,'CurrentFigure');
   updown = get(updowngui,'UserData');
   tm = get(updown.thisfile,'UserData');
end;

switch selector;
   case 'init';
      updowngui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','updowndlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-712)/2 (realscrsz(4)-height)/2 712 height], ...
         'Tag','FigParent', ...
         'Resize','off', ...
         'Name',name, ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','updowndlg exit', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(updowngui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 710 height-14],axlist,axlength,[0.5 1]);

      %
      % Displaying the model files
      %

      updown.thisfile = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[20 60 600 height-94], ...
         'FontSize',cu_ini.gui_fontsize+2, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'Style','listbox', ...
         'Enable','on', ...
         'Max',1, ...
         'HorizontalAlignment','center', ...
         'Callback','updowndlg modelinfo', ...
         'String',tm.modelfiles, ...
         'Tag','Text1',...
         'Value',1);

      updown.icon = uicontextmenu('Callback','set(gcf,''Pointer'',''arrow'')');
         uimenu('Parent',updown.icon,'Label','View Model Summary','Callback','updowndlg(''modelinfo'')');
         uimenu('Parent',updown.icon,'Separator','on','Label','Move up','Callback','updowndlg(''up'')');
         uimenu('Parent',updown.icon,'Label','Move down','Callback','updowndlg(''down'')');
         uimenu('Parent',updown.icon,'Separator','on','Label','Delete model from list','Callback','updowndlg(''deletemodel'')');
         set(updown.thisfile,'UIContextMenu',updown.icon);
      %
      % Move models up and down
      %

      checkh = (realscrsz(4)-100)/2;

      updown.up = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[642 min(size(tm.modelfiles,1)*9+76,checkh+19) 50 22], ...
         'TooltipString','Move up', ...
         'CData',myup, ...
         'CallBack','updowndlg up', ...
         'Enable','on', ...
         'Tag','UpButton');


      updown.down = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[642 min(size(tm.modelfiles,1)*9+50,checkh-7) 50 22], ...
         'TooltipString','Move down', ...
         'CData',mydown, ...
         'CallBack','updowndlg down', ...
         'Enable','on', ...
         'Tag','DownButton');

      updown.deletemodel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[642 min(size(tm.modelfiles,1)*9+16,checkh-41) 50 22], ...
         'TooltipString','Delete model from list', ...
         'CData',mydel, ...
         'CallBack','updowndlg deletemodel', ...
         'Enable','on', ...
         'Tag','DeleteFileButton');

      %
      % OK, Cancel and Apply buttons
      %

      updown.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[211 14 90 25], ...
         'String','OK', ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'CallBack','updowndlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      updown.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[311 14 90 25], ...
         'String','Cancel', ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'CallBack','updowndlg cancel', ...
         'Enable','on', ...
         'UserData',files, ...
         'Tag','CancelButton');

      updown.apply = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[411 14 90 25], ...
         'String','Apply', ...
         'FontSize',cu_ini.gui_fontsize, ...
         'FontName',cu_ini.gui_fontname, ...
         'FontWeight',cu_ini.gui_fontweight, ...
         'FontAngle',cu_ini.gui_fontangle, ...
         'CallBack','updowndlg apply', ...
         'Enable','off', ...
         'UserData','waiting', ...
         'Tag','ApplyButton');

      set(updowngui,'UserData',updown,...
         'HandleVisibility','callback');

   case 'modelinfo'
      currval = get(updown.thisfile,'Value');
      currfile = strtrim(tm.modelfiles(currval,:));
      sel_type = get(updowngui,'SelectionType');
      if (strcmp(sel_type,'open')==1)|(gcbo~=updown.thisfile);
         curr_text = readtext(currfile);
         %
         show_text = strvcat(' ','                    MODEL INFORMATION','                    =================',' ');
         show_text = strvcat(show_text,['Model file:                    ' currfile],' ');
         this_txt = strrep(curr_text{4},'endogenous=','');
         this_txt = strrep(this_txt,' ','#');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         this_txt = vecstr(strtrim(this_txt));
         show_text = strvcat(show_text,['Endogenous variables data:     ' strrep(strtrim(this_txt(1,:)),'#',' ')]);
         if size(this_txt,1)>1;
            show_text = strvcat(show_text,['Data sheet:                    ' strtrim(this_txt(2,:))]);
         end;
         %
         this_txt = strrep(curr_text{11},'evars=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         show_text = strvcat(show_text,['Variable names:                ' strtrim(this_txt)],' ');
         %
         this_txt = strrep(curr_text{6},'exogenous1=','');
         this_txt = strrep(this_txt,' ','#');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         if strcmp(strtrim(this_txt),'')==0;
            this_txt = vecstr(strtrim(this_txt));
            show_text = strvcat(show_text,['Exogenous I(1) variables data: ' strrep(strtrim(this_txt(1,:)),'#',' ')]);
            if size(this_txt,1)>1;
               show_text = strvcat(show_text,['Data sheet:                    ' strtrim(this_txt(2,:))]);
            end;
            this_txt = strrep(curr_text{12},'xvars1=','');
            this_txt = strrep(this_txt,'$',' ');
            this_txt = strrep(this_txt,'|','');
            show_text = strvcat(show_text,['Variable names:                ' strtrim(this_txt)],' ');
         end;
         %
         this_txt = strrep(curr_text{7},'exogenous0=','');
         this_txt = strrep(this_txt,' ','#');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         if strcmp(strtrim(this_txt),'')==0;
            this_txt = vecstr(strtrim(this_txt));
            show_text = strvcat(show_text,['Exogenous I(0) variables data: ' strrep(strtrim(this_txt(1,:)),'#',' ')]);
            if size(this_txt,1)>1;
               show_text = strvcat(show_text,['Data sheet:                    ' strtrim(this_txt(2,:))]);
            end;
            this_txt = strrep(curr_text{13},'xvars0=','');
            this_txt = strrep(this_txt,'$',' ');
            this_txt = strrep(this_txt,'|','');
            show_text = strvcat(show_text,['Variable names:                ' strtrim(this_txt)],' ');
         end;
         %
         this_txt = strrep(curr_text{8},'deterministic=','');
         this_txt = strrep(this_txt,' ','#');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         if strcmp(strtrim(this_txt),'')==0;
            this_txt = vecstr(strtrim(this_txt));
            show_text = strvcat(show_text,['Deterministic variables data: ' strrep(strtrim(this_txt(1,:)),'#',' ')]);
            if size(this_txt,1)>1;
               show_text = strvcat(show_text,['Data sheet:                    ' strtrim(this_txt(2,:))]);
            end;
            this_txt = strrep(curr_text{14},'deter=','');
            this_txt = strrep(this_txt,'$',' ');
            this_txt = strrep(this_txt,'|','');
            show_text = strvcat(show_text,['Variable names:                ' strtrim(this_txt)],' ');
         end;
         %
         %
         this_txt = strrep(curr_text{1},'lag=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         show_text = strvcat(show_text,['   * Lag order:                ' strtrim(this_txt)]);
         %
         this_txt = strrep(curr_text{15},'sample=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         this_vec = mystr2num(strtrim(this_txt));
         if size(this_vec,2)==4;
            show_text = strvcat(show_text,['   * Actual sample:            ' num2str(this_vec(1),'%0.0f') ':' num2str(this_vec(2),'%0.0f') ' - ' num2str(this_vec(3),'%0.0f') ':' num2str(this_vec(4),'%0.0f')]);
         elseif size(this_vec,2)==2;
            show_text = strvcat(show_text,['   * Actual sample:            ' num2str(this_vec(1),'%0.0f') ':1 - ' num2str(this_vec(2),'%0.0f') ':1']);
         end;
         %
         this_txt = strrep(curr_text{16},'subsample=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         this_vec = mystr2num(strtrim(this_txt));
         if size(this_vec,2)==4;
            show_text = strvcat(show_text,['   * Selected sample:          ' num2str(this_vec(1),'%0.0f') ':' num2str(this_vec(2),'%0.0f') ' - ' num2str(this_vec(3),'%0.0f') ':' num2str(this_vec(4),'%0.0f')]);
         elseif size(this_vec,2)==2;
            show_text = strvcat(show_text,['   * Selected sample:          ' num2str(this_vec(1),'%0.0f') ':1 - ' num2str(this_vec(2),'%0.0f') ':1']);
         end;
         %
         this_txt = strrep(curr_text{21},'frequency=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strtrim(strrep(this_txt,'|',''));
         if strcmp(this_txt,'q')==1;
            freq = 'quarterly';
         elseif strcmp(this_txt,'m')==1;
            freq = 'monthly';
         else;
            freq = 'annual';
         end;
         show_text = strvcat(show_text,['   * Data frequency:           ' freq]);
         %
         this_txt = strrep(curr_text{18},'deterselect=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         this_vec = mystr2num(strtrim(this_txt));
         if this_vec(1)==1;
            const_type = 'Unrestricted constant';
         elseif this_vec(2)==1;
            const_type = 'Restricted constant';
         elseif this_vec(3)==1;
            const_type = 'Unrestricted linear trend';
         elseif this_vec(4)==1;
            const_type = 'Restricted linear trend';
         end;
         show_text = strvcat(show_text,['   * ' const_type]);
         %
         this_txt = strrep(curr_text{20},'seasonal=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         this_vec = mystr2num(strtrim(this_txt));
         if this_vec(1)==1;
            const_type = 'Centered seasonal dummies';
         elseif this_vec(2)==1;
            const_type = 'No seasonal dummies';
         end;
         show_text = strvcat(show_text,['   * ' const_type]);
         %
         this_txt = strrep(curr_text{10},'cointegration=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         this_vec = mystr2num(strtrim(this_txt));
         if this_vec(1)==1;
            const_type = 'Cointegration estimation';
         else;
            const_type = 'No cointegration estimation';
         end;
         show_text = strvcat(show_text,['   * ' const_type]);
         %
         this_txt = strrep(curr_text{19},'modeltype=','');
         this_txt = strrep(this_txt,'$',' ');
         this_txt = strrep(this_txt,'|','');
         model_vec = mystr2num(strtrim(this_txt));
         if model_vec(1)==1;
            const_type = 'Structural form model';
         else;
            const_type = 'Reduced form model';
         end;
         show_text = strvcat(show_text,['   * ' const_type]);
         %
         if (size(curr_text,2)>21)&(model_vec(1)==1);
            this_txt = strrep(curr_text{22},'shocks=','');
            this_txt = strrep(this_txt,'$',' ');
            this_txt = strtrim(strrep(this_txt,'|',''));
            if strcmp(this_txt,'')==0;
               show_text = strvcat(show_text,['   * Structural shocks:        ' this_txt]);
            end;
         end;
         show_text = strvcat(show_text,' ');
         %
         set(updowngui,'Visible','off');
         drawnow;
         %
         % display model information
         %
         realscrsz = get(0,'ScreenSize');
         width = 800;
         height = 500;
         if realscrsz(3)>1024;
            width = 1000;
         end;
         textgui(show_text,['Model file - ' getfilename(currfile)],[(realscrsz(3)-width)/2 (realscrsz(4)-height)/2 width height],'Model Information',85);
         %
         set(updowngui,'Visible','on');
         drawnow;
      end;
   case 'deletemodel'
      currval = get(updown.thisfile,'Value');
      currfile = strtrim(tm.modelfiles(currval,:));
      if strcmp(currfile,tm.curr_modelfile)==0;
         txt = ['Are you sure you want to delete the model file "' getname(currfile) '" from the list?'];
         answer = query(txt,'delete',120,'Delete File From List',350,'no');
         if strcmp(lower(answer),'yes')==1;
            set(updown.apply,'Enable','on');
            if currval==1;
               tm.modelfiles = tm.modelfiles(2:size(tm.modelfiles,1),:);
               newval = 1;
            elseif currval==size(tm.modelfiles,1);
               tm.modelfiles = tm.modelfiles(1:size(tm.modelfiles,1)-1,:);
               newval = currval-1;
            else;
               tm.modelfiles = strvcat(tm.modelfiles(1:currval-1,:),tm.modelfiles(currval+1:size(tm.modelfiles,1),:));
               newval = currval;
            end;
            set(updown.thisfile,'String',tm.modelfiles);
            set(updown.thisfile,'Value',newval);
         end;
      else;
         txt = 'You are not allowed to delete the current model file from the list.';
         about(txt,'information','Illegal action...',120,350);
      end;
   case 'up';
      currval = get(updown.thisfile,'Value');
      if currval>1;
         set(updown.apply,'Enable','on');
         if currval==2;
            if size(tm.modelfiles,1)>2;
               tm.modelfiles = strvcat(tm.modelfiles(2,:),tm.modelfiles(1,:),tm.modelfiles(3:size(tm.modelfiles,1),:));
            else;
               tm.modelfiles = strvcat(tm.modelfiles(2,:),tm.modelfiles(1,:));
            end;
         elseif currval==size(tm.modelfiles,1);
            tm.modelfiles = strvcat(tm.modelfiles(1:currval-2,:),tm.modelfiles(currval,:),tm.modelfiles(currval-1,:));
         else;
            tm.modelfiles = strvcat(tm.modelfiles(1:currval-2,:),tm.modelfiles(currval,:),tm.modelfiles(currval-1,:),tm.modelfiles(currval+1:size(tm.modelfiles,1),:));
         end;
         set(updown.thisfile,'String',tm.modelfiles);
         set(updown.thisfile,'Value',currval-1);
      end;
   case 'down';
      currval = get(updown.thisfile,'Value');
      if currval<size(tm.modelfiles,1);
         set(updown.apply,'Enable','on');
         if currval==1;
            if size(tm.modelfiles,1)>2;
               tm.modelfiles = strvcat(tm.modelfiles(2,:),tm.modelfiles(1,:),tm.modelfiles(3:size(tm.modelfiles,1),:));
            else;
               tm.modelfiles = strvcat(tm.modelfiles(2,:),tm.modelfiles(1,:));
            end;
         elseif currval==size(tm.modelfiles,1)-1;
            tm.modelfiles = strvcat(tm.modelfiles(1:currval-1,:),tm.modelfiles(currval+1,:),tm.modelfiles(currval,:));
         else;
            tm.modelfiles = strvcat(tm.modelfiles(1:currval-1,:),tm.modelfiles(currval+1,:),tm.modelfiles(currval,:),tm.modelfiles(currval+2:size(tm.modelfiles,1),:));
         end;
         set(updown.thisfile,'String',tm.modelfiles);
         set(updown.thisfile,'Value',currval+1);
      end;
   case 'cancel';
      tm.modelfiles = tm.oldmodelfiles;
      set(updown.cancel,'UserData',tm.modelfiles);
      set(updown.thisfile,'String',tm.modelfiles);
      set(updown.thisfile,'Value',1);
      set(updown.apply,'Enable','off');
      set(updowngui,'UserData',updown);
%      updowndlg('apply');
   case 'apply';
      set(updown.cancel,'UserData',tm.modelfiles);
      set(updown.apply,'Enable','off');
      set(updowngui,'UserData',updown);
   case 'done';
      set(updown.done,'UserData','done');
   case 'exit';
      set(updown.done,'UserData','done');
%      if strcmp(get(updown.apply,'UserData'),'done')==1;
%         delete(updowngui);
%      end;
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(cu_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(updowngui,'Name') ' ' fix_filepath([pwd '\icons\contents.ico'])];
      dos(prog);
      if (strcmp(cu_ini.pstoolsdir,'')==0)&(appexist([cu_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(updown.icon,'UserData',pid);
      end;
   end;
   set(updowngui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(updowngui,'Name') ' 3'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(updown.done);
   end;
end;

if nargout==1;
   h = updowngui;
   drawnow;
end;

if strcmp(selector,'exit')==0;
   set(updown.thisfile,'UserData',tm);
end;

if thisexit==0;
   waitfor(updown.done,'UserData','done');
end;

%
% end of updowndlg.m
%
