function wk1write_fcn(filename, m, r, c)
%WK1WRITE Write spreadsheet (WK1) file.
%   WK1WRITE('FILENAME',M) writes matrix M into a Lotus WK1 spreadsheet
%   file with the name.  '.wk1' is appended to the filename if no
%   extension is given.
%
%   WK1WRITE('FILENAME',M,R,C) writes matrix M into a Lotus WK1 spreadsheet
%   file, starting at offset row R, and column C in the file.  R and C
%   are zero-based so that R=C=0 is the first cell in the spreadsheet.
%
%   See also WK1READ, DLMREAD, DLMWRITE.

%   Brian M. Bourgault 10/22/93
%   Copyright (c) 1984-98 by The MathWorks, Inc.
%   $Revision: 5.10 $  $Date: 1997/11/21 23:37:02 $
%
% include WK1 constants
%
%wk1const
if 0
LOTTABLE    = [ 24 24 ];
LOTQRANGE   = [ 25 25 ];
LOTPRANGE   = [ 26 8 ];
LOTSRANGE   = [ 27 8 ];
LOTFRANGE   = [ 28 8 ];
LOTKRANGE1  = [ 29 9 ];
LOTDRANGE   = [ 32 0 ];
LOTKRANGE2  = [ 35 9 ];
LOTPROTECT  = [ 36 1 ];
LOTFOOTER   = [ 37 242 ];
LOTHEADER   = [ 38 242 ];
LOTSETUP    = [ 39 40 ];
LOTTITLES   = [ 42 16 ];
LOTGRAPH    = [ 45 437 ];
LOTNGRAPH   = [ 46 455 ];
LOTFORMAT   = [ 48 1 ];
LOTWINDOW   = [ 50 144 ];
LOTSTRING   = [ 51 0 ];
LOTLOCKPASS = [ 55 0 ];
LOTLOCKED   = [ 56 0 ];
LOTQSET     = [ 60 0 ];
LOTCQUERY   = [ 61 0 ];
LOTPSET     = [ 62 0 ];
LOTCPRINT   = [ 63 0 ];
LOTSGRAPH   = [ 64 0 ];
LOTCGRAPH   = [ 65 0 ];
LOTZOOM     = [ 66 0 ];
LOTNSPLIT   = [ 67 0 ];
LOTNSROWS   = [ 68 0 ];
LOTNSCOLS   = [ 69 0 ];
LOTRULER    = [ 70 0 ];
LOTSNRANGE  = [ 71 0 ];
LOTACOMM    = [ 72 0 ];
LOTAMACRO   = [ 73 0 ];
LOTQPARSE   = [ 74 0 ];
LOTWKSPASS  = [ 75 0 ];
LOTHIDCOL2  = [ 101 32 ];
LOTPARSE    = [ 102 16 ];
LOTRRANGES  = [ 103 25 ];
LOTMRANGES  = [ 105 40 ];
else
%
% currently used
%
LOTBEGIN    = [ 0 0 ];
LOTEND      = [ 1 0 ];
LOTCALCMOD  = [ 2 1 ];
LOTCALCORD  = [ 3 1 ];
LOTSPLTWM   = [ 4 1 ];
LOTSPLTWS   = [ 5 1 ];
LOTDIMENSIONS   = [ 6 8 ];
LOTWINDOW1  = [ 7 32 ];
LOTCOLWID1      = [ 8 3 ];
LOTCOLW1        = [ 8 3 ];
LOTCOLW2    = [ 10 3 ];
LOTNMRNG    = [ 11 24 ];
LOTNRANGE   = [ 11 24 ];
LOTBLANK    = [ 12 5 ];
LOTINTEGER  = [ 13 7 ];
LOTNUMBER   = [ 14 13 ];
LOTLABEL    = [ 15 -1 ];
LOTFORMULA  = [ 16 -1 ];
%
LOTMARGINS  = [ 40 10 ];
LOTLABELFMT = [ 41 1 ];
%
LOTCALCCNT  = [ 47 1 ];
%
LOTCURSORW12    = [ 49 1 ];
%
LOTHIDCOL   = [ 100 32 ];
LOTCPI      = [ 150 6 ];
end
%
% constants
%
LOTWK1BOFSTR = [0 0 2 0 6 4];
LOTEOFSTR   = [1 0 0 0];

%
% test for proper filename
%
if ~isstr(filename),
    error('FILENAME must be a string.');
end;

if nargin < 2
    error('Requires at least 2 input arguments.');
end

%
% open the file Lotus uses Little Endian Format ONLY
%
if ~isempty(filename) & all(filename~='.'),
  filename = [filename '.wk1'];
end
comp=computer;
if strcmp(comp(1:3),'MAC'),
  fid = fopen(filename,'wt', 'l');
else
  fid = fopen(filename,'wb', 'l');
end

if fid == (-1)
    error(['Could not open file ' filename '.']);
end
%
% check for row,col offsets
%
if nargin < 3
    r = 0;
end
if nargin < 4
    c = 0;
end

%
% Lotus WK1 BOF
%
fwrite(fid, LOTWK1BOFSTR,'uchar');

%
% Lotus WK1 dimensions size of matrix
[br,bc] = size(m);
LOTrng = [0 0 bc br];
wk1wrec(fid, LOTDIMENSIONS, 0);
fwrite(fid, LOTrng, 'ushort');

%
% Lotus WK1 cpi
%
wk1wrec(fid, LOTCPI, 0);
fwrite(fid, [0 0 0 0 0 0], 'uchar');

%
% Lotus WK1 calcount
%
wk1wrec(fid, LOTCALCCNT, 0);
fwrite(fid, 0, 'uchar');

% Lotus WK1 calcmode
wk1wrec(fid, LOTCALCMOD, 0);
fwrite(fid, -1, 'char');

%
% Lotus WK1 calorder
%
wk1wrec(fid, LOTCALCORD, 0);
fwrite(fid, 0, 'char');

%
% Lotus WK1 split
%
wk1wrec(fid, LOTSPLTWM, 0);
fwrite(fid, 0, 'char');

%
% Lotus WK1 sync
%
wk1wrec(fid, LOTSPLTWS, 0);
fwrite(fid, 0, 'char');

%
% Lotus WK1 cursor12
%
wk1wrec(fid, LOTCURSORW12, 0);
fwrite(fid, 1, 'char');

%
% Lotus WK1 window1, for now but needs work !!!
%
deffmt = 113;
wk1wrec(fid, LOTWINDOW1, 0);
fwrite(fid, [0 0], 'ushort');
fwrite(fid, deffmt, 'char');    
fwrite(fid, 0, 'char'); 
fwrite(fid, 10, 'ushort');
fwrite(fid, [bc br], 'ushort');
fwrite(fid, [0 0 0 0], 'ushort');
fwrite(fid, [0 0 0 0], 'ushort');
fwrite(fid, [72 0], 'ushort');

%
% Lotus WK1 hidcol
%
x = 1:LOTHIDCOL(2);
buf = ones(size(x)) * 0;
wk1wrec(fid, LOTHIDCOL, 0);
fwrite(fid, buf, 'char');

%
% Lotus WK1 margins
%
buf = [4 76 66 2 2];
wk1wrec(fid, LOTMARGINS, 0);
fwrite(fid, buf, 'ushort');

%
% Lotus WK1 labelfmt
%
wk1wrec(fid, LOTLABELFMT, 0);
fwrite(fid, '''', 'char');

%
% start dumping the array, for now number format float
%
for i = 1:br
    for j = 1:bc
       if (iscell(m)==1)&(ischar(m{i,j})==1);
          my_str = m{i,j};
          wk1wrec(fid, LOTLABEL, 0);
%          wk1wrec(fid, LOTNUMBER, 0);
%          wk1wrec(fid, [15 -1], 0);
%          wk1wrec(fid, LOTLABELFMT, 0);
          fwrite(fid, deffmt, 'uchar');
          fwrite(fid, [ c+j-1 r+i-1 ], 'ushort');
          count = fwrite(fid, m{i,j}, 'uchar');
       else;
          wk1wrec(fid, LOTNUMBER, 0);
          fwrite(fid, deffmt, 'char');    
          fwrite(fid, [ c+j-1 r+i-1 ], 'ushort');
          if iscell(m)==1;
             fwrite(fid, m{i,j}, 'double');
          else;
             fwrite(fid, m(i,j), 'double');
          end;
       end;
    end
end


%
% Lotus WK1 EOF
%
fwrite(fid, LOTEOFSTR,'uchar');

% close files
fclose(fid);

%
% end of wk1write.fcn
%
