%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI line                                       %
%                                                                %
%          Function: yline                                       %
%                                                                %
%          Input:                                                %
%                       boxvec: 1x3 vector with left, bottom,    %
%          		    and height                           %
%                       mycolors: 1x2 vector with dark and       %
%                          light color values (0 -> 1)           %
%                       cu_ini: structure with ini settings      %
%                                                                %
%          Output:                                               %
%          		y axis boxhandle                         %
%                                                                %
%          This version:        May 30, 2003                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2002          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function boxhandle = yline(boxvec,mycolors,cu_ini)

if nargin==2;
   cu_ini = find_inisettings;
end;

left = boxvec(1,1);
bottom = boxvec(1,2);
height = boxvec(1,3);

if nargin==1;
   dark = 0.5;
   light = 1.0;
else;
   dark = mycolors(1,1);
   light = mycolors(1,2);
   if light<dark;
      check = dark;
      dark = light;
      light = check;
   elseif light==dark;
      light = 1.0;
      if dark>=light;
         dark = 0.5;
      end;
   end;
end;

lightvec = [light light light];
darkvec = [dark dark dark];

if isalmostequal(darkvec,cu_ini.gui_shade_color)==0;
   darkvec = cu_ini.gui_shade_color;
end;

if isalmostequal(lightvec,cu_ini.gui_light_color)==0;
   lightvec = cu_ini.gui_light_color;
end;

% the left y axis

h1 = axes('Units','pixels', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[left bottom 1 height], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','Y1', ...
   'XColor',darkvec, ...
   'YColor',darkvec, ...
   'ZColor',darkvec);

h2 = axes('Units','pixels', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[left+1 bottom 1 height], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','Y2', ...
   'XColor',lightvec, ...
   'YColor',lightvec, ...
   'ZColor',lightvec);

if matlabnumber>=6.0;
   h3 = axes('Units','pixels', ...
      'TickLength',[0 0], ...
      'CameraUpVector',[0 1 0], ...
      'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
      'Position',[left+2 bottom 1 1], ...
      'XTickLabelMode','manual', ...
      'YTickLabelMode','manual', ...
      'Tag','Y2', ...
      'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      'ZColor',get(0,'defaultuicontrolbackgroundcolor'));
end;

if nargout>0;
   if matlabnumber>=6.0;
      boxhandle = [h1 h2 h3];
   else;
      boxhandle = [h1 h2];
   end;
end;

%
% end of yline.m
%
